@@ -32,10 +32,9 @@ void setVariable(const string& in, map<string, string>& variables)
 {
 	if(in[0] != '-')
 	{
-		Log::warning() << "argument " << in << " ignored.\n";
+		MagLog::warning() << "argument " << in << " ignored."<<endl;
 		return;
 	}
-
 	variables.insert(magml::cut(in.substr(1, in.size())));
 }
 
@@ -43,34 +42,33 @@ void setVariable(const string& in, map<string, string>& variables)
 
 int main(int argc, char **argv)
 {
-    try
+  try
     {
-	    if(argc<2)
-	    {
-		    Log::error() << " Usage: " << argv[0] << " file.magml [-varname=varvalue]\n";		   
-		    exit (1);
-	    }
+	if(argc<2)
+	{
+		cout << " Usage: " << argv[0] << " file.json [-varname=varvalue]"<< endl;
+		exit (1);
+	}
 
-	    map<string, string> variables;
+	map<string, string> variables;
 
-	    for( int i = 2; i < argc; i++ )
-		       magml::setVariable(argv[i], WebInterpretor::parameters());
+	for( int i = 2; i < argc; i++ )
+		magml::setVariable(argv[i], WebInterpretor::parameters());
 
 	//for ( int i = 0; i < 10; i++) 
 	{
-		
-        try {
-		    WebInterpretor::json(argv[1]);
-		
-	    }
-        catch (...) {
-            exit(1);
-        }
-    }
+		try {
+			WebInterpretor::json(argv[1]);
+		}
+		catch (...) {
+			cout << argv[0] << " FAILED to dispatch JSON file!"<< endl;
+			exit(1);
+		}
+	}
     }
     catch (MagicsException& e)
     {
-		Log::error() << "Magics: Catch Exception " << e << "\n";
+		cout << "MagJson: Catch Exception " << e << endl;
 		exit(1);
     }
 }
