@@ -29,11 +29,9 @@ BasicGraphicsObject::BasicGraphicsObject() :
 {
 }
 
-
-
 BasicGraphicsObject::~BasicGraphicsObject() 
 {
-	
+
 }
 
 void BasicGraphicsObject::print(ostream& out) const
@@ -41,9 +39,6 @@ void BasicGraphicsObject::print(ostream& out) const
 	out << "BasicGraphicsObject[";
 	out << "]";
 }
-BasicGraphicsObjectContainer::~BasicGraphicsObjectContainer() 
-{
-}
 
 void BasicGraphicsObjectContainer::print(ostream& out) const
 {
@@ -54,77 +49,57 @@ void BasicGraphicsObjectContainer::print(ostream& out) const
 }
 
 
+bool BasicGraphicsObjectContainer::buildTree(const Layout& parent, unsigned int frame, const BaseDriver& driver) const
+{
+	bool more = false;
+	for (vector<BasicGraphicsObject*>::const_iterator object = objects_.begin(); object!= objects_.end(); ++object){
+		if ( (*object)->buildTree(parent, frame, driver ) )
+			more = true;
+	}
+	return more;
+}
+
 void BasicGraphicsObjectContainer::visit(const BaseDriver& driver) const
 {
 	for (vector<BasicGraphicsObject*>::const_iterator object = objects_.begin(); object!= objects_.end(); ++object) 
 			(*object)->redisplay(driver);
-		
+	for (vector<BasicGraphicsObject*>::const_iterator object = last_.begin(); object!= last_.end(); ++object)
+				(*object)->redisplay(driver);
 }
 
-
-
-// Specialisation!
-/*
-#include <Text.h>
-#include <Polyline.h>
-#include <BaseDriver.h>
-#include <Flag.h>
-#include <Arrow.h>
-
-namespace magics
+BasicGraphicsObjectContainer::~BasicGraphicsObjectContainer()
 {
-template<>
-void Text::redisplay(const BaseDriver& driver) const
-{
-	string text = getText();
-	if ( !text.empty() ) {
-		NiceText nice;
-		MagFont font = font_;
-		font.size(getHeight());
-		font.colour(colour_);	
-    	nice.font(font);
-    	nice.text(text);
-    	addNiceText(nice);
+	MagLog::debug() << "delete CONTAINER" << objects_.size() << endl;
+	for (vector<BasicGraphicsObject*>::iterator object = objects_.begin(); object!= objects_.end(); ++object) {
+		if (*object)
+			delete *object;
+		*object = 0;
 	}
-	const_cast<Text *>(this)->setText("");
-	
-	driver.redisplay(*this);
+	objects_.clear();
 }
 
-template<>
-void Polyline::redisplay(const BaseDriver& driver) const
+void BasicGraphicsObjectContainer::clear()
 {
-	if ( box_ == "legend" ) {
-		Log::debug() << "Warning---> wrong box!" << endl;
-		const_cast<Polyline * >(this)->box_ = "drawing_area";
+	MagLog::debug() << "CLEAR CONTAINER" << objects_.size() << endl;
+	for (vector<BasicGraphicsObject*>::iterator object = objects_.begin(); object!= objects_.end(); ++object) {
+		if (*object) {
+
+			delete *object;
+		}
+		*object = 0;
 	}
-	driver.redisplay(*this);
+	objects_.clear();
 }
-
-template <>	
-bool Flag::reproject(const Transformation&, GraphicsList& out) const
+void BasicGraphicsObjectContainer::release()
 {
-	out.push_back(const_cast<Flag*>(this));
-
-	return true;
+	MagLog::debug() << "CLEAR CONTAINER" << objects_.size() << endl;
+	for (vector<BasicGraphicsObject*>::iterator object = objects_.begin(); object!= objects_.end(); ++object) {
+		(*object)->release();
+	}
+	objects_.clear();
 }
-
-template <>
-bool Arrow::reproject(const Transformation&, GraphicsList& out) const
+void BasicGraphicsObject::check()
 {
-
-	out.push_back(const_cast<Arrow*>(this));
-	
-    return true;
+	if (parent_)
+		parent_->remove(this);
 }
-
-template <>
-bool Polyline::reproject(const Transformation&, GraphicsList& out) const
-{
-	
-	out.push_back(const_cast<Polyline*>(this));
-	
-    return true;
-}
-}
-*/
