@@ -18,7 +18,7 @@
 
 /*! \file BasicGraphicsObject.h
     \brief Implementation of BasicGraphicsObject class.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: March 2004
 
@@ -28,7 +28,7 @@
 
 #include "magics.h"
 
-#include "Log.h"
+#include "MagLog.h"
 
 
 #include "VectorOfPointers.h"
@@ -49,60 +49,108 @@ public:
 	virtual ~BasicGraphicsObject();
 
 	virtual bool reproject(BasicGraphicsObjectContainer&) const 
-	{ Log::error() << "BasicGraphicsObject::reproject(...)--->Need to be implemented!\n"; assert(0); return false; }
+	{ MagLog::error() << "BasicGraphicsObject::reproject(...)--->Need to be implemented!\n"; assert(0); return false; }
 
 	virtual void redisplay(const BaseDriver&) const 
-		{ Log::dev() << "BasicGraphicsObject::redisplay(...)--->Not yet implemented\n"; }
+	{ MagLog::dev() << "BasicGraphicsObject::redisplay(...)--->Not yet implemented\n"; }
 
-	void parent(BasicGraphicsObjectContainer* parent) {
+	void parent(BasicGraphicsObjectContainer* parent)
+	{
 		//assert(parent_ == 0);
 		parent_ = parent; 
 	}
-	
+	void check();
 	BasicGraphicsObjectContainer& parent() { assert(parent_); return *parent_; }
 
+	void makeBrother(const BasicGraphicsObject& brother)
+	{ parent_ = brother.parent_; }
+
+	virtual void release() {}
+
 	bool isOrphan() { return parent_== 0; } 
-	void orphan() {		
-		parent_ = 0; 
-	}
+	void orphan() {	parent_ = 0; }
 
 	void name(const string& name) { name_ = name; }
 	const string& name() const    { return name_; }
 	bool root() { return parent_ == 0; }
+	
+	void widthResolution(int width) { widthResolution_ = width; }
+	void heightResolution(int height) { heightResolution_ = height; }
+	int widthResolution() const { return widthResolution_; }
+	int heightResolution() const { return heightResolution_; }
+
+
+
+	virtual bool buildTree(const Layout&,  unsigned int, const BaseDriver&) const
+	{ return false; }
 
 protected:
 	virtual	void print(ostream&) const;
+	int widthResolution_;
+	int heightResolution_;
 	
 	BasicGraphicsObjectContainer* parent_;  // Just for reference : do not delete!
 	string name_;
 private:
 // No copy allowed
-	BasicGraphicsObject& operator=(const BasicGraphicsObject&);
+	//BasicGraphicsObject& operator=(const BasicGraphicsObject&);
 
 // -- Friends
 	friend ostream& operator<<(ostream& s,const BasicGraphicsObject& p)
 		{ p.print(s); return s; }
 };
 
+
+
+/*!
+
+ Inherited by SceneLayer
+ 
+ \sa SceneLayer
+*/
 class BasicGraphicsObjectContainer : public BasicGraphicsObject
 {
 public:
 	BasicGraphicsObjectContainer() {}
-	~BasicGraphicsObjectContainer();
+	virtual ~BasicGraphicsObjectContainer();
 	
-	virtual void push_back(BasicGraphicsObject* object) {
+	void push_back(BasicGraphicsObject* object)
+	{
+		object->check(); // here we make sure that the object is not in 2 containres!
 		objects_.push_back(object);
 		object->parent(this);
 	}
+	void push_last(BasicGraphicsObject* object)
+		{
+			object->check(); // here we make sure that the object is not in 2 containres!
+			last_.push_back(object);
+			object->parent(this);
+		}
+
+	
+	void clear();
+	bool buildTree(const Layout&,  unsigned int, const BaseDriver&) const;
+	void release();
+	void remove(BasicGraphicsObject* object) {
+		std::remove(objects_.begin(), objects_.end(), object);
+
+	}
+
 
 	void visit(const BaseDriver&) const;
 
-	virtual double absoluteX()      		const //absolute position from the root
+	double absoluteX() const //absolute position from the root
 	{
 		assert(parent_); return parent_->absoluteX();
 	}
-
-	virtual double absoluteY()      		const //absolute position from the root
+	
+	virtual void getDriverInfo(double& x, double& y, double& width, double& height)
+	{
+		if ( parent_ ) 
+			parent_->getDriverInfo(x, y, width, height);
+	}
+	
+	virtual double absoluteY() const //absolute position from the root
 	{
 		assert(parent_); return parent_->absoluteY();
 	}
@@ -131,10 +179,17 @@ public:
 	{
 		assert(parent_); return parent_->transformation();
 	}
-
+	const vector<BasicGraphicsObject*>& objects() { //
+		//first we add
+		for (vector<BasicGraphicsObject*>::iterator l = last_.begin(); l != last_.end(); ++l)
+			objects_.push_back(*l);
+		last_.clear();
+		return objects_;
+	}
 protected:
 	virtual	void print(ostream&) const;
 	vector<BasicGraphicsObject*> objects_;
+	vector<BasicGraphicsObject*> last_;
 };
 
 } // namespace magics
