@@ -31,17 +31,19 @@
 #define BasicSceneObject_H
 
 #include "magics.h"
-#include "Log.h"
+#include "MagLog.h"
 #include "VectorOfPointers.h"
 #include "BasicGraphicsObject.h"
 #include "Layout.h"
 #include "DisplayManager.h"
+#include "MagicsEvent.h"
+
 //#include "SceneVisitor.h"
 
 namespace magics {
 	
 class XmlNode;
-class GeoPoint;
+class UserPoint;
 class UserPoint;
 
 class Transformation;
@@ -60,20 +62,28 @@ class TextVisitor;
 class MetaDataVisitor;
 class LegendVisitor;
 class PreviewVisitor;
+class HistoVisitor;
 class MagnifierVisitor;
 class SceneVisitor;
 class LayoutManager;
 class AnimationStep;
+class SceneLayer;
 
-template <class P> class Data;
-template <class P> class Visdef;
+class MetaDataCollector;
+class MagnifierCollector;
+class ValuesCollector;
+class DataIndexCollector;
+class BinningObject;
+
+class Data;
+class Visdef;
 
 class BasicPositionalObject;
 class BasicSceneNode;
 
-enum MagicsMode { interactif, paper };
+enum MagicsMode { interactif, paper, basic};
 
-class BasicSceneObject  {
+class BasicSceneObject : public MetviewIcon {
 
 public:
 	BasicSceneObject(BasicSceneObject* parent = 0);
@@ -89,18 +99,16 @@ public:
 	virtual void text(TextVisitor* text) { assert(parent_); parent_->text(text); }
 	virtual void legend(LegendVisitor* legend) { assert(parent_); parent_->legend(legend); }
 
-		
-	virtual void data(Data<GeoPoint>*)     { Log::warning() << "data(Data<GeoPoint>*) to be checked" << endl; }
-	virtual void visdef(Visdef<GeoPoint>*) { Log::warning() << "visdef(Data<GeoPoint>*) to be checked" << endl; }
-	
-	virtual void data(Data<UserPoint>*)     { Log::warning() << "data(Data<UserPoint>*) to be checked" << endl; }
-	virtual void visdef(Visdef<UserPoint>*) { Log::warning() << "visdef(Data<UserPoint>*) to be checked" << endl; }
+	virtual void binning(BinningObject*)   { MagLog::warning() << "binning(BinningObject*) to be checked" << endl; }
+	virtual void data(Data*)     { MagLog::warning() << "data(Data*) to be checked" << endl; }
+	virtual void visdef(Visdef*) { MagLog::warning() << "visdef(Data*) to be checked" << endl; }
 	
+
 	
 	void addVisitor(SceneVisitor* visitor) { visitors_.push_back(visitor); }
 	
 	void parent(BasicSceneObject* parent) { 
-		assert(parent_ == 0); 
+
 		parent_ = parent;  
 	}
 	void reparent(BasicSceneObject* parent) { 
@@ -110,20 +118,29 @@ public:
 	void orphan() { parent_ = 0; }
 	BasicSceneObject& parent() const { assert(parent_); return *parent_; }
 	virtual void set(const XmlNode&) 
-		{ Log::dev() << "Warning:  BasicSceneObject::set(const XmlNode&)-->Not implemented!" << endl; }
+		{ MagLog::dev() << "Warning:  BasicSceneObject::set(const XmlNode&)-->Not implemented!" << endl; }
 	virtual void resolve();
 	virtual Layout* execute(AnimationStep&,  Layout&)
-		{ Log::dev() << "Warning:  BasicGraphicsObject* BasicSceneObject::execute(AnimsationStep&))-->Not implemented!" << endl; return 0;}
+		{ MagLog::dev() << "Warning:  BasicGraphicsObject* BasicSceneObject::execute(AnimsationStep&))-->Not implemented!" << endl; return 0;}
 	virtual void getReady() { dispatch(&BasicSceneObject::getReady); }
- 
-    
+	virtual void release() { dispatch(&BasicSceneObject::release); }
+
 	virtual void visit(MetaDataVisitor& meta)  		{ dispatch(meta); } 
 	virtual void visit(PreviewVisitor& preview)  		{ dispatch(preview); }
+	virtual void visit(HistoVisitor& histo)  		{ dispatch(histo); }
 	virtual void visit(MagnifierVisitor& magnifier)  	{ dispatch(magnifier); }
 	
+	virtual void visit(DateDescription& timestamp)  	{ dispatch(timestamp); }
+	virtual void visit(LevelDescription& level)  	{ dispatch(level); }
+
 	virtual void visit(Transformation& transformation)  	{ dispatch(transformation); }
 	virtual void visit(AnimationRules& rules)  { dispatch(rules); }
 	
+	virtual void visit(MetaDataCollector& infos)  { dispatch(infos); }
+	virtual void visit(MagnifierCollector& infos)  { dispatch(infos); }
+	virtual void visit(ValuesCollector& infos)  { dispatch(infos); }
+	virtual void visit(DataIndexCollector& infos)  { dispatch(infos); }
+	
 	virtual void visit(DrawingVisitor& drawing)  { dispatch(drawing); }
 	virtual void visit(TopAxisVisitor& top)  { dispatch(top); }
 	virtual void visit(BottomAxisVisitor& bottom)  { dispatch(bottom); }
@@ -133,9 +150,12 @@ public:
 	
 	virtual void visit(TextVisitor& text)  { dispatch(text); }
 	virtual void visit(LegendVisitor& legend)  { dispatch(legend); }
-	
+	virtual void visit(SceneLayer& tree)  { dispatch(tree); }
 	virtual void visit(BasicGraphicsObjectContainer& tree)  { dispatch(tree); }
 	virtual void visit(AnimationStep& step)  { dispatch(step); }
+	virtual void visit(SceneLayer& visitor, vector<LayoutVisitor*>& args)  { dispatch(visitor, args); }
+
+	virtual bool needLegend();
 
 	virtual void execute()
 	{
@@ -143,12 +163,19 @@ public:
 	  return parent_->execute(); 
 	} 
 
+	virtual  BasicGraphicsObject* visualise()
+	{
+		  assert(parent_);
+		  return parent_->visualise();
+	}
 	virtual MagicsMode mode() { assert(parent_); return parent_->mode(); }
 
 	virtual Transformation& transformation() const
 		{ assert(parent_); return parent_->transformation(); } 
 	virtual const Layout& layout() const 
 		{ assert(parent_); return parent_->layout(); } 
+	virtual BasicGraphicsObject* toDisplay()
+		{ assert(parent_); return parent_->toDisplay(); }
 	virtual double absoluteWidth()    const 
 		{ assert ( parent_ ); return  parent_->absoluteWidth(); } 
    	virtual double absoluteHeight()   const 
@@ -157,30 +184,48 @@ public:
 		{ assert ( parent_ ); return  parent_->absoluteRootWidth(width); } 
    	virtual void absoluteRootHeight(double height)   
    		{ assert ( parent_ ); return  parent_->absoluteRootHeight(height); }
+ 	virtual int rootWidthResolution()   const
+		{ assert ( parent_ ); return  parent_->rootWidthResolution(); } 
+   	virtual int rootHeightResolution() const   
+   		{ assert ( parent_ ); return  parent_->rootHeightResolution(); }
+   	virtual int widthResolution()   const
+   			{ assert ( parent_ ); return  parent_->rootWidthResolution(); } 
+   	virtual int heightResolution() const   
+   	   		{ assert ( parent_ ); return  parent_->rootHeightResolution(); }
    
 	const string& name() const { return name_; }
 	void name(const string& name)  { name_ = name; }
 
 	virtual BasicSceneNode* insert(BasicPositionalObject*) { assert (false);  return 0;}
-    
+
+
 protected:
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	virtual void print(ostream&) const; 
 	typedef void (BasicSceneObject::*Function)();
 
+
+
+
+
 	template <class T>
 	void dispatch(T& visitor) {
 		for ( vector<BasicSceneObject*>::iterator item = items_.begin(); item != items_.end(); ++item) 
 			(*item)->visit(visitor); 		
 	}
-
+	template <class T1, class T2>
+		void dispatch(T1& visitor, T2& args) {
+			for ( vector<BasicSceneObject*>::iterator item = items_.begin(); item != items_.end(); ++item) 
+				(*item)->visit(visitor, args); 		
+		}
 	void dispatch(Function function)
 	{
 		for ( vector<BasicSceneObject*>::iterator item = items_.begin(); item != items_.end(); ++item) 
 			((*item)->*function)(); 		
 	}
 
-VectorOfPointers<vector<BasicSceneObject*> >          items_;	 
+
+	VectorOfPointers<vector<BasicSceneObject*> >          items_;
 	vector<SceneVisitor*>     visitors_;	 
  
 	 BasicSceneObject* parent_; //Do not delete! Only a reference
@@ -213,6 +258,7 @@ class BasicSceneNode : public BasicPositionalObject
 {
 public:
 	BasicSceneNode();
+	BasicSceneNode(Layout*);
 	virtual ~BasicSceneNode();
 
 	virtual BasicSceneNode* insert(BasicPositionalObject*); // Return the node tinto which the object has been inserted! 
@@ -235,6 +281,7 @@ public:
 	double absoluteWidth()    const;
 	double absoluteHeight()   const;
 
+
 	void manager(LayoutManager* manager) { manager_ = manager; }
 
 protected:
@@ -243,5 +290,36 @@ protected:
 	friend class LayoutManager;
 };
 
+class EmptySceneObject: public BasicSceneObject {
+
+public:
+	EmptySceneObject();
+	virtual ~EmptySceneObject();
+    
+ 
+    
+    void visit(SceneLayer&, vector<LayoutVisitor*>&);
+  
+   
+
+protected:
+     //! Method to print string about this class on to a stream of type ostream (virtual).
+	 virtual void print(ostream&) const; 
+    
+   
+     
+private:
+    //! Copy constructor - No copy allowed
+	EmptySceneObject(const EmptySceneObject&);
+    //! Overloaded << operator to copy - No copy allowed
+	EmptySceneObject& operator=(const EmptySceneObject&);
+
+// -- Friends
+    //! Overloaded << operator to call print().
+	friend ostream& operator<<(ostream& s,const EmptySceneObject& )
+		{ s << "EmptySceneObject";  return s; }
+
+};
+
 } // namespace magics
 #endif
