@@ -176,9 +176,27 @@ BottomHorizontalLayoutManager::~BottomHorizontalLayoutManager()
 {
 }
   
-BasicSceneNode*  BottomHorizontalLayoutManager::operator()(BasicSceneNode* , BasicPositionalObject* )
+BasicSceneNode*  BottomHorizontalLayoutManager::operator()(BasicSceneNode* parent, BasicPositionalObject* node)
 {
-	assert(false);
+	  Layout& layout = node->layout();
+	// first fits in width! 
+
+	   if (y_ + layout.height() <=  100 ) { 
+		   if ( x_ + layout.width()  <=100 ) {
+			// it fits perfectly! we set the postion for the node
+			layout.x(x_);				
+			x_=  x_ + layout.width();
+			layout.y(y_);
+			return parent;
+	   }
+	   else { // create another row
+		   x_ = 0;
+		  y_ = y_ + layout.height() + gapy_;
+		   return (*this)(parent, node);	
+	   }
+	}
+	// Do not fit on this Layout!
+	return parent->newNode(node);
 }
 
 TopVerticalLayoutManager::TopVerticalLayoutManager()
