@@ -15,13 +15,13 @@
 
 #include "magics.h"
 #include "MagicsObserver.h"
-#include "Layer.h"
+
 
 namespace magics {
 
-#ifdef MAGICS_OPENGL
-class OpenGLNode;
-#endif
+class Layer;
+class MetaDataCollector;
+class DateTime;
 
 class MagicsEvent
 {
@@ -185,45 +185,6 @@ protected:
 	vector<MagicsAnimationStepData>  steps_;
 };
 
-#ifdef MAGICS_OPENGL
-
-class MagicsLayerTreeEvent:  public MagicsEvent
-{
-// From Uplot to driver!
-public:
-	MagicsLayerTreeEvent(OpenGLNode *rootNode) : rootNode_(rootNode) {}	
-	virtual ~MagicsLayerTreeEvent() {}	
-	void notify(MagicsObserver& observer) { observer.notify(*this); }
-	OpenGLNode* rootNode() { return rootNode_; }
-protected:
-	//! Method to print string about this class on to a stream of type ostream (virtual).
-	virtual void print(ostream& out) const
-	{ 
-		out << "MagicsLayerTreeEvent[" << "]";
-	}	
-	OpenGLNode* rootNode_;
-};
-
-class MagicsLayerSwapEvent:  public MagicsEvent
-{
-// From Uplot to driver!
-public:
-	MagicsLayerSwapEvent(OpenGLNode *first,OpenGLNode* second) : firstNode_(first), secondNode_(second) {}	
-	virtual ~MagicsLayerSwapEvent() {}	
-	void notify(MagicsObserver& observer) { observer.notify(*this); }
-	OpenGLNode* firstNode() { return firstNode_; }
-	OpenGLNode* secondNode() { return secondNode_; }
-protected:
-	//! Method to print string about this class on to a stream of type ostream (virtual).
-	virtual void print(ostream& out) const
-	{ 
-		out << "MagicsLayerTreeEvent[" << "]";
-	}	
-	OpenGLNode* firstNode_;
-	OpenGLNode* secondNode_;
-};
-
-#endif
 
 class MagicsLayerUpdateEvent:  public MagicsEvent
 {
@@ -238,70 +199,88 @@ protected:
 		out << "MagicsLayerUpdateEvent[" << "]"; }	
 };
 
-#ifdef MAGICS_OPENGL
-
-class MagicsZoomUpdateEvent :  public MagicsEvent
+class LevelDescription
 {
 public:
-	MagicsZoomUpdateEvent(int levelNum, int actLevel, string layoutId,OpenGLNode* node) : 
- 		levelNum_(levelNum), actLevel_(actLevel), layoutId_(layoutId), node_(node) {};
-	virtual ~MagicsZoomUpdateEvent() {}	
-	void notify(MagicsObserver& observer) { observer.notify(*this); }
-	int levelNum() const {return levelNum_;} 
-	int actLevel() const {return actLevel_;}
-	const string& layoutId() const {return layoutId_;}
-	OpenGLNode* node() const {return node_;}
-private:	
-	int levelNum_;
-	int actLevel_;
-	string layoutId_;
-	OpenGLNode* node_;
-};
+	LevelDescription();
+	~LevelDescription();
+	static LevelDescription surface(int set, int index) {
+		LevelDescription level;
+		level.surface_ = true;
+		level.index_ = index;
+		level.set_ = set;
+		return level;
+	}
+	static LevelDescription level(const string& unit, double l, int set, int index) {
+		LevelDescription newlevel;
+		newlevel.surface_ = false;
+		newlevel.unit_ = unit;
+		newlevel.level_ = l;
+		newlevel.set_ = set;
+		newlevel.index_ = index;
 
+		return newlevel;
+	}
 
-class MagicsRenderZoomPreviewEvent :  public MagicsEvent
-{
-public:
-	MagicsRenderZoomPreviewEvent(OpenGLNode *node,int x,int y, int tw, int th) : 
-		node_(node), x_(x), y_(y), tw_(tw), th_(th) {};
-
-	virtual ~MagicsRenderZoomPreviewEvent() {}	
-	void notify(MagicsObserver& observer) { observer.notify(*this); }
-	int x() const {return x_;} 
-	int y() const {return y_;}
-	int tw() const {return tw_;} 
-	int th() const {return th_;}
-	OpenGLNode* node() const {return node_;}
-
+	bool operator < (const LevelDescription& other) const; // Code function in Layer.cc
+	void update(const LevelDescription&) const;
 protected:
-	//! Method to print string about this class on to a stream of type ostream (virtual).
-	virtual void print(ostream& out) const { 
-		out << "MagicsRenderZoomPreviewEvent[" << "]"; }	
-
-private:	
-	OpenGLNode* node_;
-	int x_;
-	int y_;
-	int tw_;
-	int th_;
+	bool surface_;
+	string unit_;
+	double level_;
+	mutable int set_;
+	mutable int index_;
 };
+class DateDescription
+{
+public:
+	DateDescription() {}
+	DateDescription(const string& valid, int set, int index) : valid_(valid), set_(set), index_(index) {}
+	~DateDescription();
 
-#endif
 
+	void update(const DateDescription&) const;
+	bool operator < (const DateDescription& other) const; // Code function in Layer.cc
+protected:
+	string valid_;
+	mutable int set_;
+	mutable int index_;
+};
 struct MetviewIcon
 {
 public:
-	MetviewIcon(const string& name = "", const string& cname = "" ) : iconName_(name), iconClass_(cname) {}
-	void icon(const string& name, const string& cname) {  iconName_ = name; iconClass_ = cname; } 
-	void visit(Layer& layer)
-	{ 
-		if ( !iconClass_.empty() && !iconName_.empty() )
-			layer.icon(iconName_, iconClass_); 
-	} 
-	void icon(const MetviewIcon& other) { iconName_ = other.iconName_; iconClass_ = other.iconClass_; }
+	MetviewIcon(const string& name = "", const string& cname = "", const string& id="unknown") :
+		iconName_(name), iconClass_(cname), iconId_(id) {};
+		
+	void icon(const string& name, const string& cname, const string& id="unknown") {
+		iconName_ = name;
+		iconClass_ = cname;
+		iconId_ = id;
+	}
+	virtual void visit(Layer& layer);
+
+	virtual void visit(MetaDataCollector& collector);
+
+	
+	virtual void initInfo() {}
+	string info(const string& name) {return (information_.find(name) != information_.end())?information_[name]:"";}
+	void setInfo(const string& name, const string& value) { information_[name]=value; }
+	void icon(const MetviewIcon& other) {
+		iconName_ = other.iconName_;
+		iconClass_ = other.iconClass_;
+		iconId_ = other.iconId_;
+	}	
+	string iconName() const {return iconName_;}
+        string iconClass() const {return iconClass_;}
+        string iconId() const {return iconId_;}
+
+
+			
 protected:
 	string iconName_;
 	string iconClass_;
+	string iconId_;
+	map<string, string> information_;
 };
 
 } // namespace magics
