@@ -50,13 +50,19 @@ public:
     }
 };
 
+struct KeyInfo
+{
+	string key_;
+	string what_;
+	string format_;
 
+};
 
 class TextVisitor: public TextVisitorAttributes, 
 	public TagHandler,
 	public LayoutVisitor, 
 	public BasicPositionalObject,
-	public VectorOfPointers<vector<TextEntry* > >
+	public map<string, vector<TextEntry* > >
 {
 
 public:
@@ -64,22 +70,31 @@ public:
 	virtual ~TextVisitor();
 	
 	TextVisitor* clone();
-	void add(TextEntry* entry) 
-		{ VectorOfPointers<vector<TextEntry* > >::push_back(entry); }
+	void add(const string& line, TextEntry* entry);
+	void add(TextEntry* entry) { add("<magics_title/>", entry); }
+	void addAutomaticTitle(const string&);
 	
-
+	
+	void addToTags(const string&,  const string&);
+			
     virtual void titles(vector<string>&);
-	virtual Layout& layout() const { return *(const_cast<TextVisitor*>(this)); }
-	Layout* execute(AnimationStep& step,  const Layout* visitor);
+	virtual Layout& layout() const { return LayoutVisitor::layout(); }
+	virtual Layout* layoutPtr() const { return LayoutVisitor::layoutPtr(); }
 	
 
-    void visit(BasicGraphicsObjectContainer&);
+
+    map<string, vector<Text*> > texts() { return currentTexts_; }
+    void visit();
     void visit(BasicSceneObject&);
-    void visit(AnimationStep&);
+    void finish(BasicGraphicsObjectContainer&);
+    void update(vector<Text*>&);
+    void start();
+    
     void visit(MetaDataVisitor&);
     bool positional() const { return positional_; } 
     virtual void getReady();
-
+    void update(const string& family, const string& definition, const string& value)
+    	{ TagHandler::update(family, definition, value); }
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
@@ -87,12 +102,18 @@ protected:
 	 virtual void decode() {}
 	 
 	bool positional_;
+	double font_size_;
 	
+	 static map<string, string> tags_;
+	 void extract(const string&, vector<KeyInfo>&);
 
     
     void set(const map<string, string>& map ) { TextVisitorAttributes::set(map); }
 	void set(const XmlNode& node ) { TextVisitorAttributes::set(node); }
 	string label_;
+	mutable vector<Text*> texts_;
+	mutable map<string, vector<Text*> > currentTexts_;
+	VectorOfPointers<vector<TextEntry* > > currentLine_;
 	
      
 private:
@@ -110,7 +131,8 @@ public:
 	XmlTextVisitor();
 	~XmlTextVisitor() {}
 	void set(const XmlNode&);
-	void getReady();	
+	void getReady();
+	void decode();
 };
 
 class FortranTextVisitor : public TextVisitor
@@ -141,6 +163,13 @@ public:
 	~FortranPositionalTextVisitor() {}
 	void getReady();	
 };
-
+/*
+class MvTextVisitor : 
+{
+	MvTextVisitor();
+	~MvTextVisitor()
+		
+}
+*/
 } // namespace magics
 #endif
