@@ -42,6 +42,7 @@
 namespace magics {
 
 class Transformation;
+class AnimationRules;
 
 class ViewNode: public BasicSceneNode {
 
@@ -61,58 +62,53 @@ public:
 	
 	virtual void text(TextVisitor* text);
 	virtual void legend(LegendVisitor* legend);
-	Layout* execute(AnimationStep&, Layout&);
-	
-	// Called to resolve the animation/overlay and getReady!
-	void resolve(); 
 	
 	
 	
+	
+	
+	void visit(MetaDataCollector&);
 
 
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 void print(ostream&) const; 
-	 void buildLayout(Layout& layout, ViewNode&);
-	 
+	 void prepareLayout(SceneLayer&);
+	 virtual void updateLayout() {}
 	
 	 void getReady();
 	 
-	 void visit(BasicGraphicsObjectContainer&);	
+	 void visit(SceneLayer&);	
 	 void visit(MetaDataVisitor&);
 	 void visit(PreviewVisitor&);
+	 void visit(HistoVisitor&);
 	
 	 virtual void copy(const ViewNode&);
 
-	 Transformation& transformation() const { assert(transformation_); return *transformation_; }
-	 /*
-	 void attachLegend(BasicSceneObject*);
-	 void attachTitle(BasicSceneObject*);
-
-     void addLegend(BasicSceneObject*);
-	 void addTitle(BasicSceneObject*);
-	*/
+	 Transformation& transformation() const { assert(viewTransformation_); return *viewTransformation_; }
 	 
-	 Transformation* transformation_;
+	 Transformation* viewTransformation_;
 	 
 	 // The margin are used to position the drawing area, inside the node! 
 	 double drawing_top_; //In automatic mode : The top should always be greater that 15% to allow a space for the automatic title, and automatic legend!
 	 double drawing_bottom_;
 	 double drawing_left_;
 	 double drawing_right_;
+	 string animation_; 
 	 
 	 DrawingVisitor* 		drawing_;
-	 MagnifierVisitor* 		magnify_;
-	 LegendVisitor* 		legend_;
-	 vector<TextVisitor*>     texts_;
 	 TopAxisVisitor*    		topAxis_;
 	 BottomAxisVisitor*   bottomAxis_;
 	 LeftAxisVisitor*    	leftAxis_;
 	 RightAxisVisitor*    	rightAxis_;
+	 double                 vaxis_;
+	 double                 haxis_;
 	 
-	 vector<TextVisitor*>  textVisitor_;
-	 LegendVisitor*           legendVisitor_;
-	 vector<Layout*>       components_;
+	 AnimationRules*         rules_;
+
+	 LegendVisitor* 		legend_;
+	 vector<TextVisitor*>     texts_;
+	 vector<LayoutVisitor*> components_;
 	
 	
 	 // Infomation about the zoom! 
@@ -144,6 +140,7 @@ public:
 	
 	void getReady();
 
+
 protected:
 	void print(ostream&) const;
 	
@@ -165,7 +162,7 @@ public:
 	}
 
 	
-	
+	void updateLayout();
 	void getReady();
 protected:
 	void print(ostream&) const;
