@@ -18,16 +18,17 @@
 
 /*! \file XmlMagics.cc
     \brief Implementation of the Template class XmlMagics.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Apr-2007
 
 */
 
+
 #include "magics_ecmwf_log.h"
 #include "XmlMagics.h"
 #include "XmlReader.h"
-#include "Exception.h"
+#include "MagException.h"
 #include "Timer.h"
 
 #include "RootSceneNode.h"
@@ -37,23 +38,29 @@
 #include "BinaryObject.h"
 
 #include "TaylorGrid.h"
+#ifdef MAGICS_GRIB
 #include "GribDecoder.h"
-#include "NetcdfDecoder.h"
+#endif
+#include "TableDecoder.h"
 #include "VisualAction.h"
 #include "XYList.h"
 #include "LandgramDecoder.h"
+#include "EpsXmlInput.h"
+#ifdef MAGICS_JSON
+#include "WrepJSon.h"
+#endif
 #include "LandgramBox.h"
 #include "ImagePlotting.h"
 #include "SymbolPlotting.h"
 #include "Contour.h"
 #include "Wind.h"
-#include "Histogram.h"
-#include "DataConverter.h"
+//#include "Histogram.h"
+
 #include "LegendVisitor.h"
 #include "TextVisitor.h"
 
 #include "UserPoint.h"
-#include "GeoPoint.h"
+#include "UserPoint.h"
 #include "Axis.h"
 #include "MatrixTestDecoder.h"
 
@@ -65,12 +72,13 @@
 
 
 #ifdef MAGICS_SPOT
-#include "EpsGraph.h"
-#include "EpsgramDecoder.h"
 #include "ClassicMtgDecoder.h"
+#include "EpsgramDecoder.h"
 #include "MetgramGraph.h"
 #endif
 
+#include "EpsGraph.h"
+
 using namespace magics;
 
 XmlMagics::XmlMagics() : root_(0),
@@ -78,18 +86,12 @@ XmlMagics::XmlMagics() : root_(0),
 	geographical_(true), 
 	driversToSet_(true)
 {
-	if(getEnvVariable("MAGPLUS_HOME").empty())
-	{
-		Log::error() << "FATAL ERROR: $MAGPLUS_HOME has to be defined!!!"<< "\n";
-		exit(0);
-	}
-
-	writeLog("magml");
+	writeMagLog("magml");
 
 	actions_["magics"] = &XmlMagics::magics;
 	actions_["page"] = &XmlMagics::page;
 	
-	actions_["binary"] = &XmlMagics::binary;
+	actions_["mgb"] = &XmlMagics::binary;
 
 	actions_["text"] = &XmlMagics::text;
 	actions_["map"] = &XmlMagics::map;
@@ -106,9 +108,11 @@ XmlMagics::XmlMagics() : root_(0),
 
 	actions_["grib"] = &XmlMagics::grib;
 	actions_["netcdf"] = &XmlMagics::netcdf;
+	actions_["table"] = &XmlMagics::table;
 
-	actions_["odb"] = &XmlMagics::oda;
+	actions_["odb"] = &XmlMagics::odb;
 	actions_["xyinput"] = &XmlMagics::xyinput;
+	actions_["input"] = &XmlMagics::input;
 	actions_["landgram"] = &XmlMagics::landgram;
 
 	actions_["wind"] = &XmlMagics::wind;
@@ -124,6 +128,7 @@ XmlMagics::XmlMagics() : root_(0),
 
 	actions_["plot"] = &XmlMagics::layer;
 	actions_["geopoints"] = &XmlMagics::geopoints;
+	actions_["cdfgraph"] = &XmlMagics::cdfgraph;
 	actions_["efigraph"] = &XmlMagics::efigraph;
 	actions_["efigram"] = &XmlMagics::efigram;
 
@@ -133,10 +138,15 @@ XmlMagics::XmlMagics() : root_(0),
 	actions_["gribloop"] = &XmlMagics::gribloop;
 
 	actions_["epsxml"] = &XmlMagics::epsxml;
+	actions_["wrepjson"] = &XmlMagics::wrepjson;
+
+	
 	actions_["epsbufr"] = &XmlMagics::epsbufr;
 	actions_["epsgraph"] = &XmlMagics::epsgraph;
 	actions_["epsgram"] = &XmlMagics::epsgram;
 	actions_["epswind"] = &XmlMagics::epswind;
+	actions_["epscloud"] = &XmlMagics::epscloud;
+	actions_["epsbar"] = &XmlMagics::epsbar;
 	actions_["epswave"] = &XmlMagics::epswave;
 	actions_["epsshade"] = &XmlMagics::epsshading;
 	actions_["epsplume"] = &XmlMagics::epsplume;
@@ -150,6 +160,7 @@ XmlMagics::XmlMagics() : root_(0),
 
 	actions_["layer"] = &XmlMagics::split;
 	actions_["thread"] = &XmlMagics::thread;
+	actions_["import"] = &XmlMagics::import;
 }
 
 
@@ -168,11 +179,13 @@ void XmlMagics::print(ostream& out)  const
 
 void XmlMagics::execute(XmlTree& tree) 
 {
+	{
+	Timer timer("Total", "Execution of the tree");
 	try {
 		tree.visit(*this);
 	}
 	catch (MagicsException& e) {
-		Log::debug() << e.what() << endl;
+		MagLog::debug() << e.what() << endl;
 	}
 
 	if ( driversToSet_ ) 
@@ -196,7 +209,7 @@ void XmlMagics::execute(XmlTree& tree)
 
 		drivers_.closeDrivers();
 	}
-
+	}
 	MetaDataVisitor::collect();
 	//root_->metadata(meta);
 	/* later 
@@ -218,6 +231,7 @@ void XmlMagics::execute(XmlTree& tree)
 
 void XmlMagics::execute(const string& file, std::map<string, string>& )
 {
+	
 	XmlReader parser(true);
 	XmlTree tree;	
 	ParameterManager::set(string("layout"), string("magml")); 
@@ -226,7 +240,7 @@ void XmlMagics::execute(const string& file, std::map<string, string>& )
 		execute(tree);
 	}
 	catch (MagicsException& e) {
-		Log::debug() << e.what() << endl;
+		MagLog::debug() << e.what() << endl;
 	}
 }
 
@@ -240,7 +254,7 @@ void XmlMagics::display(const string& file)
 		tree.visit(*this);
 	}
 	catch (MagicsException& e) {
-			Log::debug() << e.what() << endl;
+			MagLog::debug() << e.what() << endl;
 	}
 	
 	if ( driversToSet_ ) 
@@ -264,7 +278,7 @@ void XmlMagics::visit(const XmlNode& node)
 
 	if ( action == actions_.end() )
 	{
-		Log::dev() << " MagML - ignoring tag: " << node.name()<< endl;
+		MagLog::dev() << " MagML - ignoring tag: " << node.name()<< endl;
 		return;
 	}
 	(this->*action->second)(node);
@@ -272,13 +286,14 @@ void XmlMagics::visit(const XmlNode& node)
 
 void XmlMagics::magics(const XmlNode& node)
 {
-	float version_ = 3.0;
-	Log::debug() << " You are using the " << fixed << setprecision(1) << version_ << " version of the magml interpreter\n";
+	ParameterManager::set(string("layout"), string("magml")); 
+	const float version_ = 3.0;
+	MagLog::debug() << " You are using the " << fixed << setprecision(1) << version_ << " version of the magml interpreter\n";
 	string version = node.getAttribute("version");
 	if ( version.empty() )
 	{
-		Log::error() << " No version defined in your magml file" << endl;
-		Log::error() << " Compatibilty issue : check your magml file and add a version number" << endl;
+		MagLog::error() << " No version defined in your magml file\n"
+		             << " Compatibilty issue: check your magml file and add a version number" << endl;
 		return;
 	}
 	float v;
@@ -286,44 +301,26 @@ void XmlMagics::magics(const XmlNode& node)
 	sv >> v;
 	if ( v < version_ )
 	{
-		Log::error() << " The version defined in the file is " << fixed << setprecision(1) << v << endl;
-		Log::error() << " Compatibilty issue : check your magml file and update the version number " << endl;
+		MagLog::error() << " The version defined in the file is " << fixed << setprecision(1) << v 
+		             << "\n Compatibilty issue: check your magml file and update the version number" << endl;
 		return;
 	}
 	string method = node.getAttribute("method") ;
 	
 	root_ =  ( method == "wrep" ) ? new  WrepRootSceneNode() : new  XmlRootSceneNode();
-	
-	
+
 	push(root_);
 	root_->set(node);
 	root_->getReady();
 	node.visit(*this);
 }
 
-//class OpenGLHelper
-//{
-//public:
-//	OpenGLHelper();
-//	~OpenGlHelper() {}
-//protected:
-//
-//};
-
-//OpenGLHelper::OpenGLHelper()
-//{
-//	Log::dev()<< " Create Motif Window! " << endl;
-//}
-//
-
 
 
 void XmlMagics::driver(const XmlNode& node)
 {
-	for ( XmlNode::ElementIterator driver = node.firstElement(); driver != node.lastElement(); ++driver) {
-//		if (*node.firstElement())->name() ) 
-//			new OpenGLhelper();
-		
+	for ( XmlNode::ElementIterator driver = node.firstElement(); driver != node.lastElement(); ++driver)
+	{
 		output_.set(**driver, drivers_);
 	}
 	driversToSet_ = false;
@@ -380,6 +377,7 @@ void XmlMagics::legend(const XmlNode& node)
 {
 	LegendVisitor* legend = new XmlLegendVisitor();
 	legend->set(node);
+	
 	top()->legend(legend);	
 	node.visit(*this);
 }
@@ -448,8 +446,6 @@ void XmlMagics::cartesian(const XmlNode&)
 	geographical_ = false;
 }
 
-
-
 void XmlMagics::geographical(const XmlNode&)
 {	
 	geographical_ = true;
@@ -459,13 +455,13 @@ void XmlMagics::layer(const XmlNode& node)
 {
 	if ( geographical_ )
 	{
-		VisualAction<GeoPoint>* action = new VisualAction<GeoPoint>();
+		VisualAction* action = new VisualAction();
 		action->set(node);
 		top()->push_back(action);
 		push(action);	
 	}
 	else {
-		VisualAction<UserPoint>* action = new VisualAction<UserPoint>();
+		VisualAction* action = new VisualAction();
 		action->set(node);
 		top()->push_back(action);
 		push(action);
@@ -475,7 +471,7 @@ void XmlMagics::layer(const XmlNode& node)
 	pop();
 }
 
-
+#ifdef MAGICS_GRIB
 GribDecoder* grib_handler;
 
 void XmlMagics::gribloop(const XmlNode& node)
@@ -492,6 +488,7 @@ void XmlMagics::gribloop(const XmlNode& node)
 	}
 }
 
+
 void XmlMagics::splitinloop(const XmlNode& ) 
 {
 	/* later!
@@ -513,18 +510,31 @@ void XmlMagics::gribinloop(const XmlNode&)
 	top()->data(gribloop_->current());
 }
 
-
 void XmlMagics::grib(const XmlNode& node)
 {
 	GribDecoder* grib = new GribDecoder();
 	grib->set(node);
-	if (geographical_) {
-		top()->data(grib);
-	}
-	else {
-		top()->data(new DataConverter(grib));
-	}
+
+	top()->data(grib);
+
 }
+#else
+void XmlMagics::gribloop(const XmlNode&)
+{
+	MagLog::warning() <<" Attempt to call GribLoop but GRIB support is disabled!"<< endl;
+}
+
+void XmlMagics::gribinloop(const XmlNode&)
+{	
+	MagLog::warning() <<" Attempt to call GribInLoop but GRIB support is disabled!"<< endl;
+}
+
+void XmlMagics::grib(const XmlNode&)
+{
+	MagLog::warning() <<" Attempt to call Grib decoding but GRIB support is disabled!"<< endl;
+}
+#endif
+
 
 #include "GeoPointsDecoder.h"
 void XmlMagics::geopoints(const XmlNode& node)
@@ -537,25 +547,40 @@ void XmlMagics::geopoints(const XmlNode& node)
 #include "MapGenDecoder.h"
 void XmlMagics::mapgen(const XmlNode& node)
 {	
-	MapGenDecoder<UserPoint>* mapgen = new MapGenDecoder<UserPoint>();
-	mapgen->set(node);
-	top()->data(mapgen);
+	if (geographical_) {
+		MapGenDecoder* mapgen = new MapGenDecoder();
+		mapgen->set(node);
+		top()->data(mapgen);
+	}
+	else {
+		MapGenDecoder* mapgen = new MapGenDecoder();
+		mapgen->set(node);
+		top()->data(mapgen);
+	}
 }
 
 void XmlMagics::xyinput(const XmlNode& node)
 {	
 	if (geographical_) {
-			XYList<GeoPoint>* list = new XYList<GeoPoint>();
+			XYList* list = new XYList();
 			list->set(node);
 			top()->data(list);
 	}
 	else {
-			XYList<UserPoint>* list = new XYList<UserPoint>();
+			XYList* list = new XYList();
 			list->set(node);
 			top()->data(list);
 	}
 }
 
+#include "InputData.h"
+void XmlMagics::input(const XmlNode& node)
+{
+	InputData* data = new InputData();
+	data->set(node);
+	top()->data(data);
+
+}
 void XmlMagics::landgram(const XmlNode& node)
 {	
 	LandgramDecoder* list = new LandgramDecoder();
@@ -580,9 +605,11 @@ void XmlMagics::graph(const XmlNode& node)
 
 void XmlMagics::histogram(const XmlNode& node)
 {
+	/*
 	Histogram* histogram =  new Histogram();
 	histogram->set(node);
 	top()->visdef(histogram);
+	*/
 }
 
 
@@ -592,22 +619,18 @@ void XmlMagics::histogram(const XmlNode& node)
 
 ***************************************************************************/
 
-
-
-void XmlMagics::odb(const XmlNode& )
-{
-}
+#ifdef MAGICS_ODB
 #include "OdaDecoder.h"
+#endif
 
-void XmlMagics::oda(const XmlNode& node)
+void XmlMagics::odb(const XmlNode& node)
 {
 #ifdef MAGICS_ODB
-
-	OdaDecoder* oda = new OdaDecoder();
-	oda->set(node);
-    top()->data(oda);
+	OdaGeoDecoder* odb = new OdaGeoDecoder();
+	odb->set(node);
+	top()->data(odb);
 #else 
-	Log::warning() << "ODB support is NOT enabled! (Forgotten -lMagPlusODB ?)\n";
+	MagLog::warning() << "ODB support is NOT enabled!\n";
 #endif
 
 }
@@ -624,18 +647,28 @@ void XmlMagics::efigram(const XmlNode& node)
 #endif
 }
 
+void XmlMagics::cdfgraph(const XmlNode& node)
+{	
+#ifdef MAGICS_JSON
+	CdfGraph* cdf = new CdfGraph();
+	cdf->set(node);
+	top()->visdef(cdf);
+#endif
+}
+
 void XmlMagics::efigraph(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
+#ifdef MAGICS_JSON
 	EfiGraph* efi = new EfiGraph();
 	efi->set(node);
 	top()->visdef(efi);
 #endif
 }
 
+
 void XmlMagics::epsplume(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
+#ifdef MAGICS_JSON
 	EpsPlume* plume = new EpsPlume();
 	plume->set(node);
 	top()->visdef(plume);
@@ -644,14 +677,13 @@ void XmlMagics::epsplume(const XmlNode& node)
 
 void XmlMagics::epsdirection(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
+#ifdef MAGICS_JSON
 	EpsDirection* direction = new EpsDirection();
 	direction->set(node);
 	top()->visdef(direction);
 #endif
 }
 
-
 void XmlMagics::metgram(const XmlNode& node)
 {	
 #ifdef MAGICS_SPOT
@@ -660,6 +692,7 @@ void XmlMagics::metgram(const XmlNode& node)
 	top()->data(metgram);
 #endif
 }
+
 void XmlMagics::metgraph(const XmlNode& node)
 {	
 #ifdef MAGICS_SPOT
@@ -678,18 +711,27 @@ void XmlMagics::epsgram(const XmlNode& node)
 	top()->data(eps);
 #endif
 }
+
 void XmlMagics::epsxml(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
 	EpsXmlInput* eps = new EpsXmlInput();
 	eps->set(node);
 	top()->data(eps);
+}
+void XmlMagics::wrepjson(const XmlNode& node)
+{	
+
+#ifdef MAGICS_JSON
+	WrepJSon* wrep = new WrepJSon();
+	wrep->set(node);
+	top()->data(wrep);
 #endif
 }
+
+
 #ifdef MAGICS_BUFR
 #include "EpsBufr.h"
 #endif
-
 void XmlMagics::epsbufr(const XmlNode& node)
 {	
 #ifdef MAGICS_BUFR
@@ -698,69 +740,85 @@ void XmlMagics::epsbufr(const XmlNode& node)
 	top()->data(eps);
 #endif
 }
+
 void XmlMagics::epsgraph(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
 	EpsGraph* epsgraph = new EpsGraph();
 	epsgraph->set(node);
 	top()->visdef(epsgraph);
-#endif
 }
+
 void XmlMagics::epswave(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
 	EpsWave* eps = new EpsWave();
 	eps->set(node);
 	top()->visdef(eps);
-#endif
 }
 
 void XmlMagics::epswind(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
 	EpsWind* epswind = new EpsWind();
 	epswind->set(node);
 	top()->visdef(epswind);
-#endif
 }
 
+void XmlMagics::epscloud(const XmlNode& node)
+{	
+	EpsCloud* epscloud = new EpsCloud();
+	epscloud->set(node);
+	top()->visdef(epscloud);
+}
+void XmlMagics::epsbar(const XmlNode& node)
+{
+	EpsBar* epsbar = new EpsBar();
+	epsbar->set(node);
+	top()->visdef(epsbar);
+}
 void XmlMagics::epsshading(const XmlNode& node)
 {	
-#ifdef MAGICS_SPOT
 	EpsShade* eps = new EpsShade();
 	eps->set(node);
 	top()->visdef(eps);
-#endif
 }
 
-
+#ifdef MAGICS_NETCDF
+#include "NetcdfDecoder.h"
+#endif
 void XmlMagics::netcdf(const XmlNode& node)
-{	
+{
+#ifdef MAGICS_NETCDF
 	if ( geographical_ ) {
-		NetcdfDecoder<GeoPoint>* netcdf = new NetcdfDecoder<GeoPoint>();
+		NetcdfDecoder* netcdf = new NetcdfDecoder();
 		netcdf->set(node);
 		top()->data(netcdf);
 	}
 	else {
-		NetcdfDecoder<UserPoint>* netcdf = new NetcdfDecoder<UserPoint>();
+		NetcdfDecoder* netcdf = new NetcdfDecoder();
 		netcdf->set(node);
 		top()->data(netcdf);
 	}
+#endif
+}
+
+void XmlMagics::table(const XmlNode& node)
+{
+	TableDecoder* table = new TableDecoder();
+	table->set(node);
+	top()->data(table);
 }
 
 #include "InputMatrix.h"
-
 void XmlMagics::matrix(const XmlNode& node)
 {	
 		//MatrixTestDecoder* matrix = new MatrixTestDecoder();
 		//matrix->set(node);
 		if ( geographical_ ) {
-			InputMatrix<GeoPoint>* data =  new InputMatrix<GeoPoint>();
+			InputMatrix* data =  new InputMatrix();
 			data->set(node);
 			top()->data(data);
 		}
 		else {
-			InputMatrix<UserPoint>* data =  new InputMatrix<UserPoint>();
+			InputMatrix* data =  new InputMatrix();
 			data->set(node);
 			top()->data(data);
 		}
@@ -775,22 +833,21 @@ void XmlMagics::polyinput(const XmlNode& node)
 
 void XmlMagics::polyline(const XmlNode& node)
 {
-		SimplePolylineVisualiser* poly =  new SimplePolylineVisualiser();
-		poly->set(node);
-		top()->visdef(poly);
+	SimplePolylineVisualiser* poly =  new SimplePolylineVisualiser();
+	poly->set(node);
+	top()->visdef(poly);
 }
 
 
-
 void  XmlMagics::symbol(const XmlNode& node)
 {	
 	if ( geographical_ ) {
-		SymbolPlotting<GeoPoint>* symbol =  new SymbolPlotting<GeoPoint>();
+		SymbolPlotting* symbol =  new SymbolPlotting();
 		symbol->set(node);
 		top()->visdef(symbol);
 	}
 	else {
-		SymbolPlotting<UserPoint>* symbol =  new SymbolPlotting<UserPoint>();
+		SymbolPlotting* symbol =  new SymbolPlotting();
 		symbol->set(node);
 		top()->visdef(symbol);
 	}
@@ -798,15 +855,14 @@ void  XmlMagics::symbol(const XmlNode& node)
 
 
 void XmlMagics::contour(const XmlNode& node)
-{	
-	
+{
 	if ( geographical_ ) {
-		Contour<GeoPoint>* contour =  new Contour<GeoPoint>();
+		Contour* contour =  new Contour();
 		contour->set(node);
 		top()->visdef(contour);
 	}
 	else {
-		Contour<UserPoint>* contour =  new Contour<UserPoint>();
+		Contour* contour =  new Contour();
 		contour->set(node);
 		top()->visdef(contour);
 	}
@@ -814,30 +870,38 @@ void XmlMagics::contour(const XmlNode& node)
 
 
 void XmlMagics::image(const XmlNode& node)
-{	
-
+{
 	if ( geographical_ ) {
-		ImagePlotting<GeoPoint>* image =  new ImagePlotting<GeoPoint>();
+		ImagePlotting* image =  new ImagePlotting();
 		image->set(node);
 		top()->visdef(image);
 	}
 	else {
-		ImagePlotting<UserPoint>* image =  new ImagePlotting<UserPoint>();
+		ImagePlotting* image =  new ImagePlotting();
 		image->set(node);
 		top()->visdef(image);
 	}
-
 }
 
+#include "ImportObjectHandler.h"
+void XmlMagics::import(const XmlNode& node)
+{
+	ImportObjectHandler* object = new ImportObjectHandler();
+	object->set(node);
+	top()->push_back(object);
+
+}
 void XmlMagics::wind(const XmlNode& node)
 {	
 	if ( geographical_ ) {
-		Wind<GeoPoint>* wind =  new Wind<GeoPoint>();
+		Wind* wind =  new Wind();
 		wind->set(node);
 		top()->visdef(wind);
 	}
 	else {
-		Log::warning() << " wind not yet implemented for cartesian system" << endl;
+		MagLog::warning() << " wind not yet implemented for cartesian system" << endl;
 	}
+#ifdef MAGICS_GRIB
 	if ( gribloop_ ) gribloop_->next();
+#endif
 }
