@@ -45,18 +45,10 @@ class GribLoop;
 class XmlTree;
 
 
-#ifdef MAGICS_OPENGL
-#include "OpenGLDriverObserver.h"
 class XmlMagics : public XmlNodeVisitor, 
 	public stack<BasicSceneObject*>, 
-	public OpenGLDriverObserver, 
-	public vector<MagicsObserver*> {
-#else 
-class XmlMagics : public XmlNodeVisitor, 
-	public stack<BasicSceneObject*>, 
-	public vector<MagicsObserver*> {
-#endif	
-
+	public vector<MagicsObserver*>
+{
 public:
 	XmlMagics(); 
 	~XmlMagics();
@@ -75,7 +67,7 @@ public:
 
 	void page(const XmlNode&);
 	void binary(const XmlNode&);
-	
+	void import(const XmlNode&);
 	void text(const XmlNode&);
 	void map(const XmlNode&);
 	void coastlines(const XmlNode&);
@@ -83,9 +75,11 @@ public:
 	void landgram(const XmlNode&);
 	void layer(const XmlNode&);
 	void grib(const XmlNode&);
+	void table(const XmlNode&);
 	void geopoints(const XmlNode&);
 	void netcdf(const XmlNode&);
 	void xyinput(const XmlNode&);
+	void input(const XmlNode&);
 	void odb(const XmlNode&);
 	void oda(const XmlNode&);
 	void contour(const XmlNode&);
@@ -96,7 +90,6 @@ public:
 	void landgrambox(const XmlNode&);
 	void cartesian(const XmlNode&);
 	void geographical(const XmlNode&);
-	
 
 	void driver(const XmlNode&);
 
@@ -107,6 +100,7 @@ public:
 	void verticalAxis(const XmlNode&);
 
 	void efigraph(const XmlNode&);
+	void cdfgraph(const XmlNode&);
 	void efigram(const XmlNode&);
 
 	void metgraph(const XmlNode&);
@@ -116,7 +110,11 @@ public:
 	void epsgram(const XmlNode&);
 	void epsbufr(const XmlNode&);
 	void epsxml(const XmlNode&);
+	void wrepjson(const XmlNode&);
+	
 	void epswind(const XmlNode&);
+	void epscloud(const XmlNode&);
+	void epsbar(const XmlNode&);
 	void epswave(const XmlNode&);
 	void epsshading(const XmlNode&);
 	void epsplume(const XmlNode&);
@@ -137,25 +135,7 @@ public:
 	void metadata(const XmlNode&);
 	void thread(const XmlNode&);
 	void split(const XmlNode&);
-#ifdef MAGICS_OPENGL
-//	void opengl(const string&, Widget, Widget); 
-  
-	//void redraw();
-	//! Selection of Interactive mode: tooltip : user callback called when mouse moves.
-	void tooltip(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::tooltip, properties); }
-	//! Selection of Interactive mode: pointSelection : user callback called when a point is selected. 
-	void pointSelection(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::pointSelection, properties);}
-	//! Selection of Interactive mode: areaSelection : user callback called when an area is selected.
-	void areaSelection(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::areaSelection, properties);}	
-	//! Selection of Interactive mode: polygoneSelection : user callback called when a polygone is selected.
-	void polygonSelection(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::polygonSelection, properties);}	
-	//! Selection of Interactive mode: lineSelection : user callback called when a line is selected.
-	void lineSelection(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::lineSelection, properties);}
-	//! Selection of Interactive mode: polylineSelection : user callback called when a line is selected.
-	void polylineSelection(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::polylineSelection, properties);}
-	//! Selection of Interactive mode: polylineSelection : user callback called when a line is selected.
-	void pickSelection(const SelectionMode& properties) { drivers_.dispatch(&BaseDriver::pickSelection, properties);}
-#endif	
+
 	void display(const string&);
 
 protected:
