@@ -32,7 +32,7 @@
 
 
 #include "magics.h"
-#include "Exception.h"
+#include "MagException.h"
 #include <PaperPoint.h>
 #include "Matrix.h"
 
@@ -53,8 +53,8 @@
 typedef grib_handle* GribHandlePtr ;
 #endif
 
-#ifndef MAGICS_EXCEPTION
-#include <Log.h>
+#ifndef MAGICS_Exception
+#include <MagLog.h>
 #endif
 
 namespace magics {
@@ -63,7 +63,7 @@ class MistmatchType : public MagicsException
 {
 public:
 	MistmatchType(const string& name, const string& type, const string& wait) : 
-		MagicsException("Parameter " + name + ": type mismatch--> type received \"" + type + "\", expected type \"" + wait + "\"- ignored setting") {}
+		MagicsException("Parameter " + name + ": type mismatch -> type received \"" + type + "\", expected type \"" + wait + "\" - setting ignored!") {}
 };
 
 
@@ -124,6 +124,10 @@ public:
 	virtual void setLocal(ListPolicy) { throw MistmatchType(name_, "LineStyle", type()); }
 	virtual void get(ListPolicy&) const { throw MistmatchType(name_, "ListPolicy", type()); }
 	
+	virtual void set(AxisAutomaticSetting) { throw MistmatchType(name_, "AxisAutomaticSetting", type()); }
+	virtual void setLocal(AxisAutomaticSetting) { throw MistmatchType(name_, "AxisAutomaticSetting", type()); }
+	virtual void get(AxisAutomaticSetting&) const { throw MistmatchType(name_, "AxisAutomaticSetting", type()); }
+	
 	virtual void set(ArrowPosition) { throw MistmatchType(name_, "ArrowPosition", type()); }
 	virtual void setLocal(ArrowPosition) { throw MistmatchType(name_, "ArrowPosition", type()); }
 	virtual void get(ArrowPosition&) const { throw MistmatchType(name_, "ArrowPosition", type()); }
@@ -132,53 +136,58 @@ public:
 	virtual void setLocal(const Matrix&) { throw MistmatchType(name_, "Matrix", type()); }
 	virtual void get(Matrix&) const { throw MistmatchType(name_, "Matrix", type()); }
 #else	
-	virtual void set(const double&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
-	virtual void setLocal(const double&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
-	virtual void get(double&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void set(const double&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void setLocal(const double&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void get(double&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
 
-	virtual void set(bool) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "bool", type()) << "\n";}
-	virtual void setLocal(bool) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "bool", type()) << "\n"; }
-	virtual void get(bool&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "bool", type()) << "\n"; }
+	virtual void set(bool) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "bool", type()) << "\n";}
+	virtual void setLocal(bool) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "bool", type()) << "\n"; }
+	virtual void get(bool&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "bool", type()) << "\n"; }
 
-	virtual void set(const magvector<double>&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
-	virtual void setLocal(const magvector<double>&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
-	virtual void get(magvector<double>&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void set(const magvector<double>&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void setLocal(const magvector<double>&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void get(magvector<double>&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
 
-	virtual void set(const int&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
-	virtual void setLocal(const int&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
-	virtual void get(int&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
+	virtual void set(const int&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
+	virtual void setLocal(const int&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void get(int&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
 
-	virtual void set(const magvector<int>&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
-	virtual void setLocal(const magvector<int>&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
-	virtual void get(magvector<int>&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
+	virtual void set(const magvector<int>&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "real", type()) << "\n"; }
+	virtual void setLocal(const magvector<int>&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
+	virtual void get(magvector<int>&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "integer", type()) << "\n"; }
 
-	virtual void set(const string&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
-	virtual void setLocal(const string&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
-	virtual void get(string&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
+	virtual void set(const string&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
+	virtual void setLocal(const string&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
+	virtual void get(string&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
 
-	virtual void set(const magvector<string>&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
-	virtual void setLocal(const magvector<string>&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
-	virtual void get(magvector<string>&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
+	virtual void set(const magvector<string>&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
+	virtual void setLocal(const magvector<string>&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
+	virtual void get(magvector<string>&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "string", type()) << "\n"; }
 
-	virtual void set(LineStyle) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "LineStyle", type()) << "\n"; }
-	virtual void setLocal(LineStyle) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "LineStyle", type()) << "\n"; }
-	virtual void get(LineStyle&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "LineStyle", type()) << "\n"; }
+	virtual void set(LineStyle) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "LineStyle", type()) << "\n"; }
+	virtual void setLocal(LineStyle) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "LineStyle", type()) << "\n"; }
+	virtual void get(LineStyle&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "LineStyle", type()) << "\n"; }
 
-	virtual void set(Justification) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
-	virtual void setLocal(Justification) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
-	virtual void get(Justification&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
+	virtual void set(Justification) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
+	virtual void setLocal(Justification) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
+	virtual void get(Justification&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
 	
-	virtual void set(ListPolicy) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "ListPolicy", type()) << "\n"; }
-	virtual void setLocal(ListPolicy) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
-	virtual void get(ListPolicy&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "ListPolicy", type()) << "\n"; }
-		
-	virtual void set(ArrowPosition) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }
-	virtual void setLocal(ArrowPosition) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }	
-	virtual void get(ArrowPosition&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }
+	virtual void set(ListPolicy) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "ListPolicy", type()) << "\n"; }
+	virtual void setLocal(ListPolicy) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "Justification", type()) << "\n"; }
+	virtual void get(ListPolicy&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "ListPolicy", type()) << "\n"; }
 	
-	virtual void set(const Matrix&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "Matrix", type()) << "\n"; }
-	virtual void setLocal(Matrix&) { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }	
-	virtual void get(Matrix&) const { Log::dev()<< "Magics-warning:" << MistmatchType(name_, "Matrix", type()) << "\n"; }
+	virtual void set(AxisAutomaticSetting) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "AxisAutomaticSetting", type()) << "\n"; }
+	virtual void setLocal(AxisAutomaticSetting) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "AxisAutomaticSetting", type()) << "\n"; }
+	virtual void get(AxisAutomaticSetting&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "AxisAutomaticSetting", type()) << "\n"; }
+			
+
+	virtual void set(ArrowPosition) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }
+	virtual void setLocal(ArrowPosition) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }	
+	virtual void get(ArrowPosition&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }
+	
+	virtual void set(const Matrix&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "Matrix", type()) << "\n"; }
+	virtual void setLocal(Matrix&) { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "ArrowPosition", type()) << "\n"; }	
+	virtual void get(Matrix&) const { MagLog::dev()<< "Magics-warning:" << MistmatchType(name_, "Matrix", type()) << "\n"; }
 #endif
 	virtual string type() const = 0;
 	
@@ -213,6 +222,7 @@ public:
 	string getType(const magvector<int>&) const { return "array of integer"; }
 	string getType(const magvector<double>&) const { return "array of real"; }
 	string getType(LineStyle) const { return "LineStyle"; }
+	string getType(AxisAutomaticSetting) const { return "AxisAutomaticSetting"; }
 	string getType(Justification) const { return "Justification"; }
 	string getType(ArrowPosition) const { return "ArrowPosition"; }
 	string getType(Matrix) const { return "2DMatrix"; }
