@@ -44,17 +44,24 @@ public:
 	CartesianTransformation();
 	virtual ~CartesianTransformation();
     virtual void operator()(Layout&) const;
-    
+    void init() {
+    	x_->set();
+    	y_->set();
+    }
+    virtual void toxml(ostream& out) const { CartesianTransformationAttributes::toxml(out); }
     virtual void set(const map<string, string>& map) { CartesianTransformationAttributes::set(map); }
     virtual void set(const XmlNode& node) { CartesianTransformationAttributes::set(node); }
     virtual bool accept(const string& node) { return CartesianTransformationAttributes::accept(node); }
 	virtual double x(double x) const { return (*x_)(x); }
 	virtual double y(double y) const { return (*y_)(y); }
+	virtual double rx(double x) const { return (*x_).revert(x); }
+	virtual double ry(double y) const { return (*y_).revert(y); }
+	virtual void fast_reproject(double& x, double& y) const { x = (*x_)(x); y = (*y_)(y);  }
 	
 	virtual PaperPoint operator()(const UserPoint& xy) const 
                  { return PaperPoint((*x_)(xy.x()), (*y_)(xy.y()),  xy.value()); }
 		
-   void aspectRatio(double&, double&) {}
+   void aspectRatio(double&, double&);
 
     virtual Transformation* clone() const {
 		CartesianTransformation* object = new CartesianTransformation();
@@ -93,24 +100,41 @@ public:
 	virtual void setDataMinY(double miny) const { y_->dataMin(miny); }
 	virtual void setDataMaxY(double maxy) const { y_->dataMax(maxy); }
 	
+	vector<double> getDataVectorMinX() const  { return  x_->mins(); }
+	vector<double> getDataVectorMaxX() const { return  x_->maxs(); }
+	vector<double> getDataVectorMinY() const  { return  y_->mins(); }
+	vector<double> getDataVectorMaxY() const { return  y_->maxs(); }
+	
+
+	Polyline& getPCBoundingBox()   const  {  boxes(); return *PCEnveloppe_; }
+
+	void boxes() const;
+
 
     virtual void setDataMinX(double minx, const string& ref) const { 
-    	x_->dataMin(minx, ref); 
+    	dataMinX_ = std::min(minx, dataMinX_);
+    	x_->dataMin(dataMinX_, ref); 
     	referenceX_ = x_->reference();
     }
 	virtual void setDataMaxX(double maxx, const string& ref) const { 
-		x_->dataMax(maxx, ref); 
+		dataMaxX_ = std::max(maxx, dataMaxX_);
+		x_->dataMax(dataMaxX_, ref); 
 		referenceX_ = x_->reference();
 	}
 	virtual void setDataMinY(double miny, const string& ref) const { 
-		y_->dataMin(miny, ref); 
+		dataMinY_ = std::min(miny, dataMinY_);
+		y_->dataMin(dataMinY_, ref); 
 		referenceY_ = y_->reference();
 	}
 	virtual void setDataMaxY(double maxy, const string& ref) const { 
-		y_->dataMax(maxy, ref); 
+		dataMaxY_ = std::max(maxy, dataMaxY_);
+		y_->dataMax(dataMaxY_, ref); 
 		referenceY_ = y_->reference();
 	}; 
-
+	virtual void setAutomaticX(bool automatic) { x_->automatic(automatic); }
+		virtual void setAutomaticY(bool automatic) { y_->automatic(automatic); }
+		virtual bool getAutomaticX() const { return  x_->automatic(); } 
+		virtual bool getAutomaticY() const { return y_->automatic(); }
 	
 	virtual bool in(double x, double y) const 
          {  double minx = std::min(x_->min(), x_->max() );
@@ -118,7 +142,12 @@ public:
          	double miny = std::min(y_->min(), y_->max() );
          	double maxy = std::max(y_->min(), y_->max() );
          	return ( minx <= x && x <=  maxx && miny <= y && y <= maxy ); }
-
+	
+	string xAxisType() const { return x_->type(); }
+	   string yAxisType() const { return y_->type(); }
+	void visit(MetaDataVisitor&, double, double, double, double,double, double);
+	void getNewDefinition(const UserPoint&, const UserPoint&, string&) const;
+	void setDefinition(const string&);
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
