@@ -18,7 +18,7 @@
 
 /*! \file Clipper.h
     \brief Definition of the clipping.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: 2004
 
@@ -81,7 +81,7 @@ class Clipper
 {
 public:
 	Clipper(const Transformation& transformation) : 
-		lastX_(int_MIN), lastY_(int_MIN), 
+		lastX_(INT_MIN), lastY_(INT_MIN), 
 		lastIn_(false),  current_(0), 
 		transformation_(transformation),
 		missing_(true) 
@@ -116,17 +116,17 @@ public:
     	current_ = poly.getShade();
     	
     	if ( current_ ) { 
-    		vector<GeoPoint>& geo = poly.coastlines();
+    		vector<UserPoint>& geo = poly.coastlines();
     		pointin_ = false;
-    		for ( vector<GeoPoint>::const_iterator point = geo.begin(); point != geo.end(); ++point)    		
+    		for ( vector<UserPoint>::const_iterator point = geo.begin(); point != geo.end(); ++point)    		
     			close(transformation_(*point));
     		current_->push_back(current_->front());   	
         	 helper_.push_back(current_);
     	}
 
     	current_ = poly.getContour();
-    	vector<GeoPoint>& geo = poly.coastlines();     	
-    	for ( vector<GeoPoint>::const_iterator point = geo.begin(); point != geo.end(); ++point)    		
+    	vector<UserPoint>& geo = poly.coastlines();     	
+    	for ( vector<UserPoint>::const_iterator point = geo.begin(); point != geo.end(); ++point)    		
     		(*this)( transformation_(*point));   	    		
     	  if ( !current_->empty() ) {
     		  helper_.push_back(current_); 
@@ -141,33 +141,17 @@ public:
     	//if  ( poly.allMissing() ) return;
         current_ = new Polyline();   
     	current_->copy(poly);  	
-    	lastX_ = int_MIN;
-    	lastY_ = int_MIN; 
+    	lastX_ = INT_MIN;
+    	lastY_ = INT_MIN; 
 		lastIn_ = false;
     	value_ = poly.front().value();
     	allmissing_ = poly.allMissing();
         static int i = 0;
         i++;
 
-        // Useful print to debug clipping problems...
-        //Log::dev()<< "[" << poly.front().x() << ", " << poly.front().y() << "]--->[" <<  poly.back().x() << ", " << poly.back().y() << "] at " << i << " with " << poly.size() << " points" << endl;
-    	//if ( i != 1 )
-    		//return;
 
-		transformation_.wrapAround(poly);
-
-    	/*Polyline projected;
-	    typename Polyline::const_iterator point = poly.begin();
-    	for ( ; point != poly.end(); ++point) {
-    		//if (i == 1) Log::dev()<< "[" << point->x() << ", " << point->y() << "]" << endl;
-    		projected.push_back(*point);
-    	}
-    	*/
-    	
-    	
     	for ( Polyline::const_iterator point = poly.begin(); point != poly.end(); ++point) {
-    		//PaperPoint clip = *point;
-    		//transformation_.clip(clip);
+
     	 	(*this)(*point);
     	}
     	
@@ -191,19 +175,19 @@ public:
 
     MAGICS_NO_EXPORT void push_intersection(double x, double y) 
     {
-        if (lastX_ == int_MIN) return;
+        if (lastX_ == INT_MIN) return;
 
         double a = (lastX_ == x) ? 0 : A(lastX_, lastY_, x, y);
         double b = B(a, x, y);
 
         // Intersect left?
-        if  ( (x <= left_ && lastX_ >= left_) || (lastX_ <= left_ && x >= left_) )
+        if  ( (x < left_ && lastX_ > left_) || (lastX_ < left_ && x > left_) )
 	{
             current_->push_back(PaperPoint(left_, Y(a, b, left_), value_, allmissing_));
             return;
         }
         // Intersect top?
-    if  ( (y <= top_ && lastY_ >= top_) || ( lastY_ >= bottom_ && lastY_ <= top_ && y >= top_) )
+    if  ( (y < top_ && lastY_ > top_) || ( lastY_ > bottom_ && lastY_ < top_ && y > top_) )
 	{
         double nx = (!a) ? x : X(a, b, top_);
         
@@ -216,14 +200,14 @@ public:
 	}
 
         // Intersect right?
-        if ( (x <= right_ && lastX_ >= right_) || (lastX_ <= right_ && x >= right_) )
+        if ( (x < right_ && lastX_ > right_) || (lastX_ < right_ && x > right_) )
 	{
             current_->push_back(PaperPoint(right_, Y(a, b, right_), value_, allmissing_));
             return;
         }
 
         // intersect bottom?
-        if ( (y <= bottom_ && lastY_ >= bottom_ && lastY_ <= top_ ) || ( lastY_ <= bottom_ && y >= bottom_) )
+        if ( (y < bottom_ && lastY_ > bottom_ && lastY_ < top_ ) || ( lastY_ < bottom_ && y > bottom_) )
 	{
             if (!a ) current_->push_back(PaperPoint(x, bottom_));
             else current_->push_back(PaperPoint(X(a, b, bottom_), bottom_, value_, allmissing_));
@@ -293,8 +277,8 @@ public:
         else {
             // push_back intersection
             push_intersection(point.x(), point.y());
-            lastX_ = current_->back().x();
-            lastY_ = current_->back().y();
+            lastX_ = point.x();
+            lastY_ = point.y();
             // create new Polyline...
            Polyline* poly = new Polyline();
             if ( !transformation_.concatenate(helper_, current_) ) 
