@@ -28,7 +28,7 @@
 
 */
 #include "Colour.h"
-#include "Log.h"
+#include "MagLog.h"
 #include "BaseParameter.h"
 #include "XmlNode.h"
 
@@ -43,7 +43,7 @@
    
    \todo Where are the web colours defined?
    
-   Exception of type BadRGBFormat and BadHSLFormat are thrown in case
+   MagException of type BadRGBFormat and BadHSLFormat are thrown in case
    values look wrong.
 */
 
@@ -93,7 +93,7 @@ Colour::Colour(const string& name) : name_(lowerCase(name)), automatic_(false)
 
 void Colour::set(const XmlNode& xml)
 {
-	Log::debug() << "set Colour-->" << xml << "\n";
+	MagLog::debug() << "set Colour-->" << xml << "\n";
 	try {
 		setColour(xml.data());
 	}
@@ -108,11 +108,13 @@ void Colour::setColour(const string& name)
 {
 	if ( name.empty() )
 	{
-		Log::warning() << "No colour name given. Red used.\n";
+		MagLog::warning() << "No colour name given. Red used.\n";
 		name_ = "red";
-		rgb_ = Rgb(1., 0., 0.);	
+		rgb_ = Rgb(1., 0., 0.);
+		MagLog::broadcast();
 		return;
 	}
+	try {
 	// get rid of white space...
 	string::size_type first = name.find_first_not_of(" ");  
 	string::size_type last = name.find_last_not_of(" ");  
@@ -151,7 +153,7 @@ void Colour::setColour(const string& name)
 			    rgb_ = Rgb(red, green, blue, 1.);
             }
             else {
-                Log::warning() << "The color [" << name_ << "] is not valid. Red used.\n";
+                MagLog::warning() << "The color [" << name_ << "] is not valid. Red used.\n";
 			    name_ = "red";
 			    rgb_ = Rgb(1., 0., 0.,1.);	
             }
@@ -159,12 +161,20 @@ void Colour::setColour(const string& name)
 		}
 		else
 		{
-			Log::warning() << "The color [" << name_ << "] is not valid. Red used.\n";
+			MagLog::warning() << "The color [" << name_ << "] is not valid. Red used.\n";
 			name_ = "red";
 			rgb_ = Rgb(1., 0., 0.,1.);	
 		}
 	}
 	else rgb_ = (*colour).second;
+	}
+	catch (...) {
+		MagLog::warning() << " Invalid colour name " << name  << ". Red used.\n";
+				name_ = "red";
+				rgb_ = Rgb(1., 0., 0.);
+				MagLog::broadcast();
+				return;
+	}
 }
 
 void Colour::setColour(float red, float green, float blue, float alpha)
@@ -217,6 +227,7 @@ Hsl Rgb::hsl() const
     float diff = max - min;             //Delta RGB value
 
     hsl.light_ = ( max + min ) / 2;
+    hsl.alpha_ = alpha_;
 
     if ( diff == 0 )                     //This is a gray, no chroma...
     {
@@ -256,6 +267,7 @@ inline float hue2rgb(float v1, float v2, float vH )             //Function Hue_2
 Rgb Hsl::rgb() const 
 {
     Rgb rgb;
+    rgb.alpha_ = alpha_;
     float var1, var2;
     if ( saturation_ == 0 ) {
         rgb.red_ = light_;
@@ -279,6 +291,7 @@ Rgb Hsl::rgb() const
     return rgb;
 }
 
+
 bool Colour::valid(const string& name)
 {
 	init();
@@ -360,7 +373,7 @@ void Colour::init() {
 }
 
 #include "BackgroundColour.h"
-Colour* Translator<string, Colour>::operator()(const string& val )
+Colour* MagTranslator<string, Colour>::operator()(const string& val )
 {
  	 if (Colour::valid(val) ) return new Colour(val);
  	 if ( val == "colour" ) return new Colour();
@@ -370,15 +383,15 @@ Colour* Translator<string, Colour>::operator()(const string& val )
 
 Rgb::Rgb(const string& name): red_(1.), green_(1.), blue_(1.),alpha_(1.)
 {
-	istringstream in(name);
-	stringbuf token;
-	in >> ws;
-	in.get(token, '(');
+  istringstream in(name);
+  stringbuf token;
+  in >> ws;
+  in.get(token, '(');
 
-	if (in.eof()) throw BadRgbFormat();
+  if (in.eof()) throw BadRgbFormat();
 
-	if (magCompare(token.str(), "rgb"))
-	{
+  if (magCompare(token.str(), "rgb"))
+  {
 	      in.ignore(name.length(), '(');
 	      in >> red_; 
 	      if (red_ <0 || red_>1) throw BadRgbFormat();
@@ -391,9 +404,9 @@ Rgb::Rgb(const string& name): red_(1.), green_(1.), blue_(1.),alpha_(1.)
 	      if (in.eof()) throw BadRgbFormat();
 	      in >> blue_; 
 	      if (blue_ <0 || blue_>1) throw BadRgbFormat();
-    }
-    else if (magCompare(token.str(), "rgba")) {
-         in.ignore(name.length(), '(');
+  }
+  else if (magCompare(token.str(), "rgba")) {
+	      in.ignore(name.length(), '(');
 	      in >> red_; 
 	      if (red_ <0 || red_>1) throw BadRgbFormat();
 
@@ -414,16 +427,17 @@ Rgb::Rgb(const string& name): red_(1.), green_(1.), blue_(1.),alpha_(1.)
           throw BadRgbFormat();
 }
 
+
 Hsl::Hsl(const string& name)
 {
-	istringstream in(name);
-	stringbuf token;
-	in >> ws;
-	in.get(token, '(');
+    istringstream in(name);
+    stringbuf token;
+    in >> ws;
+    in.get(token, '(');
 
-	if (in.eof()) throw BadHslFormat();
+    if (in.eof()) throw BadHslFormat();
 
-	if (magCompare(token.str(),"HSL")) {
+    if (magCompare(token.str(),"HSL")) {
 
         in.ignore(name.length(), '(');
         in >> hue_; 
@@ -438,6 +452,7 @@ Hsl::Hsl(const string& name)
         if (in.eof()) throw BadHslFormat();	    
         in >> light_; 
         if ( light_ <0 || light_>1) throw BadHslFormat();
+        alpha_=1.;
     }
     else if (magCompare(token.str(),"HSLA")) {
         in.ignore(name.length(), '(');
@@ -459,7 +474,7 @@ Hsl::Hsl(const string& name)
         in >> alpha_; 
         if ( alpha_ <0 || alpha_>1) throw BadHslFormat();
     }
-    else BadHslFormat();
+    else throw BadHslFormat();
 }
 
 
@@ -507,3 +522,8 @@ istream& operator>>(istream& s, Hsl& p)
 	if (p.light_ <0 || p.light_>1) throw BadHslFormat();
 	return s; 
 }
+bool Colour::none() const
+{
+	static Colour none("none");
+	return *this == none;
+}
