@@ -73,6 +73,7 @@ public:
 };
 
 class Layout;
+class LayoutVisitor;
 
 class LayoutFrame 
 {
@@ -85,20 +86,38 @@ protected:
 	int thickness_;
 	LineStyle style_;
 	Colour colour_;
-	bool blanking_;	
+	bool blanking_;
 	bool visible_;
-	friend class Layout;	
-	
+	friend class Layout;
 };
 
+
+class StartPage : public BasicGraphicsObject
+{
+public:
+	StartPage() {}
+	virtual ~StartPage() {}
+	void redisplay(const BaseDriver& driver) const;
+};
+
+
+class EndPage : public BasicGraphicsObject
+{
+public:
+	EndPage() {}
+	virtual ~EndPage() {}
+	void redisplay(const BaseDriver& driver) const;
+};
+
+
 class Layout : public BasicGraphicsObjectContainer
 {
 public:
 	Layout();
 	virtual ~Layout();
 
-	virtual Layout*  clone();
-	virtual Layout*  newLayout() { return new Layout(); }
+	virtual Layout*  clone() const;
+	virtual Layout*  newLayout() const { return new Layout(); }
 	
 	bool reproject(BasicGraphicsObjectContainer&) const;	
 	void redisplay(const BaseDriver& driver) const;
@@ -124,7 +143,11 @@ public:
 	double absoluteX()      		const; //absolute position from the root
 	double absoluteY()      		const; //absolute position from the root
 	double absoluteWidth()  const; //absolute position from the root
-	double absoluteHeight() const; //absolute position from the root
+	double absoluteHeight()  const; //absolute position from the root
+	
+	bool buildTree(const Layout&, unsigned int, const BaseDriver&) const;
+	void release(); // Release the graphical objects!
+	
 	virtual Layout* execute(AnimationStep& step,  const Layout* visitor);
 
 	
@@ -155,6 +178,10 @@ public:
 			width = driverInfos_.front().width_;
 			height = driverInfos_.front().height_;
 		}
+		else {
+			if ( parent_) 
+				parent_->getDriverInfo(x, y, width, height);
+		}
 	}
 
 	void frame(const Layout& other)
@@ -177,6 +204,9 @@ public:
 	void zoomable(bool zoom = true) { zoomable_ = zoom; }
 	bool isZoomable() const { return zoomable_; }
 	
+	void setNavigable() { navigable_ = true; }
+	bool isNavigable() const { return navigable_; }
+
 	const string& id() const { return id_; }
 	void id(const string& id) { id_ = id; }
     int zoomLevels() const { return zoomLevels_; }
@@ -199,7 +229,9 @@ public:
   
     void frameIt() { frame_.frame(*this); }
     void blankIt() { frame_.blank(*this); }
-
+    void resolve(bool resolve) { resolve_  = resolve; }
+    bool resolve() const { return resolve_; }
+    virtual void resize(double, double) {}
 protected:
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	virtual void print(ostream&) const; 
@@ -222,54 +254,32 @@ protected:
 	double ymax_;
 	
 	bool zoomable_;
+	bool navigable_;
 	// Zoom information : only used if mode is ZOOMABLE
 	int zoomLevels_;
 	int zoomCurrentLevel_;
 	string id_;
 	
 	bool resizable_;
+	bool resolve_;
 	
 	LayoutFrame frame_;
 		
-	mutable vector<DriverInfo> driverInfos_;
+	static vector<DriverInfo> driverInfos_;
+	friend class LayoutVisitor;
+	friend class TopAxisVisitor;
+	friend class BottomAxisVisitor;
+	friend class LeftAxisVisitor;
+	friend class RightAxisVisitor;
 	
 private:
 	//! Copy constructor - No copy allowed
 	Layout(const Layout&);
 	//! Overloaded << operator to copy - No copy allowed
 	Layout& operator=(const Layout&);
-	
 // 
 };
 
-class StandaloneLayout : public Layout
-{
-public:
-	StandaloneLayout();
-	virtual ~StandaloneLayout() {}
-	void redisplay(const BaseDriver& driver) const;
-	
-	const string& path() const { return path_; }
-	void path(const string& path)  { path_ = path; }
-
-	const string& format() const { return format_; }
-	void format(const string& format)  { format_ = format; }
-	
-	double outputWidth() const { return output_width_; }
-	void outputWidth(double width)  { output_width_ = width; }
-	
-	double outputHeight() const { return output_height_; }
-	void outputHeight(double height)  { output_height_ = height; }
-	
-	virtual void print(ostream&) const; 
-
-protected :
-	string path_;
-	string format_;
-	double output_width_;
-	double output_height_;
-	
-};
 
 class PreviewLayout : public Layout 
 {
@@ -277,6 +287,31 @@ public:
 	PreviewLayout();
 	virtual ~PreviewLayout();
 	void redisplay(const BaseDriver& driver) const;
+	
+};
+class LegendLayout : public Layout
+{
+public:
+	LegendLayout();
+	virtual ~LegendLayout();
+	void redisplay(const BaseDriver& driver) const;
+
+};
+class HistoLayout : public Layout 
+{
+public:
+	HistoLayout();
+	virtual ~HistoLayout();
+	void redisplay(const BaseDriver& driver) const;
+};
+
+class SceneLayout : public Layout
+{
+public:
+	SceneLayout();
+	virtual ~SceneLayout();
+	void redisplay(const BaseDriver& driver) const;
+	
 };
 
 
@@ -287,7 +322,7 @@ public:
 	MagnifierLayout();
 	virtual ~MagnifierLayout();
 	void redisplay(const BaseDriver& driver) const;
-	virtual void redisplay(const BaseDriver&, AnimationStep*, vector<PaperPoint>&,float,float) const {}
+	virtual void redisplay(const BaseDriver&, vector<PaperPoint>&,float,float) const {}
 };
 
 class RootLayout : public Layout
@@ -299,26 +334,36 @@ public:
 	double absoluteHeight() const { return absoluteHeight_; }
 	virtual bool childOfRoot() { return false; }
 	void redisplay(const BaseDriver& driver) const;
-	
-	
+
+	 virtual void resize(double width, double height) { absoluteWidth_ = width; absoluteHeight_ = height; }
+
 protected:
 	double absoluteWidth_;
 	double absoluteHeight_; 
+
 };
+class BasicLayout : public Layout
+{
+public:
+	BasicLayout();
+	virtual ~BasicLayout();
+	bool buildTree(const Layout&, unsigned int, const BaseDriver&) const;
+};
+class LayoutVisitor;
+
+
 class LayoutHelper
 {
 public:
 	LayoutHelper();
 	~LayoutHelper();
-	void add(Layout*);
-	void attachTop(Layout*);
-	void attachNoConstraintTop(Layout*);
-	void attachLeft(Layout*);
-	void attachRight(Layout*);
-	void attachBottom(Layout*);
-	
-	
-	
+	void add(LayoutVisitor*);
+	void attachTop(LayoutVisitor*);
+	void attachNoConstraintTop(LayoutVisitor*);
+	void attachLeft(LayoutVisitor*);
+	void attachRight(LayoutVisitor*);
+	void attachBottom(LayoutVisitor*);
+
 protected :
 	virtual void print(ostream&) const; 
 	double xmin_;
@@ -326,9 +371,8 @@ protected :
 	double ymin_;
 	double ymax_;
 	friend ostream& operator<<(ostream& s,const LayoutHelper& p)
-			{ p.print(s); return s; }
+		{ p.print(s); return s; }
 };
 
-
 } // namespace magics
 #endif
