@@ -16,8 +16,8 @@
 
  ******************************** LICENSE ********************************/
 
-/*! \file LogoPlottingBase.h
-    \brief Definition of the Template class LogoPlottingBase.
+/*! \file MagLogoPlottingBase.h
+    \brief Definition of the Template class MagLogoPlottingBase.
     
     Magics Team - ECMWF 2006
     
@@ -27,11 +27,11 @@
     
 */
 
-#ifndef LogoPlottingBase_H
-#define LogoPlottingBase_H
+#ifndef MagLogoPlottingBase_H
+#define MagLogoPlottingBase_H
 
 #include "magics.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 #include "Factory.h"
 #include "BasicGraphicsObject.h"
 
