@@ -25,7 +25,7 @@ namespace magics {
 class MagicsFormat 
 {
 public:
-	MagicsFormat(const string& format) : format_(format) {}
+	MagicsFormat(const string& format,double value) : format_(format), value_(value) {}
 	virtual ~MagicsFormat() {}	
 	bool valid(ostream& out) const;
 
@@ -38,6 +38,7 @@ public:
 		return manip.format(out);
 	}
 	string format_;
+	double value_;
 };
 
 
