@@ -30,7 +30,9 @@
 #ifndef Matrix_H
 #define Matrix_H
 
+#include <cfloat>
 #include "magics.h"
+#include "MagException.h"
 
 namespace magics {
 
@@ -51,13 +53,16 @@ public :
     virtual double  real_column(double, double) const = 0;
     virtual int    lowerRow(double) const = 0;
     virtual int    lowerColumn(double) const = 0;    
-    virtual double  interpolate(double  i, double  j) const = 0;   
+    virtual double  interpolate(double  i, double  j) const = 0; 
+    virtual double  nearest(double i, double j, double &iOut, double &jOut) const = 0;  
+    virtual double  nearest(double i, double j) const =0 ;
     virtual double  missing() const = 0;
     virtual double  XResolution() const = 0;
     virtual double  YResolution() const = 0;
     virtual double  width() const = 0;
     virtual double   height() const = 0;
     virtual bool akimaEnable() const { return false; }
+
   
 
    
@@ -75,17 +80,19 @@ public :
      template <class O> 
      void for_each(int xf, int yf, const O& object)
      {
-         int i,j; // STEPHAN: I assume that is right ... ???
-     	 for ( i = firstRow(); i > 0; i = nextRow(i, xf) )
-	 	for ( j = firstColumn(); j > 0; j = nextColumn(j, yf) )
-			object(row(i,j), column(i,j), (*this)(i, j));
+        
+     	 for ( int i = firstRow(); i > 0; i = nextRow(i, xf) )
+     		 for ( int j = firstColumn(); j > 0; j = nextColumn(j, yf) )
+     			 	object(row(i,j), column(i,j), (*this)(i, j));
      }
 
     virtual double  minX() const = 0;
     virtual double  minY() const = 0;
     virtual double  maxX() const = 0;
     virtual double  maxY() const = 0;
-    
+    virtual double  min() const = 0;
+    virtual double  max() const = 0;
+     
     virtual double  left() const = 0;
        virtual double  top() const = 0;
        virtual double  right() const = 0;
@@ -161,12 +168,11 @@ public:
 	Matrix(int rows, int columns): 
 		rows_(rows), 
 		columns_(columns),
-		missing_(int_MIN),
-		akima_(false)
-		
+		missing_(DBL_MIN),
+		akima_(false),
+		min_(DBL_MAX), max_(DBL_MIN)
 	{  
-         reserve(rows_*columns_); 
-         
+         set(rows, columns);
 	}
 	
 	Matrix* clone() { return new Matrix(); }
@@ -174,7 +180,7 @@ public:
 	
     Matrix(int rows, int columns, double val):     
         rows_(rows), 
-        columns_(columns), missing_(int_MIN), akima_(false)
+        columns_(columns), missing_(DBL_MIN), akima_(false), min_(DBL_MAX), max_(DBL_MIN)
      {       
          
          resize(rows_ * columns_, val); 
@@ -182,7 +188,7 @@ public:
          columnsAxis_.resize(columns_, val); 
 	}
     
-    Matrix(): missing_(int_MIN), akima_(false) {}
+    Matrix(): missing_(DBL_MIN), akima_(false), min_(DBL_MAX), max_(DBL_MIN) {}
     
     void set(int rows, int columns) 
     {
@@ -193,7 +199,10 @@ public:
          columnsAxis_.reserve(columns);  
     }
     
-    
+     double min() const;
+     double max() const;
+     void min(double m) {min_=m;}
+     void max(double m) {max_=m;}
     
     virtual ~Matrix() {}
     
@@ -208,26 +217,19 @@ public:
     double real_row(double r, double) const { return r; }
     double real_column(double, double c) const { return c; }
     
-    int  lowerRow(double r) const {    
-		map<double, int>::const_iterator bound = rowsMap_.find(r);
-		if ( bound != rowsMap_.end() ) return (*bound).second;
-    	
-		bound = rowsMap_.lower_bound(r);
-        if ( bound == rowsMap_.end() ) return -1;
-		return (*bound).second - 1;
-   
-    }  
-      
-  
-   
-    int  lowerColumn(double c) const {   	
-		map<double, int>::const_iterator bound = columnsMap_.find(c);
-		if ( bound != columnsMap_.end() ) return (*bound).second;
-	    	
-		bound = columnsMap_.lower_bound(c);
-	        if ( bound == columnsMap_.end() ) return -1;  
-		return (*bound).second - 1;
+
+    void release()
+    {
+    	rows_ = 0;
+    	columns_ = 0;
+    	rowsAxis_.clear();
+    	rowsAxis_.resize(0);
+    	columnsAxis_.clear();
+    	this->clear();
+    	columnsAxis_.resize(0);
+		this->resize(0);
     }
+ 
     
     double regular_column(int j) const { return columnsAxis_[j];  }
     double column(int, int j) const { return columnsAxis_[j];  }
@@ -274,70 +276,9 @@ public:
     }
     
   
-    double interpolate(double r, double c) const 
-    {   
-        int ii, jj;
-        //Log::debug() << " interpolate---> " << r << " , " << c << "\n";
-       
-            ii = row_ind(r);
-        
-       if (ii == -1) {
-           
-            map<double, int>::const_iterator bound = rowsMap_.lower_bound(r);
-            
-            if ( bound == rowsMap_.end() ) return missing_;
-            double xr = (*bound).first;
-            if (bound == rowsMap_.begin()) 
-                ii = (*bound).second;
-                
-            else {
-                double xl = (*--bound).first;
-          
-           //Log::debug() << xl << " < " << r << " < " << xr << "\n";
-            
-            double a = (*this).interpolate(xl, c);
-            double b = (*this).interpolate(xr, c);
-          
-            if ( same(a, missing_) || same(b, missing_) ) return missing_;
-            
-            double da = (xr-r)/(xr-xl);
-            double db = (r-xl)/(xr-xl);
-            double val = (a*da) + (b*db);
-            //Log::debug() << a << "*" << da << "+" << b << "*" << db << "=" << val << "\n";
-            return val;
-            }
-        }    
-       
-        jj = column_ind(c);
-        
-        if ( jj == -1 ) {
-             map<double, int>::const_iterator bound = columnsMap_.lower_bound(c);
-            if ( bound == columnsMap_.end() ) return missing_;
-            
-            double yr = (*bound).first;
-          
-            
-            if (bound == columnsMap_.begin()) 
-                jj = (*bound).second;
-            else {
-            double yl = (*--bound).first;
-            //Log::debug() << yl << " < " << c << " < " << yr << "\n";
-            double a = (*this)(ii, column_ind(yl));           
-            double b = (*this)(ii, column_ind(yr));
-            
-            if ( same(a, missing_) || same(b, missing_) ) return missing_;
-            
-            double da = ((yr-c)/(yr-yl));
-            double db = (c-yl)/(yr-yl);
-            double val = (a*da) + (b*db);
-            //Log::debug() << a << "*" << da << "+" << b << "*" << db << "=" << val << "\n";
-            return val;
-            }
-            
-        }    
-	
-        return (*this)(ii, jj);
-    }
+    double interpolate(double r, double c) const;
+    double nearest(double i, double j) const {double d1, d2; return nearest(i,j,d1,d2);}
+    double nearest(double i, double j,double &iOut, double &jOut) const;
     
    void multiply(double factor) 
     {
@@ -367,19 +308,15 @@ public:
      }
      
     
-    double operator()(int row, int column) const
-    { 
-           // Here for perfrormance we are trusting the user we do not catch exception 
-				return (*this)[ row * columns_ + column];
-          
-    }
-    
+    double operator()(int row, int column) const;
+
     double YResolution() const {
+    	
            magvector<double> diff;
            diff.reserve(rowsAxis_.size());
            std::adjacent_difference(rowsAxis_.begin(), rowsAxis_.end(), back_inserter(diff));
            double resol = std::accumulate(diff.begin()+1, diff.end(), 0.)/(diff.size()-1);
-           //Log::dev() << "Matrix::YResolution()--->" << resol << "\n";
+           //MagLog::dev() << "Matrix::YResolution()--->" << resol << "\n";
            return resol;
     }
      double XResolution() const {
@@ -387,7 +324,7 @@ public:
            diff.reserve(columnsAxis_.size());
            std::adjacent_difference(columnsAxis_.begin(), columnsAxis_.end(), back_inserter(diff));
            double resol = std::accumulate(diff.begin()+1, diff.end(), 0.)/(diff.size()-1);
-           //Log::dev() << "Matrix::XResolution()--->" << resol << "\n";
+           //MagLog::dev() << "Matrix::XResolution()--->" << resol << "\n";
            return resol;
     }
    
@@ -416,26 +353,63 @@ public:
     virtual void boundRow(double r, 
     	double& row1, int& index1, double& row2, int& index2) const { 
     	
-    	map<double, int>::const_iterator bound = rowsMap_.lower_bound(r);
-    	row1 = bound->first;
-    	index1 = bound->second;
-    	bound++;
-    	row2 = bound->first;
-    	index2 = bound->second;
+    	index1 = this->lowerRow(r);
+           		row1 = this->regular_row(index1);
+           		index2 = this->upperRow(r);
+           		row2 = this->regular_row(index2);
     } 
     
     virtual void boundColumn(double r, 
     	double& column1, int& index1, double& column2, int& index2) const { 
     	
-    	map<double, int>::const_iterator bound = columnsMap_.lower_bound(r);
-    	column1 = bound->first;
-    	index1 = bound->second;
-    	bound++;
-    	column2 = bound->first;
-    	index2 = bound->second;
+    	index1 = this->lowerColumn(r);
+    	column1 = this->regular_column(index1);
+        index2 = this->upperColumn(r);
+        column2 = this->regular_column(index2);
     } 
     
-    
+	int    lowerRow(double r) const {
+
+		int last = -1;
+		for ( map<double, int>::const_iterator i = rowsMap_.begin(); i != rowsMap_.end(); ++i) { 	
+			if ( i->first >  r  ) {				
+				return last;
+			}				
+			last = i->second;
+		}		
+		return last;	
+	}
+	int    lowerColumn(double c) const { 
+		
+		int last = -1;
+		for ( map<double, int>::const_iterator i = columnsMap_.begin(); i != columnsMap_.end(); ++i) {
+				if ( i->first > c  ) 
+					return last;
+				last = i->second;
+		}
+		return last;
+    } 
+	int    upperRow(double r) const {
+		
+		
+		for ( map<double, int>::const_iterator i = rowsMap_.begin(); i != rowsMap_.end(); ++i) { 	
+			if ( i->first >=  r  ) {				
+				return i->second;
+			}				
+		
+		}		
+		return -1;	
+	}
+	int    upperColumn(double c) const { 
+		
+		
+		for ( map<double, int>::const_iterator i = columnsMap_.begin(); i != columnsMap_.end(); ++i) {
+				if ( i->first >= c  ) 
+					return i->second;
+		}
+		return -1;
+    } 
+
     
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
@@ -460,17 +434,32 @@ protected:
      bool akima_;
 
     int row_ind(double row) const {
-        map<double, int>::const_iterator i = rowsMap_.find(row);
-        if ( i != rowsMap_.end() ) return (*i).second;
-        return -1;
+        map<double, int>::const_iterator i = rowsMap_.lower_bound(row);
+    	if ( same(i->first, row) )
+    			return i->second;
+        if ( i == rowsMap_.end() ) {
+            map<double, int>::const_reverse_iterator i = rowsMap_.rbegin();
+            if ( same(i->first, row) )
+                return i->second;
+        }
+    	return -1;
     }
     int column_ind(double column) const {
-        map<double, int>::const_iterator j = columnsMap_.find(column);
-        if ( j != columnsMap_.end() ) return (*j).second;
-        return -1;
+	  
+       map<double, int>::const_iterator i = columnsMap_.lower_bound(column);
+	   
+    	if ( same(i->first, column) )
+    	    	return i->second;
+        if ( i == columnsMap_.end() ) {
+            map<double, int>::const_reverse_iterator i = columnsMap_.rbegin();
+            if ( same(i->first, column) )
+                return i->second;
+        }
+    	return -1;
     }    
 private:
-   
+   mutable double min_;
+   mutable double max_;
     
 // -- Friends
     //! Overloaded << operator to call print().
@@ -484,52 +473,58 @@ class ProjectedMatrix: public Matrix
 {
 public:
 	    
-		ProjectedMatrix(int rows, int columns): Matrix( rows, columns) {
-			 rows_ = rows;
-			         columns_ = columns;
-			         reserve(rows_*columns_); 
-			         rowsAxis_.reserve(rows);  
-			         rowsArray_.reserve(rows_*columns_);  
-			         columnsArray_.reserve(rows_*columns_);  
-			         columnsAxis_.reserve(columns);  
-		}
+		ProjectedMatrix(int rows, int columns);
 	
-		double row(int row, int column) const {
-			 return rowsArray_[ row * columns_ + column];
-		}
 		
-		double column(int row, int column) const {
-					 return columnsArray_[ row * columns_ + column];
-				}
-		  double real_column(int row, int column) const { 
-			  return columnsArray_[ row * columns_ + column];
-		      
-		  }
-		  double real_row(int row,  int column) const { 
-			  return rowsArray_[ row * columns_ + column];
-		   }
-		  vector<double>&  rowsArray() const { return rowsArray_; }
-		  vector<double>&  columnsArray() const { return columnsArray_; }
-		  void setMapsAxis() 
-		    {
-		        
-		        for (int i = 0; i < rows_; i++) {
-		            rowsAxis_.push_back(i);
-		        	rowsMap_[i] = i;
-		        }
-		        for (int i = 0; i < columns_; i++) {
-		        	columnsAxis_.push_back(i);
-		        	columnsMap_[i] = i;
-		       }
-		        
-		        
-		       
-		    }
+		void getReady(); // Prepare the matrix ... 
+		  
+		vector<double>&  values() const { return values_; }
+		vector<double>&  rowsArray() const { return rowsArray_; }
+		vector<double>&  columnsArray() const { return columnsArray_; }
+		
+		int index(int r, int c) { return (r* origColumns_) + c; }
+		  
+		  
 protected:
+	      void build();
+	      
+	      int origColumns_;
+	      int origRows_;
 		  mutable vector<double> rowsArray_;
 		  mutable vector<double> columnsArray_; 
-		 
+		  mutable vector<double> values_; 
+		  
+		  double minx_;
+		  double miny_;
+		  double maxx_;
+		  double maxy_;
+		  
+		  double stepx_;
+		  double stepy_;
+		  
 			
+};
+class RotatedMatrix: public Matrix
+{
+
+public:
+		RotatedMatrix(int, int);
+
+		vector<double>&  values() const { return values_; }
+		vector<double>&  rowsArray() const { return rowsArray_; }
+		vector<double>&  columnsArray() const { return columnsArray_; }
+
+		double operator()(int r, int c) const { return values_[r* columns_ + c]; }
+		double row(int r, int c) const { return rowsArray_[r* columns_ + c]; }
+		double column(int r, int c) const { return columnsArray_[r* columns_ + c]; }
+protected:
+
+		mutable vector<double> rowsArray_;
+		mutable vector<double> columnsArray_;
+		mutable vector<double> values_;
+
+
+
 };
 
 } // namespace magics
