@@ -18,7 +18,7 @@
 
 /*! \file OutputFactory.cc
     \brief Implementation of the Template class OutputFactory.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Wed July 2006
 */
