@@ -18,7 +18,7 @@
 
 /*! \file OutputFactory.h
     \brief Definition of the Template class OutputFactory.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Wed July-2006
 */
@@ -27,7 +27,7 @@
 #define OutputFactory_H
 
 #include "magics.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 #include "Factory.h"
 
 
@@ -46,15 +46,15 @@ public:
 
 	virtual void set(const XmlNode&)
 	{
-		Log::dev() << "(const XmlNode&)---> to be checked!...\n";
+		MagLog::dev() << "(const XmlNode&)---> to be checked!...\n";
 	}
 	virtual void set(const map<string, string>&)
 	{
-		Log::dev() << "(const map<string, string&)---> to be checked!...\n";
+		MagLog::dev() << "(const map<string, string&)---> to be checked!...\n";
 	}
 	virtual OutputFactory* clone() const
 	{
-		Log::dev() << "(const map<string, string&)---> to be checked!...\n";
+		MagLog::dev() << "(const map<string, string&)---> to be checked!...\n";
 		return new OutputFactory();
 	}
 	virtual void set(DriverManager&, const XmlNode&); 
@@ -95,7 +95,7 @@ public:
 	
 	virtual OutputFactory* clone() const
 	{
-		Log::dev() << "(const map<string, string&)---> to be checked!...\n";
+		MagLog::dev() << "(const map<string, string&)---> to be checked!...\n";
 		return new PS_PsOutputFactory();
 	}
 	virtual void set(DriverManager&, const XmlNode&); 
@@ -284,7 +284,7 @@ public:
 
 
 template <>
-class Translator<string, OutputFactory> { 
+class MagTranslator<string, OutputFactory> { 
 public:
 	OutputFactory* operator()(const string& val )
 	{
