@@ -18,7 +18,7 @@
 
 /*! \file OutputHandler.cc
     \brief Implementation of the Template class OutputHandler.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Wed July 2006
 */
@@ -36,7 +36,7 @@ void OutputHandler::set(DriverManager& magics)
 
 	for ( vector<string>::const_iterator format = formats_.begin(); format != formats_.end(); ++format)
 	{
-		OutputFactory* factory = Translator<string, OutputFactory>()(*format);
+		OutputFactory* factory = MagTranslator<string, OutputFactory>()(*format);
 		XmlNode node;
 		factory->set(magics, node); 
 		factories_.push_back(factory);
@@ -46,7 +46,7 @@ void OutputHandler::set(DriverManager& magics)
 void OutputHandler::set(const XmlNode& node, DriverManager& magics) 
 {
 	try {
-		OutputFactory* factory = Translator<string, OutputFactory>()(node.name());
+		OutputFactory* factory = MagTranslator<string, OutputFactory>()(node.name());
 	
 		factory->set(magics, node); 
 		factories_.push_back(factory);
@@ -73,7 +73,11 @@ static SimpleObjectMaker<PS_EpsOutputFactory, OutputFactory> eps("eps");
 static SimpleObjectMaker<PS_PdfOutputFactory,  OutputFactory> ps_pdf("ps_pdf");
 
 #ifdef MAGICS_RASTER
+#ifdef MAGICS_CAIRO
 static SimpleObjectMaker<GD_PngOutputFactory, OutputFactory> gdpng("gd_png");
+#else
+static SimpleObjectMaker<GD_PngOutputFactory, OutputFactory> gdpng("png");
+#endif
 static SimpleObjectMaker<GD_JpegOutputFactory, OutputFactory> jpeg("jpeg");
 static SimpleObjectMaker<GD_GifOutputFactory, OutputFactory> gif("gif");
 static SimpleObjectMaker<GD_GifAnimOutputFactory, OutputFactory> gif_anim("gif_animation");
@@ -81,7 +85,7 @@ static SimpleObjectMaker<GD_GifAnimOutputFactory, OutputFactory> gif_anim("gif_a
 
 static SimpleObjectMaker<SVG_SvgOutputFactory, OutputFactory> svg("svg");
 
-static SimpleObjectMaker<BinaryOutputFactory, OutputFactory> binary("binary");
+static SimpleObjectMaker<BinaryOutputFactory, OutputFactory> binary("mgb");
 
 #ifdef MAGICS_CAIRO
 static SimpleObjectMaker<CAIRO_PngOutputFactory,   OutputFactory> png("png");
