@@ -49,7 +49,12 @@ void ParameterManager::print(ostream& out)  const
 	}
 	out << "]";
 }
-
+void ParameterManager::resetAll()
+{
+	for (iterator entry = begin(); entry != end(); ++entry) { 
+		entry->second->reset();
+	}
+}
 void  ParameterManager::add(const string& name, BaseParameter* param)  
 {
 	if ( !table_ )  table_ = new ParameterManager();
