@@ -18,7 +18,7 @@
 
 /*! \file ParameterManager.h
     \brief Handles the Magics Parameters
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Jan 2004
 
@@ -31,9 +31,10 @@
 
 #include <magics.h>
 #include <BaseParameter.h>
-#include <Log.h>
+#include <MagLog.h>
 #include <Factory.h>
-#include <Exception.h>
+#include <MagException.h>
+
 
 namespace magics {
 
@@ -44,9 +45,9 @@ public :
     UnknownParameter(const string& name) :
         MagicsException(name + ": unknown parameter, the call is ignored.")
    {
-	Log::info() << "The parameter " << name << " is unknown in Magics++.\n"
-	            << "Please check the documentation to see if it will become avalaible in Magics++\n"
-	            << "or contact the Graphics Section at ECMWF.\n";
+	MagLog::info() << "The parameter " << name << " is unknown in Magics++.\n"
+	            << "Please check the documentation or contact\n"
+	            << "the Meteorological Visualisation Section at ECMWF.\n";
    }
 };
 
@@ -71,14 +72,13 @@ public:
 			}
 			catch (MagicsException& e)
 			{
-			    Log::warning() << "Magics: Catch Exception " << e << "\n";
+			    MagLog::warning() << "MagException > " << e << "\n";
 			}
 		}
 		else
-			Log::warning() << "The parameter " << name << " was not found.\n";
+			MagLog::warning() << "The parameter " << name << " was not found.\n";
 	}
 
-
 	static void set(const string& name, const char* value)
 	{
 		assert(table_);
@@ -89,10 +89,10 @@ public:
 			}
 			catch (MagicsException& e)
 			{
-			    Log::warning() << "Magics: Catch Exception " << e << "\n";
+			    MagLog::warning() << "MagException > " << e << "\n";
 			}
 		else
-			Log::warning() << "The parameter " << name << " was not found.\n";
+			MagLog::warning() << "The parameter " << name << " was not found.\n";
 	}
 	
 	static void setLocal(const BaseParameter* from)
@@ -105,10 +105,10 @@ public:
 			}
 			catch (MagicsException& e)
 			{
-			    Log::warning() << "Magics: Catch Exception " << e << "\n";
+			    MagLog::warning() << "MagException > " << e << "\n";
 			}
 		else
-			Log::warning() << "The parameter " << from->name() << " was not found.\n";
+			MagLog::warning() << "The parameter " << from->name() << " was not found.\n";
 	}
 
 	static void resetLocal(const string& name)
@@ -122,11 +122,11 @@ public:
 			}
 			catch (MagicsException& e)
 			{
-			    Log::warning() << "Magics: Catch Exception " << e << "\n";
+			    MagLog::warning() << "MagException > " << e << "\n";
 			}
 		}
 		else
-			Log::warning() << "The parameter " << name << " was not found.\n";
+			MagLog::warning() << "The parameter " << name << " was not found.\n";
 	}
 
 
@@ -146,7 +146,7 @@ public:
 	{
 	     assert(table_);
 	     BaseParameter* param = (*table_).parameter(name);
-	     return (param) ? param->clone() : 0;             
+	     return (param) ? param->clone() : 0;
 	}
 
 	template <class T>
@@ -177,12 +177,38 @@ public:
 		get(name, value);
 		return value;
 	}
+	static stringarray getStringArray(const string& name)
+		{
+			stringarray value;
+			get(name, value);
+			return value;
+		}
+	static doublearray getDoubleArray(const string& name)
+		{
+			doublearray value;
+			get(name, value);
+			return value;
+		}
+	static intarray getIntArray(const string& name)
+		{
+			intarray value;
+			get(name, value);
+			return value;
+		}
 
-	static bool getBool(const string& name) 
+	static bool getBool(const string& name)
 	{
-		bool value;
-		get(name, value);
-		return value;
+
+		string s;
+		get(name, s);
+		s = lowerCase(s);
+
+		if(s == "no" || s == "off" || s == "false") return false;
+		if(s == "yes"|| s == "on"  || s == "true")  return true;
+
+		// Catter for ints
+		return atoi(s.c_str());
+
 	}
 
 	template <class T>
@@ -191,14 +217,14 @@ public:
 		string val, def;
 
 		if (!table_) {
-     			Log::error() << "Problem in setting the parameter [" << name <<  "] ---> contact graphics team" << endl;
+     			MagLog::error() << "Problem in setting the parameter [" << name <<  "] ---> contact Magics team" << endl;
 		}
 		assert(table_);
 
 		BaseParameter* param = (*table_).parameter(name);
 		if (!param)
 		{
-			Log::warning() << "parameter \"" << name << "\" not found " << endl;
+			MagLog::warning() << "parameter \"" << name << "\" not found " << endl;
 			return;
 		}
 
@@ -211,13 +237,13 @@ public:
 		{
 			param->reset();
 			param->get(def);
-			Log::warning() << "parameter \"" << name << "\" : value [" << val << "] is not valid ---> default [" << def <<  "] used" << endl;
+			MagLog::warning() << "parameter \"" << name << "\" : value [" << val << "] is not valid ---> default [" << def <<  "] used" << endl;
 
 			try {
 				object = SimpleObjectMaker<T>::create(def);
 			}
 			catch (NoFactoryException& e2) {
-				Log::error() << "default [" << def <<  "] not found ---> contact graphics team" << endl;
+				MagLog::error() << "default [" << def <<  "] not found ---> contact Magics team" << endl;
 				throw e2;
 			}
 		}
@@ -229,18 +255,25 @@ public:
 		{
 			param->reset();
 			param->get(def);
-			Log::warning() << "parameter \"" << name << "\" : value [" << val << "] is not valid ---> default [" << def <<  "] used" << endl;
+			MagLog::warning() << "parameter \"" << name << "\" : value [" << val << "] is not valid ---> default [" << def <<  "] used" << endl;
 			object = SimpleObjectMaker<T>::create(def);	
 			if (!object)
 			{
-				Log::error() << "default [" << def <<  "] not found ---> contact graphics team" << endl;
-				Log::error() << "Serious Error --> abort" << endl;
+				MagLog::error() << "default [" << def <<  "] not found ---> contact Magics team\n"
+				             << "Serious Error --> abort" << endl;
 				throw NoFactoryException("name");
 			}
 		}
 	#endif
 	}
 
+	void resetAll();
+
+	static void reset() {
+		if (table_)
+			table_->resetAll();
+	}
+
 protected:
 	virtual void print(ostream&) const;
 	static ParameterManager* table_;
@@ -256,9 +289,9 @@ protected:
 		{
 			return (*param).second;	
 		}
-		Log::info() << "The parameter " << name << " is unknown in Magics++.\n"
-		            << "Please check the documentation to see if it will become avalaible in Magics++\n"
-		            << "or contact the Graphics Section at ECMWF.\n";
+		MagLog::info() << "The parameter " << name << " is unknown in Magics++.\n"
+		            << "Please check the documentation or contact\n"
+		            << "the Meteorological Visualisation Section at ECMWF.\n";
 		return 0;
 	}
 
