@@ -9,7 +9,7 @@
  
  	http://www.apache.org/licenses/LICENSE-2.0
  
- Unless required by applicable law or agreed to in writing, software 
+ Unless required by applicable law or agreedto in writing, software
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
@@ -20,7 +20,7 @@
 
 /*! \file PolarStereographicProjection.cc
     \brief Implementation of PolarStereographicProjection.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Fri Jan 11 15:01:45 2008
 
@@ -33,6 +33,11 @@
 #include <Text.h>
 #include <MatrixHandler.h>
 #include <MetaData.h>
+#include <ParameterSettings.h>
+#include <math.h>
+#include <MagJSon.h>
+
+
 using namespace magics;
 
 /*!
@@ -60,53 +65,81 @@ void PolarStereographicProjection::print(ostream& out) const
 double PolarStereographicProjection::unitToCm(double width, double height) const
 {
 	// Reproject 2 point and find the unit
-	int hemis = (hemisphere_ ==  NORTH) ? 1 : -1;
+	const int hemis = (hemisphere_ ==  NORTH) ? 1 : -1;
 
-	GeoPoint ll1(0,  hemis*50);
-	GeoPoint ll2(0,  hemis*51);
+	UserPoint ll1(0,  hemis*50);
+	UserPoint ll2(0,  hemis*51);
 	PaperPoint xy1 = (*this)(ll1);
 	PaperPoint xy2 = (*this)(ll2);
 	
-	double unit = abs(xy1.y() - xy2.y());
-	
-	return unit*(height/(getMaxY() -getMinY()) ); 
+	const double unit = abs(xy1.y() - xy2.y());
+	const double wid  = getMaxPCY() - getMinPCY();
+	if (!zero(wid))	return unit*(height/wid);
+	return unit;
 }
 
+double PolarStereographicProjection::height() const { 
+	// Reproject 2 point and find the unit
+		int hemis = (hemisphere_ ==  NORTH) ? 1 : -1;
+		bool pole;
+		double height;
+		UserPoint ll(0,  hemis*90);
+		PaperPoint xy = (*this)(ll);
+
+		pole =  ( xy.x() > xpcmin_ && xy.x() < xpcmax_  && xy.y() > ypcmin_ && xy.y() < ypcmax_ );
+		
+		if ( !pole )  { 
+			height = (ymax_-ymin_); 
+		}
+		else {
+			if (hemisphere_ ==  NORTH) {
+				height = 180-ymax_-ymin_;
+			}
+			else {
+			 height = 180+ymax_+ymin_;
+			}
+		}
+	return height;
+}
 
 PaperPoint PolarStereographicProjection::operator()(const UserPoint& point)  const
-{
-	Log::warning() << "PolarStereographicProjection::operator()(const UserPoint& point) should not be called!" << endl;
-	return Transformation::operator()(point);
-}
-
-PaperPoint PolarStereographicProjection::operator()(const GeoPoint& point)  const
 {
 	assert(projection_);
-	TeCoord2D geo = TeCoord2D(point.longitude()*TeCDR, point.latitude()*TeCDR);
+
+	TeCoord2D geo = TeCoord2D(point.x()*TeCDR, point.y()*TeCDR);
 	TeCoord2D xy = projection_->LL2PC(geo);
 
 	return PaperPoint(xy.x(), xy.y(), point.value(), point.missing(), point.border());
 }
 
+
+
 PaperPoint PolarStereographicProjection::operator()(const PaperPoint& point)  const
 {
 	return Transformation::operator()(point);
 }
 
-void PolarStereographicProjection::revert(const PaperPoint& xy, GeoPoint& point)  const
+void PolarStereographicProjection::revert(const vector<pair<double, double> > & in, vector<pair<double, double> > & out) const
+{
+	assert(projection_);
+	out.reserve(in.size());
+	for ( vector<pair<double, double> >::const_iterator pt = in.begin();  pt != in.end(); ++pt) {
+		TeCoord2D texy = TeCoord2D(pt->first, pt->second);
+		TeCoord2D geo = projection_->PC2LL(texy);
+		out.push_back(make_pair(geo.x()*TeCRD, geo.y()*TeCRD));
+	}
+}
+
+void PolarStereographicProjection::revert(const PaperPoint& xy, UserPoint& point)  const
 {
 	assert(projection_);
 	TeCoord2D texy = TeCoord2D(xy.x(), xy.y());
 	TeCoord2D geo = projection_->PC2LL(texy);
   
-	point = GeoPoint(geo.x()*TeCRD, geo.y()*TeCRD);
+	point = UserPoint(geo.x()*TeCRD, geo.y()*TeCRD);
 }
 
-void PolarStereographicProjection::revert(const PaperPoint& xy, UserPoint& point)  const
-{
-	Log::dev() << "PolarStereographicProjection::revert(...) needs implementing." << endl;
-	Transformation::revert(xy, point);
-}
+
 
 bool PolarStereographicProjection::needShiftedCoastlines()  const
 {
@@ -118,7 +151,14 @@ void PolarStereographicProjection::init(double width, double height)
 	if ( !projection_ ) 
 		projection_ = new TePolarStereographic(TeDatum(), vertical_longitude_*TeCDR, 0., 0., "Meters", (hemisphere_ == NORTH) ? TeNORTH_HEM : TeSOUTH_HEM);
 	
-	magCompare(area_, "corners" ) ?  corners() : centre(width, height);
+	if ( magCompare(area_, "full") ) {
+		ymin_ = ( hemisphere_ == NORTH ) ? -20. : 20.;
+		ymax_ = ( hemisphere_ == NORTH ) ? -20. : 20.;
+		xmin_ = ( hemisphere_ == NORTH ) ? -45. : 45.;
+		xmax_ = ( hemisphere_ == NORTH ) ? 135. : -135.;
+	}
+	else
+		magCompare(area_, "corners" ) ?  corners() : centre(width, height);
 	double llx, urx , lly , ury;
 		
 	if ( magCompare(system_, "projection") == false )
@@ -139,21 +179,24 @@ void PolarStereographicProjection::init(double width, double height)
 		llx = ::min(urxy.x(), llxy.x());
 		urx = ::max(urxy.x(), llxy.x());
 
-		if (llx == urx)
-		urx = llx+1000;
+		if ( (urx - llx) < 10) {
+				urx = llx+10.;
+
+		}
 
 		lly = ::min(urxy.y(), llxy.y());
 		ury = ::max(urxy.y(), llxy.y());
+		if ( ( ury - lly ) < 10) {
+			ury = lly+10.;
 
-		if (lly == ury)
-		ury = lly+1000;
-		xmin_ = int_MAX;
-		ymin_ = int_MAX;
-		xmax_ = int_MIN;
-		ymax_ = int_MIN;
+		}
+		xmin_ = DBL_MAX;
+		ymin_ = DBL_MAX;
+		xmax_ = DBL_MIN;
+		ymax_ = DBL_MIN;
 
-		double stepx= (urx - llx)/100;
-		double stepy= (ury - lly)/100;
+		double stepx= (urx - llx)/100.;
+		double stepy= (ury - lly)/100.;
 		for (double x = llx; x <= urx; x += stepx )
 		{
 		   for (double y = lly; y <= ury; y += stepy )
@@ -196,8 +239,8 @@ void PolarStereographicProjection::init(double width, double height)
 		xmax_ = ::max(ll.x()*TeCRD, ur.x()*TeCRD); 
 		ymin_ =  ::min(ll.y()*TeCRD, ur.y()*TeCRD);
 		ymax_ =  ::max(ll.y()*TeCRD, ur.y()*TeCRD);
-		double stepx= (urx - llx)/100;
-		double stepy= (ury - lly)/100;
+		double stepx= (urx - llx)/100.;
+		double stepy= (ury - lly)/100.;
 		for (double x = llx; x <= urx; x += stepx )
 		{
 		   for (double y = lly; y <= ury; y += stepy )
@@ -225,16 +268,26 @@ void PolarStereographicProjection::init(double width, double height)
 		ypcmin_ = lly;
 		xpcmax_ = urx;
 		ypcmax_ = ury; 
-		Log::dev() << " Projection definition-->[" << ymin_ << ", " << xmin_ << ", " << xmax_ << ", " << ymax_ << "]" << endl;
+
+		PCEnveloppe_->clear();
+
+		PCEnveloppe_->push_back(PaperPoint(xpcmin_, ypcmin_));
+		PCEnveloppe_->push_back(PaperPoint(xpcmin_, ypcmax_));
+		PCEnveloppe_->push_back(PaperPoint(xpcmax_, ypcmax_));
+		PCEnveloppe_->push_back(PaperPoint(xpcmax_, ypcmin_));
+		PCEnveloppe_->push_back(PaperPoint(xpcmin_, ypcmin_));
+
+		boost::geometry::correct(PCEnveloppe_->polygon_);
+
+
+		MagLog::dev() << " Projection definition-->[" << ymin_ << ", " << xmin_ << ", " << xmax_ << ", " << ymax_ << "]" << endl;
 	}
 }
+
 void PolarStereographicProjection::fill(double& width, double& height)  
 {  
 	init(width, height);
-    Transformation::fill(width, height);
-    
-    
-
+	Transformation::fill(width, height);
 }
 
 void PolarStereographicProjection::aspectRatio(double& width, double& height)  
@@ -243,12 +296,53 @@ void PolarStereographicProjection::aspectRatio(double& width, double& height)
 	Transformation::aspectRatio(width, height);
 }
 
+void PolarStereographicProjection::smallestBoundingBox(double& xmin, double& ymin, double& xmax, double& ymax)  const
+{
+		vector<pair<double, double> > geo;
+		vector<pair<double, double> > xy;
+
+		double xpcmax =  xpcmax_;
+		double xpcmin =  xpcmin_;
+		double ypcmax =  ypcmax_;
+		double ypcmin =  ypcmin_;
+
+		const double xs = (xpcmax- xpcmin)/99.;
+		const double ys = (ypcmax- ypcmin)/99.;
+		// Walk along the boundary...
+		double x,y;
+		for (int i = 0; i < 100; i++) {
+			x = xpcmin +(i*xs);
+			for (int i = 0; i < 100; i++) {
+				y = ypcmin +(i*ys);
+				xy.push_back(make_pair(x, y));
+			}
+		}
+		revert(xy, geo);
+		xmin=DBL_MAX;
+		xmax=DBL_MIN;
+		ymin=DBL_MAX;
+		ymax=DBL_MIN;
+
+		for (vector<pair<double, double> >::iterator point = geo.begin(); point != geo.end(); ++point) {
+			if ( xmin > point->first) xmin = point->first;
+			if ( xmax < point->first) xmax = point->first;
+			if ( ymin > point->second) ymin = point->second;
+			if ( ymax < point->second) ymax = point->second;
+			//userEnveloppe_->push_back(PaperPoint(point->first, point->second));
+		}
+
+
+}
+
 void PolarStereographicProjection::boundingBox(double& xmin, double& ymin, double& xmax, double& ymax)  const
 {
-	xmin=-180;
-	ymin=-90;
-	xmax=180;
-	ymax=90;
+	smallestBoundingBox(xmin, ymin, xmax, ymax);
+	xmin -= 5;
+	xmax += 5;
+	ymin -= 5;
+	ymax += 5;
+	
+	MagLog::dev() << " Projection bounding box-->[" << xmin << ", " << xmax << ", " << ymin << ", " << ymax << "]" << endl;
 }
 
 double PolarStereographicProjection::getMinX()  const
@@ -273,25 +367,25 @@ double PolarStereographicProjection::getMaxY()  const
 
 void PolarStereographicProjection::setMinX(double x)  
 {
-	Log::dev() << "PolarStereographicProjection::setMinX(...) needs implementing." << endl;
+	MagLog::dev() << "PolarStereographicProjection::setMinX(...) needs implementing." << endl;
 	Transformation::setMinX(x);
 }
 
 void PolarStereographicProjection::setMinY(double y)  
 {
-	Log::dev() << "PolarStereographicProjection::setMinY(...) needs implementing." << endl;
+	MagLog::dev() << "PolarStereographicProjection::setMinY(...) needs implementing." << endl;
 	Transformation::setMinY(y);
 }
 
 void PolarStereographicProjection::setMaxX(double x)  
 {
-	Log::dev() << "PolarStereographicProjection::setMaxX(...) needs implementing." << endl;
+	MagLog::dev() << "PolarStereographicProjection::setMaxX(...) needs implementing." << endl;
 	Transformation::setMaxX(x);
 }
 
 void PolarStereographicProjection::setMaxY(double y)  
 {
-	Log::dev() << "PolarStereographicProjection::setMaxY(...) needs implementing." << endl;
+	MagLog::dev() << "PolarStereographicProjection::setMaxY(...) needs implementing." << endl;
 	Transformation::setMaxY(y);
 }
 
@@ -321,13 +415,16 @@ void PolarStereographicProjection::gridLongitudes(const GridPlotting& grid)  con
 	const vector<double>& latitudes = grid.latitudes();
 	for (vector<double>::const_iterator lon = longitudes.begin(); lon != longitudes.end(); ++lon)
 	{
+		if (*lon < xmin_ ) continue;
+		if (*lon > xmax_ ) continue;
 		Polyline poly;
 
 		double min = ::min(latitudes.front(), latitudes.back());
-		double max = 80;
+		double max = ::max(latitudes.front(), latitudes.back());
 		
-		for (double lat = min; lat <= max; lat += 1) { 
-			poly.push_back((*this)(GeoPoint(*lon,lat)));
+
+		for (double lat = min; lat <= max; lat += 1) {
+			poly.push_back((*this)(UserPoint(*lon,lat)));
 		}  
 		grid.add(poly);     
 	}
@@ -338,9 +435,12 @@ void PolarStereographicProjection::gridLatitudes(const GridPlotting& grid)  cons
 	const vector<double>& latitudes = grid.latitudes();
 	for (vector<double>::const_iterator lat = latitudes.begin(); lat != latitudes.end(); ++lat)
 	{
+		if (*lat < ymin_ ) continue;
+		if (*lat > ymax_ ) continue;
 	    Polyline poly;        
-            for (int lon = -180; lon <= 180; lon += 1) { 
-                poly.push_back((*this)(GeoPoint(lon,*lat)));
+
+            for (int lon = -180; lon <= 180; lon += 1) {
+                poly.push_back((*this)(UserPoint(lon,*lat)));
                 
             }	
 	    grid.add(poly);        
@@ -349,7 +449,7 @@ void PolarStereographicProjection::gridLatitudes(const GridPlotting& grid)  cons
 
 inline double CA(TeCoord2D& p1, TeCoord2D& p2)
 {
-    return (p2.y() - p1.y())/(p2.x() - p1.x());
+    return (p2.x() - p1.x()) ? (p2.y() - p1.y())/(p2.x() - p1.x()) : 0;
 }
 
 inline double CB(double a, TeCoord2D& p)
@@ -359,7 +459,7 @@ inline double CB(double a, TeCoord2D& p)
 
 inline double CX(double a, double b, double y)
 {
-    return (y - b)/a;
+    return (a) ? (y - b)/a : 0;
 }
 
 inline double CY(double a, double b, double x)
@@ -369,35 +469,51 @@ inline double CY(double a, double b, double x)
 
 void PolarStereographicProjection::horizontalLabels(const LabelPlotting& label, double y, double yy, bool top) const
 {
+	double xmin, ymin, xmax, ymax;
+	boundingBox(xmin, ymin, xmax,ymax);
+
+	vector<double> done;
     const vector<double>& longitudes = label.longitudes();
     for (vector<double>::const_iterator lon = longitudes.begin(); lon != longitudes.end(); ++lon)
     {
-        // find the equation of the line using 2 points : lon/-20 -->lon/ +20
+     	// find the equation of the line using 2 points : lon/-20 -->lon/ +20
         TeCoord2D p1 = TeCoord2D((*lon)*TeCDR, 20*TeCDR);
         TeCoord2D p2 = TeCoord2D((*lon)*TeCDR, -20*TeCDR);
 
         TeCoord2D p1xy = projection_->LL2PC(p1);
         TeCoord2D p2xy = projection_->LL2PC(p2);
-    
+
         double a = CA(p1xy, p2xy);
         double b = CB(a, p1xy);
             
         TeCoord2D xy = TeCoord2D(CX(a, b, y), y);   
 
         PaperPoint point(CX(a, b, y), y);
+
         if ( !in(point) ) continue;
-        	GeoPoint geo;
-        	revert(point, geo);
-        	point.y(yy);           		
+        bool add = true;
+        for ( vector<double>::iterator x = done.begin(); x != done.end(); ++x) {
+        	if ( same(*x, point.x_ ) ) {
+        		add = false;
+        		break;
+        	}
+        }
+
+        if ( !add ) continue;
+        done.push_back(point.x_);
+    	UserPoint geo;
+       	revert(point, geo);
+       	point.y(yy);
+
         Text* text = new Text();
         text->setJustification(MCENTRE);
         text->setVerticalAlign(MBOTTOM);
-	if(top) text->setVerticalAlign(MBOTTOM);
-	else    text->setVerticalAlign(MTOP);     
-        text->setText(geo.writeLongitude());
+        text->setVerticalAlign(top ? MBOTTOM : MTOP);
+        label.add(text);
+        text->setText(writeLongitude(geo));
         text->push_back(point);
 			
-        label.add(text);         
+
      }
 }
 
@@ -410,6 +526,8 @@ void PolarStereographicProjection::horizontalLabels(const LabelPlotting& label,
 void PolarStereographicProjection::verticalLabels(const LabelPlotting& label, double x, double xx, bool left)  const
 {
     const vector<double>& longitudes = label.longitudes();
+
+    vector<double> done;
     for (vector<double>::const_iterator lon = longitudes.begin(); lon != longitudes.end(); ++lon)
     {
         // find the equation of the line using 2 points : lon/-20 -->lon/ +20
@@ -421,23 +539,36 @@ void PolarStereographicProjection::verticalLabels(const LabelPlotting& label, do
     
         double a = CA(p1xy, p2xy);
         double b = CB(a, p1xy);
+        if ( !a ) continue; // parallele line!
         
         TeCoord2D xy = TeCoord2D(x, CY(a, b, x) );       
 
         PaperPoint point(x, CY(a, b, x));
-        	if ( !in(point) ) continue;
-        GeoPoint geo;
-        revert(point, geo);
-        point.x(xx);
-        	
+        if ( !in(point) ) continue;
+        bool add = true;
+        for ( vector<double>::iterator x = done.begin(); x != done.end(); ++x) {
+            if ( same(*x, point.y_ ) ) {
+                add = false;
+                break;
+             }
+         }
+
+         if ( !add ) continue;
+         done.push_back(point.y_);
+
+         UserPoint geo;
+         revert(point, geo);
+         point.x(xx);
+
+
         Text* text = new Text();
-	if(left) text->setJustification(MRIGHT);
-	else     text->setJustification(MLEFT);
+        label.add(text);
+        text->setJustification(left ? MRIGHT: MLEFT);
         text->setVerticalAlign(MHALF);    
-        text->setText(geo.writeLongitude());
+        text->setText(writeLongitude(geo));
         text->push_back(point);
 	
-        label.add(text);
+
      }
 }
 
@@ -446,30 +577,31 @@ void PolarStereographicProjection::labels(const LabelPlotting& label, DrawingVis
 {
 	Text *text;
 	const vector<double>& latitudes = label.latitudes();
-	const vector<double>& longitudes = label.longitudes();
-	
+	//const vector<double>& longitudes = label.longitudes();
+	vector<double> longitudes;
+	longitudes.push_back(0);
 	unsigned int flat = (unsigned int) std::max(1, (int) maground(latitudes.size()/4));
 	unsigned int flon = (unsigned int) std::max(1, (int) maground(longitudes.size()/4));
-	
-	
-	
+
 	for (unsigned int lat = 0; lat < latitudes.size(); lat += flat)
 	{  
 	    for (unsigned int lon = 0 ; lon < longitudes.size(); lon += flon)
 	    { 	   
-	   	   GeoPoint point(longitudes[lon],latitudes[lat]);
+	   	   UserPoint point(longitudes[lon],latitudes[lat]);
 	   	   PaperPoint xy = (*this)(point);
 	   	   
 	   	   if ( !in(xy) ) continue;	   
 	   
 	   	   text = new Text();
-	   	   text->setText(point.writeLatitude());
-	           text->push_back(xy);
-	           text->setBlanking(false);
-	   	   label.add(text);
+	   	   label.add(text); // This will set the font!
+	   	   text->setText(writeLatitude(point));
+	       text->push_back(xy);
+	       text->setBlanking(true);
+
 	    }
 	}
 }
+
 void PolarStereographicProjection::labels(const LabelPlotting& label, TopAxisVisitor&)  const
 {
 	// Find intersection of the latitude line  with the top!
@@ -509,7 +641,10 @@ void PolarStereographicProjection::labels(const LabelPlotting& label, RightAxisV
 /*!
  This method generates meta output for web interaction through JavaScript...
 */
-void  PolarStereographicProjection::visit(MetaDataVisitor& visitor, double left, double top, double width, double height) 
+void  PolarStereographicProjection::visit(MetaDataVisitor& visitor, 
+	double left, double top, 
+	double width, double height,
+	double img_width, double img_height) 
 {
 	ostringstream java;
 
@@ -522,7 +657,9 @@ void  PolarStereographicProjection::visit(MetaDataVisitor& visitor, double left,
 	java << "\"top\" : " << top <<  ",";		
 	java << "\"left\" : " << left <<  ",";		
 	java << "\"width\" : " << width <<  ",";	
+	java << "\"img_width\" : " << img_width <<  ",";	
 	java << "\"height\" : " << height <<  ",";	
+	java << "\"img_height\" : " << img_height <<  ",";	
 	
 	java << "\"pcxmin\" : " << getMinPCX() <<  ",";		
 	java << "\"pcymin\" : " << getMinPCY() <<  ",";		
@@ -534,6 +671,15 @@ void  PolarStereographicProjection::visit(MetaDataVisitor& visitor, double left,
 
 void PolarStereographicProjection::corners()
 {
+	// For backwards compatibility!
+	if  ( min_longitude_ == -180 && max_longitude_ == 180 &&
+		   min_latitude_ == -90 && max_latitude_ == 90 ) {
+		min_latitude_ = ( hemisphere_ == NORTH ) ? -20. : 20.;
+		max_latitude_ = ( hemisphere_ == NORTH ) ? -20. : 20.;
+		min_longitude_ = ( hemisphere_ == NORTH ) ? -45. : 45.;
+		max_longitude_ = ( hemisphere_ == NORTH ) ? 135. : -135.;
+	}
+
 	xmin_ = min_longitude_;
 	xmax_ = max_longitude_;
 	ymin_ = min_latitude_;
@@ -542,28 +688,14 @@ void PolarStereographicProjection::corners()
 	if ( magCompare(system_, "projection") ) 
 		return; 
 	
-	double xmin = std::min(min_longitude_, max_longitude_);
-	double xmax = std::max(min_longitude_, max_longitude_);
 	
-	double ymin = std::min(min_latitude_, max_latitude_);
-	double ymax = std::max(min_latitude_, max_latitude_);
-   
-	// change the default ...
-	 if ( ymin == -90 ) 
-        	 ymin_ = ( hemisphere_ == NORTH ) ? -20. : 20.;
-	if ( ymax == 90 ) 
-        	 ymax_ = ( hemisphere_ == NORTH ) ? -20. : 20.;
-	if ( xmin == -180 ) 
-        	 xmin_ = ( hemisphere_ == NORTH ) ? -45. : 45.;
-	if ( xmax == 180 ) 
-        	 xmax_ = ( hemisphere_ == NORTH ) ? 135. : -135.;   
 }
 
 
 
 void PolarStereographicProjection::centre(double width, double height)
 {
-    PaperPoint centre = (*this)(GeoPoint(centre_longitude_, centre_latitude_));
+    PaperPoint centre = (*this)(UserPoint(centre_longitude_, centre_latitude_));
       
     double x = (width* map_scale_)/200;
     double y = (height * map_scale_)/200;
@@ -572,9 +704,9 @@ void PolarStereographicProjection::centre(double width, double height)
     PaperPoint llxy(centre.x() - x, centre.y() - y);
     PaperPoint urxy(centre.x() + x, centre.y() + y);
     
-    GeoPoint ll;
+    UserPoint ll;
     revert(PaperPoint(centre.x() - x, centre.y() - y), ll);
-    GeoPoint ur;
+    UserPoint ur;
     revert(PaperPoint(centre.x() + x, centre.y() + y), ur);    
 
     xmin_ = ll.x();
@@ -583,7 +715,6 @@ void PolarStereographicProjection::centre(double width, double height)
     ymax_ = ur.y();
 }
 
-#define ff(p1, p2) sqrt( ((p1.x()-p2.x())*(p1.x()-p2.x())) + ((p1.y()-p2.y())*(p1.y()-p2.y())) )
 
 
 
@@ -596,159 +727,277 @@ void PolarStereographicProjection::centre(double width, double height)
  whose distance from the current grid point is less than the thinning factor, will be ignored. 
  The default value is 2.0, e.g. the statement
 */
-
-void PolarStereographicProjection::thin(MatrixHandler<GeoPoint>& matrix, double x, double y, vector<GeoPoint>& out) const
+void PolarStereographicProjection::thin(MatrixHandler& matrix, double x, double y, vector<UserPoint>& out) const
 {
-	int xfactor = (int) ceil((float) x);
-	int yfactor = (int) ceil((float) y);
-    
-	ThinningMatrixHandler<GeoPoint> original(matrix, 1, 1);
-	double lat = hemisphere_ == NORTH ? 90 : -60;
-   
-	GeoPoint ll1(0, lat);
-	GeoPoint ll2(original.XResolution(),  lat);
-	PaperPoint xy1 = (*this)(ll1);
-	PaperPoint xy2 = (*this)(ll2);
-	
-	double factor = ff(xy1, xy2)*xfactor;
-	
-	
-	int columns = original.columns();
-	int rows = original.rows();
 
-	PaperPoint last, first;
-	double savelat = -1;
-	 
+	int yfactor = (int) ceil((float) x);
+    
+
+
+	int columns = matrix.columns();
+	int rows = matrix.rows();
+ 
 	for ( int row = 0; row < rows; row+=yfactor)
 	{
-		double savelon = -1;
-
-		for ( int column = 0; column < columns; column++)
+		const double lat = matrix.row(row, 0);
+		int incrementx;
+		if (abs(lat) == 90) {
+			incrementx = columns;
+		}
+		else {
+			double coeff = (1-((lat/90.)*(lat/90.)));
+			assert ( coeff != 0);
+			incrementx = yfactor / coeff;
+			if (incrementx < 0 )
+				incrementx = 1;
+		}
+		for ( int column = 0; column < columns; column+=incrementx)
 		{
-			double lat = original.row(row, column);	
-			double lon = original.column(row, column);
-			if (hemisphere_ == NORTH && lat < -20) continue;
-			if (hemisphere_ == SOUTH && lat > 20) continue;
-			if (savelat != -1)
-			{
-				PaperPoint previous = (*this)(GeoPoint(savelon, savelat));
-				PaperPoint current = (*this)(GeoPoint(lon, lat));
-				if ( ff(previous, current) < factor ) continue;
-				savelat = lat;
-			}			
-			GeoPoint point(lon, lat, original(row, column));
+			const double lon = matrix.column(row, column);
 
-			PaperPoint xy = (*this)(point); 
-			if ( column == 0 ) first = xy;
-
-			if ( savelon == -1)
-			{
-				savelon = lon;
-				savelat = lat;
-				last = xy;
-				if ( in(xy) )
-					out.push_back(point);
-				continue;
-			}
-			if ( ff(last, xy) > factor && ff(first, xy) > factor)
-			{						
-				last = xy;
-				if ( in(xy) )
-					out.push_back(point); 
-			}	
+			UserPoint point(lon, lat, matrix(row, column));
+			out.push_back(point); 
 		}
 	}
 }
 
-bool PolarStereographicProjection::concatenate(vector<Polyline* >& lines, Polyline* poly) const 
-{
-	if (poly->empty()) return true;
-	for ( vector<Polyline* >::iterator  line = lines.begin(); line != lines.end(); ++line)
-	{
-		Polyline* building = *line;
-		// concatenate front???
-		if ( building->empty() ) continue;
-		
-		if ( (*line)->front() == poly->back() )
-		{
-//			Log::dev()<< "concatenate to the front" << (*line)->front() << " == " <<  poly->back() << endl;
-			(*line)->insert((*line)->begin(), poly->begin(), poly->end());
-			lines.erase(line);
-			concatenate(lines, building);
-			return true;			
-		}
-		// concatenate back???
-		if ( (*line)->back() == poly->front()  )
-		{
-//			Log::dev()<< "concatenate to the back" << (*line)->back() << " == " <<  poly->front() << endl;
-			(*line)->insert((*line)->end(), poly->begin(), poly->end());
-			lines.erase(line);
-			concatenate(lines, building);			
-			return true;
-		}
-	}
-	
-	lines.push_back(poly);
-	return true;
-}
-
 
 void PolarStereographicProjection::setNewPCBox(double minx, double miny, double maxx, double maxy)
 {
 	PaperPoint p1(minx, miny);
 	PaperPoint p2(maxx, maxy);
-	GeoPoint   ll, ur;
+	UserPoint   ll, ur;
 
 	revert(p1, ll);
 	revert(p2, ur);
 
-	cout << ll << endl;
-	cout << ur << endl;
-
 	min_longitude_ = ll.x();
 	max_longitude_ = ur.x();
 	min_latitude_ = ll.y();
 	max_latitude_ = ur.y(); 
 
+
+
 	corners();
+
+	xpcmin_ = minx;
+	ypcmin_ = miny;
+	xpcmax_ = maxx;
+	ypcmax_ = maxy;
+
+	PCEnveloppe_->clear();
+
+	PCEnveloppe_->push_back(PaperPoint(xpcmin_, ypcmin_));
+	PCEnveloppe_->push_back(PaperPoint(xpcmin_, ypcmax_));
+	PCEnveloppe_->push_back(PaperPoint(xpcmax_, ypcmax_));
+	PCEnveloppe_->push_back(PaperPoint(xpcmax_, ypcmin_));
+	PCEnveloppe_->push_back(PaperPoint(xpcmin_, ypcmin_));
 }
 
-void PolarStereographicProjection::operator()(const GeoPoint& geo, vector<PaperPoint>& out) const
+void PolarStereographicProjection::operator()(const UserPoint& geo, vector<PaperPoint>& out) const
 {
 	PaperPoint pp = (*this)(geo);
 		if ( in(pp) ) 
 			out.push_back(pp);
 }
 
-void PolarStereographicProjection::reprojectComponents(const GeoPoint& point, pair<double, double>& components) const
+void PolarStereographicProjection::reprojectComponents(const UserPoint& point, pair<double, double>& components) const
 {
-	double s1 = sqrt((components.first * components.first) + (components.second * components.second));
-	GeoPoint u(point.x() + components.first, point.y());
-	GeoPoint v(point.x(), point.y() + components.second);
 
-	PaperPoint up = (*this)(u);
-	PaperPoint vp = (*this)(v);
-	PaperPoint p = (*this)(point);
+	double speed = sqrt((components.first * components.first) + (components.second * components.second));
+	double angle = atan2(components.second,components.first);
+
+
+
+
+    UserPoint pt(point.x()+0.5, point.y());
+
+    PaperPoint pp = (*this) (pt);
+	PaperPoint from = (*this)(point);
+
+
+	components.first = pp.x() - from.x();
+	components.second = pp.y() - from.y();
+
+	double rotation = atan2((pp.y() - from.y()), (pp.x() - from.x()));
+
+
+
+
+	//components.second *= f;
+	// we the angle and the spped we compute u/v...
+	components.first = speed * cos(angle+rotation);
+	components.second = speed * sin(angle+rotation);
+
+
+
+
+	speed = sqrt((components.first * components.first) + (components.second * components.second));
+	angle = DEG(atan2(components.second,components.first));
 
-	components.first = (up.x() - p.x()) + (vp.x() - p.x());
-	components.second = (up.y() - p.y()) + (vp.y() - p.y());
 
-	double s2 = sqrt((components.first * components.first) + (components.second * components.second));
-	double f = s1/s2;
-	components.first *= f;
-	components.second *= f;	
 }
 
 void PolarStereographicProjection::reprojectSpeedDirection(const PaperPoint& point, pair<double, double>& wind) const
 {
-  	double a = 90 - (wind.second); 
-  	double speed =wind.first; 
-  	double x = 3.14/180.;
+	double a = 90 - (wind.second); 
+	const double x = 3.14/180.;
 	a *= x;
-    GeoPoint geo;
-    wind.first = speed-1 * cos(a);
-    wind.second = speed-1 * sin(a);
-    reprojectComponents(geo, wind);
+	UserPoint geo;
+	const double speed =wind.first; 
+	wind.first = speed-1 * cos(a);
+	wind.second = speed-1 * sin(a);
+	reprojectComponents(geo, wind);
+}
+
+void PolarStereographicProjection::coastSetting(map<string, string>& setting, double abswidth, double absheight) const
+{
+	// work out the ratios of geographical to paper lengths
+	const double xratio = ( xpcmax_ - xpcmin_ ) / abswidth;
+	const double yratio = ( ypcmax_ - ypcmin_ ) / absheight;
+
+	// choose the smallest (smaller ratio means more detail required)
+	const double ratio = min (xratio, yratio);
+
+	std::string resol = "110m";
+	if ( ratio < 100000 )  // highest resolution
+	{
+		resol = "10m";
+	}
+	else if ( ratio < 300000)   // medium resolution
+	{
+		resol = "50m";
+	}
+	setting["resolution"]      = resol;
+	setting["lakes"]      = resol + "/" + resol + "_lakes";
+	setting["land"]       = resol + "/" + resol + "_land";
+	setting["rivers"]     = resol + "/" + resol + "_rivers_lake_centerlines";
+	setting["boundaries"] = resol + "/" + resol + "_admin_0_boundary_lines_land";
+	
+	//! \note Administraive borders hardcoded to 10m resolution (low res version do not contain all info)
+	setting["administrative_boundaries"] = "10m/10m_admin_1_states_provinces_shp";
+
+	MagLog::dev() << "GeoRectangularProjection::coastSetting[" << abswidth << ", " << absheight << "]->" <<  ratio << " resol: "<<resol<< endl;
+}
+
+MatrixHandler* PolarStereographicProjection::prepareData(const AbstractMatrix& matrix) const
+{ return new GeoBoxMatrixHandler(matrix, *this); }
+
+void PolarStereographicProjection::wraparound(const UserPoint& point, stack<UserPoint>& duplicates) const
+{
+	if ( in(point) ) {
+		duplicates.push(point);
+	}
+
+}
+
+Polyline& PolarStereographicProjection::getPCBoundingBox() const
+{
+
+
+	if ( PCEnveloppe_->empty() ) {
+		getUserBoundingBox();
+	}
+
+	return *PCEnveloppe_;
+}
+
+Polyline& PolarStereographicProjection::getUserBoundingBox() const
+{
+
+	double minlat = -90;
+	double maxlat = 90; 
+	double minlon = -180;
+	double maxlon = 180; 
+
+	if ( userEnveloppe_->empty() ) {
+		// left
+		for ( int lat = minlat; lat <= maxlat; lat++) {
+			userEnveloppe_->push_back(PaperPoint(minlon,lat));
+
+		}
+		// top
+		for ( int lon = minlon; lon <= maxlon; lon++) {
+			userEnveloppe_->push_back(PaperPoint(lon, maxlat));
+
+		}
+		// right
+		for ( int lat = maxlat; lat >= minlat; lat--) {
+
+			userEnveloppe_->push_back(PaperPoint(maxlon, lat));
+
+		}
+		// bottom
+		for ( int lon = maxlon; lon >= minlon; lon--) {
+			userEnveloppe_->push_back(PaperPoint(lon, minlat));
+
+		}
+
+	}
+
+	return *userEnveloppe_;
+
+
+
+}
+
+#include <boost/geometry/algorithms/distance.hpp>
+double PolarStereographicProjection::patchDistance(double res) const
+{
+	/*
+	UserPoint geo1(0,60);
+	UserPoint geo2(0,61);
+
+	PaperPoint xy1 = (*this)(geo1);
+	PaperPoint xy2 = (*this)(geo2);
+
+	double x = xy1.distance(xy2);
+	*/
+	return 100000*res;
+
+}
+
+void  PolarStereographicProjection::fast_reproject(double& x, double& y) const
+{
+	TeCoord2D geo = TeCoord2D(x*TeCDR, y*TeCDR);
+	TeCoord2D xy = projection_->LL2PC(geo);
+	x = xy.x();
+	y = xy.y();
+
+}
+
+void PolarStereographicProjection::getNewDefinition(const UserPoint& ll, const UserPoint& ur, string& out) const
+{
+	map<string, string> def;
+	def["subpage_map_projection"] = "polar_stereographic";
+	def["subpage_map_area_definition"] = "corners";
+
+	def["subpage_map_hemisphere"] = (hemisphere_ == NORTH) ? "north" : "south";
+	def["subpage_map_vertical_longitude"] = tostring(vertical_longitude_);
+	def["subpage_lower_left_longitude"] = tostring(ll.x_);
+	def["subpage_lower_left_latitude"] = tostring(ll.y_);
+	def["subpage_upper_right_longitude"] = tostring(ur.x_);
+	def["subpage_upper_right_latitude"] = tostring(ur.y_);
+	::toxml(out, def);
+	out = "{" + out + "}";
+
+}
+void PolarStereographicProjection::setDefinition(const string& json)
+{
+	if (json.empty())
+				return;
+
+	MagJSon helper;
+	helper.interpret(json);
+
+	XmlNode node = **helper.tree_.firstElement();
+
+	node.name("polar_stereographic");
+
+
+	set(node);
+
+
 }
 
 
