@@ -21,7 +21,7 @@
 /*!
     \file PolarStereographicProjection.h
     \brief Definition of PolarStereographicProjection.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Fri Jan 11 15:01:45 2008
 */
@@ -73,28 +73,26 @@ public:
 	}
 	virtual void fill(double&, double&) ;
 	 void setNewPCBox(double, double, double, double);
+
+	 double patchDistance(double) const;
+	 void fast_reproject(double& x, double& y) const;
 	/*!
 	\\brief Initialise the projection
 	*/
 	virtual void init(double, double) ;
+
 	/*!
 	\\brief 
 	*/
 	virtual PaperPoint operator()(const UserPoint&) const;
-	/*!
-	\\brief 
-	*/
-	virtual PaperPoint operator()(const GeoPoint&) const;
-	virtual void operator()(const GeoPoint&, vector<PaperPoint>& out) const;
+	virtual void operator()(const UserPoint&, vector<PaperPoint>& out) const;
 	virtual double unitToCm(double, double) const;
+	virtual double height() const;
 	/*!
 	\\brief 
 	*/
 	virtual PaperPoint operator()(const PaperPoint&) const;
-	/*!
-	\\brief 
-	*/
-	virtual void revert(const PaperPoint&, GeoPoint&) const;
+	void revert(const vector<pair<double, double> > &, vector<pair<double, double> > &) const;
 	/*!
 	\\brief 
 	*/
@@ -104,9 +102,9 @@ public:
 	*/
 	virtual bool needShiftedCoastlines() const;
 
-	virtual bool concatenate(vector<Polyline* >& lines, Polyline* poly) const;
-
-
+	//virtual bool concatenate(vector<Polyline* >& lines, Polyline* poly) const;
+	Polyline& getPCBoundingBox() const;
+	Polyline& getUserBoundingBox() const;
 	/*!
 	\\brief set the aspect ratio!
 	*/
@@ -115,6 +113,7 @@ public:
 	\\brief set the bounding box!
 	*/
 	virtual void boundingBox(double&, double&, double&, double&) const;
+	virtual void smallestBoundingBox(double&, double&, double&, double&) const;
 	/*!
 	\\brief return the xmin in user coordinates!
 	*/
@@ -180,16 +179,19 @@ public:
 	virtual void labels(const LabelPlotting&, TopAxisVisitor&) const;
 	virtual void labels(const LabelPlotting&, BottomAxisVisitor&) const;
 	
-	virtual void thin(MatrixHandler<GeoPoint>&, double x, double y, vector<GeoPoint>&) const; 
+	virtual void thin(MatrixHandler&, double x, double y, vector<UserPoint>&) const;
 	/*!
 	        \\brief prepare the javascript for thes particular instance of projection
 	*/
-	virtual void  visit(MetaDataVisitor&,double, double, double, double) ;
+	virtual void  visit(MetaDataVisitor&,double, double, double, double, double, double) ;
 	
 	// Needed for Image processing!
 	virtual TeProjection&  getProjection() 
 			{ return *projection_ ; }
-
+	double dimension(BasicGraphicsObjectContainer& parent) const
+        { return parent.absoluteHeight(); }
+	void getNewDefinition(const UserPoint&, const UserPoint&, string&) const;
+	void setDefinition(const string&);
 protected:
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	virtual void print(ostream&) const; 
@@ -210,8 +212,15 @@ protected:
 	double ypcmin_;
 	double xpcmax_;
 	double ypcmax_;
-	void reprojectComponents(const GeoPoint&, pair<double, double>&) const;
+	void reprojectComponents(const UserPoint&, pair<double, double>&) const;
 	void reprojectSpeedDirection(const PaperPoint& point, pair<double, double>&) const;
+
+	void coastSetting(map<string, string>&, double width, double height) const;
+
+	MatrixHandler* prepareData(const AbstractMatrix& matrix) const;
+	void wraparound(const UserPoint&, stack<UserPoint>&) const;
+
+
 private:
 	//! Copy constructor - No copy allowed
 	PolarStereographicProjection(const PolarStereographicProjection&);
