@@ -23,7 +23,7 @@
     
     Started: Jan 2004
     
-    Changes:
+    Changes: 
     
 */
 #ifndef Symbol_H
@@ -34,7 +34,7 @@
 #include "BasicGraphicsObject.h"
 #include "BaseDriver.h"
 #include "SymbolItem.h"
-#include "PaperPoint.h"
+#include "Polyline.h"
 
 namespace magics {
 
@@ -42,11 +42,11 @@ namespace magics {
 class Symbol: public BasicGraphicsObject, public vector<PaperPoint> {
 
 public:
-	enum TextPosition { M_NONE, M_BELOW , M_ABOVE , M_LEFT , M_RIGHT};
+	enum TextPosition { M_NONE, M_BELOW , M_ABOVE , M_LEFT , M_RIGHT, M_CENTRE};
 	Symbol();
 	virtual ~Symbol();
 
-	void redisplay(const BaseDriver& driver) const { driver.redisplay(*this); }
+	void redisplay(const BaseDriver& driver) const;
 
 	void setMarker(int m)
 	{ 
@@ -61,6 +61,7 @@ public:
 		return symbol.str();
 	}
 
+	    void boundingbox(const Polyline& boundingbox);
 	virtual void push_back(const PaperPoint& point,const string& ="") {
 		 vector<PaperPoint>::push_back(point);
 	}
@@ -79,20 +80,57 @@ public:
 	void   setDistanceApart(double a){apart_ = a;}
 	double getDistanceApart() const  {return apart_;}
 
+	void   outlineColour(const Colour& a){ outlineColour_ = a;}
+
+	const Colour& outlineColour() const { return outlineColour_; }
+	bool   outline() const { return outline_;}
+
+	int outlineThickness() const { return outlineThickness_; }
+	LineStyle outlineLineStyle() const { return outlineStyle_; }
+
+	void outline(bool outline, const Colour& colour, int thickness, LineStyle style) {
+		outline_ = outline;
+		outlineColour_ = colour;
+		outlineThickness_ = thickness;
+		outlineStyle_ = style;
+	}
+	void connectline(bool connect) { connectLine_ = connect; }
+	void connectline(bool connectline, const Colour& colour, int thickness, LineStyle style) {
+		connectLine_ = connectline;
+		connectLineColour_ = colour;
+		connectLineThickness_ = thickness;
+		connectLineStyle_ = style;
+	}
+	int connectLineThickness() const { return connectLineThickness_; }
+	LineStyle connectLineStyle() const { return connectLineStyle_; }
+	const Colour& connectLineColour() const { return connectLineColour_; }
+	bool connectLine() const { return connectLine_; }
+
 //private:
 // No copy allowed
-	Symbol(const Symbol&);
-	Symbol& operator=(const Symbol&);
+
 
 protected:
 	virtual void print(ostream&) const; 
-
+	Polyline boundingbox_;
 private:
 	int	marker_;
 	double	height_;
 	string	symbol_;
 	Colour colour_;
 	double apart_;
+	Colour outlineColour_;
+	LineStyle outlineStyle_;
+	int outlineThickness_;
+	bool outline_;
+
+	bool connectLine_;
+	Colour connectLineColour_;
+	LineStyle connectLineStyle_;
+	int connectLineThickness_;
+
+
+
 
 // -- Friends
 	friend ostream& operator<<(ostream& s,const Symbol& p)
@@ -108,11 +146,11 @@ public:
 
 	void push_back(const PaperPoint& point, const string& text) {
 		Symbol::push_back(point);
-		texts_.push_back(text);
+		if ( !text.empty() ) texts_.push_back(text);
 	}
 
 	void text(const vector<string>& text) { texts_ = text; }
-	void redisplay(const BaseDriver& driver) const { driver.redisplay(*this); }
+	void redisplay(const BaseDriver& driver) const;
 
 	vector<string>::const_iterator textBegin() const { return texts_.begin(); }
 	vector<string>::const_iterator textEnd() const { return texts_.end(); }
@@ -120,17 +158,35 @@ public:
 
 	void font(const MagFont& font) { font_ = font; }
 	const MagFont& font() const { return font_; }
+	void blanking(bool blanking) { blanking_ = blanking; }
+	bool blanking() const { return blanking_; }
 
 	void position(TextPosition position) { position_ = position; }
 	TextPosition position() const { return position_; }
 
 protected:
-	vector<string> texts_;
+	mutable vector<string> texts_;
 	MagFont font_;
 	TextPosition position_;
+	bool blanking_;
 };
 
 
+class ImageSymbol : public Symbol
+{
+public:
+	ImageSymbol(const string& path, const string& format) : path_(path), format_(format) {}
+	~ImageSymbol() {}
+	void redisplay(const BaseDriver& driver) const;
+
+	void set(double width, double height) { width_ = width; height_ = height; }
+
+protected:
+	string path_;
+	string format_;
+	double width_;
+	double height_;
+};
 class SimpleTextSymbol : public TextSymbol
 {
 public:
@@ -140,7 +196,9 @@ public:
 	void push_back(const PaperPoint& point) {
 		TextSymbol::push_back(point, text_);
 	}
-
+	void push_back(const PaperPoint& point, const string&) {
+			TextSymbol::push_back(point, text_);
+		}
 protected:
 	string text_;
 };
@@ -153,7 +211,7 @@ public:
 	ComplexSymbol(int rows, int columns): rows_(rows), columns_(columns) {}
 	~ComplexSymbol() {}
 	
-	void redisplay(const BaseDriver& driver) const { Log::dev() << "Redisplay -->" << *this << endl; driver.redisplay(*this); }
+	void redisplay(const BaseDriver& driver) const { MagLog::dev() << "Redisplay -->" << *this << endl; driver.redisplay(*this); }
 	void add(GraphicsItem* item) { items_.push_back(item);}
 	vector<GraphicsItem*>::const_iterator itemBegin() const { return items_.begin(); }
 	vector<GraphicsItem*>::const_iterator itemEnd() const { return items_.end(); }
@@ -173,21 +231,33 @@ struct SymbolProperties
 	string    marker_;
 	string    label_;
 	MagFont   font_;
+	bool      blanking_;
 	Symbol::TextPosition position_;
 	vector<string> text_;
 
-    SymbolProperties(Colour colour, double height, const string& marker, const string& label="") : 
-        colour_(colour), 
-        height_(height),
-        marker_(marker),
-        label_(label), position_(Symbol::M_NONE)
-        {}
-    
-    
+	bool outline_;
+	Colour outlineColour_;
+	LineStyle outlineStyle_;
+	int outlineThickness_;
+
+	bool connectLine_;
+	Colour connectLineColour_;
+	LineStyle connectLineStyle_;
+	int connectLineThickness_;
+
+
+	bool  image_;
+	string image_path_;
+	string image_format_;
+	int    image_width_;
+	int    image_height_;
+
+    SymbolProperties(Colour colour, double height, const string& marker, const string& label="");
     SymbolProperties(Colour colour, double height, int marker, const string& label=""); 
     SymbolProperties();
 
     void setSymbol(const string& symbol, int marker);
+    void position(Symbol::TextPosition position) { position_ = position; }
 
     virtual ~SymbolProperties() {}
     bool  operator<(const SymbolProperties& other) const
@@ -200,11 +270,14 @@ struct SymbolProperties
             return other.colour_ < colour_;
     }
     
+    void label(const string& label) { label_ = label; }
+    
     virtual void print(ostream& out) const {
         out << "SymbolProperty[";
         out << colour_ << "(colour),";
         out << height_ << "(height),";
         out << marker_ << "(marker)";
+        out << label_ << "(label)";
         out << "]";
     }
     
