@@ -16,7 +16,7 @@
 
  ******************************** LICENSE ********************************/
 
-#ifndef machine_H
+#ifndef marsmachine_H
 #include "magics.h"
 #endif
 
@@ -28,8 +28,9 @@
 #include "Seconds.h"
 #endif
 
-
-#include "Log.h"
+#include "Mutex.h"
+#include "AutoLock.h"
+#include "MagLog.h"
 
 namespace magics {
 
@@ -58,24 +59,28 @@ Timer::Timer(const string& name, const string& detail):
 {
 	gettimeofday(&start_,0);
 }
-
+static Mutex lockprofiles_;
 Timer::~Timer()
 {
 	const double  s   = elapsed();
 	clock_t cpu =  clock();
 	const Seconds	sec1(s);
 	const Seconds	sec2(double(cpu-cpu_)/CLOCKS_PER_SEC);
-	Log::profile() << name_ << ": " 
+	MagLog::profile() << name_ << ": " 
 		<< sec1 << " elapsed, " 
 		<< sec2 << " cpu" << "\n";
 	ostringstream e;
-			e << sec1;
+			e << elapsed();
 			
 			ostringstream c;
-			c << sec2;
+			c << double(cpu-cpu_)/CLOCKS_PER_SEC;
 			
+
+	{
+		AutoLock<Mutex> lock(lockprofiles_);
 	
-	profiles_.push_back(ProfileInfo(name_, details_, e.str(), c.str()));
+		profiles_.push_back(ProfileInfo(name_, details_, e.str(), c.str()));
+	}
 }
 
 double Timer::elapsed()
@@ -91,4 +96,13 @@ ProfileInfo::ProfileInfo(const string& name, const string& details, const string
 		
 	}
 
-vector<ProfileInfo> Timer::profiles_;
\ No newline at end of file
+ostream& operator<<(ostream& s,const ProfileInfo& p)
+			{ s << "\t\"" << p.name_ << "\" : {" << endl;
+			   s << "\t\t\"details\" : \"" << p.details_ << "\"," << endl;
+			   s << "\t\t\"elapsed\" : " << p.elapsed_ << "," << endl;
+			   s << "\t\t\"cpu\" : " << p.cpu_ << endl;
+			   s << "\t}";
+			   return s;
+			   }
+	
+vector<ProfileInfo> Timer::profiles_;
