@@ -26,8 +26,8 @@
 #include "magics.h"
 #endif
 
-//#ifndef Log_H
-//#include "Log.h"
+//#ifndef MagLog_H
+//#include "MagLog.h"
 //#endif
 
 #include <time.h>
@@ -48,17 +48,7 @@ public:
 	
 	// -- Friends
 	    //! Overloaded << operator to call print().
-		friend ostream& operator<<(ostream& s,const ProfileInfo& p)
-			{ s << p.name_ << " = function() {" << endl;
-			   s << "this.details = \"" << p.details_ << "\";" << endl;
-			   s << "this.elapsed = \"" << p.elapsed_ << "\";" << endl;
-			   s << "this.cpu = \"" << p.cpu_ << "\";" << endl;
-			   s << "return this;" << endl;
-			   s << "}" << endl;
-			   return s;
-			   }
-	
-	
+		friend ostream& operator<<(ostream& s,const ProfileInfo& p);
 	
 };
 
