@@ -59,30 +59,29 @@ Currently Magics++ supports following projections:
 #include "magics.h"
 
 #include "PaperPoint.h"
-#include "GeoPoint.h"
 #include "UserPoint.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 #include "Factory.h"
 #include "TeProjection.h"
 #include "BasicGraphicsObject.h"
 #include "SceneVisitor.h"
 
 
+
+
 namespace magics {
-    
+
 class Layout;
 class GridPlotting;
 class LabelPlotting;
 class BasicSceneObject;
-
-class Polyline;
-class PolyCoast;
-class Text;
-template <class P> class MatrixHandler;
-template <class P> class PointsHandler;
-
+class MetaDataCollector;
+class AbstractMatrix;
+class MatrixHandler;
+class PointsHandler;
 class XmlNode;
 
+class Polyline;
 
 class ViewFilter
 {
@@ -93,28 +92,27 @@ public:
 	bool in(const PaperPoint& xy);
 
     vector<bool> done;
-    double xmin_;    
-    double xmax_;  
+    double xmin_;
+    double xmax_;
     double ymin_;
-    double ymax_; 
+    double ymax_;
     double xres_;
-    double yres_;  
+    double yres_;
     int xdim_;
     int ydim_;
-    
-		
 };
 
 
 class Transformation
 {
-
 public:
 	Transformation();
 	virtual ~Transformation();
-	virtual void init() {}
-    
-	
+	virtual void init();
+
+	enum CoordinateType {GeoType,XyType};
+	CoordinateType coordinateType() const {return coordinateType_;}	
+	virtual void toxml(ostream&) const {}
 	// Xml Methods ! 
 	virtual void set(const map<string, string>& ) {}
 	virtual void set(const XmlNode& ) {}
@@ -125,56 +123,47 @@ public:
 		return object;
 	}
 	
+	virtual Polyline& getUserBoundingBox() const  { assert(false); }
+	virtual Polyline& getPCBoundingBox()   const  { assert(false); }
 	
-	
-   
+	bool needTopAxis() const   { return topAxis_; }
+	void needTopAxis(bool top) { topAxis_ = top; }
+
 	void operator()(const Polyline& poly,  BasicGraphicsObjectContainer& out) const;	
-	void operator()(PolyCoast& poly,  BasicGraphicsObjectContainer& out) const;	
-	
-	
 
-	
-	virtual bool wrapAround(const Polyline&) const { return false; }   
+	string writeLongitude(const UserPoint&) const;
+	string writeLatitude(const UserPoint&) const;
+
+	virtual bool wrapAround(const Polyline&) const { return false; }
+	virtual void coastSetting(map<string, string>&, double, double) const { assert(false); }
+	virtual bool verifyDef(const string&) const { return false; }   
+
 
- 
- 
 
-		
-	
-	virtual bool concatenate(vector<Polyline* >&, Polyline*) const
-	 	{ return false; }
-	
-	
 	virtual double x(double x) const { return x; }
 	virtual double y(double y) const { return y; }
-	
+	virtual double rx(double x) const { return x; }
+	virtual double ry(double y) const { return y; }
+
+	/*
 	// is the polyline out the projection!
-
 	virtual bool out(const Polyline&) const;
-	
 	// is the polyline in the projection?
-
 	virtual bool in(const Polyline&) const;
-	
+	*/
     // is the point in projected area?
-	bool in(const GeoPoint&) const;
 	bool in(const UserPoint&) const;
 	bool in(const PaperPoint&) const;
-	bool clip(PaperPoint&) const; 
-    
-	// is the point in the projected area?
-	bool in(double x, double y) const
-       { return ( getAbsoluteMinX() <= x && x <=  getAbsoluteMaxX() && getAbsoluteMinY() <= y && y <= getAbsoluteMaxY()); }
+
+
+	// is the point in PC in the projected area?
+	bool in(double x, double y) const;
+
     bool inX(double x) const
        { return ( getAbsoluteMinX() <= x && x <=  getAbsoluteMaxX()); }
     bool inY(double y) const
         { return ( getAbsoluteMinY() <= y && y <= getAbsoluteMaxY()); }
-	
-	
-	// make sure that the polyline starts inside the projected area!
-	virtual void rotate(vector<GeoPoint>&) const;
 
-	
 	// Needed for Image processing!
 	virtual TeProjection&  getProjection() 
 		{ static TeDatum datum; static TeLatLong dummy(datum); return dummy; }
@@ -193,48 +182,63 @@ public:
 	
 	
 	virtual void aspectRatio(double&, double&);
-	virtual void fill(double&, double&); // fill the sapec , can adapt the coordiantes to return the biggest area.. 	
+	virtual void fill(double&, double&); // fill the space , can adapt the coordiantes to return the biggest area..
 	virtual void forceNewArea(double, double, double, double, double&, double&); 
 	
 	
 // Basic reprojection method! 
 	virtual PaperPoint operator()(const UserPoint& xy) const 
 		{ return PaperPoint(xy.x(), xy.y()); }
-	virtual PaperPoint operator()(const GeoPoint& geo) const 
-		{ return PaperPoint(geo.longitude(), geo.latitude()); }
+
+
+	virtual void fast_reproject(double& x, double& y) const
+			{ }
+
+	virtual double patchDistance(double) const { assert(false); }
+
 	virtual PaperPoint operator()(const PaperPoint& xy) const 
 		{ return xy; }
-
-	virtual void operator()(const GeoPoint& geo, vector<PaperPoint>& out) const;
+	virtual void operator()(const UserPoint& geo, Polyline& out) const;
 	virtual void operator()(const UserPoint& xy, vector<PaperPoint>& out) const;
+
+	virtual void revert(const vector<pair<double, double> > &, vector<pair<double, double> > &) const;
 	
-	virtual void revert(const PaperPoint& xy, GeoPoint& point) const 
-		{ point = GeoPoint(xy.x(), xy.y()); }		
 	virtual void revert(const PaperPoint& xy, UserPoint& point) const 
-		{ point = UserPoint(xy.x(), xy.y()); }
+		{ point = UserPoint(xy.x(), xy.y()); }		
+
 
 // Does the projection needs the coastalines to be shifted!
 	virtual bool needShiftedCoastlines() const { return false; }
 
 // Set the bounding box in user coordinates 
 	virtual void boundingBox(double&, double&, double&, double&) const;
-
+	virtual void smallestBoundingBox(double& x1, double& y1, double& x2, double& y2) const
+	{
+		boundingBox(x1, y1, x2, y2);
+	}
 	
 
 	virtual void setDataMinX(double minx) const { dataMinX_ = std::min(minx, dataMinX_); }
 	virtual void setDataMaxX(double maxx) const { dataMaxX_ = std::max(maxx, dataMaxX_); }
 	virtual void setDataMinY(double miny) const { dataMinY_ = std::min(miny, dataMinY_); }
 	virtual void setDataMaxY(double maxy) const { dataMaxY_ = std::max(maxy, dataMaxY_); }
-
+	
+	virtual void setAutomaticX(bool) {}
+	virtual void setAutomaticY(bool) {}
+	virtual bool getAutomaticX() const { return false; } 
+	virtual bool getAutomaticY() const { return false; }
+	
     virtual void setDataMinX(double minx, const string& ref) const; 
 	virtual void setDataMaxX(double maxx, const string& ref) const; 
 	virtual void setDataMinY(double miny, const string& ref) const; 
 	virtual void setDataMaxY(double maxy, const string& ref) const; 
 	
-	const string& getDataReferenceX() const  { return dataReferenceX_; }
-	const string& getDataReferenceY() const  { return dataReferenceY_; }
-	void setDataReferenceX(const string& ref) const  { dataReferenceX_ = ref; }
-	void setDataReferenceY(const string& ref) const  { dataReferenceY_ = ref; }
+	virtual vector<double> getDataVectorMinX() const  { return vector<double>(); }
+	virtual vector<double> getDataVectorMaxX() const  { return vector<double>(); }
+	virtual vector<double> getDataVectorMinY() const  { return vector<double>(); }
+	virtual vector<double> getDataVectorMaxY() const  { return vector<double>(); }
+	
+	
 	
     virtual const string& getReferenceX() const  { return referenceX_; }
 	virtual const string& getReferenceY() const  { return referenceY_; }
@@ -267,6 +271,8 @@ public:
 	virtual double getMaxPCX() const { return -1; }
 	virtual double getMinPCY() const { return -1; }
 	virtual double getMaxPCY() const { return -1; }
+	virtual double dimension(BasicGraphicsObjectContainer& parent) const 
+	{ return parent.absoluteWidth(); }
 	
 	double getAbsoluteMinPCX() const { return std::min(getMinPCX(), getMaxPCX()); }
 	double getAbsoluteMaxPCX() const { return std::max(getMinPCX(), getMaxPCX()); }
@@ -281,28 +287,39 @@ public:
 	virtual void setNewPCBox(double, double, double, double) {  }
 	virtual double unitToCm(double, double) const;
 	
-
+	virtual double height() const { return getMaxX() - getMinX(); }
 	
 
-   virtual void thin(MatrixHandler<GeoPoint>&, double x, double y, vector<GeoPoint>&) const; 
+	virtual void getNewDefinition(const UserPoint&, const UserPoint&, string&) const
+		{ assert(false); }
+	virtual void setDefinition(const string&)
+		{ assert(false); }
+
+   virtual void thin(MatrixHandler&, double x, double y, vector<UserPoint>&) const;
    
-   virtual void reprojectComponents(const GeoPoint& point, pair<double, double>&) const;
+   virtual void reprojectComponents(const UserPoint& point, pair<double, double>&) const;
    virtual void reprojectSpeedDirection(const PaperPoint& point, pair<double, double>&) const;
  
-   void thin(PointsHandler<GeoPoint>& points, vector<PaperPoint>& out, vector<PaperPoint>&) const;
-   void thin(MatrixHandler<GeoPoint>& points, vector<PaperPoint>& out, vector<PaperPoint>&) const; 
-   void thin(MatrixHandler<UserPoint>& points, vector<PaperPoint>& out, vector<PaperPoint>&) const;
+   void thin(PointsHandler& points, vector<PaperPoint>& out, vector<PaperPoint>&) const;
+   void thin(MatrixHandler& points, vector<PaperPoint>& out, vector<PaperPoint>&) const;
+
    // The view is set in Projection coordinates!
   virtual void filterView(double xmin, double xmax, double ymin, double ymax, double x, double y)  const {
-	  	view_ = ViewFilter(xmin, xmax, ymin, ymax, x, y);	  
+	  	view_ = ViewFilter(xmin, xmax, ymin, ymax, x, y);
   }
-  
-   virtual  void  visit(MetaDataVisitor&, double, double, double, double);
-   
- 
+  virtual  void  collect(MetaDataCollector&) const {}
+   virtual  void  visit(MetaDataVisitor&, double, double, double, double, double, double);
+   virtual string xAxisType() const { return "regular"; }
+   virtual string yAxisType() const { return "regular"; }
+
+   virtual MatrixHandler* prepareData(const AbstractMatrix& matrix) const;
+   virtual void wraparound(const UserPoint&, stack<UserPoint>&) const;
+
 protected:
 	virtual void print(ostream&) const;    
     
+    CoordinateType coordinateType_;	
+    
     mutable double areaMinX_;
     mutable double areaMaxX_;
     mutable double areaMinY_;
@@ -319,7 +336,12 @@ protected:
     mutable string referenceY_;
     
     mutable ViewFilter view_;
-   
+
+    bool topAxis_;
+
+    mutable Polyline* userEnveloppe_;
+    mutable Polyline* PCEnveloppe_;
+
 private:
 	// No copy allowed
 	Transformation(const Transformation&);
@@ -332,7 +354,7 @@ private:
 };
 
 template<>
-class Translator<string, Transformation>
+class MagTranslator<string, Transformation>
 { 
 public:
 	Transformation* operator()(const string& val )
@@ -345,6 +367,7 @@ public:
 		ParameterManager::update(param, object);
 		return object;
 	}
+
 };
 
 } // namespace magics
