@@ -250,7 +250,7 @@ void MetgramParameter::interpretResult(spot_query_result* result, vector<Customi
 	Timer timer("access spot","");
 	size_t len = 1024;
 	char tmp[1024];
-	double missing = int_MAX; 
+	double missing = INT_MAX;
 	
 	string date = get_spot(result,"date");
 	string time = get_spot(result,"time");
@@ -279,7 +279,7 @@ void MetgramParameter::interpretResult(spot_query_result* result, vector<Customi
 	}	
 	bool rain = false;
 	
-	base_ = DateTime(date, Time(time));
+	base_ = DateTime(date, MagTime(time));
 
 	DateTime next;
 
@@ -340,11 +340,11 @@ void MetgramParameter::interpretResult(spot_query_result* result, vector<Customi
 	    		(*point)[key] = (*this)(values[info->second]);	  
 	    		ypos.push_back( (*this)(values[info->second]));	 
 	    		if (rain) (*point)["as_rain"] = 1;
-	    		Log::dev() << " SPOTBASE returns " << info->first << " for " <<(*this)(values[info->second]) << " at " << step->first <<  endl;
+	    		MagLog::dev() << " SPOTBASE returns " << info->first << " for " <<(*this)(values[info->second]) << " at " << step->first <<  endl;
 		
 	    	}
 	    	else {
-	    		Log::warning() << " SPOTBASE returns nan for " << info->first << " : data ignored for step " << (*point)["shift"]<< endl;
+	    		MagLog::warning() << " SPOTBASE returns nan for " << info->first << " : data ignored for step " << (*point)["shift"]<< endl;
 	    		ok = false;
 	    	}
 	    	
@@ -360,12 +360,12 @@ void MetgramParameter::interpretResult(spot_query_result* result, vector<Customi
 		minx_ = std::min(minx_, *std::min_element(xpos.begin(), xpos.end()));
 		maxx_ = std::max(maxx_, *std::max_element(xpos.begin(), xpos.end()));
 	
-		Log::debug() << "MIN=" << minx_ << "MAX=" << maxx_ << endl;
+		MagLog::debug() << "MIN=" << minx_ << "MAX=" << maxx_ << endl;
 	}
     miny_ = std::min(miny_, *std::min_element(ypos.begin(), ypos.end()));
 	maxy_ = std::max(maxy_, *std::max_element(ypos.begin(), ypos.end()));
 	
-	Log::debug() << "MIN=" << miny_ << "MAX=" << maxy_ << endl;
+	MagLog::debug() << "MIN=" << miny_ << "MAX=" << maxy_ << endl;
 }
 
 
@@ -519,7 +519,7 @@ Timer time("openSpot","");
 	}
 	if ( !spot_ )
 	{
-		Log::error() << database_ << ": Not a valid Epsgram Database\n";
+		MagLog::error() << database_ << ": Not a valid Epsgram Database\n";
 		throw MagicsException(database_ +": Not a valid Epsgram Database");
 	}
 
@@ -529,12 +529,12 @@ Timer time("openSpot","");
 	spot_set_method(query,"EPSGRAM");
  
 	if (!date_.empty() ) {
-		Log::debug() << "Set the date --->" << date_ << "\n";
+		MagLog::debug() << "Set the date --->" << date_ << "\n";
 		spot_set_parameter(query,"date", date_.c_str());
 	}
    
 	if (!time_.empty() ) {
-		Log::debug() << "Set the date --->" << time_ << "\n";
+		MagLog::debug() << "Set the date --->" << time_ << "\n";
 		spot_set_parameter(query,"time", time_.c_str());
 	}
 
@@ -561,7 +561,7 @@ void ClassicMtgDecoder::decode()
 	catch (NoFactoryException& e)
 	{
 		// The data do not know how to verify the criter ....
-		Log::warning() << "Eps Data access: parameter [" << param_ << "] unknown\n";   
+		MagLog::warning() << "Eps Data access: parameter [" << param_ << "] unknown\n";   
 		parameter_ = new MetgramParameter(param_, param_title_, param_);
 	} 
 #else
@@ -569,7 +569,7 @@ void ClassicMtgDecoder::decode()
 	if ( !parameter_ )
 	{
 		// The data do not know how to verify the criter ....
-		Log::warning() << "Eps Data access: parameter [" << param_ << "] unknown\n";
+		MagLog::warning() << "Eps Data access: parameter [" << param_ << "] unknown\n";
 		parameter_ = new MetgramParameter(param_, param_title_, param_);
 	}
 #endif   
@@ -582,29 +582,25 @@ void ClassicMtgDecoder::decode()
 	}
 	if ( !result )
 	{
-		Log::error() << " No EpsData\n";
+		MagLog::error() << " No EpsData\n";
 		return;
 	}
 	Timer ("prepare meta data","");
 	double lat, lon;
 
 	spot_get_location(result, &lat, &lon);
+	
 	spot_get_index_value(result, "orog", &detz_);
-//	spot_get_index_value(result, "detz", &detz_);
-//	parameter_->epsHeight(epsz_);
-//	parameter_->correction(correction_);
-//	parameter_->deterministicHeight(detz_);
 	spot_get_index_value(result, "mask", &mask_);
 
 	if ( lon > 180 ) lon -= 360.;
 
-	grid_ = GeoPoint(lon, lat);
+	grid_ = UserPoint(lon, lat);
 
 	map<string, int> columns;
 	map<int, int> rows;
 	
-//	char tmp[1024];
-//	size_t len = 1024;
+
 	long resol; 
 	
 	read(get_spot(result,"numberOfPointsAlongAMeridian"), resol);
@@ -628,7 +624,7 @@ void ClassicMtgDecoder::moreTitle(TextVisitor& title)
 {
 	if ( !long_title_ ) return;
 	
-	DateTime base(date_, Time(time_));
+	DateTime base(date_, MagTime(time_));
 	ostringstream out;
 	tm convert = base;
 	locale loc("");        
@@ -640,7 +636,7 @@ void ClassicMtgDecoder::moreTitle(TextVisitor& title)
 	vector<string> lines;
 	
 	string height = "";
-	if (height_ < int_MAX  ) {
+	if (height_ < INT_MAX  ) {
 		ostringstream sh;
 		sh << " (" << height_ << "m) ";
 		height = sh.str();
@@ -649,7 +645,7 @@ void ClassicMtgDecoder::moreTitle(TextVisitor& title)
 		height = " ";
 	}
 	
-	lines.push_back(station_ + " " + grid_.writeLatitude() + " " + grid_.writeLongitude() + " " + tostring(maground(detz_)) + " m");
+	lines.push_back(station_ + " " + grid_.asLatitude() + " " + grid_.asLongitude() + " " + tostring(maground(detz_)) + " m");
 	lines.push_back("ECMWF Forecast from " + out.str());
 	
 	string landsea = (mask_>= 0.5) ? "Nearest land grid point" : "Nearest sea grid point";
