@@ -45,15 +45,15 @@ void EpsBufr::visit(Transformation& transformation)
 	decode();
 	transformation.setDataMinX((minstep_ - shift_) * 3600, base_);
 	transformation.setDataMaxX((maxstep_ + 6) * 3600, base_);
-	transformation.setMinY(miny_);
-	transformation.setMaxY(maxy_);
+	transformation.setDataMinY(miny_);
+	transformation.setDataMaxY(maxy_);
 }
 		
 void EpsBufr::decode()
 {
 	if ( !points_.empty()) return;
 	
-     Log::dev() << "EpsBufr::decode()-->" << *this << endl;		
+     MagLog::dev() << "EpsBufr::decode()-->" << *this << endl;		
      // Test
      MvObsSet set(path_.c_str());
      
@@ -63,21 +63,24 @@ void EpsBufr::decode()
      filter.setArea(ll, ur);
      
      MvObs obs = filter();
-     map<int, vector<float> > members;
-     map<int, float > control; 
+    
+     map<int, float > curve1; 
+     map<int, float > curve2; 
      
      minstep_ = std::numeric_limits<double>::max();
      maxstep_ = -std::numeric_limits<double>::max();
      miny_ = std::numeric_limits<double>::max();
      maxy_ = -std::numeric_limits<double>::max();
 
+     double previous = 0;
      while (obs)
      {
-             //Log::dev()<< "obs---------------------------------------------------------------------------" << endl;		
-     	MvLocation loc = obs.location();
-//        int subsets = obs.msgSubsetCount();
+             //MagLog::dev()<< "obs---------------------------------------------------------------------------" << endl;		
+     		MvLocation loc = obs.location();
+//      	  int subsets = obs.msgSubsetCount();
 
-        float value = obs.value( param_descriptor_ );
+       		 float value = obs.value(param_descriptor_ );
+        	obs.value(5195); 
         	if ( value == kFortranBufrMissingValue )
 		{
         		obs = filter(NR_returnMsg);         	// Were going to the next message!
@@ -88,88 +91,113 @@ void EpsBufr::decode()
     		 
     		while ( ++i  )
 		{
-    			float step =  obs.valueByOccurrence(i, 4024);   			
+    			// 0424 is the bufr descriptor for Time Period Or Displacement [HOUR]
+                
+    			float step =  obs.valueByOccurrence(i, 4024);  
     			float value =  obs.valueByOccurrence(i, param_descriptor_);
-    			value = (value * param_scaling_factor_ ) + param_offset_factor_;
-    			base_ = DateTime(obs.obsTime().CharValue());
-    			Log::dev() << "date--->" << base_ << endl; 
     			if ( value == kFortranBufrMissingValue ) {
     				obs = filter(); // We going to the next subset!
     				break;
     			}
+    			value = (value * param_scaling_factor_ ) + param_offset_factor_;
+    			
+    			base_ = DateTime(obs.obsTime().CharValue());
+    			MagLog::dev() << "date--->" << base_ << endl; 
+    		
+    			if ( accumulated_) {
+    				double total = value;
+    				value = value - previous;
+    				previous = total;
+    			}
+
     			if  ( minstep_ > step ) minstep_ = step;
     			if  ( maxstep_ < step ) maxstep_ = step;
     			if  ( miny_ > value ) miny_ = value;
     			if  ( maxy_ < value ) maxy_ = value;
-    			if ( obs.subsetNumber() == 1 )
-			{
-    				control[step] = value;
-    			}
-    			else {
-    				map<int, vector<float> >::iterator member = members.find(step);
-    				if ( member == members.end() ) {
-    					members.insert(make_pair(step, vector<float>()));
-    					member = members.find(step);
-    				}
-    				member->second.push_back(value);      
-    			}
+
+    			curve1[step] = value;
     		}
      	}
+     if ( param_descriptor_2_) { 
+    	 MvObsSet set(path_.c_str());
      
-     	for (map<int, vector<float> >::iterator member = members.begin(); member != members.end(); ++member) {
-     	    std::sort( member->second.begin(), member->second.end());
-     	}
+     MvObsSetIterator   filter(set);
+     MvLocation ll(latitude_-0.1, longitude_-0.1);
+     MvLocation ur(latitude_+0.1, longitude_+0.1);
+     filter.setArea(ll, ur);
+    MvObs obs2 = filter();
+     while (obs2)
+         {
+                 //MagLog::dev()<< "obs---------------------------------------------------------------------------" << endl;		
+         	MvLocation loc = obs2.location();
+    //        int subsets = obs.msgSubsetCount();
 
-	for (map<int, float>::const_iterator step = control.begin(); step != control.end(); ++step) {
+            float value = obs2.value( param_descriptor_2_ );
+            
+            	if ( value == kFortranBufrMissingValue )
+    		{
+            		obs2 = filter(NR_returnMsg);         	// Were going to the next message!
+            		continue;
+            	}
+            	
+        		int i = 0;
+        		 
+        		while ( ++i  )
+    		{
+        			// 0424 is the bufr descriptor for Time Period Or Displacement [HOUR]
+        			float step =  obs2.valueByOccurrence(i, 4024); 
+        			obs2.valueByOccurrence(i, 5195);  			
+        			float value =  obs2.valueByOccurrence(i, param_descriptor_2_);
+        			if ( value == kFortranBufrMissingValue ) {
+        				obs2 = filter(); // We going to the next subset!
+        				break;
+        			}
+
+        			curve2[step] = value;
+        		}
+         	}
+     }
+    
+	for (map<int, float>::const_iterator step = curve1.begin(); step != curve1.end(); ++step) {
 		CustomisedPoint* point = new CustomisedPoint();		
 		point->longitude(step->first * 3600);		 
 		point->latitude(step->first * 3600);		 
-
+		DateTime date  = base_ +  Second(step->first * 3600);    
 		(*point)["step"]    = step->first * 3600;
-		(*point)["shift"] = shift_ *3600;
-		(*point)["year"]    = base_.date().year();
-		(*point)["month"]   = base_.date().month();
-		(*point)["day"]     = base_.date().day();
-		(*point)["hours"]   = base_.time().hours();
-		(*point)["minutes"] = base_.time().minutes();
-		(*point)["seconds"] = base_.time().seconds();
+		(*point)["shift"] = minstep_ - shift_ *3600;
+		(*point)["year"]    = date.date().year();
+		(*point)["month"]   = date.date().month();
+		(*point)["day"]     = date.date().day();
+		(*point)["hours"]   = date.time().hours();
+		(*point)["minutes"] = date.time().minutes();
+		(*point)["seconds"] = date.time().seconds();
 		(*point)["width"]    = 1 * 3600;
 		(*point)["control"] = step->second;	
-		map<int, vector<float> >::iterator member = members.find(step->first);
-
-		(*point)["min"]  =    	member->second.front();
-		(*point)["ten"]  =  	member->second[4];
-		(*point)["twentyfive"]  = member->second[12];
-		(*point)["median"]  = 	member->second[24];
-		(*point)["seventyfive"]  = 	 member->second[30];	
-		(*point)["ninty"] = member->second[45];
-		(*point)["max"]  = member->second.back();
-
+		(*point)["curve1"] = step->second;	
+		if ( param_descriptor_2_ ) 
+			(*point)["curve2"] = curve2[step->first];	
 		points_.push_back(point);
 	}
 }
 
 void EpsBufr::customisedPoints(const std::set<string>&, CustomisedPointsList& out)
 {
-	Log::dev() << "EpsBufr::customisedPoints-->need to be implemented" <<endl;
-	Log::dev() << *this << endl;
+	MagLog::dev() << "EpsBufr::customisedPoints-->need to be implemented" <<endl;
+	MagLog::dev() << *this << endl;
 	decode();
 
 	for (vector<CustomisedPoint*>::const_iterator point = points_.begin(); point != points_.end(); ++point)
 			out.push_back(*point);	
 }
 
-PointsHandler<UserPoint>& EpsBufr::points()
+PointsHandler& EpsBufr::points()
 {
 }
 
 void EpsBufr::visit(TextVisitor& title)
 {
-	bool long_title_ = true;
-	bool short_title_ = true;
-	if ( !long_title_ && !short_title_) return;
 	decode();
-	if ( long_title_ )
+	if (  EpsBufrAttributes::information_ )
 	{
 		ostringstream out;
 		tm convert = base_;	
@@ -180,8 +208,8 @@ void EpsBufr::visit(TextVisitor& title)
 		tfac.put(out, out, ' ', &convert, format.c_str(), format.c_str()+format.length()); 
 	    		
 		ostringstream line;
-		GeoPoint position(longitude_, latitude_);
-		line << station_name_ << "(" << position.writeLatitude() << ", " << position.writeLongitude() << ")" << endl;
+		UserPoint position(longitude_, latitude_);
+		line << station_name_ << "(" << position.asLatitude() << ", " << position.asLongitude() << ")" << endl;
 		title.add(new TextEntry(title_));
 		title.add(new TextEntry(line.str()));
 		title.add(new TextEntry(out.str()));
@@ -194,7 +222,8 @@ void EpsBufr::visit(TextVisitor& title)
 	}
 }
 
-void EpsBufr::visit(MetaDataVisitor&) {
+void EpsBufr::visit(MetaDataVisitor&)
+{
 }
 	
 void EpsBufr::print(ostream& out) const
