@@ -38,10 +38,12 @@
 #include "LegendVisitor.h"
 #include "XmlReader.h"
 #include "IntervalMap.h"
+#include "EfiLegendEntry.h"
 
 #include "Factory.h"
 #include <locale>
 #include <limits>
+#include <cfloat>
 
 using namespace magics;
 
@@ -128,27 +130,27 @@ public:
 	double eps(double value) const
 	{
 		reduction_ =  "reduced to the station height";
-		if ( height_ == int_MAX && detz_)
+		if ( height_ == INT_MAX && detz_)
 		{
 			//reduction_ =  "reduced to the " + detResolution_ + " orography";
-			double correction = ( detz_ - epsz_ ) * 0.0065;  
+			const double correction = ( detz_ - epsz_ ) * 0.0065;
 			return value - correction;
 		} 
-		if ( height_ == int_MAX )
+		if ( height_ == INT_MAX )
 		{
 			//reduction_ =  "no reduction";
 			return value;
 		}
 
-		double correction = (height_ - epsz_ ) * 0.0065;  
+		const double correction = (height_ - epsz_ ) * 0.0065;
 		return value - correction; 
 	}
 
 	double deterministic(double value) const
 	{
-		if ( height_ == int_MAX )
+		if ( height_ == INT_MAX )
 			return value;
-		double correction = (height_ - detz_ ) * 0.0065;  
+		const double correction = (height_ - detz_ ) * 0.0065;
 		return value - correction ; 
 	}
    
@@ -194,7 +196,7 @@ protected :
 string EpsParameter::height() 
 {
 	ostringstream height;
-	if ( height_ < int_MAX )
+	if ( height_ < INT_MAX )
 		height << " " << height_ << " m";
 	else 
 		height << " " << maground(detz_) << " m (" << detResolution_ << ")";
@@ -1000,7 +1002,7 @@ void EpsParameter::interpretResult(spot_query_result* result, vector<CustomisedP
 	vector<double> ypos;
 	
 	if ( result == 0 ) {
-		Log::warning() << "No Information for parameter " << name_ << endl;
+		MagLog::warning() << "No Information for parameter " << name_ << endl;
 		minx_ = 0;
 		maxx_ = 60;
 		miny_ = 0;
@@ -1035,7 +1037,7 @@ void EpsParameter::interpretResult(spot_query_result* result, vector<CustomisedP
     	len = 1024;    	
 	}	
 
-	DateTime base(date, Time(time));
+	DateTime base(date, MagTime(time));
 	DateTime next;
 	
 	base_ = string(base);
@@ -1071,23 +1073,26 @@ void EpsParameter::interpretResult(spot_query_result* result, vector<CustomisedP
     dets << "T" << (2*resol)-1;
     detResolution_ = dets.str() ;
     epsResolution_ = rs.str();
-	
-	Log::debug() << "Code ---> " << code_ << endl;
-	Log::debug() << "Resolution ---> " << resol << endl;
-	Log::debug() << "Det height ---> " << detz_ << endl;
-	Log::debug() << "Eps height ---> " << epsz_ << endl;
-	Log::debug() << "Det resoltion ---> " << detResolution_ << endl;
-	Log::debug() << "Eps resoltion ---> " << epsResolution_ << endl;
+	 
+	MagLog::debug() << "Code ---> " << code_ << endl;
+	MagLog::debug() << "Resolution ---> " << resol << endl;
+	MagLog::debug() << "Det height ---> " << detz_ << endl;
+	MagLog::debug() << "Eps height ---> " << epsz_ << endl;
+	MagLog::debug() << "Det resoltion ---> " << detResolution_ << endl;
+	MagLog::debug() << "Eps resoltion ---> " << epsResolution_ << endl;
   
-
-	for (map<int, int>::const_iterator step = rows.begin(); step != rows.end(); ++step) {
+	
+	for (map<int, int>::const_iterator step = rows.begin(); step != rows.end(); ++step) { 
 
 		CustomisedPoint* point = new CustomisedPoint();		
 		point->latitude(0);
-		point->longitude(x(step->first) * 3600);		
-		(*point)["shift"] = shift_;
+		point->longitude((x(step->first) ) * 3600);				
 		(*point)["resolution"] = resol;
-        
+		 
+		Second s = x(step->first) * 3600;
+		DateTime valid = base + s;
+		point->base(base);
+		point->valid(valid);      
 		spot_get_row(result, step->second,values,&nb);
 		
         bool ok = true;
@@ -1099,53 +1104,42 @@ void EpsParameter::interpretResult(spot_query_result* result, vector<CustomisedP
 				continue;
 			}
            
-            //Log::debug()  << step->first << " -> " << info->first << " = " << values[info->second] << "->" << (*this)(values[info->second], info->first) << endl;
+            MagLog::debug()  << step->first << " -> " << info->first << " = " << values[info->second] << "->" << (*this)(values[info->second], info->first) << endl;
 	    	(*point)["step"]    = x(step->first) * 3600;
 	    	stepvalues(x(step->first) * 3600, xpos);
-	    	(*point)["year"]    = base.date().year();
-			(*point)["month"]   = base.date().month();
-			(*point)["day"]     = base.date().day();
-			(*point)["hours"]   = base.time().hours();
-			(*point)["minutes"] = base.time().minutes();
-			(*point)["seconds"] = base.time().seconds();
+	 
 			map<string, EpsParameter::SpecificFunction>::const_iterator function = specifics_.find(type_);
 			(*point)["width"] = 1*3600;
 			if ( type_ == "eps10" ) 
 				specific10(*point);
 			if ( type_ == "eps15" ) 
 				specific15(*point);
-		  
-		    	
-            if ( !isnan(values[info->second]) ) {
-                (*point)[x(prefix_, info->first)] = (*this)(values[info->second], info->first);
-                //Log::debug() <<  x(info->first) << "----->" << (*this)(values[info->second], info->first) << endl;
-            
-                ypos.push_back((*this)(values[info->second], info->first));
-                
-                xvalues((*this)(values[info->second], info->first), xpos);
 
-	    		if ( info->first == "forecast"  && step->first > 240 )
-                    (*point)["forecast"] = double_MIN;                
+		if ( !isnan(values[info->second]) ) {
+		  (*point)[x(prefix_, info->first)] = (*this)(values[info->second], info->first);
+		  //MagLog::debug() <<  x(info->first) << "----->" << (*this)(values[info->second], info->first) << endl;
+
+		   ypos.push_back((*this)(values[info->second], info->first));
+
+		  xvalues((*this)(values[info->second], info->first), xpos);
+
+	    	  if ( info->first == "forecast"  && step->first > 240 )
+	    	   (*point)["forecast"] = DBL_MIN;
 	    	}
 	    	else {
-	    		Log::warning() << " SPOTBASE returns nan for " << info->first << " : data ignored for step " << (*point)["shift"]<< endl;
+	    		MagLog::warning() << " SPOTBASE returns nan for " << info->first << " : data ignored for step " << (*point)["shift"]<< endl;
 	    		ok = false;
 	    	}
 		}
     	if ( ok ) {
             out.push_back(point);
-    		
-    		
-    		 
     	}
 		nb=1024;
-	
 	}
 	
 	vector<double> maxs;
 	vector<double> allvalues;
-	
-	
+
     for (vector<CustomisedPoint*>::iterator point = out.begin(); point != out.end(); ++point) {
     	maxs.push_back((**point)["max"]);
     	allvalues.push_back((**point)["seventyfive"]);
@@ -1158,31 +1152,29 @@ void EpsParameter::interpretResult(spot_query_result* result, vector<CustomisedP
     		allvalues.push_back((**point)["control"]);  
     		maxs.push_back((**point)["control"]);
     	}
+    }
 
-    }
-    
     if ( threshold_ < 30) {
-    double limit = *std::max_element(allvalues.begin(), allvalues.end());
-    std::sort(maxs.begin(), maxs.end());
-    
-    double mm = maxs.back();
-    
-    double p1 = 1;
-    double p2;
-    int i = maxs.size() -1 ;
-    int n = maxs.size();
-    for ( vector<double>::reverse_iterator val = maxs.rbegin(); val != maxs.rend(); ++val) {
-    	p2 = (*val)/maxs.back();
+    	double limit = *std::max_element(allvalues.begin(), allvalues.end());
+    	std::sort(maxs.begin(), maxs.end());
+
+    	double mm = maxs.back();
+    	double p1 = 1;
+    	double p2;
+    	int i = maxs.size() -1 ;
+    	int n = maxs.size();
+    	for ( vector<double>::reverse_iterator val = maxs.rbegin(); val != maxs.rend(); ++val) {
+    		p2 = (*val)/maxs.back();
     	
-    	if ( p2 < (1 - i/n)*percentile_ && (p1-p2) > threshold_/n && *val >= limit ) {
-   			mm = *val;  			
-    	}
-   		p1 = p2;    
-		i--;
-    }
+    		if ( p2 < (1 - i/n)*percentile_ && (p1-p2) > threshold_/n && *val >= limit ) {
+    			mm = *val;
+    		}
+   			p1 = p2;
+			i--;
+  	  	}
     
-    miny_ = std::min(miny_, *std::min_element(ypos.begin(), ypos.end()));
-    maxy_ = std::max(mm, 1.);
+    	miny_ = std::min(miny_, *std::min_element(ypos.begin(), ypos.end()));
+        maxy_ = std::max(mm, 1.);
     }
     else {
     	 miny_ = std::min(miny_, *std::min_element(ypos.begin(), ypos.end()));
@@ -1227,21 +1219,18 @@ SpotDecoder::~SpotDecoder()
 void SpotDecoder::print(ostream& out)  const
 {
 	out << "SpotDecoder[";
-	
 	out << "]";
 }
 
 
 
 
-	
 
-
-PointsHandler<UserPoint>& SpotDecoder::points()
+PointsHandler& SpotDecoder::points()
 {
 	decode();
 	
-	pointsHandlers_.push_back(new PointsHandler<UserPoint>(*this));
+	pointsHandlers_.push_back(new PointsHandler(*this));
 	return *(pointsHandlers_.back());
 }
 
@@ -1250,8 +1239,8 @@ void SpotDecoder::visit(TextVisitor& title)
 	decode();
 	moreTitle(title);
 	
-	if ( parameter_ && !magCompare(parameter_->title(), "ignore")  )
-				title.add(new TextEntry(parameter_->title()));
+	if ( !magCompare(parameter_->title(), "ignore")  )
+			title.update("spot", "parameter",  parameter_->title());
 }
 
 static string build_date(const string& date, const string& time)
@@ -1261,7 +1250,7 @@ static string build_date(const string& date, const string& time)
 		out << "Unknown date";
 	}
 	else {
-		DateTime base(date, magics::Time(time));
+		DateTime base(date, MagTime(time));
 		tm convert = base;	
     	locale loc("");
     	out.imbue(loc);   
@@ -1271,74 +1260,59 @@ static string build_date(const string& date, const string& time)
 	}
 	return out.str();
 }
-    
+
 
 void EpsgramDecoder::visit(MetaDataVisitor& metadata)
 {
-	
 	decode();
 	ostringstream meta;
 	meta << "<epsgram>\n";
-	toxml(meta, "input", 1);
+	//toxml(meta, "input", 1);
 	meta << "\t<output\n";
 	meta << "\t\tparameter =\'" << parameter_->xml() << "\'\n";
 	meta << "\t\tdate =\'" << SpotDecoder::date_ << "\'\n";
 	meta << "\t\ttime =\'" << SpotDecoder::time_ << "\'\n";
 	
 	string date = build_date(SpotDecoder::date_, SpotDecoder::time_);
-	
-	
+
 	meta << "\t\tnice_date =\'" <<  date << "\'\n";
 	string landsea = (mask_>= 0.5) ? "no" : "yes";
 	meta << "\t\tsea_point =\'" <<  landsea << "\'\n";
-	
+
 	string height = "";
-	if (SpotDecoder::height_ < int_MAX  ) {
+	if (SpotDecoder::height_ < INT_MAX  ) {
 		ostringstream sh;
 		sh << SpotDecoder::height_ << "m";
 		height = sh.str();
 		meta << "\t\theight =\'" <<  height << "\'\n";
 	}
-	
-	
-	
 
-	GeoPoint position(SpotDecoder::longitude_, SpotDecoder::latitude_);
+	UserPoint position(SpotDecoder::longitude_, SpotDecoder::latitude_);
 	meta << "\t\tstation =\'" <<  SpotDecoder::station_ << "\'\n";
 	meta << "\t\tlatitude =\'" <<  grid_.y() << "\'\n";
 	meta << "\t\tlongitude =\'" <<  grid_.x() << "\'\n";
 	meta << "\t\tresolution =\'" <<  resolution_ << "\'/>\n";
 	
 	meta << "</epsgram>\n";
-	
-	
-	
-	
+
 	metadata.add(new MetaDataEntry(meta.str()));
-	
-	
-	
 }
 
+
 EpsgramDecoder::EpsgramDecoder()
 {
 }
 
-
-
-
 EpsgramDecoder::~EpsgramDecoder() 
 {
-	
 }
 
 void EpsgramDecoder::moreTitle(TextVisitor& title) const 
 {
-	if (long_title_ == false ) return;
 	
 	static map<string, string> titles;
 	if ( titles.empty() ) {
-		titles["eps10"] = "Deterministic Forecast and EPS Distribution ";
+		titles["eps10"] = "";
 		titles["eps15"] = "Extended Range Forecast based on EPS Distribution ";
 		titles["epsrose"] = "Deterministic Forecast and EPS Distribution ";
 		titles["epsplume"] = "ECMWF ensemble forecast ";
@@ -1356,33 +1330,22 @@ void EpsgramDecoder::moreTitle(TextVisitor& title) const
 	
 	
 	string height = long_title_height_ ? parameter_->height() : " ";
-	
 
-	
-
-	
 	string epsz = "";
 	ostringstream sd;
-    
 	//sd  <<  maground(epsz_) << " m";
     sd  <<  "undef" << " m";
 	epsz = sd.str();
 	
-	Log::dev() << "title-->" << long_title << "(" << long_title.size() << ")\n";
+	MagLog::dev() << "title-->" << long_title << "(" << long_title.size() << ")\n";
 
     string station = long_title;
     if ( long_title_point_ ) {
-    	station = station + string(" ")  + grid_.writeLatitude() + " " + grid_.writeLongitude() + landsea + height;	
+    	station = station + string(" ")  + grid_.asLatitude() + " " + grid_.asLongitude() + landsea + height;
     }
     
- 	title.add(new TextEntry(station));
-	
-	map<string, string>::const_iterator t = titles.find(SpotDecoder::type_);
-	if ( t != titles.end() )
- 		title.add(new TextEntry(t->second + date));
-	else 
-		title.add(new TextEntry(SpotDecoder::type_ + "title to implement " + date));
- 	title.add(new TextEntry());
+ 	title.update("spot", "base_date",  date);
+ 	title.update("spot", "station",  station);
 	
 }
 
@@ -1411,7 +1374,7 @@ void SpotDecoder::decode(bool check)
 	if (!date_.empty()) {
 		read(date_, date);	 
 		if ( date <= 0) {
-			Date d(0);
+			MagDate d(0);
 	 		d += date;
 	 		date_ = tostring(d.yyyymmdd());	
 		}
@@ -1423,17 +1386,13 @@ void SpotDecoder::decode(bool check)
     }
     catch (NoFactoryException& e) {
     	parameter_ = new EpsParameter(param_, param_title_, param_);
-    	
-        
-        
-      
-    } 
+    }
 #else
 	parameter_ = SimpleFactory<EpsParameter>::create(param_);
 	if ( !parameter_ )
 	{
 		// The data do not know how to verify the criter ....
-		Log::warning() << "Eps Data access: parameter [" << param_ << "] unknown\n";
+		MagLog::warning() << "Eps Data access: parameter [" << param_ << "] unknown\n";
 		return;
 	}
 #endif   
@@ -1451,7 +1410,7 @@ void SpotDecoder::decode(bool check)
 	spot_query_result *result = parameter_->prepare(*this, points_);
 	
 	if ( !result ) {
-		Log::error() << " No EpsData\n";
+		MagLog::error() << " No EpsData\n";
 		date_ = "";
 		time_ = "";
 		return;
@@ -1460,28 +1419,24 @@ void SpotDecoder::decode(bool check)
     double lat, lon;
  	
     spot_get_location(result, &lat, &lon);
-    
-  
+
     spot_get_index_value(result, "mask", &mask_);
     
     if ( type_ == "epsrose" ) {
     	// The mask is strange Seems to be 1 for sea and 0 fr land! 
-    	//Log::dev()<< "MASK--> " << mask_ << endl;
+    	//MagLog::dev()<< "MASK--> " << mask_ << endl;
     	if ( mask_ == 0 ) mask_ = 1;
     	else mask_ = 0;
     }
     if ( lon > 180 ) 
     	lon -= 360.;
 
-	grid_ = GeoPoint(lon, lat);
-	
-	
-	
-	
+	grid_ = UserPoint(lon, lat);
+
 	long resol; 
 	
 	read(get_spot(result,"numberOfPointsAlongAMeridian"), resol);
-    spot_get_index_value(result, "bathymetry", &bathymetry_);
+	spot_get_index_value(result, "bathymetry", &bathymetry_);
 
 	ostringstream rs, dets;
 	
@@ -1493,7 +1448,7 @@ void SpotDecoder::decode(bool check)
 	date_ = get_spot(result,"date");
 	time_ = get_spot(result,"time");
 		
-	Log::debug() << "Date /time ---> [" << date_ << ", " << time_ << "]\n";
+	MagLog::debug() << "Date /time ---> [" << date_ << ", " << time_ << "]\n";
 
   
 
@@ -1511,7 +1466,7 @@ void SpotDecoder::decode(bool check)
 
 
 
-MatrixHandler<UserPoint>& EpsgramDecoder::matrix()
+MatrixHandler& EpsgramDecoder::matrix()
 {
 	decode();
 	vector<double> values;
@@ -1519,8 +1474,8 @@ MatrixHandler<UserPoint>& EpsgramDecoder::matrix()
 	// Find the min and the max ..
 	for (vector<CustomisedPoint*>::const_iterator point = points_.begin(); point != points_.end(); ++point)
 	{
-		 Log::dev() << **point << endl;
-		 Log::dev() << **point << endl;
+		 MagLog::dev() << **point << endl;
+		 MagLog::dev() << **point << endl;
 		 for ( int s = 0; s != 50; s++) {		    
 				string key = tostring(s);
 			    map<string, double>::const_iterator member = (*point)->find(key);
@@ -1534,21 +1489,20 @@ MatrixHandler<UserPoint>& EpsgramDecoder::matrix()
 
 	double from = maground(*min_element(values.begin(), values.end())) - 1;
 	double to = maground(*max_element(values.begin(), values.end())) + 1 ;
-	Log::dev() << "min->" << *min_element(values.begin(), values.end()) << " = " << from << endl;
-	Log::dev() << "max->" << *max_element(values.begin(), values.end()) << " = " << to << endl;
+	MagLog::dev() << "min->" << *min_element(values.begin(), values.end()) << " = " << from << endl;
+	MagLog::dev() << "max->" << *max_element(values.begin(), values.end()) << " = " << to << endl;
 
 	IntervalMap<int> array;
-	double step = parameter_->plumesInterval();
+	const double step = parameter_->plumesInterval();
 	for ( double a = from; a <= to; a = a + step ) {
 			array.insert(make_pair(Interval(a, a+step), 0));
 	}
 
 
-	cout << "dim-->" << points_.size() << array.size() << endl; 
+
 	matrix_.set(array.size(), points_.size());
 	for (IntervalMap<int>::iterator interval = array.begin(); interval!= array.end(); ++interval) {
-		double row = (interval->first.max_ - interval->first.min_)/2 +interval->first.min_ ;
-		cout << "row-> " << row << endl;
+		const double row = (interval->first.max_ - interval->first.min_)/2 +interval->first.min_ ;
 		matrix_.rowsAxis().push_back(row);
 	}
 	for (vector<double>::iterator s = steps.begin(); s != steps.end(); ++s) {				
@@ -1561,61 +1515,63 @@ MatrixHandler<UserPoint>& EpsgramDecoder::matrix()
 	for (vector<CustomisedPoint*>::const_iterator point = points_.begin(); point != points_.end(); ++point)
 	{
 
-		for (IntervalMap<int>::iterator interval = array.begin(); interval!= array.end(); ++interval) {
+		for (IntervalMap<int>::iterator interval = array.begin(); interval!= array.end(); ++interval)
+		{
 			interval->second = 0;
 		}
-		for ( int s = 0; s != 50; s++) {		    
+
+		for ( int s = 0; s != 50; s++)
+		{
 			string key = tostring(s);
 
 			map<string, double>::const_iterator step = (*point)->find(key);
 			if (step != (*point)->end() ) {
 
-			  IntervalMap<int>::iterator interval = array.find(step->second);
+			  IntervalMap<int>::iterator interval = array.get(step->second);
 			  interval->second++;
-		     }
-
 			}
+		}
 		int row = 0;
-		cout << "Array ->" << array.size() << "row-->" << row  << endl;
-		for (IntervalMap<int>::iterator interval = array.begin(); interval!= array.end(); ++interval) {
-			cout << "index = " << column+row*steps.size() << interval->second << endl;
+
+		for (IntervalMap<int>::iterator interval = array.begin(); interval!= array.end(); ++interval)
+		{
+
 			matrix_[column+row*steps.size()] = interval->second*2; // in percentage!
 			row++;
 		}
 	    column++;
 	}
 
-
 	/*matrix_.set(steps_.size(), heights_.size());
 
 
 	for ( vector<double>::const_iterator height =heights_.begin(); height != heights_.end(); ++height) {
 				matrix_.rowsAxis().push_back(*height);
 	}	
-	Log::dev()<< " step-->" <<endl;
+	MagLog::dev()<< " step-->" <<endl;
 	for ( vector<double>::const_iterator step = steps_.begin(); step != steps_.end(); ++step) {
 				matrix_.columnsAxis().push_back(*step);
-				Log::dev()<< *step << " ";
+				MagLog::dev()<< *step << " ";
 			}
-			Log::dev()<< endl << "<--- step" <<endl;
+			MagLog::dev()<< endl << "<--- step" <<endl;
 			matrix_.setMapsAxis();
 			int i = 0;
 			for ( int height = 0; height < heights_.size(); height++) {
 
-			    Log::dev()<< endl <<  heights_[height] << " " << steps_.size() << endl;
+			    MagLog::dev()<< endl <<  heights_[height] << " " << steps_.size() << endl;
 				for ( int step = 0; step < steps_.size(); step++) {
 					matrix_.push_back(values_[(step*heights_.size()) + height]);
-					Log::dev()<< matrix_.back() << " ";
+					MagLog::dev()<< matrix_.back() << " ";
 				}
 			}
-				Log::debug() << matrix_ << endl;
+				MagLog::debug() << matrix_ << endl;
 	for (vector<CustomisedPoint*>::const_iterator point = points_.begin(); point != points_.end(); ++point)
 	{
-			 Log::dev() << **point << endl;
+			 MagLog::dev() << **point << endl;
 
 	}
 	*/
-	matrixHandlers_.push_back(new  MatrixHandler<UserPoint>(matrix_));
+	matrixHandlers_.push_back(new  MatrixHandler(matrix_));
 	return *(matrixHandlers_.back());
 }
 
@@ -1626,7 +1582,7 @@ spot_query* SpotDecoder::newQuery() const
 
 spot_query* SpotDecoder::newQuery(const string& database) const
 {
-	Log::dev() << "Accessing Database ->" << database << endl;
+	MagLog::dev() << "Accessing Database ->" << database << endl;
 
 	if (! spot_) {
 			
@@ -1634,26 +1590,26 @@ spot_query* SpotDecoder::newQuery(const string& database) const
 	}
 	if ( !spot_ )
 	{
-		Log::error() << database << ": Not a valid Epsgram Database\n";
+		MagLog::error() << database << ": Not a valid Epsgram Database\n";
 		throw MagicsException(database +": Not a valid Epsgram Database");
 	}
 
 	spot_query*  query  =  spot_new_query(spot_);
 	spot_set_location (query, latitude_, longitude_);
-    Log::dev()<< "lat---> " << latitude_ << "lon-->" << longitude_ << endl;
+	MagLog::dev()<< "lat---> " << latitude_ << "lon-->" << longitude_ << endl;
 	spot_set_method(query,"EPSGRAM");
 
-    //Log::dev()<< "resol---> " << resol << endl;
-    //Log::dev() << "bathymetry---> " << bathymetry_ << endl;
+    //MagLog::dev()<< "resol---> " << resol << endl;
+    //MagLog::dev() << "bathymetry---> " << bathymetry_ << endl;
     
  
      if (!date_.empty() ) {
-     	Log::dev() << "Set the date --->" << date_ << "\n";
+     	MagLog::dev() << "Set the date --->" << date_ << "\n";
      	spot_set_parameter(query,"date", date_.c_str());
      }
    
      if (!time_.empty() ) {
-     	Log::dev() << "Set the date --->" << time_ << "\n";
+     	MagLog::dev() << "Set the date --->" << time_ << "\n";
      	spot_set_parameter(query,"time", time_.c_str());
      }
      
@@ -1683,12 +1639,10 @@ void EpsgramDecoder::set()
 
 EfigramDecoder::EfigramDecoder() : first_(true)
 {
-	
 }
 
 EfigramDecoder::~EfigramDecoder()
 {
-
 }
 
 void EfigramDecoder::decode()
@@ -1727,7 +1681,7 @@ void EfigramDecoder::decode()
 		SpotDecoder::prefix_ = tostring(prefix) + "_"; 
 
 		try {
-			cout << "STEP--> " << *step << endl;
+
 			date_ = "";
 			time_ = "";
 			step_ = *step;
@@ -1756,9 +1710,9 @@ void EfigramDecoder::decode()
     			map<int, int> rows;
     			// build name of database!
     			string database = legend_database_ + "/" + *date + "/epsdb";;
-    			Log::dev() << "Accessing Database for legend-->" << database << endl;
+    			MagLog::dev() << "Accessing Database for legend-->" << database << endl;
     			spot_query* query = newQuery(database);		
-			spot_set_parameter(query,"param", codes[getParam()].c_str()); 
+			spot_set_parameter(query,"param", codes[SpotDecoder::param_].c_str()); 
 			spot_query_result* result = spot_handle_query(query);	
 
 			// Get the columns names;
@@ -1783,7 +1737,7 @@ void EfigramDecoder::decode()
 			string date = get_spot(result,"date");
 			string time = get_spot(result,"time");
 
-			DateTime base(date, Time(time));
+			DateTime base(date, MagTime(time));
 
 			ostringstream out;
 			tm convert = base;	
@@ -1800,20 +1754,20 @@ void EfigramDecoder::decode()
  				spot_get_row(result, s->second,values,&nb);	
 				for (map<string, int>::const_iterator info = columns.begin(); info != columns.end(); ++info)
 				{
-					Log::dev() << "legend_step--> " << legend_step
+					MagLog::dev() << "legend_step--> " << legend_step
 					           << "\nefi read --->" <<  s->first << " = " << values[info->second] << endl;
 					if ( s->first == legend_step )
 					{						
 						legend << "Eps t+ [" << legend_step - 24 << "-" << legend_step;			
 						legend << "h] ";		
 						efi_.push_back(int(maground(double(values[info->second]*100))));
-						Log::dev() << "efi--->" <<  values[info->second] << " " << efi_.back() << endl;
+						MagLog::dev() << "efi--->" <<  values[info->second] << " " << efi_.back() << endl;
 					}
 				}
 			}
 			legends_.push_back(legend.str());
 
-			Log::dev() << "legend ---" <<  legends_.back() << endl;
+			MagLog::dev() << "legend ---" <<  legends_.back() << endl;
 			spot_delete_result(result);
 			spot_delete_query(query);
 
@@ -1849,7 +1803,7 @@ void EfigramDecoder::decode()
     					values.push_back(parameter_->maxx_);
 		}
 		catch (...) {
-			Log::info() <<" No Clim" << endl;
+			MagLog::info() <<" No Clim" << endl;
 		}
     	
 		date_ = lastdate;
@@ -1876,176 +1830,11 @@ void EfigramDecoder::set()
 #include "Polyline.h"
 #include "PaperPoint.h"
 
-class EfiEntry : public LegendEntry
-{
-public:
-	EfiEntry(int efi) : LegendEntry(""), efi_(efi)
-	{
-		if ( efi_ > -200 ) label_ = tostring(efi_) + "%"; 
-	}
-
-	void set(const PaperPoint& point, BasicGraphicsObjectContainer& task)
-	{
-		Log::dev() << "LEGEND_STYLE --->" << style_ << endl; 
-		Log::dev() << "LEGEND_TYPE --->" << type_ << endl;
-		Log::dev() << "LEGEND_COLOUR --->" << colour_ << endl;
-		double x = point.x()-0.7;
-		double y = point.y();
-
-		double factor = 1.5;
-		double half = factor/2;
-		double width = efi_* factor/100.;
-		double height = 0.5; 
-		double top = y-height;
-		double bottom = y+height;
-
-		if ( magCompare(style_, "title") )
-		{
-			Text* efi = new Text();
-			efi->addText("EFI", textColour_, textHeight_);
-			efi->push_back(PaperPoint(x, y));	
-			task.push_back(efi);
-			Text* t1 = new Text();
-			t1->addText("-100%", textColour_, textHeight_);
-			t1->push_back(PaperPoint(x-factor, y));	
-			task.push_back(t1);
-			Text* t2 = new Text();
-			t2->addText("-50%", textColour_, textHeight_);
-			t2->push_back(PaperPoint(x-half, y));	
-			task.push_back(t2);
-			Text* t3 = new Text();
-			t3->addText("50%", textColour_, textHeight_);
-			t3->push_back(PaperPoint(x+half, y));	
-			task.push_back(t3);
-			Text* t4 = new Text();
-			t4->addText("100%", textColour_, textHeight_);
-			t4->push_back(PaperPoint(x+factor, y));	
-			task.push_back(t4);
-
-			return;              
-		}
-
-		Polyline* normal  = new Polyline();
-		normal->setColour(normalColour_);
-		normal->setThickness(normalThickness_);
-		normal->push_back(PaperPoint(x, bottom));
-		normal->push_back(PaperPoint(x, top));
-
-		Polyline* lmax  = new Polyline();
-		lmax->setColour(normalColour_);
-		lmax->setThickness(1);
-		lmax->push_back(PaperPoint(x+factor, bottom));
-		lmax->push_back(PaperPoint(x+factor, top));			
-
-		Polyline* lmin  = new Polyline();
-		lmin->setColour(normalColour_);
-		lmin->setThickness(1);
-		lmin->push_back(PaperPoint(x-factor, bottom));
-		lmin->push_back(PaperPoint(x-factor, top));
-
-		Polyline* l1  = new Polyline();
-		l1->setColour(normalColour_);
-		l1->setThickness(1);
-		l1->setLineStyle(M_DOT);
-		l1->push_back(PaperPoint(x+half, bottom));
-		l1->push_back(PaperPoint(x+half, top));			
-
-		Polyline* l2  = new Polyline();
-		l2->setColour(normalColour_);
-		l2->setThickness(1); 
-		l2->setLineStyle(M_DOT);
-		l2->push_back(PaperPoint(x-half, bottom));
-		l2->push_back(PaperPoint(x-half, top));
-
-		Polyline* lt  = new Polyline();
-		lt->setColour(normalColour_);
-		lt->setThickness(1); 
-		lt->setLineStyle(M_DOT);
-		lt->push_back(PaperPoint(x-factor, top));
-		lt->push_back(PaperPoint(x+factor, top));
-
-		Polyline* lb  = new Polyline();
-		lb->setColour(normalColour_);
-		lb->setThickness(1); 
-		lb->setLineStyle(M_DOT);
-		lb->push_back(PaperPoint(x-factor, bottom));
-		lb->push_back(PaperPoint(x+factor, bottom));
-
-		task.push_back(lmin);
-		task.push_back(lmax);
-		task.push_back(l1);
-		task.push_back(l2);
-		task.push_back(lt);
-		task.push_back(lb);
-		if (efi_ > -200)
-		{
-		  if ( magCompare(type_, "both") || 
-		     ( efi_ < 0  && magCompare(type_, "negative") ) || 
-		     ( efi_ > 0  && magCompare(type_, "positive") ) )
-		  {			
-			Polyline* box  = new Polyline();
-			box->setColour(normalColour_);
-			box->setFilled(true);
-			box->setFillColour(colour_);
-
-			FillShadingProperties* shading = new FillShadingProperties();                      
-			box->setShading(shading);
-
-
-			box->push_back(PaperPoint(x, bottom));
-			box->push_back(PaperPoint(x, top));
-			box->push_back(PaperPoint(x+width, top));
-			box->push_back(PaperPoint(x+width, bottom));
-			box->push_back(PaperPoint(x, bottom));
-			task.push_back(box);
-		  }
-		}
-		task.push_back(normal);
-            
-		if ( magCompare(style_, "bottom") )
-		{
-			Polyline* line  = new Polyline();
-			line->setColour(normalColour_);
-			line->setThickness(1); 
-			line->push_back(PaperPoint(x-factor, bottom));
-			line->push_back(PaperPoint(x+factor, bottom));
-			task.push_back(line);
-		} 
-		if ( magCompare(style_, "top") )
-		{
-			Polyline* line  = new Polyline();
-			line->setColour(normalColour_);
-			line->setThickness(1); 
-			line->push_back(PaperPoint(x-factor, top));
-			line->push_back(PaperPoint(x+factor, top));
-			task.push_back(line);
-		}
-	}
-
-        double textColumnPosition() { return 0.85; }
-	void type(const string& type)  { type_ = type; }
-        void style(const string& style)  { style_ = style; }
-	void colour(const Colour& colour) { colour_ = colour; }
-	void normalColour(const Colour& colour) { normalColour_ = colour; }
-	void normalThickness(int thickness) { normalThickness_ = thickness; }
-        void textColour(const Colour& colour) { textColour_ = colour; }
-        void textHeight(double height) { textHeight_ = height; }
-
-protected:
-	int efi_;
-	Colour colour_;
-	Colour textColour_;
-	double textHeight_;
-	string type_;
-	string style_;
-	Colour normalColour_;
-	int     normalThickness_;
-};
 
 
 void EfigramDecoder::visit(LegendVisitor& legend)
 { 
-    legend.setColumns(5);
+    legend.columns_ = 5;
     for ( vector<string>::iterator entry = legends_.begin(); entry != legends_.end(); ++entry) {
         LegendEntry *step = new LegendEntry(*entry);
         legend.add(step);
@@ -2057,8 +1846,8 @@ void EfigramDecoder::visit(LegendVisitor& legend)
     }    
     EfiEntry* efi = new EfiEntry(-9999);
     efi->style("title");
-    efi->textColour(legend.getColour());
-    efi->textHeight(legend.getHeight()); 
+    efi->textColour(*legend.colour_);
+    efi->textHeight(legend.getFont_size()); 
 	legend.add(efi);
 	
 	if ( legend_colours_.empty() ) {
@@ -2073,8 +1862,8 @@ void EfigramDecoder::visit(LegendVisitor& legend)
         if ( i == efi_.size()-1 ) {
             efi->style("bottom");
         }
-	    efi->textColour(legend.getColour());
-	    efi->textHeight(legend.getHeight()); 
+	    efi->textColour(*legend.colour_);
+	    efi->textHeight(legend.getFont_size()); 
 	    efi->colour(Colour(*colour));
 	    efi->type(legend_type_);
 	    efi->normalColour(*legend_normal_colour_);
@@ -2099,10 +1888,10 @@ void EfigramDecoder::visit(TextVisitor& title)
 
 	if ( long_title_ )
 	{
-		DateTime base(date_, Time(time_));
+		DateTime base(date_, MagTime(time_));
 		DateTime from = base + Second((step_-12)*3600);
 		DateTime to = base + Second((step_+12)*3600);
-		Log::debug() << "HELOOOO-->Date/time ---> [" << date_ << ", " << time_ << ", " << step_ << "]\n";
+		MagLog::debug() << "HELOOOO-->Date/time ---> [" << date_ << ", " << time_ << ", " << step_ << "]\n";
 		ostringstream out;
         
 		tm cfrom = from;	
@@ -2117,10 +1906,11 @@ void EfigramDecoder::visit(TextVisitor& title)
 		tfac.put(out, out, ' ', &cto, format.c_str(), format.c_str()+format.length()); 
 
 		ostringstream line;
-		GeoPoint position(SpotDecoder::longitude_, SpotDecoder::latitude_);
-		line << "Forecast and M-Climate cumulative distribution functions with EFI values at " << position.writeLatitude() << "/" << position.writeLongitude() << endl;
+		UserPoint position(SpotDecoder::longitude_, SpotDecoder::latitude_);
+
+		line << "Forecast and M-Climate cumulative distribution functions with EFI values at " << position.asLatitude() << "/" << position.asLongitude() << endl;
+
 		title.add(new TextEntry(line.str()));
-//        cout << out.str() << endl;
 		title.add(new TextEntry(out.str()));
 		title.add(new TextEntry(""));
 	}
@@ -2174,171 +1964,6 @@ void EpsParameter::setTransformation(Transformation& transformation)
 	transformation.setDataMaxY(maxy_); 
 }
     
-void EpsXmlInput::visit(Transformation& transformation)
-{
-	decode();
-	
-	transformation.setDataMinX((minstep_ - dateOffset_) * 3600, base_);
-	
-	transformation.setDataMaxX((maxstep_+dateOffset_) * 3600, base_);
-	
-	dateOffset_ = minstep_ - dateOffset_;
-	
-	transformation.setMinY(miny_);
-	transformation.setMaxY(maxy_);
-}
-		
-void EpsXmlInput::decode()
-{
-	if ( !points_.empty()) return;
-	minstep_ =  std::numeric_limits<double>::max();
-	maxstep_ =  std::numeric_limits<double>::min();
-	miny_ =  std::numeric_limits<double>::max();;
-	maxy_ =  std::numeric_limits<double>::min();;
-		
-	XmlReader parser(true);
-	XmlTree tree;	
-
-	try {
-		parser.interpret(path_, &tree);		
-		tree.visit(*this);
-	}
-	catch (MagicsException& e)
-	{
-		Log::debug() << e.what() << endl;
-	}
-}
-  	
-void EpsXmlInput::customisedPoints(const std::set<string>&, CustomisedPointsList& out)
-{
-	Log::dev() << "EpsXmlInput::customisedPoints-->need to be implemented" <<endl;
-	Log::dev() << *this << endl;
-	decode();
-
-	for (vector<CustomisedPoint*>::iterator point = points_.begin(); point != points_.end(); ++point)
-	{
-		(**point)["step"] -= (dateOffset_* 3600);	
-		out.push_back(*point);
-	}
-}
-
-
-void EpsXmlInput::visit(TextVisitor& title)
-{
-	if ( !long_title_ && !short_title_) return;
-	decode();
-	if ( long_title_ )
-	{
-		ostringstream out;
-		tm convert = base_;	
-		locale loc("");      
-		out.imbue(loc);   
-		const std::time_put<char>& tfac = use_facet<time_put<char> >(loc); 
-		string format = "Forecast %A %e %B %Y %H UTC";
-		tfac.put(out, out, ' ', &convert, format.c_str(), format.c_str()+format.length()); 
-
-		ostringstream line;
-		GeoPoint position(longitude_, latitude_);
-		line << station_ << "(" << position.writeLatitude() << ", " << position.writeLongitude() << ")" << endl;
-		title.add(new TextEntry(title_));
-		title.add(new TextEntry(line.str()));
-		title.add(new TextEntry(out.str()));
-	}
-	if ( short_title_ )
-	{
-		title.add(new TextEntry(""));
-		title.add(new TextEntry(parameter_));
-	}
-}
-
-void EpsXmlInput::visit(MetaDataVisitor&)
-{
-}	
-
-void EpsXmlInput::print(ostream& out) const
-{
-	out << "EpsXmlInput[";
-	EpsXmlInputAttributes::print(out);
-	out << "]";
-}
-
-EpsXmlInput::EpsXmlInput():dateOffset_(6)
-{
-}
-
-EpsXmlInput::~EpsXmlInput()
-{
-}
-
-#define value(s) (tonumber(node.getAttribute(s))*scaling) + offset
-
-void EpsXmlInput::visit(const XmlNode& node)
-{
-	Log::dev() << node.name() << endl;
-	if ( node.name() == "forecast" )
-	{
-		Date date(node.getAttribute("date"));
-		Time time(node.getAttribute("time"));
-		station_ = node.getAttribute("station");
-		title_ = node.getAttribute("title");
-		latitude_ = tonumber(node.getAttribute("latitude"));
-		longitude_ = tonumber(node.getAttribute("longitude"));
-		base_ = DateTime(date, time);
-		node.visit(*this);
-	}
-
-	static double scaling;
-	static double offset;
-
-	if ( node.name() == "param" && node.getAttribute("name") == param_ )
-	{
-		string s = node.getAttribute("scaling");
-		parameter_ = node.getAttribute("title");
-		scaling = (s.empty()) ? 1 :  tonumber(s);
-		string o = node.getAttribute("offset");
-		offset = (o.empty()) ? 0 :  tonumber(o);
-		
-		node.visit(*this);
-	}
-
-	if ( node.name() == "step" )
-	{
-		Log::dev() << node.getAttribute("value") << endl;
-		double step = tonumber(node.getAttribute("value"));
-		if ( step < minstep_) minstep_ = step;
-		if ( step > maxstep_) maxstep_ = step;
-		double min = value("min");
-		double max = value("max");
-		if ( min < miny_) miny_ = min;
-		if ( max > maxy_) maxy_ = max;
-		CustomisedPoint* point = new CustomisedPoint();						
-		point->longitude(step * 3600);		
-		//point->insert(make_pair("shift", dateOffset_*3600));		
-		point->insert(make_pair("shift", 0 ));
-		point->insert(make_pair("step", step* 3600));
-		point->insert(make_pair("width", 1.5 * 3600));	
-
-		point->insert(make_pair("year",  base_.date().year()));	
-		point->insert(make_pair("month",  base_.date().month()));	
-		point->insert(make_pair("day",  base_.date().day()));	
-		point->insert(make_pair("hours",  base_.time().hours()));	
-		point->insert(make_pair("minutes",  base_.time().minutes()));		    
-		point->insert(make_pair("seconds",  base_.time().seconds()));	
-		point->insert(make_pair("min", value("min")));	
-		point->insert(make_pair("max", value("max")));	
-		point->insert(make_pair("ten", value("ten")));	
-		point->insert(make_pair("twentyfive", value("twentyfive")));	
-		point->insert(make_pair("median", value("median")));	
-		point->insert(make_pair("seventyfive", value("seventyfive")));	
-		point->insert(make_pair("ninty", value("ninety")));
-		if ( !node.getAttribute("forecast").empty() )
-			point->insert(make_pair("forecast", value("forecast")));	
-		if ( !node.getAttribute("control").empty() )
-			point->insert(make_pair("control", value("control")));	
-
-		points_.push_back(point);   	
-	}
-}
 
 void EfigramDecoder::visit(Transformation& transformation)
 {	
