@@ -24,20 +24,20 @@
     Started: Tue 16-Mar-2004
     
     Changes:
-    
+
 */
 
 #ifndef GribDecoder_H
 #define GribDecoder_H
 
 #include "magics.h"
-#include "Exception.h"
+#include "MagException.h"
 
 
 #include "Decoder.h"
 #include "Data.h"
 #include "GribDecoderAttributes.h"
-#include "GeoPoint.h"
+#include "UserPoint.h"
 
 
 #include "GribLoopAttributes.h"
@@ -48,18 +48,24 @@
 
 
 namespace magics {
-	
+class GribDecoder;
+struct MatchCriteria
+{
+    MatchCriteria() {}
+    virtual ~MatchCriteria() {}
+    virtual bool verify(const GribDecoder&, const string&, const string&) { return false; }
+};
 
-class GribException : public MagicsException
+class GribMagException : public MagicsException
 {
 public:
-	 GribException( const string& why ):
+	 GribMagException( const string& why ):
 		MagicsException("Grib API error message: " +  why){}
 }; 
-class GribFileException : public MagicsException
+class GribFileMagException : public MagicsException
 {
 public:
-	GribFileException(const string& file, int index)
+	GribFileMagException(const string& file, int index)
 	{ 
 		ostringstream s;
 		s << "Grib decoding failed: field " << index << " in " << file << endl;
@@ -67,10 +73,11 @@ public:
 	}
 }; 
 
+class GribLoop;
 
 class GribDecoder: 
     public Decoder, 
-    public Data<GeoPoint>, 
+    public Data,
     public GribDecoderAttributes
 {
 public:
@@ -80,12 +87,15 @@ public:
 	// implements BaseSceneObject interface
 	virtual void set(const map<string, string>& params) { GribDecoderAttributes::set(params); }
 	virtual void set(const XmlNode& node) { GribDecoderAttributes::set(node); }
-
+	void set(const GribLoop&, int);
 	// implements Decoder interface
 	void decode();
 	void decode2D();
-	void decodeRaster();
+	void decode(const Transformation&);
+		void decode2D(const Transformation&);
+	void decodeRaster(const Transformation&);
 	void decodePoints();
+	void release();
 	
 	// Data Interface : info for the layer managment! 
 	 string layerId()  { decode(); return layerId_; }
@@ -96,65 +106,92 @@ public:
 	string title() {
 		return title_;
 	}
-	
+
+	bool getExpver() const { return expver_; }
 	void version();
 	
 	
 	// implements Decoder
 		void visit(AnimationRules&);
-		void visit(AnimationStep&);
-		void visit(MagnifierVisitor&);
+		void visit(MetaDataCollector&);
+		void visit(MagnifierCollector&);
+		void visit(ValuesCollector&);
 		
+		 const DateDescription& timeStamp();
+		 const LevelDescription& level() ;
+
 	// implements Decoder
 	void visit(TextVisitor&);
     
-	PointsHandler<GeoPoint>& points()
+	PointsHandler& points()
 	{
 		decodePoints();
-		pointsHandlers_.push_back(new PointsHandler<GeoPoint>(points_)); 
+		pointsHandlers_.push_back(new PointsHandler(points_));
 		return *(pointsHandlers_.back()); 
 	} 
-    
-	MatrixHandler<GeoPoint>& matrix()
-	{
-		decode();
-		matrixHandlers_.push_back(new MatrixHandler<GeoPoint>(*matrix_));         
-		return *(matrixHandlers_.back()); 
+	PointsHandler& points(const Transformation& transformation)
+		{
+			decodePoints();
+			pointsHandlers_.push_back(new BoxPointsHandler(points_, transformation, true));
+			return *(pointsHandlers_.back()); 
+		} 
+	PointsHandler& points(const Transformation& transformation, bool all) {
+		decodePoints();
+		pointsHandlers_.push_back(new BoxPointsHandler(points_, transformation, !all));
+		return *(pointsHandlers_.back());
 	}
 
-	MatrixHandler<GeoPoint>& xComponent()
+	MatrixHandler& matrix()
 	{
-		decode2D();
-		matrixHandlers_.push_back(new MatrixHandler<GeoPoint>(*xComponent_)); 
-		return *(matrixHandlers_.back());   
-	} 
-
-	MatrixHandler<GeoPoint>& yComponent()
+		if ( !context_) 
+			throw MagicsException("No data to display ");
+		decode();
+		matrixHandlers_.push_back(new MatrixHandler(*matrix_));
+		return *(matrixHandlers_.back()); 
+	}
+	MatrixHandler& matrix(const Transformation& transformation)
+		{
+		if ( !context_) 
+			throw MagicsException("No data to display ");
+			decode(transformation);
+			matrixHandlers_.push_back(new MatrixHandler(*matrix_));
+			return *(matrixHandlers_.back()); 
+		}
+	
+	void setPath(const string& path) { file_name_ = path; }
+	
+	RasterData&  raster(const Transformation& transformation)
 	{
-		decode2D();
-		matrixHandlers_.push_back(new MatrixHandler<GeoPoint>(*yComponent_)); 
-		return *(matrixHandlers_.back());   
-	} 
-    
-	RasterData<GeoPoint>&  raster()
-	{
-		decodeRaster();
+		decodeRaster(transformation);
 		return raster_; 
 	} 
+	void customisedPoints(const Transformation& t, const std::set<string>& n, CustomisedPointsList& out, bool all)
+	{
 
-	
+	}
 
-	virtual void open();
+	void customisedPoints(const AutomaticThinningMethod&, const Transformation&, const std::set<string>&, CustomisedPointsList&);
+	void customisedPoints(const BasicThinningMethod&, const Transformation&, const std::set<string>&, CustomisedPointsList& );
+
+
+	virtual void open(bool sendMsg = true);
 	virtual void openFirstComponent();
 	virtual void openSecondComponent();
+	virtual void openThirdComponent();
+	virtual void readColourComponent();
 
 	grib_handle*  id() const { if (!handle_) const_cast<GribDecoder*>(this)->decode(); return handle_; }
-	long      getLong(const string&) const;
-	string getString(const string&) const;
-	double    getDouble(const string&) const;
-    void    setDouble(const string&, double) const;
+	long      getLong(const string&,   bool warnIfKeyAbsent = true) const;
+	string    getString(const string&, bool warnIfKeyAbsent = true) const;
+	double    getDouble(const string&, bool warnIfKeyAbsent = true) const;
+    void      setDouble(const string&, double) const;
+
+    string    getstring(const string&, bool warnIfKeyAbsent = true) const;
+
 	void      read(Matrix **matrix);
-	bool     id(const string&) const;
+	void      read(Matrix **matrix, const Transformation&);
+	bool      id(const string&) const;
+
 
 	static grib_context* gribContext();
 	static void releaseContext() {  
@@ -163,6 +200,7 @@ public:
 		context_ = 0;
 	}
 	grib_handle*  handle() const { return handle_; }
+        void initInfo();
 
 protected:
 	//! Method to print string about this class on to a stream of type ostream (virtual).
@@ -171,8 +209,14 @@ protected:
 	mutable Matrix*     matrix_;
 	mutable Matrix*     xComponent_;
 	mutable Matrix*     yComponent_;     
-	mutable RasterData<GeoPoint> raster_;
-	mutable PointsList<GeoPoint> points_;
+	mutable Matrix*     colourComponent_;
+	mutable RasterData raster_;
+	mutable PointsList points_;
+
+	int internalIndex_;
+	GribInterpretor* interpretor_;
+
+
 
 	
  
@@ -181,7 +225,7 @@ protected:
 	
 	string title_;
 	static int count_;
-     
+	friend class GribInterpretor;
 private:
 	//! Copy constructor - No copy allowed
 	GribDecoder(const GribDecoder&);
@@ -198,14 +242,32 @@ private:
 class GribEntryDecoder: public GribDecoder
 {
 public:
-	GribEntryDecoder(grib_handle* handle) { handle_ = handle; }
+	GribEntryDecoder(grib_handle* handle) {
+		handle_ = handle;
+		handle1_ = 0;
+		handle2_ = 0;
+		handle3_ = 0;
+		Data::dimension_ = 1;
+
+	}
 	GribEntryDecoder(grib_handle* handle1, grib_handle* handle2) { 
 		handle_ = handle1;
 		handle1_ = handle1; 
-		handle2_ = handle2; }
+		handle2_ = handle2;
+		handle3_ = 0;
+
+		Data::dimension_ = 2;
+	}
+	GribEntryDecoder(grib_handle* handle1, grib_handle* handle2, grib_handle* handle3) {
+		handle_ = handle1;
+		handle1_ = handle1;
+		handle2_ = handle2;
+		handle3_ = handle3;
+		Data::dimension_ = 3;
+	}
 	~GribEntryDecoder() {}
 
-	void open() { assert(handle_); }
+	void open(bool sendMsg = true) { assert(handle_); }
 	
 	void openFirstComponent() { 
 		assert(handle1_);
@@ -216,17 +278,30 @@ public:
 		assert(handle2_);
 		handle_ = handle2_;
 	}
+	void openThirdComponent() {
+			assert(handle3_);
+			handle_ = handle3_;
+		}
+	void readColourComponent() {
+		if ( handle3_ ) {
+			handle_ = handle3_;
+			read(&colourComponent_);
+		}
+		else
+			colourComponent_ = 0;
+	}
 	
 protected:
 	grib_handle*  handle1_;
 	grib_handle*  handle2_;
+	grib_handle*  handle3_;
 };
 
-class GribLoop : public GribLoopAttributes, public DataLoop<GeoPoint>, public MetviewIcon
+class GribLoop : public GribLoopAttributes, public DataLoop
 {
 public:
 	GribLoop();
-	virtual ~GribLoop() {}
+	virtual ~GribLoop();
 	
 	
 	
@@ -234,7 +309,7 @@ public:
 	void set(const XmlNode& node) { GribLoopAttributes::set(node); }
 	
 	
-	Data<GeoPoint>* current();
+	Data* current();
 	bool         hasMore();
 	void         next();
 	void setToFirst();
@@ -244,16 +319,18 @@ public:
 
 protected:
 	virtual void print(ostream&) const; 
-	GribDecoder* grib_;
-	
-	vector<int>::iterator current_;
-	vector<int>::iterator current1_;
-	vector<int>::iterator current2_;
+	vector<GribDecoder*> gribs_;
+    GribDecoder* currentgrib_;
+	friend class GribDecoder;
+	vector<int>::iterator currentDim_;
+	vector<int>::iterator currentPos_;
+
 	
 	FILE* file_;
 	static map<string, string>  ids_;
 	static int  index_;
-	
+	int uniqueId_;
+	int counter_;
 	
 private:
 	//! Copy constructor - No copy allowed
