@@ -43,8 +43,10 @@ public:
 	virtual ~GribRegularInterpretor();
 	
 	void interpretAsMatrix(const GribDecoder&, Matrix**) const; 
+	virtual double XResolution(const GribDecoder& grib) const { return  longitudeIncrement(grib); }
 	
-	virtual void interpretAsRaster(const GribDecoder&, RasterData<GeoPoint>&) const;
+	void interpretAsMatrix(const GribDecoder&, Matrix**, const Transformation&) const; 
+	virtual void interpretAsRaster(const GribDecoder&, RasterData&, const Transformation&) const;
     virtual void latitudes(const GribDecoder&, vector<double>&) const;
     virtual double longitudeIncrement(const GribDecoder&) const;
 protected:
@@ -70,8 +72,10 @@ public:
 	GribReducedGaussianInterpretor() {}
 	virtual ~GribReducedGaussianInterpretor() {}
 	
+	virtual double XResolution(const GribDecoder& grib) const;
 	void interpretAsMatrix(const GribDecoder&, Matrix**) const; 
-	void interpretAsRaster(const GribDecoder&, Matrix**) const; 
+	void interpretAsMatrix(const GribDecoder&, Matrix**, const Transformation&) const; 
+	void interpretAsRaster(const GribDecoder&, RasterData&, const Transformation&) const;
 	
 	
 protected:
@@ -98,7 +102,7 @@ public:
 	virtual ~GribReducedLatLonInterpretor() {}
 	
 	void interpretAsMatrix(const GribDecoder&, Matrix**) const; 
-	
+	virtual double XResolution(const GribDecoder& grib) const;
 	
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
@@ -120,18 +124,25 @@ private:
 class GribRotatedInterpretor: public GribRegularInterpretor {
 
 public:
-	GribRotatedInterpretor() {}
+	GribRotatedInterpretor(): original_(false) {}
 	virtual ~GribRotatedInterpretor() {}
 	
-	GeoPoint unrotate(double lat, double lon) const;
+	pair<double, double> unrotate(double lat, double lon) const;
+	pair<double, double> rotate(double lat, double lon) const;
+	void raw(const GribDecoder&, const Transformation&, const string&, CustomisedPointsList&) const;
+	void keepOriginal(bool original) { original_ = original; }
 	void interpretAsMatrix(const GribDecoder&, Matrix**) const; 
-	
+	void interpret2D(double&, double&, double&, double&) const;
+
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
 	 mutable double southPoleLat_;
 	 mutable double southPoleLon_;
-   
+	 mutable double angle_;
+	 mutable bool uvRelativeToGrid_;
+
+	 bool original_;
 };
 
 class GribLambertAzimutalInterpretor: public GribRegularInterpretor {
@@ -140,7 +151,7 @@ public:
 	GribLambertAzimutalInterpretor() {}
 	virtual ~GribLambertAzimutalInterpretor() {}
 	
-	GeoPoint unrotate(double lat, double lon) const;
+	UserPoint unrotate(double lat, double lon) const;
 	void interpretAsMatrix(const GribDecoder&, Matrix**) const; 
 	
 protected:
