@@ -18,7 +18,7 @@
 
 /*! \file GribSatelliteInterpretor.cc
     \brief Implementation of the Template class GribSatelliteInterpretor.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Mon 18-Apr-2005
 
@@ -26,7 +26,7 @@
 */
 
 
-
+#include "CustomisedPoint.h"
 #include "GribSatelliteInterpretor.h"
 #include "GribDecoder.h"
 #include "RasterData.h"
@@ -53,9 +53,9 @@ void GribSatelliteInterpretor::print(ostream& out)  const
 }
 
 
-void GribSatelliteInterpretor::interpretAsRaster(const GribDecoder& grib, RasterData<GeoPoint>& raster) const
+void GribSatelliteInterpretor::interpretAsRaster(const GribDecoder& grib, RasterData& raster,const Transformation&) const
 {
-     Log::dev() << "GribSatelliteInterpretor::interpretAsRaster" << "\n";
+     MagLog::dev() << "GribSatelliteInterpretor::interpretAsRaster" << "\n";
 /*
      \param datum:    planimetric datum
      \param offx:     x offset
@@ -80,8 +80,8 @@ void GribSatelliteInterpretor::interpretAsRaster(const GribDecoder& grib, Raster
      double offy = grib.getDouble("yCoordinateOfOriginOfSectorImage");
      double prj  = 2*asin(1/altitude)/grib.getDouble("dx");
      double pri  = 2*asin(1/altitude)/grib.getDouble("dy");
-     double pjs  = grib.getDouble("xCoordinateOfSubSatellitePoint");
-     double pis  = grib.getDouble("yCoordinateOfSubSatellitePoint");
+     double pjs  = grib.getDouble("XpInGridLengths");
+     double pis  = grib.getDouble("YpInGridLengths");
      double lao  = grib.getDouble("latitudeOfSubSatellitePointInDegrees") *TeCDR;
      double lono = grib.getDouble("longitudeOfSubSatellitePointInDegrees")*TeCDR;
      double prs  = altitude * TeEARTHRADIUS;
