@@ -34,11 +34,11 @@
 
 #include "ImportActionAttributes.h"
 #include "Data.h"
-
+#include "TextVisitor.h"
 namespace magics {
 
-template <class P>
-class ImportAction: public ImportActionAttributes, public Data<P> {
+
+class ImportAction: public ImportActionAttributes, public Data {
 
 public:
 	ImportAction() {}
@@ -47,12 +47,50 @@ public:
 	
 	
 	string path() { return path_; }
-	void visit(AnimationStep& step) { step.update("grib", "shortName", "W200");  step.update("grib", "valid-date", valid_time_); }
 	
+	void visit(MetaDataCollector& collector) {
+		MetviewIcon::visit(collector);
 
+		MetaDataCollector::iterator format = collector.find("MV_Format");
+		if ( format != collector.end() ) {
+			format->second = service_;
+		}
+
+		MetaDataCollector::iterator name = collector.find("shortName");
+		if ( name != collector.end() ) {
+			name->second = short_name_;
+		}
+	 }
+
+	 void visit(TextVisitor& text)
+	 {
+		MetaDataCollector collector;
+		collector["service_name"]="";
+		collector["service_title"]="";
+		collector["title"]="";
+		collector["date"]="";
+		collector["time"]="";
+
+		MetviewIcon::visit(collector);
+
+		if(collector["service_name"] == "WMS")
+		{
+			string s="WMS: " + collector["title"] + " " + collector["date"] + " " + collector["time"];
+
+			text.addAutomaticTitle(s);
+		}	
+		else if(!url_.empty() && !service_.empty() )
+		{
+			text.addAutomaticTitle(service_ + " :" + url_);
+		} 
+	 }
+	   virtual void customisedPoints(const Transformation&, const std::set<string>&, CustomisedPointsList&, bool ) {}
+	    //! Method to access the data as a list of points
+	     // needMissing : if true the list will contain all the points (If they are outside the area: They will be flagged missing)
+	     virtual PointsHandler& points(const Transformation&, bool) { assert(false); }
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
-	 virtual void print(ostream& out) const { out << "ImportAction<P>["; ImportActionAttributes::print(out); out << "]"; }  
+	 virtual void print(ostream& out) const { out << "ImportAction["; ImportActionAttributes::print(out); out << "]"; }
 
 private:
     //! Copy constructor - No copy allowed
@@ -62,13 +100,12 @@ private:
     
 // -- Friends
     //! Overloaded << operator to call print().
-	friend ostream& operator<<(ostream& s,const ImportAction<P>& p)
+	friend ostream& operator<<(ostream& s,const ImportAction& p)
 		{ p.print(s); return s; }
-
 };
 
-template <class P>
-class ImportLoop : public DataLoop<P>, vector<Data<P> * >
+
+class ImportLoop : public DataLoop, public vector<Data * >
 {
 public :
 	ImportLoop() {}
@@ -78,15 +115,13 @@ public :
 	virtual void set(LayerNode&) {}
 	
 	virtual void setToFirst() { current_ = this->begin(); }
-	virtual Data<P>* current() { return *current_; }
+	virtual Data* current() { return *current_; }
 	virtual bool         hasMore() { return current_!= this->end(); }
 	virtual void         next() { ++current_; }
-	void add(Data<P>* data) { push_back(data); } 
+	void add(Data* data) { push_back(data); }
 protected:
-	typename vector<Data<P> * >::iterator current_;
+	vector<Data * >::iterator current_;
 };
-
 } // namespace magics
 
-
 #endif
