@@ -37,8 +37,8 @@
 
 namespace magics {
 
-template <class P>
-class InputMatrix: public Data<P>, public InputMatrixAttributes {
+
+class InputMatrix: public Data, public InputMatrixAttributes {
 
 public:
 	InputMatrix();
@@ -47,10 +47,20 @@ public:
 	virtual void set(const map<string, string>& map) { InputMatrixAttributes::set(map); }
 	virtual void set(const XmlNode& node) { InputMatrixAttributes::set(node); }
 
-	virtual MatrixHandler<P>& matrix();
-	virtual MatrixHandler<P>& xComponent();
-	virtual MatrixHandler<P>& yComponent();
+    void getReady(const Transformation& transformation);
 
+	virtual MatrixHandler& matrix();
+	virtual MatrixHandler& xComponent();
+	virtual MatrixHandler& yComponent();
+	void prepareComponents();
+	pair<double, double> sd2uv(double s, double d);
+	virtual PointsHandler& points(const Transformation&);
+	
+	void customisedPoints(const BasicThinningMethod&, const Transformation&, const std::set<string>&, CustomisedPointsList& );
+	void customisedPoints(const Transformation& t, const std::set<string>& n, CustomisedPointsList& out, bool all)
+	{}
+	PointsHandler& points(const Transformation& t, bool) { return points(t); }
+	
 	bool defined()
 	{
 	   return !field_.empty() || 
@@ -61,25 +71,33 @@ public:
 	           );
 	}
 	void filter(Matrix&);
+	void release();
 
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
+     Matrix* matrix_;
+     Matrix* u_;
+     Matrix* v_;
+     Matrix* speed_;
+     Matrix* direction_;
+     
+     friend class InputMatrixRegularInterpretor;
 
 private:
     //! Copy constructor - No copy allowed
-	InputMatrix(const InputMatrix<P>&);
+	InputMatrix(const InputMatrix&);
     //! Overloaded << operator to copy - No copy allowed
-	InputMatrix& operator=(const InputMatrix<P>&);
+	InputMatrix& operator=(const InputMatrix&);
 
 // -- Friends
     //! Overloaded << operator to call print().
-	friend ostream& operator<<(ostream& s,const InputMatrix<P>& p)
+	friend ostream& operator<<(ostream& s,const InputMatrix& p)
 		{ p.print(s); return s; }
 
 };
 } // namespace magics
 
-#include "InputMatrix.cc"
+
 
 #endif
