@@ -35,19 +35,21 @@
 
 #include "Decoder.h"
 #include "NetcdfDecoderAttributes.h"
-#include "GeoPoint.h"
+#include "UserPoint.h"
 #include "Data.h"
 #include "PointsHandler.h"
 #include "Factory.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 
 
 
 namespace magics {
 
-template <class P>
+class Transformation;
+
+
 class NetcdfDecoder: public Decoder,
-                     public Data<P>,           
+                     public Data,
                      public NetcdfDecoderAttributes
 {
 
@@ -61,20 +63,26 @@ public:
     virtual void set(const XmlNode& node) { NetcdfDecoderAttributes::set(node); }
     virtual void visit(MagnifierVisitor&); 
     
-  
+    virtual PointsHandler& points(const Transformation&, bool);
 
-    PointsHandler<P>& points() 
-    { 
-    	(*interpretor_).interpretAsPoints(points_); 
-    	this->pointsHandlers_.push_back(new PointsHandler<P>(points_));
-    	return *(this->pointsHandlers_.back()); 
+    void customisedPoints(const Transformation& t, const std::set<string>& n, CustomisedPointsList& out, bool all)
+       {
+       	customisedPoints(t, n, out);
+       }
+
+    void getReady(const Transformation& transformation)
+    {
+    	(*interpretor_).getReady(transformation);
     }
 
-
-    MatrixHandler<P>& matrix() {
-        Log::dev() << "NetcdfDecoder::matrix! " << "\n";
+    void visit(Transformation& transformation) {
+    	 (*interpretor_).visit(transformation);    
+    }
+    
+    MatrixHandler& matrix() {
+        MagLog::dev() << "NetcdfDecoder::matrix! " << "\n";
         (*interpretor_).interpretAsMatrix(&data_); 
-        this->matrixHandlers_.push_back(new MatrixHandler<P>(*data_)); 
+        this->matrixHandlers_.push_back(new MatrixHandler(*data_));
         return *(this->matrixHandlers_.back());
     } 
     
@@ -87,9 +95,9 @@ public:
     
     void visit(AnimationStep& step) {
     	   try {
-    		   MatrixHandler<P>& data = matrix() ; 
+    		   MatrixHandler& data = matrix() ;
     		   // Information about contains...
-    		   	Log::dev() << "Netcdf::visit(AnimationRules&) --> " << endl;
+    		   	MagLog::dev() << "Netcdf::visit(AnimationRules&) --> " << endl;
     		   	
     		
     		   	step.xResolution(abs(data.XResolution()));
@@ -104,13 +112,15 @@ public:
     	         
     	
     }
-    
+    void visit(MetaDataCollector&);
+    void visit(ValuesCollector&);
+    void visit(TextVisitor&);
     
 protected:
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
 	 
-	 PointsList<P> points_;
+	 PointsList points_;
 	 Matrix*     data_;
 
 private:
@@ -125,19 +135,19 @@ private:
 		{ p.print(s); return s; }
 };
 
-template < class P>
-class NetcdfLoop : public DataLoop<P>
+
+class NetcdfLoop : public DataLoop
 {
 public:
-	NetcdfLoop(NetcdfDecoder<P>* netcdf): netcdf_(netcdf) {}
+	NetcdfLoop(NetcdfDecoder* netcdf): netcdf_(netcdf) {}
 	virtual ~NetcdfLoop() {}
 	void set(const map<string, string>& map) { } // NetcdfLoopAttributes::set(map); }
 	void set(const XmlNode& node) { } // NetcdfLoopAttributes::set(node); }
 	
-    void set(NetcdfDecoder<P>* netcdf) { netcdf_ = netcdf; }
+    void set(NetcdfDecoder* netcdf) { netcdf_ = netcdf; }
 	
-	Data<P>* current() {
-		NetcdfDecoder<P>* current = netcdf_;
+	Data* current() {
+		NetcdfDecoder* current = netcdf_;
 		netcdf_ = 0;
 		return current;
 	}
@@ -147,12 +157,12 @@ public:
 	void         next() { netcdf_ = 0; }
 	
 
-	
+   void visit(Transformation& transformation) { if ( netcdf_ ) netcdf_->visit(transformation); }
 	
 
 protected:
 	virtual void print(ostream&) const {}
-	NetcdfDecoder<P>* netcdf_;
+	NetcdfDecoder* netcdf_;
 	
 	
 	
@@ -161,17 +171,16 @@ private:
 	//! Copy constructor - No copy allowed
 	NetcdfLoop(const NetcdfLoop&);
 	//! Overloaded << operator to copy - No copy allowed
-	NetcdfLoop<P>& operator=(const NetcdfLoop<P>&);
+	NetcdfLoop& operator=(const NetcdfLoop&);
 
 // -- Friends
 	//! Overloaded << operator to call print().
-	friend ostream& operator<<(ostream& s,const NetcdfLoop<P>& p)
+	friend ostream& operator<<(ostream& s,const NetcdfLoop& p)
 		{ p.print(s); return s; }
 	
 };
 
 
-#include "NetcdfDecoder.cc"
 
 } // namespace magics
 #endif
