@@ -31,6 +31,7 @@
 #include "Factory.h"
 #include "Netcdf.h"
 #include <limits>
+#include "Layer.h"
 
 using namespace magics;
 
@@ -41,58 +42,76 @@ NetcdfGeoMatrixInterpretor::NetcdfGeoMatrixInterpretor()
 NetcdfGeoMatrixInterpretor::~NetcdfGeoMatrixInterpretor() 
 {}
 
+
+
 void NetcdfGeoMatrixInterpretor::interpretAsMatrix(Matrix** data)
 {
 	if ( *data ) return;
 	
-	Matrix* matrix = new Matrix();
-	
-	*data = matrix;
-	matrix->missing(std::numeric_limits<double>::max());
-    matrix->missing(100000002004087734272.0);
+	matrix_ = new Matrix();
+	matrix_->akimaEnabled();
+	*data = matrix_;
+
+   
 	Netcdf netcdf(path_);
+
+	double missing_value = netcdf.getDefaultMissing(field_);
+
+	missing_value = netcdf.getVariableAttribute(field_, missing_attribute_, missing_value);
+
+
 	// get the data ...
 	try
 	{
 		map<string, string> first, last;
 		setDimensions(dimension_, first, last);
-       
-		netcdf.get(longitude_, matrix->columnsAxis(), first, last);
-		netcdf.get(latitude_, matrix->rowsAxis(), first, last);
+
+		netcdf.get(longitude_, matrix_->columnsAxis(), first, last);
+		netcdf.get(latitude_, matrix_->rowsAxis(), first, last);
 		
+
+
+		matrix_->missing(missing_value);
+
 		if  ( magCompare(primary_index_, "latitude") ) {
-			// WE reserve the matrix since we are used to lat/lon matrix!
+			// WE reserve the matrix_ since we are used to lat/lon matrix_!
 			vector<double> data;
 			netcdf.get(field_, data, first, last);
-			int columns =  matrix->columnsAxis().size();
-			int rows =  matrix->rowsAxis().size();
+			int columns =  matrix_->columnsAxis().size();
+			int rows =  matrix_->rowsAxis().size();
  
 		    for (int lon = 0; lon < columns; lon++)
 			     for (int lat = 0; lat < rows; lat++) {
-					
-			    	matrix->push_back(data[lon + lat*rows]);
+			    	matrix_->push_back(data[lon + lat*rows]);
 			     }
 		}
-		else 		
-			netcdf.get(field_, *matrix, first, last);
-		
-		Log::debug() << "data[" << matrix->size() << ", " << scaling_ << ", " << offset_ << "\n";
+		else 	{
+			vector<double> data;	
+			netcdf.get(field_, data, first, last);
+			int i = 0;
+			for (vector<double>::iterator d = data.begin(); d != data.end(); ++d ) {
+				if ( !isnan(*d) ) {
+					matrix_->push_back(*d);
+				}
+				else 
+					matrix_->push_back(missing_value);
+			   i++;
+			}
+		}
 
-		matrix->multiply(scaling_);
-		matrix->plus(offset_);
+
+		matrix_->multiply(scaling_);
+		matrix_->plus(offset_);
         
-		Log::debug() << "data[" << matrix->size() << ", " << scaling_ << ", " << offset_ << "\n";
+
         
-		
+	        matrix_->setMapsAxis();
         
-        matrix->setMapsAxis();
-        
-		Log::dev() << matrix << "\n";
+
 	}
-    
 	catch (MagicsException& e)
 	{
-		Log::error() << e << "\n";
+		MagLog::error() << e << "\n";
 	}
 }
 
@@ -107,7 +126,7 @@ void NetcdfGeoMatrixInterpretor::print(ostream& out)  const
 	out << "]";
 }
 
-void NetcdfGeoMatrixInterpretor::interpretAsPoints(PointsList<GeoPoint>& list) 
+void NetcdfGeoMatrixInterpretor::interpretAsPoints(PointsList& list)
 {
 	Netcdf netcdf(path_);
 	
@@ -119,31 +138,78 @@ void NetcdfGeoMatrixInterpretor::interpretAsPoints(PointsList<GeoPoint>& list)
 		vector<double> values;
 		map<string, string> first, last;
 		setDimensions(dimension_, first, last);
+		double missing_value = missing(netcdf);
 		
 		netcdf.get(field_, values, first, last);
 		netcdf.get(longitude_, longitudes, first, last);
 		netcdf.get(latitude_, latitudes, first, last);
-		int val = 0;
+		unsigned int val = 0;
 		
-		for (int  lat  =0 ; lat < latitudes.size(); lat+=latitude_sample_) {
-			for ( int lon = 0; lon < longitudes.size(); lon+=longitude_sample_) {
+		for (unsigned int  lat  =0 ; lat < latitudes.size(); lat+=latitude_sample_) {
+			for ( unsigned int lon = 0; lon < longitudes.size(); lon+=longitude_sample_) {
 				val = (lat* longitudes.size() + lon);
 				if (val >= values.size() ) return;
 				if ( values[val] < suppress_below_ ) continue;
 				if ( values[val] > suppress_above_ ) continue;
-				if ( same(values[val], missing_) ) continue;
-				list.push_back(GeoPoint(longitudes[lon],latitudes[lat],values[val]));		
-				
-				
+				if ( same(values[val], missing_value ) ) continue;
+				list.push_back(new UserPoint(longitudes[lon], latitudes[lat], (values[val]*scaling_) + offset_));
 			}
 		}
-				
-       Log::dev()<< "everything ok" << endl;
+ 		MagLog::dev()<< "everything ok" << endl;
 	}
-    
 	catch (MagicsException& e)
 	{
-		Log::error() << e << "\n";
+		MagLog::error() << e << "\n";
 	}
 }
 
+void NetcdfGeoMatrixInterpretor::statsData(map<string,vector<double> >& stats)
+{
+	if(matrix_)
+	{
+	  	for(unsigned int i=0; i < matrix_->size(); i++)
+		{  
+			if(matrix_->at(i) != matrix_->missing())
+			{
+			  	stats["value"].push_back(matrix_->at(i));
+			}
+		}
+	}	
+}
+
+void NetcdfGeoMatrixInterpretor::visit(MetaDataCollector& mdc)
+{
+	mdc["_datatype"]="NetCDF_geomatrix";
+	mdc["path"]=path_;
+	mdc["latitude"]=latitude_;
+	mdc["longitude"]=longitude_;
+	mdc["value"]=field_;	
+	mdc["statsType"]="scalar";
+	
+	Netcdf nc(path_);
+	
+	string attrKey;
+	string attrVal;		
+
+	//Value attributes
+	getAttributes(nc,field_,attrKey,attrVal);		
+	if(!attrKey.empty())
+	{
+		mdc["valueAttrKey"]=attrKey;
+		mdc["valueAttrValue"]=attrVal;
+	}
+}
+
+void NetcdfGeoMatrixInterpretor::visit(ValuesCollector& vcp,PointsList&)
+{	
+	vcp.setCollected(true);
+	
+  	assert(matrix_); 
+  	const Transformation& transformation = vcp.transformation();
+  	MatrixHandler* box =  transformation.prepareData(*matrix_);
+	for (ValuesCollector::iterator point =  vcp.begin(); point != vcp.end(); ++point) {
+		point->push_back(new ValuesCollectorData(point->x(),point->y(),box->nearest(point->y(), point->x()),-1.));
+	}
+}
+
+
