@@ -47,12 +47,12 @@ public:
 	virtual ~NetcdfGeoMatrixInterpretor();
     
     void set(const map<string, string>& params) { 
-        Log::debug() << "NetcdfGeoMatrixInterpretor::set(params)" << "\n";
+        MagLog::debug() << "NetcdfGeoMatrixInterpretor::set(params)" << "\n";
         NetcdfInterpretorAttributes::set(params); 
         NetcdfGeoMatrixInterpretorAttributes::set(params);
     }
     void set(const XmlNode& node) { 
-        Log::debug() << "NetcdfGeoMatrixInterpretor::set(params)" << "\n";
+        MagLog::debug() << "NetcdfGeoMatrixInterpretor::set(params)" << "\n";
         NetcdfInterpretorAttributes::set(node); 
         XmlNode netcdf = node;
         netcdf.name("netcdf");
@@ -69,14 +69,16 @@ public:
     	NetcdfGeoMatrixInterpretorAttributes::copy(other); 
     }
     virtual void interpretAsMatrix(Matrix**);
-    void interpretAsPoints(PointsList<GeoPoint>&);
+    void interpretAsPoints(PointsList&);
+
+    virtual void statsData(map<string,vector<double> >&);
+    virtual void visit(MetaDataCollector&);
+    virtual void visit(ValuesCollector&,PointsList&); 
 
-    
-    
-   
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
+	 Matrix* matrix_;
     
 private:
     //! Copy constructor - No copy allowed
