@@ -31,9 +31,9 @@
 #define NetcdfInterpretor_H
 
 #include "magics.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 #include "Factory.h"
-#include "GeoPoint.h"
+#include "UserPoint.h"
 #include "UserPoint.h"
 #include "MatrixHandler.h"
 #include "NetcdfInterpretorAttributes.h"
@@ -41,29 +41,31 @@
 
 namespace magics {
 
-template <class P> class RasterData;
+class RasterData;
 
+class Netcdf;
 class NetcdfInterpretor: public NetcdfInterpretorAttributes {
 
 public:
 	NetcdfInterpretor();
 	virtual ~NetcdfInterpretor();
+	virtual void visit(Transformation&) {}
+	virtual void getReady(const Transformation&) {}
     virtual void interpretAsMatrix(Matrix**)
-    	{ Log::dev() << "Method  NetcdfInterpretor::interpretAsMatrix() --> Not yet implemented.\n"; }
+    	{ MagLog::dev() << "Method  NetcdfInterpretor::interpretAsMatrix() --> Not yet implemented.\n"; }
     virtual void interpretAsVectors(Matrix**, Matrix**) 
-    	{ Log::dev() << "Method  NetcdfInterpretor::interpretAsVectors() --> Not yet implemented.\n"; }
-    virtual void interpretAsRaster(RasterData<GeoPoint>&) 
-    	{ Log::dev() << "Method  NetcdfInterpretor::interpretAsRaster() --> Not yet implemented.\n"; }
-    virtual void interpretAsPoints(PointsList<GeoPoint>&)  
-    	{ Log::dev() << "Method  NetcdfInterpretor::interpretAsPoints() --> Not yet implemented.\n"; }
+    	{ MagLog::dev() << "Method  NetcdfInterpretor::interpretAsVectors() --> Not yet implemented.\n"; }
+    virtual void interpretAsRaster(RasterData&)
+    	{ MagLog::dev() << "Method  NetcdfInterpretor::interpretAsRaster() --> Not yet implemented.\n"; }
+    virtual void interpretAsPoints(PointsList&)
+    	{ MagLog::dev() << "Method  NetcdfInterpretor::interpretAsPoints() --> Not yet implemented.\n"; }
     virtual void customisedPoints(const std::set<string>&, CustomisedPointsList&)  
-        	{ Log::dev() << "Method  NetcdfInterpretor::customisedPoints() --> Not yet implemented.\n"; }
+        	{ MagLog::dev() << "Method  NetcdfInterpretor::customisedPoints() --> Not yet implemented.\n"; }
     virtual void customisedPoints(const Transformation&, const std::set<string>&, CustomisedPointsList&)  
-            	{ Log::dev() << "Method  NetcdfInterpretor::customisedPoints() --> Not yet implemented.\n"; }
-    virtual void interpretAsRaster(RasterData<UserPoint>&) 
-    	{ Log::dev() << "Method  NetcdfInterpretor::interpretAsRaster() --> Not yet implemented.\n"; }
-    virtual void interpretAsPoints(PointsList<UserPoint>&)  
-    	{ Log::dev() << "Method  NetcdfInterpretor::interpretAsPoints() --> Not yet implemented.\n"; }
+            	{ MagLog::dev() << "Method  NetcdfInterpretor::customisedPoints() --> Not yet implemented.\n"; }
+
+    virtual void interpretAsPoints(PointsList& points, const Transformation&)
+    	{ interpretAsPoints(points);}
     virtual void set(const map<string, string>& params) { NetcdfInterpretorAttributes::set(params); }
     virtual void set(const XmlNode& node) { NetcdfInterpretorAttributes::set(node); }
      virtual bool accept(const string& node) { return NetcdfInterpretorAttributes::accept(node); }
@@ -72,13 +74,19 @@ public:
     	object->copy(*this);
     	return object;
     }
-
+    
+    virtual void statsData(map<string,vector<double> >&) {}
+    virtual void visit(MetaDataCollector&) {}
+    virtual void visit(ValuesCollector&,PointsList&) {};
+    virtual void visit(TextVisitor&) {}    
+    double missing(Netcdf&) const;
 
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
      void setDimensions(const stringarray&, map<string, string>& first, map<string, string>& last);
-
+     void getAttributes(Netcdf&,const string&,string&,string&);
+	
 private:
     //! Copy constructor - No copy allowed
 	NetcdfInterpretor(const NetcdfInterpretor&);
@@ -91,7 +99,7 @@ private:
 		{ p.print(s); return s; }
 };
 template<>
-class Translator<string, NetcdfInterpretor> { 
+class MagTranslator<string, NetcdfInterpretor> { 
 public:
 	NetcdfInterpretor* operator()(const string& val ) {
 		 return SimpleObjectMaker<NetcdfInterpretor>::create(val);
@@ -103,6 +111,7 @@ public:
 		ParameterManager::update(param, object);
 		return object;
 	}
+
 };
 
 } // namespace magics
