@@ -48,14 +48,14 @@ public:
     
     void set(const map<string, string>& params)
     { 
-        Log::debug() << "NetcdfMatrixInterpretor::set(params)" << "\n";
+        MagLog::debug() << "NetcdfMatrixInterpretor::set(params)" << "\n";
         NetcdfInterpretorAttributes::set(params); 
         NetcdfMatrixInterpretorAttributes::set(params);
     }
     
     void set(const XmlNode& node)
     { 
-        Log::debug() << "NetcdfMatrixInterpretor::set(params)" << "\n";
+        MagLog::debug() << "NetcdfMatrixInterpretor::set(params)" << "\n";
         XmlNode netcdf = node;
          netcdf.name("netcdf");
          NetcdfInterpretorAttributes::set(netcdf); 
@@ -83,12 +83,35 @@ public:
     	NetcdfMatrixInterpretorAttributes::copy(*this); 
     }
     virtual void interpretAsMatrix(Matrix**);
-   
+    virtual void interpretAsPoints(PointsList& points, const Transformation&);
+    virtual void visit(Transformation&);
+    virtual void getReady(const Transformation&);
+    virtual void visit(MetaDataCollector&);
+    virtual void visit(ValuesCollector&,PointsList&);
+    virtual void visit(TextVisitor&); 
+    virtual void statsData(map<string,vector<double> >&);
+	virtual void x();
+	virtual void y(); 
+	bool cf_date(Netcdf& netcdf, const string&, const string&, string&, vector<double>&, vector<double>&);
+	//return true, if the the data is date CF-compliant and the date axis has been set
+	bool reference_date(Netcdf& netcdf, const string&, const string&, string&, vector<double>&, vector<double>&);
+	//return true, if the the data is Metview-date compliant and the date axis has been set
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
-     
-    
+	 Matrix* matrix_;
+	 vector<double> columns_;
+	 vector<double> rows_;
+	 vector<double> dateRows_; // Used when dealing with date dimensions to store the original values
+	 vector<double> dateColumns_; // Used when dealing with date dimensions to store the original values
+	 string baseDateX_;
+	 string baseDateY_;
+	 string geoMinX_;
+	 string geoMaxX_;
+	 string geoMinY_;
+	 string geoMaxY_;
+	 string refDateX_;
+	 string refDateY_;
 private:
     //! Copy constructor - No copy allowed
 	NetcdfMatrixInterpretor(const NetcdfMatrixInterpretor&);
