@@ -32,16 +32,16 @@
 
 #include "magics.h"
 
-#include "NetcdfGeoMatrixInterpretorAttributes.h"
+#include "NetcdfOrcaInterpretorAttributes.h"
 #include "NetcdfInterpretor.h"
 #include "Matrix.h"
-#include "Exception.h"
+#include "MagException.h"
 #include "VectorOfPointers.h"
 
 namespace magics {
 
 class NetcdfOrcaInterpretor: 
-        public NetcdfGeoMatrixInterpretorAttributes, 
+        public NetcdfOrcaInterpretorAttributes,
 		public NetcdfInterpretor {
 
 public:
@@ -49,12 +49,12 @@ public:
 	virtual ~NetcdfOrcaInterpretor();
     
     void set(const map<string, string>& params) { 
-        Log::debug() << "NetcdfOrcaInterpretor::set(params)" << "\n";
+        MagLog::debug() << "NetcdfOrcaInterpretor::set(params)" << "\n";
         NetcdfInterpretorAttributes::set(params); 
-        NetcdfGeoMatrixInterpretorAttributes::set(params);
+        NetcdfOrcaInterpretorAttributes::set(params);
     }
 
-    virtual void interpretAsPoints(PointsList<GeoPoint>&);
+    virtual void interpretAsPoints(PointsList&);
     virtual void interpretAsMatrix(Matrix**);
    
     
