@@ -50,7 +50,7 @@ NetcdfVectorInterpretor::~NetcdfVectorInterpretor()
 
 void NetcdfVectorInterpretor::customisedPoints(const std::set<string>&, CustomisedPointsList&)
 {
-	Log::dev() << "NetcdfVectorInterpretor::customisedPoints()--> to be implemented!" << endl;
+	MagLog::dev() << "NetcdfVectorInterpretor::customisedPoints()--> to be implemented!" << endl;
 }
 
 
@@ -100,7 +100,7 @@ void NetcdfGeoVectorInterpretor::customisedPoints(const Transformation& transfor
 			
 			//If the lat-lon units is specified as "radians" convert lat-lon 
 			//to degrees. By default the units are sipposed to be "degrees"
-			const char *units;
+			const char *units = 0;
 			if ( magCompare(netcdf.getVariableAttribute(latitude_,"units",units), "radians") )
 			{			
 				while ( lat!= latitudes.end()) {
@@ -136,10 +136,9 @@ void NetcdfGeoVectorInterpretor::customisedPoints(const Transformation& transfor
 			}
 	}
 	catch (MagicsException& e)
-		{
-			Log::error() << e << "\n";
-		}
-			
+	{
+		MagLog::error() << e << "\n";
+	}
 }
 
 
@@ -188,7 +187,7 @@ void NetcdfGeoPolarMatrixInterpretor::customisedPoints(const Transformation&, co
 			
 			//If the lat-lon units is specified as "radians" convert lat-lon 
 			//to degrees. By default the units are sipposed to be "degrees"
-			const char *units;
+			const char *units = 0;
 			if ( magCompare(netcdf.getVariableAttribute(latitude_,"units",units), "radians") )
 			{			
 				while ( lat!= latitudes.end()) {
@@ -223,14 +222,12 @@ void NetcdfGeoPolarMatrixInterpretor::customisedPoints(const Transformation&, co
 			*/
 	}
 	catch (MagicsException& e)
-		{
-			Log::error() << e << "\n";
-		}
-			
+	{
+		MagLog::error() << e << "\n";
+	}
 }
 
 
-
 /*!
  Class information are given to the output-stream.
 */		
