@@ -35,12 +35,12 @@
 #include <TitleField.h>
 #include <Data.h>
 #include <Factory.h>
-#include <Title.h>
 #include <VectorOfPointers.h>
 
 #include <stack>
 
 namespace magics {
+class GribDecoder;
 
 
 class SimpleStringFormat
@@ -70,16 +70,14 @@ protected:
 	}
 };
 
-template <class D>
+
 class TitleFieldHandler 
 {
 public :
     TitleFieldHandler() {}
     virtual ~TitleFieldHandler() {};
-    virtual void operator()(TitleField& field, ostream& out, const D&)
-    {
-        out << field; 
-    }
+    virtual void operator()(TitleField& field, vector<string>& lines, const GribDecoder&) { assert (false); }
+
 };
 
 
@@ -96,7 +94,7 @@ public:
 	void add(TitleField* field) { template_.push_back(field); }
 
 	template <class D>
-	static void title(ostream& out,  const D& data)
+	static void title(vector<string>& out,  const D& data)
 	{
 		if ( !singleton_) new TitleTemplate();
 		(*singleton_)(out, data);
@@ -111,15 +109,19 @@ public:
 	}
 
 	template <class D>
-	void operator()(ostream& out, const D& data) const
+	void operator()(vector<string>& out, const D& data) const
 	{
 		if ( !verify(data) ) return;
+		vector<string> lines;
+		lines.push_back("");
 		for (const_iterator child = begin(); child != end(); ++child)
 		{
 			if ((*child)->verify(data))
 			{
-				(*(*child))(out, data);
-				return;
+				(*(*child))(lines, data);
+				for (vector<string>::iterator line = lines.begin(); line != lines.end(); ++line)
+					if ( !line->empty() ) out.push_back(*line);
+				return; 
 			}
 		}
 
@@ -128,57 +130,39 @@ public:
 #ifdef MAGICS_EXCEPTION
 			try
 			{
-				auto_ptr<TitleFieldHandler<D> > object(SimpleObjectMaker<TitleFieldHandler<D> >::create((*entry)->name()));
-				(*object)(*(*entry), out, data);
+				auto_ptr<TitleFieldHandler > object(SimpleObjectMaker<TitleFieldHandler >::create((*entry)->name()));
+				(*object)(*(*entry), lines, data);
 			}
 			catch (NoFactoryException& e)
 			{
 				// The data do not know how to verify the criter ....
-				Log::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
-				(*(*entry))(out);
+				MagLog::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
+				(*(*entry))(lines);
 			}
 #else
-			TitleFieldHandler<D>* object = SimpleObjectMaker<TitleFieldHandler<D> >::create((*entry)->name());
+			TitleFieldHandler* object = SimpleObjectMaker<TitleFieldHandler >::create((*entry)->name());
 			if (object)
 			{
-				(*object)(*(*entry), out, data);
+				(*object)(*(*entry), lines, data);
 			}
 			else
 			{
 				// The data do not know how to verify the criter ....
-				Log::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
-				(*(*entry))(out);
+				MagLog::debug() << "Can Not Create the TitleFieldHandler for " << (*entry)->name() << "\n";
+				(*(*entry))(lines);
 			}
 #endif
 		}
+		
+		for (vector<string>::iterator line = lines.begin(); line != lines.end(); ++line)
+			if ( !line->empty()) out.push_back(*line);
 	}
 
 /*!
  \todo code needed if MAGICS_EXCEPTION are disabled!
 */
-	template <class D>
-	bool verify(const D& data) const
-	{
-		for (map<string, string>::const_iterator criter = criteria_.begin(); criter != criteria_.end(); ++criter)
-		{
-//#ifdef MAGICS_EXCEPTION
-			try{
-				Log::debug() << "Try  to create the MatchCriteria for " << criter->first << "\n";
-				auto_ptr<MatchCriteria<D> >  object(SimpleObjectMaker<MatchCriteria<D> >::create(criter->first));
-				Log::debug() << "Found the MatchCriteria for " << criter->first << "\n";
-				if (!(*object).verify(data, criter->first, criter->second)) return false;
-			}
-			catch (NoFactoryException& e) 
-			{ // The data do not know how to verify the criter ....
-				Log::warning() << "Can Not Create the MatchCriteria for " << criter->first << "\n";
-				return false; 
-			}
-//#else
-			
-//#endif
-		}
-		return true;
-	}
+
+	bool verify(const GribDecoder& data) const;
 
 	map<string, string>& criteria() { return criteria_; }
 
