@@ -18,7 +18,7 @@
 
 /*! \file BaseDriver.cc
     \brief Implementation of driver base class.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Jan 2004
 
@@ -38,13 +38,12 @@
 #include <Image.h>
 #include <Symbol.h>
 #include <PaperPoint.h>
-//#include <InteractiveSet.h>
-//#include <FrameLoop.h>
 #include "AnimationRules.h"
 
 #include <BaseDriverSymbols.h>
 #include <BaseDriverWind.h>
 #include <BaseDriverImages.h>
+#include <BaseDriverBinaryReader.cc>
 
 #include <System.h>
 
@@ -58,21 +57,21 @@ int BaseDriver::numFiles_ = 0;
 /*!
   Main job is to initialise most variables and read font information
 */
-BaseDriver::BaseDriver():currentPage_(-1),fileName_(""),currentLayer_(""),currentGroup_(""),
+BaseDriver::BaseDriver():currentPage_(-1),fileName_(""),currentLayer_(""),
         currentLineType_(M_SOLID),currentLineWidth_(-1), currentLineStyle_(1),currentColour_(Colour("white")),
 	coordRatioX_(1),coordRatioY_(1),
-	lastAreaHeightPercentage_(0.),lastAreaWidthPercentage_(0.),
-	newPage_(true),newLayout_(true),external_(false),polylineAntialiasing_(false),disabled_(false),alphaEnabled_(false),obs_distance_(-1.),
+//	lastAreaHeightPercentage_(0.),lastAreaWidthPercentage_(0.),
+	newPage_(true),//newLayout_(true), // external_(false),
+	disabled_(false),alphaEnabled_(false),
 	indexHatch_(0),currentShading_(M_SH_NONE),cmScale_(1.),
-	xDeviceLength_(Translator<double, double>().magics("SUPER_PAGE_X_LENGTH")),
-	yDeviceLength_(Translator<double, double>().magics("SUPER_PAGE_Y_LENGTH"))
+	xDeviceLength_(MagTranslator<double, double>().magics("SUPER_PAGE_X_LENGTH")),
+	yDeviceLength_(MagTranslator<double, double>().magics("SUPER_PAGE_Y_LENGTH")),obs_distance_(-1.)
+//	polylineAntialiasing_(false),
 {
-	readFonts();
-
         //width in pixels  = cm * 2.54 * dpi
         //height in pixels = cm * 2.54 * dpi
 /*        const string size = getSize();
-        const float cmdpi = 2.54 * 72.;
+        const MFloat cmdpi = 2.54 * 72.;
         if(!magCompare(size,"a4"))
         {
             string::size_type posX = size.find_first_of("x");
@@ -82,8 +81,8 @@ BaseDriver::BaseDriver():currentPage_(-1),fileName_(""),currentLayer_(""),curren
                 string::size_type posCM = size.find_first_of("c",posX+1);
                 if(posCM != string::npos) // assume CM
                 {
-                   float xcm=0.;
-                   float ycm=0.;
+                   MFloat xcm=0.;
+                   MFloat ycm=0.;
                    istringstream is(first);
                    is>>xcm;
                    xDeviceLength_ = static_cast<int>(xcm * cmdpi);
@@ -114,18 +113,18 @@ BaseDriver::~BaseDriver()
 }
 
 
-//! Method set solid fill shading properties
-/*
+/*! \brief Method set solid fill shading properties
+
  \sa renderSimplePolygon
 */
-void BaseDriver::shade(const FillShadingProperties& properties)  const 
+void BaseDriver::shade(const FillShadingProperties& properties)  const
 {
 	currentShading_ = M_SH_SOLID;
 	currentShadingProperties_ = &properties;
 }
 
-//! Method set hatch fill shading properties
-/*
+/*! \brief Method set hatch fill shading properties
+
  \sa renderSimplePolygon
 */
 void BaseDriver::shade(const HatchShadingProperties& properties) const
@@ -134,8 +133,8 @@ void BaseDriver::shade(const HatchShadingProperties& properties) const
 	currentShadingProperties_ = &properties;
 }
 
-//! Method set dot fill shading properties
-/*
+/*! \brief Method set dot fill shading properties
+
  \sa renderSimplePolygon
 */
 void BaseDriver::shade(const DotShadingProperties& properties)   const
@@ -149,17 +148,17 @@ void BaseDriver::shade(const DotShadingProperties& properties)   const
 */
 void BaseDriver::printOutputName(const std::string & str) const
 {
-	if(getFilelist())
+	if(filelist_)
 	{
 		const SystemInfo info;
 		fstream fs;
 		if(numFiles_ == 0)
 		{
-			fs.open(getFilelist_name().c_str(),fstream::out);
+			fs.open(filelist_name_.c_str(),fstream::out);
 			fs << "# "<< getMagicsVersionString()<< " "<<info.getHostName() <<" " << info.getTime()<< "\n";
 		}
 		else
-			fs.open(getFilelist_name().c_str(),fstream::out | fstream::app);
+			fs.open(filelist_name_.c_str(),fstream::out | fstream::app);
 
 		fs << info.getTime()<< " "<<str<< "\n";
 		fs.close();
@@ -170,8 +169,7 @@ void BaseDriver::printOutputName(const std::string & str) const
 
 //! Method to read font information
 /*!
- A hash table is produce to map font names to file names of TTF
- files.
+ A hash table is produce to map font names to file names of TTF files.
 
   \todo make this a singleton!?
 */
@@ -208,7 +206,7 @@ void BaseDriver::readFonts() const
 		}
 		psfile.close();
 	}
-	else Log::fatal() << "BaseDriver::readFonts() --> Cannot open Font file " << s << " ! No text can be plotted.\n";
+	else MagLog::fatal() << "BaseDriver::readFonts() --> Cannot open Font file " << s << " ! No text can be plotted.\n";
 }
 
 /*! \brief formulating a filename
@@ -222,31 +220,29 @@ void BaseDriver::readFonts() const
 string BaseDriver::getFileName(const string &extension, const unsigned int no) const
 {
 	string ext = "."+extension;
-	bool set      = false;
 	bool full     = false;
 	bool legacy   = false;
 
-	string filename = getName();
-	if(filename=="")
+	string filename = name_;
+	if(filename.empty())
 	{
-	  filename = getFullname();
-	  if(filename!="") full = true;
+	  filename = fullname_;
+	  if(!filename.empty()) full = true;
 	}
-	if(filename=="")
+	if(filename.empty())
 	{
-	  filename = getLegacyname();
-	  if(filename!="") legacy = true;
+	  filename = legacyname_;
+	  if(!filename.empty()) legacy = true;
 	}
-	if(filename=="")
+	if(filename.empty())
 	{
 	  filename = extension;
-	  if(filename!="") set = true;
 	}
 
 	// name stays the same as given
 	if( no==0 && (full || (legacy && extension=="ps")) ) return filename;
 	if( no==0 )  { filename += ext; return filename;}
-	if( external_ )   { filename += "_legend"+ext; return filename;}
+//	if( external_ )   { filename += "_legend"+ext; return filename;}
 
 
 	// if nothing is set
@@ -264,16 +260,16 @@ string BaseDriver::getFileName(const string &extension, const unsigned int no) c
 		}
 	}
 
-	const int numberWidth = getNumberingwidth();
+	const int numberWidth = numberingwidth_;
 
-	if( (no > 1) || getFirstnumber() || legacy )
+	if( (no > 1) || firstnumber_ || legacy )
 	{
 		char *ostr;
 		if(numberWidth==4)      {ostr=new char[5]; sprintf(ostr,"%04u",no);}
 		else if(numberWidth==3) {ostr=new char[4]; sprintf(ostr,"%03u",no);}
 		else if(numberWidth==2) {ostr=new char[3]; sprintf(ostr,"%02u",no);}
 		else                    {ostr=new char[2]; sprintf(ostr,"%u",no);}
-		filename += getSeparator()+ostr+ext;
+		filename += separator_+ostr+ext;
 		delete [] ostr;
 	}
 	else // if first page no number
@@ -284,231 +280,6 @@ string BaseDriver::getFileName(const string &extension, const unsigned int no) c
 	return filename;
 }
 
-/*!
-
-  Overwritten in KMLDriver
-  
-  \sa Layer
-
-*/
-MAGICS_NO_EXPORT void BaseDriver::redisplay(const Layer& layer) const 
-{
-	if(!magCompare(currentLayer_,layer.name()))
-	{
-		currentLayer_=layer.name();
-		
-		newLayer(layer);
-		layer.visit(*this);
-		closeLayer(layer);
-	}
-	else
-		layer.visit(*this);
-}
-
-
-/*!
- \brief Method to read and execute binary file
- 
- This method is implmented for performance in Metview 4 and WREP
- 
-*/
-void BaseDriver::redisplay(const BinaryObject& binary) const 
-{
-  ifstream in(binary.getPath().c_str());
-  char mag [6];
-  in.read((char *)(&mag), 6);
-  if(strncmp( "MAGICS", mag,6) !=0 )
-  {
-	  Log::error() << "Magics number of binary file " <<binary.getPath().c_str()<<" is wrong!\n"
-			<< "   Please check endiness and if you gave the write filename."<< endl;
-	  return;
-  }
-  int version;
-  in.read((char *)(&version), sizeof(int));
-  cout << "BINARY version = "<<version<< endl;
-
-  char c;
-  while( in.get(c) )
-  {
-	int n;
-	switch(c) {
-	case 'T':
-        	float x;
-        	float y;
-        	float s;
-		int len;
-		{ 
-			in.read((char *)(&x), sizeof(float));
-			in.read((char *)(&y), sizeof(float));
-			Text text;
-			PaperPoint pp(x,y);
-			text.push_back(pp);
-
-			float angle;
-			in.read((char *)(&angle), sizeof(float));
-			in.read((char *)(&s), sizeof(float));
-			bool bl;
-			in.read((char *)(&bl), sizeof(bool));
-			enum Justification horizontal;
-			enum VerticalAlign vertical;
-			in.read((char *)(&horizontal), sizeof(enum Justification));
-			in.read((char *)(&vertical), sizeof(enum VerticalAlign));
-			in.read((char *)(&len), sizeof(int));
-			char *tex = new char[len+1];
-			in.read(tex, sizeof(char)*len);
-			tex[len]='\0';
-
-			string str(tex);
-			text.addText(str,Colour("black"),s);
-			text.setBlanking(bl);
-			text.setJustification(horizontal);
-			text.setVerticalAlign(vertical);
-			text.setAngle(angle);
-			renderText(text);
-			delete [] tex;
-		}
-        	break;
-
-	case 'A':
-        	in.read((char *)(&n), sizeof(int));
-        	{
-			Arrow arrow;
-			double sc;
-			in.read((char *)(&sc), sizeof(double));
-			arrow.setScale(sc);
-			int index;
-			in.read((char *)(&index), sizeof(int));
-			arrow.setHeadIndex(index);
-			LineStyle ls;
-			in.read((char *)(&ls), sizeof(LineStyle));
-			arrow.setStyle(ls);
-			ArrowPosition ap;
-			in.read((char *)(&ap), sizeof(ArrowPosition));
-			arrow.setArrowPosition(ap);
-			int hi;
-			in.read((char *)(&hi), sizeof(int));
-			arrow.setHeadIndex(hi);
-			double hr;
-			in.read((char *)(&hr), sizeof(double));
-			arrow.setHeadRatio(hr);
-			float r,g,b;
-			in.read((char *)(&r), sizeof(float));
-			in.read((char *)(&g), sizeof(float));
-			in.read((char *)(&b), sizeof(float));
-			setNewColour(Colour(r,g,b));
-
-			for(int pts=0;pts<n;pts++)
-			{
-			  double x,y;
-			  in.read((char *)(&x), sizeof(double));
-			  in.read((char *)(&y), sizeof(double));
-			  PaperPoint p;
-			  in.read((char *)(&p), sizeof(PaperPoint));
-			  ArrowPoint ap(x, y, p);
-			  arrow.push_back(ap);
-			} 
-			renderWindArrow(arrow);
-		}
-		break;
-	case 'H':
-		in.read((char *)(&n), sizeof(int));
-		{
-			float *x = new float[n];
-			float *y = new float[n];
-			in.read((char *)(x), sizeof(float)*n);
-			in.read((char *)(y), sizeof(float)*n);
-			renderPolyline( n, x, y);
-			delete [] x;
-			delete [] y;
-		}
-		break;
-
-	case 'B':
-		in.read((char *)(&n), sizeof(int));
-		{
-			float *x = new float[n];
-			float *y = new float[n];
-			in.read((char *)(x), sizeof(float)*n);
-			in.read((char *)(y), sizeof(float)*n);
-			renderPolyline2( n, x, y);
-			delete [] x;
-			delete [] y;
-		}
-		break;
-
-	case 'S':
-        	in.read((char *)(&n), sizeof(int));
-        	{
-			float *x = new float[n];
-			float *y = new float[n];
-			in.read((char *)(x), sizeof(float)*n);
-			in.read((char *)(y), sizeof(float)*n);
-			renderSimplePolygon( n, x, y);
-			delete [] x;
-			delete [] y;
-		}
-        	break;
-
-	case 'W':
-        	float width;
-        	in.read((char *)(&width), sizeof(float));
-        	setNewLineWidth(width);
-        	break;
-
-	case 'L':
-        	float w;
-        	LineStyle linestyle;
-        	in.read((char *)(&linestyle), sizeof(LineStyle));
-        	in.read((char *)(&w), sizeof(float));
-        	setLineParameters(linestyle, w);
-        	break;
-
-	case 'C':
-        	float r,g,b;
-        	in.read((char *)(&r), sizeof(float));
-        	in.read((char *)(&g), sizeof(float));
-        	in.read((char *)(&b), sizeof(float));
-        	setNewColour(Colour(r,g,b));
-        	break;
-	case 'N':
-	//		  cout<<"N"<< endl;
-        	startPage();
-        	break;
-	case 'E':
-	//		  cout<<"E"<< endl;
-        	endPage();
-        	break;
-	case 'P':
-		{
-			Layout l;
-			double x,y,w,h,minx,miny,maxx,maxy;
-			in.read((char *)(&x), sizeof(double));
-			in.read((char *)(&y), sizeof(double));
-			in.read((char *)(&w), sizeof(double));
-			in.read((char *)(&h), sizeof(double));
-			in.read((char *)(&minx), sizeof(double));
-			in.read((char *)(&miny), sizeof(double));
-			in.read((char *)(&maxx), sizeof(double));
-			in.read((char *)(&maxy), sizeof(double));
-			l.x(x);
-			l.y(y);
-			l.width(w);
-			l.height(h);
-			l.minX(minx);
-			l.minY(miny);
-			l.maxX(maxx);
-			l.maxY(maxy);
-			project(l);
-		}
-        	break;
-	case 'U':
-        	unproject();
-        	break;
-	}
-   }
-   in.close();
-}
-
 
 /*!
   \brief processing layouts
@@ -517,77 +288,64 @@ void BaseDriver::redisplay(const BinaryObject& binary) const
 */
 MAGICS_NO_EXPORT void BaseDriver::redisplay(const Layout& layout) const 
 {
-	if(layout.childOfRoot()) startPage();  // New page detected
-
 	project(layout);
 	staLayouts_.push(&layout);
 	layout.visit(*this);  // visit this layout!
 	unproject();
-
-	if(layout.childOfRoot())
-	{
-		endPage();
-		vecPoints_.clear();
-	}
 }
 
-/*!
-  \brief processing standalone layouts
-  This methods processes the Layout objects. It will mostly requires a new page!
-  Only the KMLDriver will have the legend included.
-  \sa Layout UnLayout
-*/
-void BaseDriver::redisplay(const StandaloneLayout& layout) const
-{
-	external_=true;
-	startAdditionalPage(layout);  // New page detected
-
-	project(layout);
-	staLayouts_.push(&layout);
-	layout.visit(*this);  // visit this layout!
-	unproject();
-
-	endAdditionalPage();
-        external_=false;
-}
 
 MAGICS_NO_EXPORT void BaseDriver::redisplay(const RootLayout& root) const
 {
-	root.visit(*this);  // visit this ROOT layout! 
+	root.visit(*this);  // visit this ROOT layout!
+}
+
+MAGICS_NO_EXPORT void BaseDriver::redisplay(const LegendLayout& legend) const
+{
+	redisplay((const Layout&) legend);
+}
+
+MAGICS_NO_EXPORT void BaseDriver::redisplay(const SceneLayout& scene) const
+{
+	redisplay((const Layout&) scene);
+}
+
+MAGICS_NO_EXPORT void BaseDriver::redisplay(const StartPage& ) const
+{
+   startPage();
+}
+
+MAGICS_NO_EXPORT void BaseDriver::redisplay(const EndPage& ) const
+{
+   endPage();
+   vecPoints_.clear();
+}
+
+
+void BaseDriver::redisplay(const PolylineSet& line) const
+{
+      line.visit(*this);
 }
 
 /*!
- \brief Method to trigger rendering of animation step tree
-
- \sa OpenGLDriver 
+  \brief Decision how to stroke/fill simple polygon
+  
+  Overwritten in SVGDriver::redisplay(const Polyline& line) const
 */
-void BaseDriver::redisplay(const StepRenderer& step) const
-{
-	Log::dev() << "BaseDriver::redisplay(const StepRenderer& step)"  << step << "--> needs implementing!" << endl;
-}
-
 void BaseDriver::redisplay(const Polyline& line) const
 {
-	if(line.isFilled())
-	{
-		setNewColour(line.getFillColour());
-		vector<PaperPoint> points;
-		const unsigned int size = line.size();
-		for(unsigned int i=0;i<size;i++) points.push_back(line[i]);
-		line.getShading()->draw(*this);
-		renderSimplePolygon(points);
-	}
-	printLine(line);
+	if(line.isFilled())   renderSimplePolygon(line);
+	if(line.isStroked())  printLine(line);
 }
 
 void BaseDriver::redisplay(const Arrow& arrow) const
 {
-	renderWindArrow(arrow);	
+	renderWindArrow(arrow);
 }
 
 void BaseDriver::redisplay(const Flag& flag) const
 {
-	renderWindFlag(flag);	
+	renderWindFlag(flag);
 }
 
 void BaseDriver::redisplay(const ImportObject& object) const
@@ -644,7 +402,7 @@ void BaseDriver::redisplay(const FlagItem& text, const ComplexSymbol& symbol) co
 
  \sa printLine
 */
-double BaseDriver::LSF(float *x,float *y, int i0) const
+double BaseDriver::LSF(MFloat *x,MFloat *y, int i0) const
 {
 	double angle = 0.;
 	double x_sum = 0.;
@@ -670,7 +428,7 @@ double BaseDriver::LSF(float *x,float *y, int i0) const
 		sxy  += (xi*yi);
 	}
 	if(sxx != 0) angle = atan2( (sxy/sxx) ,1.);
-	else Log::debug() << "BaseDriver: Devision through zero prevented in calculation of Label angle!" << endl;
+	else MagLog::debug() << "BaseDriver: Devision through zero prevented in calculation of Label angle!" << endl;
 
 	return angle;
 }
@@ -678,7 +436,6 @@ double BaseDriver::LSF(float *x,float *y, int i0) const
 /*!
   \brief Method plotting Polylines with Labels.
 
-
   \sa renderPolyline renderText
 
   \todo location memory for labels.
@@ -688,13 +445,13 @@ void BaseDriver::printLine(const Polyline &line) const
     const unsigned long n = line.size();
     if(n < 1) return;
 
-    float *x      = new float[n];
-    float *y      = new float[n];
+    MFloat *x      = new MFloat[n];
+    MFloat *y      = new MFloat[n];
 
     for(unsigned long s=0;s<n;s++)
     {
-        x[s] = line[s].x();
-        y[s] = line[s].y();
+        x[s] = line.get(s).x();
+        y[s] = line.get(s).y();
     }
 
     // render line - driver specific part
@@ -703,62 +460,75 @@ void BaseDriver::printLine(const Polyline &line) const
 	setNewColour(line.getColour());
 	currentLineStyle_ = setLineParameters(line.getLineStyle(),line.getThickness());
 
-	if(line.getAntiAliasing()) polylineAntialiasing_=true;
+//	if(line.getAntiAliasing()) polylineAntialiasing_=true;
 	renderPolyline(n, x, y);
-	polylineAntialiasing_=false;
+//	polylineAntialiasing_=false;
+    
+	Polyline::Holes::const_iterator h  = line.beginHoles();
+	Polyline::Holes::const_iterator he = line.endHoles();
+
+	for (; h != he; ++h)
+	{
+		vector<double> xx;
+		vector<double> yy;
+		line.hole(h,xx,yy);
+		const unsigned long nn = xx.size();
+		MFloat *nx      = new MFloat[nn];
+		MFloat *ny      = new MFloat[nn];
+		for(unsigned int is=0;is<nn;is++)
+		{
+		  nx[is] = xx[is];
+		  ny[is] = yy[is];
+		}
+		renderPolyline(nn, nx, ny);
+		delete [] nx;
+		delete [] ny;
+	}
     }
 
-    const unsigned int minimum_points_for_labelling = 16;  // must be at least -15 : see function LSF above
+    const unsigned int minimum_points_for_labelling = 32;  // must be at least -15 : see function LSF above
 
     if (line.getLabel().isVisible() && line.getLabel().getText() != "" && (n > minimum_points_for_labelling))
     {
-    //const unsigned int minimum_points_for_labelling = 16;  // must be at least -15 : see function LSF above
+	assert(staLayouts_.empty() == false);
 
-    // Calculate how far apart the labels should be.
-    // Our algorithm is to take the average of the width and height and divide by 2.
-    // This should give us approximately 3 labels over the width of the page.
-    
-    assert(staLayouts_.empty() == false);
-  
-    const float  coords_range_x = staLayouts_.top()->maxX()-staLayouts_.top()->minX();
-    const float  coords_range_y = staLayouts_.top()->maxY()-staLayouts_.top()->minY();
+	MFloat *labelx = new MFloat[n];  // in theory, we shouldn't need this many entries...
+	MFloat *labely = new MFloat[n];  // in theory, we shouldn't need this many entries...
 
-    //const float  coords_range_x = getBox(line.getBox())->getXmax() - getBox(line.getBox())->getXmin();
-    //const float  coords_range_y = getBox(line.getBox())->getYmax() - getBox(line.getBox())->getYmin();
-    const float  min_distance_between_labels = (coords_range_x + coords_range_y) / 4;
-    const float  min_square_distance_between_labels = (min_distance_between_labels * min_distance_between_labels);
-    float *labelx = new float[n];  // in theory, we shouldn't need this many entries...
-    float *labely = new float[n];  // in theory, we shouldn't need this many entries...
-    int num_labels = 0;
-        unsigned int i = 1;
-        // Store the minimum x,y corner, pretending that we're going to plot a label there.
-        // We won't actually plot a label there though - it's just to make sure that
-        // we don't plot one too close to that point.
-        labelx [0] = -180.;
-        labely [0] = -90.;
-	//labelx [0] = getBox(line.getBox())->getXmin();
-        //labely [0] = getBox(line.getBox())->getYmin();
-        num_labels = 1;
+	// Store the minimum x,y corner, pretending that we're going to plot a label there.
+	// We won't actually plot a label there though - it's just to make sure that
+	// we don't plot one too close to that point.
+	int num_labels = 1;
+	labelx [0] = staLayouts_.top()->minX();
+	labely [0] = staLayouts_.top()->minY();
 
+	// Calculate how far apart the labels should be.
+	// Our algorithm is to take the average of the width and height and divide by 2.
+	// This should give us approximately 3 labels over the width of the page.
+	const MFloat  coords_range_x = fabs(staLayouts_.top()->maxX()-staLayouts_.top()->minX());
+	const MFloat  coords_range_y = fabs(staLayouts_.top()->maxY()-staLayouts_.top()->minY());
+	const MFloat  min_distance_between_labels = (coords_range_x + coords_range_y) / 4.;
+	const MFloat  min_square_distance_between_labels = (min_distance_between_labels * min_distance_between_labels);
 
+	unsigned int i = 15;
 	while(i < n-minimum_points_for_labelling)
 	{
 	  const double angle = LSF(x,y,i);
 	  const double angle2 = LSF(x,y,i+1);
 	  if(fabs(angle-angle2)< 0.01)
 	  {
-	    const float THIS_X = x[i];
-	    const float THIS_Y = y[i];
-	    const float PREV_X = labelx[num_labels - 1];
-	    const float PREV_Y = labely[num_labels - 1];
+	    const MFloat THIS_X = x[i];
+	    const MFloat THIS_Y = y[i];
+	    const MFloat PREV_X = labelx[num_labels - 1];
+	    const MFloat PREV_Y = labely[num_labels - 1];
 
 	    const double distance_squared = ((THIS_X - PREV_X) * (THIS_X - PREV_X)) + ((THIS_Y - PREV_Y) * (THIS_Y - PREV_Y));
 
 	    if (distance_squared > min_square_distance_between_labels)
 	    {
                 // line fitting
-                float pro_x = x[i+2];
-                float pro_y = y[i+2];  // lines not needed
+                MFloat pro_x = x[i+2];
+                MFloat pro_y = y[i+2];  // lines not needed
                 // end of line fitting
 		
                 Text text;
@@ -780,7 +550,7 @@ void BaseDriver::printLine(const Polyline &line) const
 		num_labels++;
 	     }
 	  }//angles are not the same
-	  i++;
+	  i++;i++;
         }
         delete [] labelx;
         delete [] labely;
@@ -795,8 +565,8 @@ void BaseDriver::printLine(const Polyline &line) const
 void BaseDriver::renderPolyline(vector<PaperPoint> &vP) const
 {
 	const unsigned int size = vP.size();
-	float *x = new float[size];
-	float *y = new float[size];
+	MFloat *x = new MFloat[size];
+	MFloat *y = new MFloat[size];
 	for(unsigned int i=0;i<size;i++)
 	{
 		x[i] = vP[i].x();
@@ -811,8 +581,8 @@ void BaseDriver::renderPolyline(vector<PaperPoint> &vP) const
 void BaseDriver::renderPolyline2(vector<PaperPoint> &vP) const
 {
 	const unsigned int size = vP.size();
-	float *x = new float[size];
-	float *y = new float[size];
+	MFloat *x = new MFloat[size];
+	MFloat *y = new MFloat[size];
 	for(int unsigned i=0;i<size;i++)
 	{
 		x[i] = projectX(vP[i].x());
@@ -827,8 +597,8 @@ void BaseDriver::renderPolyline2(vector<PaperPoint> &vP) const
 void BaseDriver::renderSimplePolygon(vector<PaperPoint> &vP) const
 {
 	const unsigned int size = vP.size();
-	float *x = new float[size];
-	float *y = new float[size];
+	MFloat *x = new MFloat[size];
+	MFloat *y = new MFloat[size];
 	for(unsigned int i=0;i<size;i++)
 	{
 		x[i] = vP[i].x();
@@ -868,12 +638,48 @@ string BaseDriver::getTmpName() const
 
 bool BaseDriver::renderCellArray(const Image& ) const
 {
-    Log::dev() << "BaseDriver::renderCellArray ..." << endl;
-    return true;
+//	MagLog::dev() << "BaseDriver::renderCellArray" << endl;
+	return true;
 }
 
-bool BaseDriver::renderPixmap(float, float, float, float, int, int, unsigned char*, int, bool) const
+bool BaseDriver::renderPixmap(MFloat, MFloat, MFloat, MFloat, int, int, unsigned char*, int, bool) const
 {
-    Log::dev() << "BaseDriver::renderPixmap(float, float, float, float, int, int, unsigned char*, int) const" << endl;
-    return true;
+//	MagLog::dev() << "BaseDriver::renderPixmap" << endl;
+	return true;
+}
+
+
+/*!
+  Overwritten in KMLDriver
+  
+  \sa Layer
+*/
+MAGICS_NO_EXPORT void BaseDriver::redisplay(const Layer& layer) const 
+{
+  MagLog::dev() << "BaseDriver::redisplay( layer) > " << layer.name()<< endl;
+}
+
+void BaseDriver::redisplay(const SceneLayer& layer) const
+{
+	for (vector<Layer*>::iterator l = layer.beginLayer(); l != layer.endLayer(); ++l)
+	{
+		(*l)->redisplay(*this);
+	}
+}
+
+void BaseDriver::redisplay(const StaticLayer& layer) const
+{
+	currentLayer_ = layer.name();
+	layer.visit(*this);
+}
+
+void BaseDriver::redisplay(const NoDataLayer& layer) const
+{
+	currentLayer_ = layer.name();
+	layer.visit(*this);
+}
+
+void BaseDriver::redisplay(const StepLayer& layer) const
+{
+  MagLog::dev() << "BaseDriver::redisplay( StepLayer&)" << layer.name()<< endl;
 }
