@@ -53,7 +53,7 @@
 
 /*! \file BaseDriver.h
     \brief Definition of driver base class.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Jan 2004
 
@@ -62,24 +62,28 @@
 #define MPP_BaseDriver_H
 
 #include <magics.h>
-#include <Translator.h>
+#include <MagTranslator.h>
 #include <Colour.h>
 #include <BaseDriverAttributes.h>
-
-class MtInputEvent;
 #include "MagicsObserver.h"
 
 namespace magics{
 
 class Layer;
+class SceneLayer;
+class StaticLayer;
+class NoDataLayer;
+class StepLayer;
+
 class Layout;
+class SceneLayout;
+class LegendLayout;
+class StartPage;
+class EndPage;
 class RootLayout;
-class PreviewLayout;
-class StandaloneLayout;
-class MagnifierLayout;
-class StepRenderer;
 class BinaryObject;
 class Polyline;
+class PolylineSet;
 class Text;
 class Symbol;
 class TextSymbol;
@@ -93,7 +97,6 @@ class Image;
 class Arrow;
 class Flag;
 class ImportObject;
-class SelectionMode;
 class PaperPoint;
 
 struct ShadingProperties;
@@ -101,19 +104,18 @@ struct FillShadingProperties;
 struct DotShadingProperties;
 struct HatchShadingProperties;
 
+class SelectionMode;
+class MtInputEvent;
+class PreviewLayout;
+class HistoLayout;
+class MagnifierLayout;
 
 /*!
  Codes for Brinary reading and writing
 */
-#define BINARY_VERSION 1
-
-enum BinaryTypes{
-    op_start_page = 1,
-    op_end_page = 2,
-    op_new_color = 3,
-    op_polyline = 4,
-};
+#define BINARY_VERSION 2               // last updated with version 2.15.0
 
+typedef double MFloat;
 
 
 /*!
@@ -148,7 +150,7 @@ inline void parsePoints(vector<PaperPoint> &vPP, string &points)
 	}
 
 	istringstream totalString(points);
-	float x,y;
+	MFloat x,y;
 
 	while(!totalString.eof())
 	{
@@ -224,6 +226,7 @@ public:
 	{
 		BaseDriverAttributes::set(node);
 	}
+	void setWidth(double width) { width_ = width; }
 
 	virtual void set(const std::map<string, string>& map)
 	{
@@ -234,52 +237,47 @@ public:
 	virtual void close() {}
 	void printOutputName(const std::string & str) const; 
 
-#ifdef MAGICS_OPENGL
-	//! Selection of Interactive mode: tooltip : user callback called when mouse moves.
-	virtual void tooltip(const SelectionMode&) {}
-	//! Selection of Interactive mode: pointSelection : user callback called when a point is selected.
-	virtual void pointSelection(const SelectionMode&) {}
-	//! Selection of Interactive mode: areaSelection : user callback called when an area is selected.
-	virtual void areaSelection(const SelectionMode&) {}
-	//! Selection of Interactive mode: polygoneSelection : user callback called when a polygone is selected.
-	virtual void polygonSelection(const SelectionMode&) {}
-	//! Selection of Interactive mode: lineSelection : user callback called when a line is selected.
-	virtual void lineSelection(const SelectionMode&) {}
-	//! Selection of Interactive mode: polylineSelection : user callback called when a line is selected.
-	virtual void polylineSelection(const SelectionMode&) {}
-	//! Selection of Interactive mode: polylineSelection : user callback called when a line is selected.
-	virtual void pickSelection(const SelectionMode&) {}
-	//! Selection of Interactive mode: magnification
-	virtual void magnification(const SelectionMode&) {}
-	//! Selection of Interactive mode: magnification
-	virtual void noModes() {}
-	
-
-
-	//! Restore the framebuffer contents from the background texture (OpenGL only)
-	virtual void restoreFb() {}
-	//! Input event handling
-	virtual void inputEvent(MtInputEvent*) {}
-#endif
-
 	virtual void project(const Layout& ) const {}
 	virtual void unproject() const {}
 
-	virtual void newLayer() const { debugOutput("newLayer");}
-	virtual void closeLayer() const { debugOutput("closeLayer");}
-	virtual void newLayer(const Layer&) const {debugOutput("newLayer");}
-	virtual void closeLayer(const Layer&) const {debugOutput("closeLayer");}
+	virtual void newLayer() const { debugOutput("BaseDriver::newLayer");}
+	virtual void closeLayer() const { debugOutput("BaseDriver::closeLayer");}
+	virtual void newLayer(Layer&) const {debugOutput("BaseDriver::newLayer()");}
+	virtual void closeLayer(Layer&) const {debugOutput("BaseDriver::closeLayer()");}
+	virtual void newLayer(StaticLayer&) const {debugOutput("BaseDriver::newStaticLayer");}
+	virtual void closeLayer(StaticLayer&) const {debugOutput("BaseDriver::closeStaticLayer");}
+	virtual void newLayer(StepLayer&) const {debugOutput("BaseDriver::newStepLayer");}
+	virtual void closeLayer(StepLayer&) const {debugOutput("BaseDriver::closeStepLayer");}
+
+	intarray frames() const  { return frame_list_; }
 
 	MAGICS_NO_EXPORT void redisplay(const Layout&) const;
 	MAGICS_NO_EXPORT void redisplay(const RootLayout&) const;
-	virtual MAGICS_NO_EXPORT void redisplay(const StandaloneLayout&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const LegendLayout&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const SceneLayout&) const;
+	MAGICS_NO_EXPORT void redisplay(const StartPage&) const;
+	MAGICS_NO_EXPORT void redisplay(const EndPage&) const;
 	virtual MAGICS_NO_EXPORT void redisplay(const Layer&) const;
-	MAGICS_NO_EXPORT void redisplay(const Polyline&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const SceneLayer&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const StaticLayer&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const NoDataLayer&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const StepLayer&) const;
+	
+	virtual MAGICS_NO_EXPORT void redisplay(const Polyline&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const PolylineSet&) const;
 
+#ifdef MAGICS_QT
 	virtual MAGICS_NO_EXPORT void redisplay(const PreviewLayout&) const {};
+	virtual MAGICS_NO_EXPORT void redisplay(const HistoLayout&) const {};
 	virtual MAGICS_NO_EXPORT void redisplay(const MagnifierLayout&) const {};
-	virtual MAGICS_NO_EXPORT void redisplay(const StepRenderer&) const;
+#endif
+	typedef void (BaseDriver::*ModeFunction)(const SelectionMode&);
+	typedef void (BaseDriver::*ControlFunction)(bool);
+	typedef void (BaseDriver::*InputEventFunction)(MtInputEvent*);
+
 	virtual void redisplay(const BinaryObject&) const;
+	void setDimensionsFromBinary(string mbg_tmpl,MFloat &ratio,int &width) const;
+
 	//! Method to redisplay a Text.
 	MAGICS_NO_EXPORT void redisplay(const Text&) const;
 	//! Method to redisplay a Symbol.
@@ -293,9 +291,9 @@ public:
 	void redisplay(const SymbolItem&, const ComplexSymbol&) const;
 	
 	//! Method to redisplay an Arrow.
-	MAGICS_NO_EXPORT void redisplay(const Arrow&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const Arrow&) const;
 	//! Method to redisplay an Flag.
-	MAGICS_NO_EXPORT void redisplay(const Flag&) const;
+	virtual MAGICS_NO_EXPORT void redisplay(const Flag&) const;
 	//! Method to redisplay a external file.
 	void redisplay(const ImportObject&) const;
 	//! Method to redisplay a external file.
@@ -310,28 +308,25 @@ public:
 
 
 	/*!
-	 \param SUPER_PAGE_X_LENGTH Default: 29.7
+	 SUPER_PAGE_X_LENGTH Default: 29.7
 	*/
-	MAGICS_NO_EXPORT void setXDeviceLength(float xdevicelength)
+	MAGICS_NO_EXPORT void setXDeviceLength(MFloat xdevicelength)
 		{ xDeviceLength_ =  xdevicelength; }
-	MAGICS_NO_EXPORT float getXDeviceLength() const
+	MAGICS_NO_EXPORT MFloat getXDeviceLength() const
 		{ return xDeviceLength_; }
 
 	/*!
-	 \param SUPER_PAGE_Y_LENGTH Default: 21.0
+	 SUPER_PAGE_Y_LENGTH Default: 21.0
 	*/
-	MAGICS_NO_EXPORT void setYDeviceLength(float ydevicelength)
+	MAGICS_NO_EXPORT void setYDeviceLength(MFloat ydevicelength)
 		{ yDeviceLength_ =  ydevicelength; }
-	MAGICS_NO_EXPORT float getYDeviceLength() const
+	MAGICS_NO_EXPORT MFloat getYDeviceLength() const
 		{ return yDeviceLength_; }
 
-	typedef void (BaseDriver::*ModeFunction)(const SelectionMode&);
-	typedef void (BaseDriver::*ControlFunction)(bool);
-	typedef void (BaseDriver::*InputEventFunction)(MtInputEvent*);
 
 protected:
-	float convertCM(const float cm) const {return cm*cmScale_;}
-	void setCMscale(const float scale) const {cmScale_ = scale;}
+	MFloat convertCM(const MFloat cm) const {return cm*cmScale_;}
+	void setCMscale(const MFloat scale) const {cmScale_ = scale;}
 
 	/*!
 	  \brief set Y values positive or negative
@@ -339,13 +334,13 @@ protected:
 	  Some drivers hava a Y axis which is positive towards the bottom of the plot
 	  and these need to set this method negative.
 	*/
-	virtual MAGICS_NO_EXPORT float setY(const float y) const {return y;}
+	virtual MAGICS_NO_EXPORT MFloat setY(const MFloat y) const {return y;}
 
 	/*!
 	  \brief set Y values positive or negative dependent how symbols are interpreted
 
 	*/
-	virtual MAGICS_NO_EXPORT float setSymbolY(const float y) const {return y;}
+	virtual MAGICS_NO_EXPORT MFloat setSymbolY(const MFloat y) const {return y;}
 
 	/*!
 	  \brief set Y values positive or negative dependent how angles are interpreted
@@ -353,99 +348,85 @@ protected:
 	  Some drivers hava a angle orientation different to PS
 	  and these need to set this method negative.
 	*/
-        virtual MAGICS_NO_EXPORT float setAngleY(const float y) const {return y;}
-        virtual MAGICS_NO_EXPORT float setFlagY(const float y) const {return y;}
+        virtual MAGICS_NO_EXPORT MFloat setAngleY(const MFloat y) const {return y;}
+        virtual MAGICS_NO_EXPORT MFloat setFlagY(const MFloat y) const {return y;}
 
 	string getFileName(const string &extension, const unsigned int no = 0) const;
 
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	virtual void print(ostream&) const;
         virtual void startPage() const{};
-	virtual void startAdditionalPage(const StandaloneLayout&) const{};
         virtual void endPage() const{};
-        virtual void endAdditionalPage() const{};
 	virtual void setNewColour(const Colour &) const {};
 	virtual void printLine(const Polyline &line) const;
 
 	virtual void renderText(const Text &) const {}
 	virtual void debugOutput(const string &s) const 
 	{
-		if(getDebug()) Log::debug() <<" DRIVERS: "<<s<< "\n";
+		if(debug_) MagLog::debug() <<" DRIVERS: "<<s<< "\n";
 	}
 
-	virtual float projectX(const float x) const {return coordRatioX_*x;}
-	virtual float projectY(const float y) const {return coordRatioY_*y;}
+	virtual MFloat projectX(const MFloat x) const {return coordRatioX_*x;}
+	virtual MFloat projectY(const MFloat y) const {return coordRatioY_*y;}
 		
 	string getTmpName() const;
 
-	double LSF(float *x,float *y, int i0) const;
+	double LSF(MFloat *x,MFloat *y, int i0) const;
 
 	mutable int	currentPage_;
 	mutable string	fileName_;
 	mutable string	currentLayer_;  // from Layer (open & close)
-	mutable string	currentGroup_;  // from property object layer
 
 	mutable LineStyle currentLineType_;
-	mutable float	currentLineWidth_;
+	mutable MFloat	currentLineWidth_;
 	mutable int	currentLineStyle_;
 	mutable Colour	currentColour_;
 
-	mutable float	coordRatioX_;
-	mutable float	coordRatioY_;
-	mutable float	dimensionX_;
-        mutable float	dimensionY_;
-        mutable float	offsetX_;
-        mutable float	offsetY_;
+	mutable MFloat	coordRatioX_;
+	mutable MFloat	coordRatioY_;
+	mutable MFloat	dimensionX_;
+        mutable MFloat	dimensionY_;
+        mutable MFloat	offsetX_;
+        mutable MFloat	offsetY_;
 
-	mutable std::stack<float>  dimensionStack_;
-	mutable std::stack<float>  scalesX_;
-	mutable std::stack<float>  scalesY_;
+	mutable std::stack<MFloat>  dimensionStack_;
+	mutable std::stack<MFloat>  scalesX_;
+	mutable std::stack<MFloat>  scalesY_;
 
-	mutable float	lastAreaHeightPercentage_;
-	mutable float	lastAreaWidthPercentage_;
+//	mutable MFloat	lastAreaHeightPercentage_;
+//	mutable MFloat	lastAreaWidthPercentage_;
 	mutable bool	newPage_;
-	mutable	bool	newLayout_;
-	mutable	bool    external_;
-	mutable bool    polylineAntialiasing_;
+//	mutable bool	firstPage_;
+//	mutable	bool	newLayout_;
+//	mutable	bool    external_;
+//	mutable bool    polylineAntialiasing_;
+
 	bool    disabled_;
 	bool    alphaEnabled_;
-
 	mutable vSymbols  sym_;
-	mutable stack<const Layout *>       staLayouts_;
-	mutable vector<const PaperPoint *>  vecPoints_;
-	mutable double                      obs_distance_;
 
-	virtual MAGICS_NO_EXPORT void setNewLineWidth(const float w) const {currentLineWidth_ = w;}
-	MAGICS_NO_EXPORT float getNewLineWidth() const {return currentLineWidth_;}
+	virtual MAGICS_NO_EXPORT void setNewLineWidth(const MFloat w) const {currentLineWidth_ = w;}
+	MAGICS_NO_EXPORT MFloat getNewLineWidth() const {return currentLineWidth_;}
 
-	// Load svg symbols from file
+	//! Load svg symbols from file
 	void loadSymbols() const;
 
-	// Symbol calculating methods
-	MAGICS_NO_EXPORT bool checkDistanceMoreThan(const PaperPoint *pp, double distance) const;
 	virtual MAGICS_NO_EXPORT void renderTextSymbols(const TextSymbol& symbol) const;
 	
 	virtual void renderComplexSymbols(const ComplexSymbol& symbol) const;
 	virtual void renderTextItem(const TextItem&, const ComplexSymbol&) const;
 	virtual void renderFlagItem(const FlagItem&, const ComplexSymbol& symbol) const;
 	virtual void renderSymbolItem(const SymbolItem&, const ComplexSymbol& symbol) const;
-
-
-	
 	virtual void renderSymbols(const Symbol& symbol) const;
-	virtual void renderPolyline(const int, float *, float *) const {};
-	virtual void renderPolyline2(const int, float *, float *) const {}
-	MAGICS_NO_EXPORT void renderPolyline(vector<PaperPoint> &vP) const;
+	virtual void renderPolyline(const int, MFloat *, MFloat *) const {};
+	virtual void renderPolyline2(const int, MFloat *, MFloat *) const {}
+	void renderPolyline(vector<PaperPoint> &vP) const;
 	void renderPolyline2(vector<PaperPoint> &vP) const;
-	virtual MAGICS_NO_EXPORT void renderSimplePolygon(const int, float *, float *) const = 0;
-	MAGICS_NO_EXPORT void renderSimplePolygon(vector<PaperPoint> &vP) const;
-	virtual void circle(const float, const float, const float, const int) const {}
-	void snowflake(const float, const float, const float) const;
-	void drizzle(const float, const float, const float) const;
-	void triangle(const float, const float, const float, const int, const int) const;
-	void lightning(const float x, const float y, const float size) const;
 
-	virtual MAGICS_NO_EXPORT int setLineParameters(const LineStyle st, const float w) const {currentLineType_=st; setNewLineWidth(w);return 0;}
+	virtual MAGICS_NO_EXPORT void renderSimplePolygon(const int, MFloat *, MFloat *) const = 0;
+	virtual MAGICS_NO_EXPORT void renderSimplePolygon(const Polyline& line) const = 0;
+	virtual void circle(const MFloat, const MFloat, const MFloat, const int) const {}
+	virtual MAGICS_NO_EXPORT int setLineParameters(const LineStyle st, const MFloat w) const {currentLineType_=st; setNewLineWidth(w);return 0;}
 
 	// Filling calculating methods
 	// PolylineSets rendering methods
@@ -462,20 +443,32 @@ protected:
 	virtual void renderImage(const ImportObject& object) const;
 	MAGICS_NO_EXPORT void renderImage(const Image& obj) const {renderCellArray(obj);}
         virtual MAGICS_NO_EXPORT bool convertToPixmap(const string &fname, const GraphicsFormat format, const int reso,
-	             const float wx0, const float wy0,const float wx1,const float wy1) const;
-	virtual MAGICS_NO_EXPORT bool renderPixmap(float,float,float,float,int,int,unsigned char*,int,bool hasAlpha=false) const;
+	             const MFloat wx0, const MFloat wy0,const MFloat wx1,const MFloat wy1) const;
+	virtual MAGICS_NO_EXPORT bool renderPixmap(MFloat,MFloat,MFloat,MFloat,int,int,unsigned char*,int,bool hasAlpha=false) const;
 	virtual MAGICS_NO_EXPORT bool renderCellArray(const Image& ) const;
 
 	mutable std::map<string, magFont, RuntimeStringCompare> FontMap_;
 	typedef std::map<string, magFont, RuntimeStringCompare>::const_iterator fontMapIter;
 
-private:
-	void readFonts() const;
+	void terminate() const {exit(1);}
 
-	mutable float cmScale_;
-	mutable float xDeviceLength_;
-	mutable float yDeviceLength_;
+	void readFonts() const;
+	void relieveFonts() const {FontMap_.clear();}
+
+private:
+	mutable MFloat cmScale_;
+	MFloat xDeviceLength_;
+	MFloat yDeviceLength_;
 	static int numFiles_;
+	mutable double obs_distance_;
+	mutable stack<const Layout *>  staLayouts_;
+	mutable vector<const PaperPoint *>  vecPoints_;
+	bool checkDistanceMoreThan(const PaperPoint *pp, double distance) const;
+	void renderSimplePolygon(vector<PaperPoint> &vP) const;
+	void snowflake(const MFloat, const MFloat, const MFloat) const;
+	void drizzle(const MFloat, const MFloat, const MFloat) const;
+	void triangle(const MFloat, const MFloat, const MFloat, const int, const int) const;
+	void lightning(const MFloat x, const MFloat y, const MFloat size) const;
 
 	//! Copy constructor - No copy allowed
 	BaseDriver(const BaseDriver&);
@@ -491,7 +484,7 @@ private:
  \brief translates a string to a driver
 */
 template<>
-class Translator<string, BaseDriver>
+class MagTranslator<string, BaseDriver>
 {
 public:
 	MAGICS_NO_EXPORT BaseDriver* operator()(const string& val )
