@@ -20,7 +20,7 @@
 
 /*! \file BaseDriverSymbols.h
     \brief Implementation of Symbol methods of driver base class.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Jan 2005
 
@@ -88,7 +88,7 @@ MAGICS_NO_EXPORT void BaseDriver::loadSymbols() const
 		FILE* in  = fopen(filename.c_str(), "r");
 		if(!in)
 		{
-			Log::error() << "BaseDriver::loadSymbols() -> Symbol file " << filename
+			MagLog::error() << "BaseDriver::loadSymbols() -> Symbol file " << filename
 			             << "could NOT been found! NO symbols will be plotted!" << endl;
 			return;
 		}
@@ -101,7 +101,7 @@ MAGICS_NO_EXPORT void BaseDriver::loadSymbols() const
 			done = len < sizeof(buf);
 			if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR)
 			{
-				Log::error() << "BaseDriver::loadSymbols()-> XML ERROR: "
+				MagLog::error() << "BaseDriver::loadSymbols()-> XML ERROR: "
 				             << XML_ErrorString(XML_GetErrorCode(parser))
 				             << " at line  " << XML_GetCurrentLineNumber(parser) << endl;
 			}
@@ -113,11 +113,12 @@ MAGICS_NO_EXPORT void BaseDriver::loadSymbols() const
 }
 
 MAGICS_NO_EXPORT void BaseDriver::renderTextSymbols(const TextSymbol& symbol) const
-{	
+{
 	const int nPos  = symbol.size();
 	const int nText = symbol.text().size();
 
 	assert(nText >= nPos);
+	if(nText < nPos) MagLog::debug() << " BaseDriverSymbols > Not enough texts ("<<nText<<") for "<<nPos<<" symbols!";
 
 	//Loop for all positions
 	for(int i=0;i< nPos ;i++)
@@ -125,35 +126,41 @@ MAGICS_NO_EXPORT void BaseDriver::renderTextSymbols(const TextSymbol& symbol) co
 		Text text;
 		text.setJustification(MCENTRE);
 		text.setVerticalAlign(MHALF);
+		text.setBlanking(symbol.blanking());
 
-		float shiftX = 0.;
-		float shiftY = 0;
+		MFloat shiftX = 0.;
+		MFloat shiftY = 0;
 
 		// default is M_NONE
-		if( symbol.position() == Symbol::M_RIGHT )
+		if( symbol.position() == Symbol::M_NONE )
+		{
+//			cout<<"M_NONE"<<endl;
+		}
+		else if( symbol.position() == Symbol::M_RIGHT )
 		{
 			shiftX = convertCM(symbol.getHeight()*.5)/coordRatioX_;
 			text.setJustification(MLEFT);
 		}
 		else if( symbol.position() == Symbol::M_LEFT )
 		{
-			shiftX = convertCM(symbol.getHeight()*.5)/coordRatioX_;
+			shiftX = -convertCM(symbol.getHeight()*.5)/coordRatioX_;
 			text.setJustification(MRIGHT);
 		}
 		else if( symbol.position() == Symbol::M_BELOW )
 		{
-			shiftY = -convertCM(symbol.getHeight()*.5)/coordRatioY_;
+			shiftY = setY(-convertCM(symbol.getHeight()*.6)/coordRatioY_);
+			text.setVerticalAlign(MTOP);
 		}
 		else if( symbol.position() == Symbol::M_ABOVE )
 		{
-			shiftY = convertCM(symbol.getHeight()*.5)/coordRatioY_;
+			shiftY = setY(convertCM(symbol.getHeight()*0.8)/coordRatioY_);
+			text.setVerticalAlign(MBOTTOM);
 		}
 
-// Log::dev()<<"x: " << symbol[i].x()<<"  y: "<<symbol[i].y()<<"+"<< shiftY<< endl;
-		text.push_back(PaperPoint(symbol[i].x()+shiftX, symbol[i].y()+shiftY));
-
-		text.setAngle(0.);			
-		text.addText(symbol.text().at(i), symbol.font()); //75 dpi		
+// MagLog::dev()<<"x: " << symbol[i].x()<<"  y: "<<symbol[i].y()<<"+"<< shiftY<< endl;
+		text.push_back(PaperPoint(symbol[i].x()+shiftX, symbol[i].y()+setFlagY(shiftY) ));
+		text.setAngle(0.);
+		text.addText(symbol.text().at(i), symbol.font()); //75 dpi
 		renderText(text);
 	}
 	renderSymbols(symbol);
@@ -177,12 +184,12 @@ MAGICS_NO_EXPORT void BaseDriver::renderComplexSymbols(const ComplexSymbol& symb
 MAGICS_NO_EXPORT void BaseDriver::renderTextItem(const TextItem& textItem, const ComplexSymbol& owner) const
 {
 	Text text;
-	const float scaling = convertCM(owner.getHeight()*1.5);
-	const float pX = 1. / coordRatioX_;
-	const float pY = 1. / coordRatioY_;
+	const MFloat scaling = convertCM(owner.getHeight()*1.5);
+	const MFloat pX = 1. / coordRatioX_;
+	const MFloat pY = 1. / coordRatioY_;
 
-	const float posX = owner[0].x() + (textItem.x() * scaling *pX);
-	const float posY = owner[0].y() + (textItem.y() * scaling *pY);
+	const MFloat posX = owner[0].x() + (           textItem.x()  * scaling *pX);
+	const MFloat posY = owner[0].y() + (setSymbolY(textItem.y()) * scaling *pY);
 	text.push_back(PaperPoint(posX, posY));
 	text.setJustification(MCENTRE);
 //
@@ -204,12 +211,15 @@ MAGICS_NO_EXPORT void BaseDriver::renderFlagItem(const FlagItem& flagItem, const
 	flag.setOriginMarker(flagItem.getOriginMarker());
 	flag.setOriginHeight(owner.getHeight());           // NOT flagItem.getOriginHeight() !!!
 	flag.setColour(flagItem.getColour());
-	const float scaling = convertCM(owner.getHeight());
-	const float pX = 1. / coordRatioX_;
-	const float pY = 1. / coordRatioY_;
 
-	const float posX = owner[0].x() + (flagItem.x() * scaling *pX);
-	const float posY = owner[0].y() + (flagItem.y() * scaling *pY);			
+	flag.setHemisphere(flagItem.getHemisphere());
+
+	const MFloat scaling = convertCM(owner.getHeight());
+	const MFloat pX = 1. / coordRatioX_;
+	const MFloat pY = 1. / coordRatioY_;
+
+	const MFloat posX = owner[0].x() + (           flagItem.x()  * scaling *pX);
+	const MFloat posY = owner[0].y() + (setSymbolY(flagItem.y()) * scaling *pY);
 	ArrowPoint a(PaperPoint(posX, posY));
 	a.set(flagItem.speed(),flagItem.direction());
 	flag.push_back(a);
@@ -222,7 +232,7 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbolItem(const SymbolItem& symbol, con
 	setNewColour(symbol.colour());
 	setLineParameters(M_SOLID, 1.); // reset line
 	loadSymbols();
-	if(sym_.empty()) {Log::error() << "BaseDriver::renderSymbols("<<symbol.symbol()<<")-> NO symbols available!"<<endl; return;}
+	if(sym_.empty()) {MagLog::error() << "BaseDriver::renderSymbols("<<symbol.symbol()<<")-> NO symbols available!"<<endl; return;}
 
 	const int noSymbols = sym_.size();
 	svgSymbol sym;
@@ -235,52 +245,52 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbolItem(const SymbolItem& symbol, con
 	if(ii==noSymbols) sym = sym_[0];
 
 	const unsigned int si = sym.elements.size();
-	const float pX = 1. / coordRatioX_;
-	const float pY = 1. / coordRatioY_;
+	const MFloat pX = 1. / coordRatioX_;
+	const MFloat pY = 1. / coordRatioY_;
 	
-	float scaling = convertCM(owner.getHeight()*1.5);  // convertCM scales them up!
+	MFloat scaling = convertCM(owner.getHeight()*1.5);  // convertCM scales them up!
 
-	const float posX = owner[0].x() + (symbol.x() * scaling * pX);
-	const float posY = owner[0].y() + (symbol.y() * scaling * pY);
+	const MFloat posX = owner[0].x() + (           symbol.x()  * scaling * pX);
+	const MFloat posY = owner[0].y() + (setSymbolY(symbol.y()) * scaling * pY);
 
 	for(unsigned int i=0;i<si;i++)  // for all elements in the symbol description
 	{
 		if(sym.elements[i].name == "circle")
 		{
-			const float r  = atof(sym.elements[i].attributes["r"].c_str())  * scaling;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r  = atof(sym.elements[i].attributes["r"].c_str())  * scaling;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			const int s = atoi(sym.elements[i].attributes["fill"].c_str());
 			circle(posX+cx,posY+cy,r,s);
 		}
 		else if(sym.elements[i].name == "snowflake")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str())   * scaling * pX;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str())   * scaling * pX;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			snowflake(posX+cx,posY+cy,r);
 		}
 		else if(sym.elements[i].name == "drizzle")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str())   * scaling * pX;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str())   * scaling * pX;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			drizzle(posX+cx,posY+cy,r);
 		}
 		else if(sym.elements[i].name == "triangle")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
 			const int s = atoi(sym.elements[i].attributes["fill"].c_str());
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			const int li = atoi(sym.elements[i].attributes["line"].c_str());
 			triangle(posX+cx,posY+cy,r,s,li);
 		}
 		else if(sym.elements[i].name == "lightning")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			lightning(posX+cx,posY+cy,r);
 		}
 		if(sym.elements[i].name == "polyline")
@@ -291,16 +301,19 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbolItem(const SymbolItem& symbol, con
 
 			if(size > 0)
 			{
-			  const float sX = posX;
-			  const float sY = posY;
+			  const MFloat sX = posX;
+			  const MFloat sY = posY;
 			  for(int s=0;s<size;s++)
 			  {
 				vPP[s].x( (vPP[s].x() * scaling * pX) + sX);
 				vPP[s].y( setSymbolY(setY((vPP[s].y()) * scaling * pY)) + sY);
 			  }
-			}
-			if(magCompare("none",sym.elements[i].attributes["fill"]) ){ renderPolyline(vPP);}
-			else {renderSimplePolygon(vPP);}
+			}			
+				
+		        if(!magCompare("none",sym.elements[i].attributes["fill"]) )
+				renderSimplePolygon(vPP);
+			else
+			  renderPolyline(vPP);
 		}
 	}// endfor 
 }
@@ -315,12 +328,10 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbolItem(const SymbolItem& symbol, con
 MAGICS_NO_EXPORT void BaseDriver::renderSymbols(const Symbol& symbol) const
 {
 	debugOutput("Start Symbols");
-	setLineParameters(M_SOLID, 1.); // reset line
-
 	loadSymbols();
 
 	// if still empty
-	if(sym_.empty()) {Log::error() << "BaseDriver::renderSymbols("<<symbol.getSymbol()<<")-> NO symbols available!"<<endl; return;}
+	if(sym_.empty()) {MagLog::error() << "BaseDriver::renderSymbols("<<symbol.getSymbol()<<")-> NO symbols available!"<<endl; return;}
 
 	const int noSymbols = sym_.size();
 	svgSymbol sym;
@@ -334,10 +345,11 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbols(const Symbol& symbol) const
 
 	const unsigned int si = sym.elements.size();
 	setNewColour(symbol.getColour());
+	setLineParameters(M_SOLID, 1.); // reset line
 
-	const float pX = 1. / coordRatioX_;
-	const float pY = 1. / coordRatioY_;
-	const float scaling = convertCM(symbol.getHeight()*.5);
+	const MFloat pX = 1. / coordRatioX_;
+	const MFloat pY = 1. / coordRatioY_;
+	const MFloat scaling = convertCM(symbol.getHeight()*.5);
 
 	for(unsigned int i=0;i<si;i++)  // for all elements in the symbol description
 	{
@@ -345,40 +357,65 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbols(const Symbol& symbol) const
 
 		if(sym.elements[i].name == "circle")
 		{
-			const float r  = atof(sym.elements[i].attributes["r"].c_str())  * scaling;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = setY(atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY);
+			const MFloat r  = atof(sym.elements[i].attributes["r"].c_str())  * scaling;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = setY(atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY);
 			const int s = atoi(sym.elements[i].attributes["fill"].c_str());
-			for(long l=0;l<n;l++) circle(symbol[l].x()+cx,symbol[l].y()+cy,r,s);
+			for(long l=0;l<n;l++)
+			{
+				circle(symbol[l].x()+cx,symbol[l].y()+cy,r,s);
+				if(s==8 && symbol.outline())
+				{
+					setNewColour(symbol.outlineColour());
+					setLineParameters(symbol.outlineLineStyle(),symbol.outlineThickness());
+					circle(symbol[l].x()+cx,symbol[l].y()+cy,r,0);
+					//We need to go back to the original values
+					setNewColour(symbol.getColour());
+					setLineParameters(M_SOLID, 1.);
+				}
+			}	
 		}
 		else if(sym.elements[i].name == "snowflake")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			for(long l=0;l<n;l++) snowflake(symbol[l].x()+cx,symbol[l].y()+cy,r);
 		}
 		else if(sym.elements[i].name == "drizzle")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			for(long l=0;l<n;l++) drizzle(symbol[l].x()+cx,symbol[l].y()+cy,r);
 		}
 		else if(sym.elements[i].name == "triangle")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
 			const int s = atoi(sym.elements[i].attributes["fill"].c_str());
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			const int li = atoi(sym.elements[i].attributes["line"].c_str());
-			for(long l=0;l<n;l++) triangle(symbol[l].x()+cx,symbol[l].y()+cy,r,s,li);
-		}
+
+			for(long l=0;l<n;l++)
+			{
+				triangle(symbol[l].x()+cx,symbol[l].y()+cy,r,s,li);
+				if(s == 1 && symbol.outline())
+				{
+					setNewColour(symbol.outlineColour());
+					setLineParameters(symbol.outlineLineStyle(),symbol.outlineThickness());
+					triangle(symbol[l].x()+cx,symbol[l].y()+cy,r,0,li);
+					//We need to go back to the original values
+					setNewColour(symbol.getColour());
+					setLineParameters(M_SOLID, 1.);
+				}	
+			}
+		}	
 		else if(sym.elements[i].name == "lightning")
 		{
-			const float r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
-			const float cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
-			const float cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
+			const MFloat r = atof(sym.elements[i].attributes["r"].c_str()) * scaling * pX;
+			const MFloat cx = atof(sym.elements[i].attributes["cx"].c_str()) * scaling * pX;
+			const MFloat cy = atof(sym.elements[i].attributes["cy"].c_str()) * scaling * pY;
 			for(long l=0;l<n;l++) lightning(symbol[l].x()+cx,symbol[l].y()+cy,r);
 		}
 		if(sym.elements[i].name == "polyline")
@@ -391,16 +428,28 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbols(const Symbol& symbol) const
 
 				if(size > 0)
 				{
-				  const float sX = symbol[l].x();
-				  const float sY = symbol[l].y();
-				  for(int s=0;s<size;s++)
-				  {
-					vPP[s].x( (vPP[s].x() * scaling * pX) + sX);
-					vPP[s].y( setSymbolY(setY((vPP[s].y()) * scaling * pY)) + sY);
-				  }
+					const MFloat sX = symbol[l].x();
+					const MFloat sY = symbol[l].y();
+					for(int s=0;s<size;s++)
+					{
+						vPP[s].x( (vPP[s].x() * scaling * pX) + sX);
+						vPP[s].y( setSymbolY(setY((vPP[s].y()) * scaling * pY)) + sY);
+					}
+				
+					if(!magCompare("none",sym.elements[i].attributes["fill"]) )
+						renderSimplePolygon(vPP);
+					else
+						renderPolyline(vPP);
+					if(!magCompare("none",sym.elements[i].attributes["fill"]) && symbol.outline())
+					{
+						setNewColour(symbol.outlineColour());
+						setLineParameters(symbol.outlineLineStyle(),symbol.outlineThickness());
+						renderPolyline(vPP);
+						//We need to go back to the original values
+						setNewColour(symbol.getColour());
+						setLineParameters(M_SOLID, 1.);
+					}
 				}
-				if(magCompare("none",sym.elements[i].attributes["fill"]) ){ renderPolyline(vPP);}
-				else {renderSimplePolygon(vPP);}
 			}
 		}
 	}// endfor 
@@ -409,10 +458,10 @@ MAGICS_NO_EXPORT void BaseDriver::renderSymbols(const Symbol& symbol) const
 /*!
   Plotting a snowflake
 */
-MAGICS_NO_EXPORT void BaseDriver::snowflake(const float x, const float y, const float size) const
+MAGICS_NO_EXPORT void BaseDriver::snowflake(const MFloat x, const MFloat y, const MFloat size) const
 {
-	const float s5 = size;
-	const float s3 = size * 0.78;
+	const MFloat s5 = size;
+	const MFloat s3 = size * 0.78;
 
 	vector<PaperPoint> line;
 	  line.push_back(PaperPoint(x-s5,y));
@@ -431,9 +480,9 @@ MAGICS_NO_EXPORT void BaseDriver::snowflake(const float x, const float y, const
 /*!
   Plotting drizzle
 */
-MAGICS_NO_EXPORT void BaseDriver::drizzle(const float x, const float y, const float size) const
+MAGICS_NO_EXPORT void BaseDriver::drizzle(const MFloat x, const MFloat y, const MFloat size) const
 {
-	const float s2 = size*.5;
+	const MFloat s2 = size*.5;
 
 	circle(x,y,s2 * coordRatioX_,8);
 
@@ -446,7 +495,7 @@ MAGICS_NO_EXPORT void BaseDriver::drizzle(const float x, const float y, const fl
 /*!
   Plotting lightning symbol
 */
-MAGICS_NO_EXPORT void BaseDriver::lightning(const float x, const float y, const float size) const
+MAGICS_NO_EXPORT void BaseDriver::lightning(const MFloat x, const MFloat y, const MFloat size) const
 {
 	vector<PaperPoint> line;
 	  line.push_back(PaperPoint(x-(size*.9),y-(size*.9)));
@@ -468,9 +517,9 @@ MAGICS_NO_EXPORT void BaseDriver::lightning(const float x, const float y, const
 /*!
   Plotting a triangle
 */
-MAGICS_NO_EXPORT void BaseDriver::triangle(const float x, const float y, const float size, const int fill, const int l) const
+MAGICS_NO_EXPORT void BaseDriver::triangle(const MFloat x, const MFloat y, const MFloat size, const int fill, const int l) const
 {
-	const float s = 0.5 * size;
+	const MFloat s = 0.5 * size;
 
 	vector<PaperPoint> line;
 	  line.push_back(PaperPoint( x+s, y-s) );
