@@ -36,17 +36,17 @@ class rotate
 {
 public:
 	rotate():angle_(0){}
-	rotate(float a, float ratio=1.):angle_(a),ratio_(ratio){}
+	rotate(MFloat a, MFloat ratio=1.):angle_(a),ratio_(ratio){}
 	void operator()(PaperPoint &p)
 	{
-		float x =  p.x()         * cos(angle_) + (p.y() / ratio_) * -sin(angle_);
-		float y = (p.x()*ratio_) * sin(angle_) +  p.y()           *  cos(angle_);
+		MFloat x =  p.x()         * cos(angle_) + (p.y() / ratio_) * -sin(angle_);
+		MFloat y = (p.x()*ratio_) * sin(angle_) +  p.y()           *  cos(angle_);
 		p.x(x);
 		p.y(y);
 	}
 private:
-	float angle_;
-	float ratio_;
+	MFloat angle_;
+	MFloat ratio_;
 };
 
 class translate
@@ -56,8 +56,8 @@ public:
 	translate(PaperPoint point):point_(point){}
 	void operator()(PaperPoint &p)
 	{
-		float x = p.x() + point_.x();
-		float y = p.y() + point_.y();
+		MFloat x = p.x() + point_.x();
+		MFloat y = p.y() + point_.y();
 		p.x(x);
 		p.y(y);
 	}
@@ -76,27 +76,30 @@ using namespace magics;
 */
 MAGICS_NO_EXPORT void BaseDriver::renderWindArrow(const Arrow &arrow) const
 {
-	const float base = 0.7;
-	const float sca = (fabs(arrow.getScale()) < 0.001) ? 1. : arrow.getScale();
-	const float scaling = convertCM(1.) / (sca * ((coordRatioX_+coordRatioX_)*.5 ));
+	const MFloat base = 0.7;
+	const MFloat sca = (fabs(arrow.getScale()) < 0.001) ? 1. : arrow.getScale();
+	const MFloat scaling = convertCM(1.) / (sca * ((coordRatioX_+coordRatioX_)*.5 ));
 
 	const unsigned int arrPoNo = arrow.size();
 	if(arrPoNo<1) return;
 
-	const float thickness = (arrow.getThickness()==1) ? 1 : 2.*arrow.getThickness();
+	const MFloat thickness = (arrow.getThickness()==1) ? 1 : 2.*arrow.getThickness();
 	const LineStyle style = arrow.getStyle();
 	const ArrowPosition pos = arrow.getArrowPosition();
 
 	setNewColour(arrow.getColour());
 	Arrow::const_iterator arr = arrow.begin();
 	int index  = arrow.getHeadIndex();
-	const float ratio = (coordRatioY_==0) ? 1. : coordRatioX_/coordRatioY_; 
+	const MFloat ratio = (coordRatioY_==0) ? 1. : coordRatioX_/coordRatioY_;
 
 	for(unsigned int pts=0;pts<arrPoNo;pts++)
 	{
 	  const double angle = setAngleY(arr->angle());
 	  double norm  = arr->norm()*scaling;
-	  if(fabs(norm) < 0.01) norm = scaling;
+
+	  // The next line was making very small wind artificially bigger!
+	  // if(fabs(norm) < 0.01) norm = scaling;
+
 	  const double norm2 = norm * base;
 
 	  vector<PaperPoint> line;
@@ -117,12 +120,12 @@ MAGICS_NO_EXPORT void BaseDriver::renderWindArrow(const Arrow &arrow) const
 	  currentLineStyle_ = old_currentColourIndex;
 
 	// Arrow head
-	  const float bx = (1.0-base) * norm; // a third of the length of arrow
-	  const float by = (1.0-base) * ratio * norm * arrow.getHeadRatio(); // 10% of the length
+	  const MFloat bx = (1.0-base) * norm; // a third of the length of arrow
+	  const MFloat by = (1.0-base) * ratio * norm * arrow.getHeadRatio(); // 10% of the length
 
 	  if( index>3 || index<0)
 	  {
-	  	Log::warning() << "Wind: Wind arrow head index "<<index<<"out of range (0-3)! Use 2 as default" << endl;
+	  	MagLog::warning() << "Wind: Wind arrow head index "<<index<<"out of range (0-3)! Use 2 as default" << endl;
 		index = 0;
 	  }
 
@@ -163,40 +166,34 @@ MAGICS_NO_EXPORT void BaseDriver::renderWindArrow(const Arrow &arrow) const
 
 MAGICS_NO_EXPORT void BaseDriver::renderWindFlag(const Flag &flag) const
 {
-	const float thickness = (flag.getThickness()==1) ? 1 : 2.*flag.getThickness();
+	const MFloat thickness = (flag.getThickness()==1) ? 1 : 2.*flag.getThickness();
 	const LineStyle style = flag.getStyle();
-	const float length = convertCM(flag.getLength() / coordRatioX_ );
+	const MFloat length = convertCM(flag.getLength() / coordRatioX_ );
 	const string marker = flag.getOriginMarker();
-	const float markerHeight = convertCM(flag.getOriginHeight() *.5 / coordRatioX_);
+	const MFloat markerHeight = convertCM(flag.getOriginHeight() *.5 / coordRatioX_);
 	setNewColour(flag.getColour());
 	const unsigned int flaPoNo = flag.size();
 	Flag::const_iterator fla = flag.begin();
 
 	Symbol origin;
-	if(markerHeight>0.)
-	{
-		origin.setSymbol(marker);
-		origin.setHeight(flag.getOriginHeight()*2.);
-		origin.setColour(flag.getColour());
-	}
 
 	for(unsigned int pts=0;pts<flaPoNo;pts++)
 	{
 		vector<PaperPoint> line;
-		const float angle = setAngleY(fla->angle());
+		const MFloat angle = setAngleY(fla->angle());
 		line.push_back(PaperPoint(-markerHeight,0.) );
 		line.push_back(PaperPoint(-length,0.) );
-		const float ratio = (coordRatioY_==0) ? 1. : coordRatioX_/coordRatioY_; 
+		const MFloat ratio = (coordRatioY_==0) ? 1. : coordRatioX_/coordRatioY_;
 		for_each(line.begin(),line.end(),rotate(angle,ratio) );
 		for_each(line.begin(),line.end(),translate(fla->point_) );
-		float len = fla->norm();
+		MFloat len = fla->norm();
 
-		float lev1 = 1.25;  // 2.5
-		float lev2 = 3.75;  // 5.
-		float lev3 = 23.;  // 25.
-		float slev1 = 2.5;
-		float slev2 = 5.;
-		float slev3 = 25.;
+		MFloat lev1 = 1.25;  // 2.5
+		MFloat lev2 = 3.75;  // 5.
+		MFloat lev3 = 23.;  // 25.
+		MFloat slev1 = 2.5;
+		MFloat slev2 = 5.;
+		MFloat slev3 = 25.;
 		
 		if(flag.getConvention()==KNOTS)
 		{
@@ -214,29 +211,29 @@ MAGICS_NO_EXPORT void BaseDriver::renderWindFlag(const Flag &flag) const
 		const int old_currentColourIndex = currentLineStyle_;
 		currentLineStyle_ = setLineParameters(style,thickness);
 		renderPolyline2(line);
+
 		currentLineStyle_ = old_currentColourIndex;
-		
-		float tmp = 0.;
+		MFloat barbFraction = 0.;
 		int i = 0;
-		const float lengthY = setY(length * ratio);
-                const float multi =  setFlagY((flag.getHemisphere()==NORTH) ? (0.2*lengthY) : (-0.2*lengthY));
+		const MFloat lengthY = setY(length * ratio);
+		const MFloat barbHeight = setFlagY((flag.getHemisphere()==NORTH) ? (0.4*lengthY) : (-0.4*lengthY));
 		bool fl=false;
 
 		if(len<lev2) i++;
 		while(len>(lev1))
 		{
-			if(len > lev3)      {tmp=1.0; len-= slev3;fl=true;}
-			else if(len >= lev2) {tmp=1.0; len-= slev2;fl=false;}
-			else                 {tmp=0.5; len-= slev1;fl=false;}
+			if(len > lev3)       {barbFraction=1.0; len-= slev3;fl=true;}
+			else if(len >= lev2) {barbFraction=1.0; len-= slev2;fl=false;}
+			else                 {barbFraction=0.6; len-= slev1;fl=false;}
 
-			const float dx   = sin(RAD(30.)) * tmp * 0.2 * length; // angle of flags
-			const float step = i * (0.1*length);
+			const MFloat dx   = sin(RAD(30.)) * barbFraction * 0.2 * length; // angle of flags
+			const MFloat step = i * (0.1*length);
 
 			if(!fl)
 			{
 				line.clear();
 				line.push_back(PaperPoint( -(length-step),0.) );
-				line.push_back(PaperPoint( -(length-step+dx),multi*tmp) );
+				line.push_back(PaperPoint( -(length-step+dx),barbHeight*barbFraction) );
 
 				for_each(line.begin(),line.end(),rotate(angle,ratio) );
 				for_each(line.begin(),line.end(),translate(fla->point_) );
@@ -246,7 +243,7 @@ MAGICS_NO_EXPORT void BaseDriver::renderWindFlag(const Flag &flag) const
 			{
 				line.clear();
 				line.push_back(PaperPoint( -(length-step)   ,0.) );
-				line.push_back(PaperPoint( -(length-step),multi*tmp) );
+				line.push_back(PaperPoint( -(length-step),barbHeight*barbFraction) );
 				line.push_back(PaperPoint( -(length-step-(0.1*length)),0.) );
 				for_each(line.begin(),line.end(),rotate(angle,ratio) );
 				for_each(line.begin(),line.end(),translate(fla->point_) );
@@ -258,5 +255,12 @@ MAGICS_NO_EXPORT void BaseDriver::renderWindFlag(const Flag &flag) const
 		}// end while
 		++fla;
 	}// end for
-	if(markerHeight>0.) renderSymbols(origin);
+
+	if(markerHeight>0. && !magCompare(marker,"none"))
+	{
+		origin.setSymbol(marker);
+		origin.setHeight(flag.getOriginHeight());
+		origin.setColour(flag.getColour());
+		renderSymbols(origin);
+	}
 }
