@@ -21,7 +21,7 @@
 /*!
     \file CairoDriver.h
     \brief Definition of CairoDriver.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Mon Oct 15 20:49:32 2007
 */
@@ -29,11 +29,15 @@
 #ifndef _CairoDriver_H
 #define _CairoDriver_H
 
-#include <cairo.h>
 #include <BaseDriver.h>
 #include <CairoDriverAttributes.h>
 #include <XmlNode.h>
 
+#ifdef _AIX
+#undef HAVE_STDLIB_H
+#endif
+#include <cairo.h>
+
 namespace magics
 {
 
@@ -59,10 +63,10 @@ public:
 	{
 		if ( magCompare(node.name(), "png") ||
 		     magCompare(node.name(), "pdf") ||
-		     magCompare(node.name(), "cps") ||
-		     magCompare(node.name(), "csvg") ||
+		     magCompare(node.name(), "cairo_ps") ||
+		     magCompare(node.name(), "cairo_svg") ||
 		     magCompare(node.name(), "x") ||
-		     magCompare(node.name(), "ceps") )
+		     magCompare(node.name(), "cairo_eps") )
 		{
 			XmlNode basic = node;
 			basic.name("driver");
@@ -92,28 +96,27 @@ public:
 private:
 	MAGICS_NO_EXPORT void startPage() const;
         MAGICS_NO_EXPORT void endPage() const;
-	MAGICS_NO_EXPORT void startAdditionalPage(const StandaloneLayout& lay) const;
-        MAGICS_NO_EXPORT void endAdditionalPage() const;
 	MAGICS_NO_EXPORT void project(const Layout& lay) const;
 	MAGICS_NO_EXPORT void unproject() const;
 	MAGICS_NO_EXPORT void setupNewSurface() const;
 
-        MAGICS_NO_EXPORT void setNewLineWidth(const float) const;
+	MAGICS_NO_EXPORT void setNewLineWidth(const MFloat) const;
 	MAGICS_NO_EXPORT void setNewColour(const Colour &col) const;
 	MAGICS_NO_EXPORT void setColour(cairo_t* ct, const Colour &col) const;
-	MAGICS_NO_EXPORT int  setLineParameters(const LineStyle style, const float w) const;
+	MAGICS_NO_EXPORT int  setLineParameters(const LineStyle style, const MFloat w) const;
 
-	MAGICS_NO_EXPORT void renderPolyline(const int, float *, float *) const;
-	MAGICS_NO_EXPORT void renderPolyline2(const int n, float *x, float *y) const;
-	MAGICS_NO_EXPORT void renderSimplePolygon(const int, float *, float *) const;
+	MAGICS_NO_EXPORT void renderPolyline(const int, MFloat *, MFloat *) const;
+	MAGICS_NO_EXPORT void renderPolyline2(const int n, MFloat *x, MFloat *y) const;
+	MAGICS_NO_EXPORT void renderSimplePolygon(const int, MFloat *, MFloat *) const;
+	MAGICS_NO_EXPORT void renderSimplePolygon(const Polyline& line) const;
 	MAGICS_NO_EXPORT void renderText(const Text& text) const;
-	MAGICS_NO_EXPORT void circle(const float x, const float y, const float r, const int) const;
-	MAGICS_NO_EXPORT bool renderPixmap(float,float,float,float,int,int,unsigned char*,int,bool) const;
+	MAGICS_NO_EXPORT void circle(const MFloat x, const MFloat y, const MFloat r, const int) const;
+	MAGICS_NO_EXPORT bool renderPixmap(MFloat,MFloat,MFloat,MFloat,int,int,unsigned char*,int,bool) const;
 	MAGICS_NO_EXPORT bool renderCellArray(const Image& obj) const;
 	MAGICS_NO_EXPORT void renderImage(const ImportObject& obj) const;
 	MAGICS_NO_EXPORT void renderSymbols(const Symbol& symbol) const;
 	MAGICS_NO_EXPORT bool convertToPixmap(const string &fname, const GraphicsFormat format, const int reso,
-		             const float wx0, const float wy0,const float wx1,const float wy1) const;
+			     const MFloat wx0, const MFloat wy0,const MFloat wx1,const MFloat wy1) const;
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	void print(ostream&) const;
 	MAGICS_NO_EXPORT void debugOutput(const string &s) const;
@@ -125,17 +128,17 @@ private:
 	mutable cairo_surface_t* surface_;
 	mutable cairo_surface_t* tmp_surface_;
 
-	MAGICS_NO_EXPORT float projectX(const float x) const {return offsetX_+(x*coordRatioX_);}
-	MAGICS_NO_EXPORT float projectY(const float y) const {return offsetY_+(y*coordRatioY_);}
-	MAGICS_NO_EXPORT float setAngleY(const float y) const {return -y;}
-        MAGICS_NO_EXPORT float setSymbolY(const float y) const {return -y;}
-        MAGICS_NO_EXPORT float setFlagY(const float y) const {return -y;}
-	MAGICS_NO_EXPORT float setY(const float y) const {return y;}
+	MAGICS_NO_EXPORT MFloat projectX(const MFloat x) const {return offsetX_+(x*coordRatioX_);}
+	MAGICS_NO_EXPORT MFloat projectY(const MFloat y) const {return offsetY_+(y*coordRatioY_);}
+	MAGICS_NO_EXPORT MFloat setAngleY(const MFloat y) const {return -y;}
+	MAGICS_NO_EXPORT MFloat setSymbolY(const MFloat y) const {return -y;}
+	MAGICS_NO_EXPORT MFloat setFlagY(const MFloat y) const {return -y;}
+	MAGICS_NO_EXPORT MFloat setY(const MFloat y) const {return y;}
 
-	mutable float offsetX_;
-	mutable float offsetY_;
-	mutable stack<float> offsetsX_;
-	mutable stack<float> offsetsY_;
+	mutable MFloat offsetX_;
+	mutable MFloat offsetY_;
+	mutable stack<MFloat> offsetsX_;
+	mutable stack<MFloat> offsetsY_;
 
 	mutable string backend_;
 	mutable int dimensionXglobal_;
