@@ -18,7 +18,7 @@
 
 /* \file DriverManager.cc
    \brief Implementation of Drivermanager class.
-   \author Graphics Section, ECMWF
+   \author Meteorological Visualisation Section, ECMWF
 
    Started: Jan 2004
 
@@ -44,6 +44,8 @@ void DriverManager::print(ostream& out)  const
 
 void DriverManager::dispatch(BasicGraphicsObject* object) const
 {
+
+	if ( !object ) return;
 	for (const_iterator driver = begin(); driver != end(); ++driver)
 		if ( !(*(*driver)).disable() ) (*object).redisplay(*(*driver));
 }
@@ -71,6 +73,7 @@ void DriverManager::dispatch(void (MagicsEvent::*notify)(MagicsObserver&), Magic
 	for (const_iterator driver = begin(); driver != end(); ++driver)
 		if ( !(*(*driver)).disable() ) (event.*notify)(*(*driver));
 }
+
 void DriverManager::dispatch(BaseDriver::InputEventFunction mode, MtInputEvent* event) const
 {
 	for (const_iterator driver = begin(); driver != end(); ++driver)
@@ -89,6 +92,7 @@ void DriverManager::closeDrivers() const
 	for (; driver != end(); ++driver) 
 		if ( !(*(*driver)).disable() ) (*(*driver)).close();
 }
+
 /*
 void DriverManager::clearDrivers() const 
 {
@@ -102,6 +106,7 @@ void DriverManager::setDriversWidth(double width) const
 	for (const_iterator driver = begin(); driver != end(); ++driver) 
 		if ( !(*(*driver)).disable() ) (*(*driver)).setXDeviceLength(width); 
 }
+
 void DriverManager::setOutputWidth(double width) const
 {
 	for (const_iterator driver = begin(); driver != end(); ++driver) 
@@ -110,6 +115,7 @@ void DriverManager::setOutputWidth(double width) const
 			(*(*driver)).setWidth(width); 
 		}
 }
+
 void DriverManager::setDriversHeight(double height) const
 {
 	for (const_iterator driver = begin(); driver != end(); ++driver) 
