@@ -20,7 +20,7 @@
 
 /*! \file GDDriver.cc
     \brief Implementation of GDDriver.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Mon Oct 29 16:05:47 2007
 
@@ -61,6 +61,7 @@ GDDriver::GDDriver() : filename_("magics_output.gif"),currentFont_(""),
                       animated_(false),jpg_(false),gif_(false),png_(false),
                       clipping_(true),offsetX_(0),offsetY_(0)
 {
+  readFonts();
 }
 
 /*!
@@ -68,6 +69,7 @@ GDDriver::GDDriver() : filename_("magics_output.gif"),currentFont_(""),
 */
 GDDriver::~GDDriver() 
 {
+  relieveFonts();
 }
 
 /*!
@@ -83,12 +85,18 @@ void GDDriver::open()
 	currentPage_ = -1;
 	//needed for multithread apps!
 #ifndef MAGICS_GIF	
-	if(gdFontCacheSetup()!=0)  Log::error() << "GDDriver::readFonts() --> Cannot initialize font cache!\n";
+	if(gdFontCacheSetup()!=0)  MagLog::error() << "GDDriver::readFonts() --> Cannot initialize font cache!\n";
 #endif
-	setCMscale(30.);// cm -> pixel
+//	setCMscale(30.);// cm -> pixel
 
-	const float ratio = getYDeviceLength() / getXDeviceLength();
-	const int width   = getWidth();
+	string mbg_tmpl = mgb_template_;
+	MFloat ratio = getYDeviceLength() / getXDeviceLength();
+	int width   = width_;
+	if(!mbg_tmpl.empty())
+	{
+		setDimensionsFromBinary(mbg_tmpl,ratio,	width);
+	}
+	setCMscale(MFloat(width)/getXDeviceLength());
 	scaleFactor_      = .8;
 	dimensionXglobal_ = width;
 	dimensionYglobal_ = static_cast<int>(ratio*width);
@@ -109,7 +117,7 @@ void GDDriver::close()
 	{
 #ifndef MAGICS_GIF
 #ifndef MAGICS_GIF_ANIMATED
-		Log::warning() << "GIF and Animated GIF are not supported in this version (Too old version of GD library!)";
+		MagLog::warning() << "GIF and Animated GIF are not supported in this version (Too old version of GD library!)";
 		animated_ = false;
 		png_ = true;
 #else
@@ -119,7 +127,7 @@ void GDDriver::close()
 #endif
 #endif
 	}
-	if( jpg_ ) Log::info() << "JEPG is not a good format for scientific plots. Please consider using GIF or PNG.\n";
+	if( jpg_ ) MagLog::info() << "JEPG is not a good format for scientific plots. Please consider using GIF or PNG.\n";
 
 	currentPage_ = 0;
 	currentPageCount_ = 0;
@@ -138,7 +146,7 @@ void GDDriver::close()
   for formats with multiple output files a new file is set up.
 
  A new page in GD means a normally a new file.
- An exception are GIF animation.
+ An MagException are GIF animation.
 
  In this method the binary output file is opened.
  This includes the formation of the file name.
@@ -149,8 +157,8 @@ MAGICS_NO_EXPORT void GDDriver::startPage() const
 {
 	if(currentPage_==-1)
 	{
-		dimensionX_ = static_cast<float>(dimensionXglobal_);
-		dimensionY_ = static_cast<float>(dimensionYglobal_);
+		dimensionX_ = static_cast<float>(dimensionXglobal_-1);
+		dimensionY_ = static_cast<float>(dimensionYglobal_-1);
 		offsetY_ = dimensionY_;
 		coordRatioY_ = -1;
 	}
@@ -168,7 +176,7 @@ MAGICS_NO_EXPORT void GDDriver::startPage() const
 	}
 
 	// No TrueColor otherwise transparent background does not work
-	currentImage_ = gdImageCreateTrueColor(dimensionXglobal_+1,dimensionYglobal_+1);
+	currentImage_ = gdImageCreateTrueColor(dimensionXglobal_,dimensionYglobal_);
 	if(png_)
 	{	
 		gdImageAlphaBlending(currentImage_, 0);
@@ -189,17 +197,14 @@ MAGICS_NO_EXPORT void GDDriver::startPage() const
 #endif
 
 	int white = gdImageColorAllocate(currentImage_, 255, 255, 255);
-	if(white == -1) Log::error() << "GDDriver: Could NOT allocate background colour!\n";
+	if(white == -1) MagLog::error() << "GDDriver: Could NOT allocate background colour!\n";
 
-	gdImageFilledRectangle(currentImage_,0,0,dimensionXglobal_+1,dimensionYglobal_+1,white);
-	if(png_)
+	gdImageFilledRectangle(currentImage_,0,0,dimensionXglobal_,dimensionYglobal_,white);
+	if(png_ && transparent_)
 	{
 		gdImageColorTransparent(currentImage_, white);
-		if(getTransparent())
-		{
-			white = gdImageColorAllocateAlpha(currentImage_, 255, 255, 255,127);
-			gdImageFilledRectangle(currentImage_,0,0,dimensionXglobal_+1,dimensionYglobal_+1,white);
-		}
+		white = gdImageColorAllocateAlpha(currentImage_, 255, 255, 255,127);
+		gdImageFilledRectangle(currentImage_,0,0,dimensionXglobal_,dimensionYglobal_,white);
 	}
 	setNewColour(Colour(0,0,0)); // set colour to black as default
 }
@@ -211,7 +216,7 @@ MAGICS_NO_EXPORT void GDDriver::startPage() const
   files are closed.
 
  A page in GD means normally a output file.
- An exception are GIF animation for which are
+ An MagException are GIF animation for which are
  closed in close().
 
  In this method the binary output file is opened.
@@ -225,8 +230,8 @@ MAGICS_NO_EXPORT void GDDriver::endPage() const
 #ifdef MAGICS_GIF_ANIMATED
 	if(animated_)
 	{
-		if(currentPage_==0) gdImageGifAnimAdd(currentImage_,outFile_,1,0,0,getDelay(),1,NULL);
-		else                gdImageGifAnimAdd(currentImage_,outFile_,1,0,0,getDelay(),1,Images_[currentPage_-1]);
+		if(currentPage_==0) gdImageGifAnimAdd(currentImage_,outFile_,1,0,0,delay_,1,NULL);
+		else                gdImageGifAnimAdd(currentImage_,outFile_,1,0,0,delay_,1,Images_[currentPage_-1]);
 	}
 	else
 #endif
@@ -237,7 +242,7 @@ MAGICS_NO_EXPORT void GDDriver::endPage() const
 		if( gif_ )
 		{
 #ifndef MAGICS_GIF
-			Log::warning() << "GIF and Animated GIF are not supported in this version (Too old version of GD library!). You get a PNG instead.";
+			MagLog::warning() << "GIF and Animated GIF are not supported in this version (Too old version of GD library!). You get a PNG instead.";
 			gif_ = false;
 			png_ = true;
 #else
@@ -250,55 +255,36 @@ MAGICS_NO_EXPORT void GDDriver::endPage() const
 				printOutputName(fileName);
 			}
 			else
-				Log::error() << "GIF: cannot open file "<< fileName <<"!"<< endl;
+				MagLog::error() << "GIF: cannot open file "<< fileName <<"!"<< endl;
 #endif
 		}
 
 		if( png_ )
 		{
 			fileName = getFileName("png",currentPageCount_);
-			string::size_type pos = fileName.find_last_of(".");
-			if(pos != string::npos)
-			{
-				const string f = fileName.substr(0,pos);
-				const string t = fileName.substr(pos);
-				if(t==".gif") fileName.replace(pos,4,".png"); 
-			}
 			outFile_ = fopen(fileName.c_str(),"wb");
 			if(outFile_)
 			{
-				if(getTransparent())
-				{
-					int white = gdImageColorAllocate(currentImage_, 255, 255, 255);
-					gdImageColorTransparent(currentImage_, white);
-				}
 				gdImagePng(currentImage_,outFile_);
 				fclose(outFile_);
 				printOutputName(fileName);
 			}
 			else
-				Log::error() << "PNG: cannot open file "<< fileName <<"!"<< endl;
+				MagLog::error() << "PNG: cannot open file "<< fileName <<"!"<< endl;
 		}
 
 		if( jpg_ )
 		{
 			fileName = getFileName("jpg",currentPageCount_);
-			string::size_type pos = fileName.find_last_of(".");
-			if(pos != string::npos)
-			{
-				const string f = fileName.substr(0,pos);
-				const string t = fileName.substr(pos);
-				if(t==".gif") fileName.replace(pos,4,".jpg"); 
-			}
 			outFile_ = fopen(fileName.c_str(),"wb");
 			if(outFile_)
 			{
-				gdImageJpeg(currentImage_,outFile_, getQuality()); // quality = -1 is default
+				gdImageJpeg(currentImage_,outFile_, quality_); // quality = -1 is default
 				fclose(outFile_);
 				printOutputName(fileName);
 			}
 			else
-				Log::error() << "JPEG: cannot open file "<< fileName <<"!"<< endl;
+				MagLog::error() << "JPEG: cannot open file "<< fileName <<"!"<< endl;
 		}
 		outFile_ = 0;
 	}
@@ -315,8 +301,7 @@ MAGICS_NO_EXPORT void GDDriver::endPage() const
 */
 MAGICS_NO_EXPORT void GDDriver::project(const magics::Layout& layout) const
 {
-newLayout_ = true;
-if(newPage_) newPage_=false;
+	if(newPage_) newPage_=false;
 
 	// push current state
 	dimensionStack_.push(dimensionX_);
@@ -394,11 +379,13 @@ MAGICS_NO_EXPORT void GDDriver::setNewColour(const Colour &colour) const
   \param w width of the line
 
 */
-MAGICS_NO_EXPORT int GDDriver::setLineParameters(const LineStyle linestyle, const float w) const
+MAGICS_NO_EXPORT int GDDriver::setLineParameters(const LineStyle linestyle, const MFloat w) const
 {
-	int width = 1;
-	if      (w > 4.) width=static_cast<int>(w/4.);
-	else if (w > 1.) width=2;
+//	int width = 1;
+//	if      (w > 3.) width=static_cast<int>(w*.25);
+//	else 
+	int width=w*.25;
+	if(width<1) width=1;
 
 	if(width==1 && linestyle==M_SOLID) return gdAntiAliased;
 
@@ -462,7 +449,7 @@ MAGICS_NO_EXPORT int GDDriver::setLineParameters(const LineStyle linestyle, cons
 /*!
   \brief renders polylines
 
-  This method renders a polyline given as two float arrays. The two 
+  This method renders a polyline given as two MFloat arrays. The two
   arrays given as X and Y values have to be at least the length of
   <i>n</i>. All values beyond <i>n</i> will be ignored. The style is
   determined by what is described in the current LineStyle.
@@ -472,17 +459,17 @@ MAGICS_NO_EXPORT int GDDriver::setLineParameters(const LineStyle linestyle, cons
   \param x array of x values
   \param y array of y values
 */
-MAGICS_NO_EXPORT void GDDriver::renderPolyline(const int n, float *x, float *y) const
+MAGICS_NO_EXPORT void GDDriver::renderPolyline(const int n, MFloat *x, MFloat *y) const
 {
 	if(n<2) return;
 
-	float old_x = projectX(x[0]);
-	float old_y = projectY(y[0]);
+	MFloat old_x = projectX(x[0]);
+	MFloat old_y = projectY(y[0]);
 
 	for(int l = 1; l<n; l++)
 	{
-		const float xx = projectX(x[l]);
-		const float yy = projectY(y[l]);
+		const MFloat xx = projectX(x[l]);
+		const MFloat yy = projectY(y[l]);
 
 		gdImageLine(currentImage_,(int)old_x,(int)old_y,(int)xx,(int)yy,currentLineStyleIndex_);
 
@@ -505,19 +492,19 @@ MAGICS_NO_EXPORT void GDDriver::renderPolyline(const int n, float *x, float *y)
   \param x array of x values
   \param y array of y values
 */
-MAGICS_NO_EXPORT void GDDriver::renderPolyline2(const int n, float* x, float* y) const
+MAGICS_NO_EXPORT void GDDriver::renderPolyline2(const int n, MFloat* x, MFloat* y) const
 {
 	if(n != 2) return;
 
-	float *xx = x;
-	float *yy = y;
+	MFloat *xx = x;
+	MFloat *yy = y;
 
-	const float ix0 = *xx;
+	const float ix0 = (float)*xx;
 	xx++;
-	const float iy0 = *yy;
+	const float iy0 = (float)*yy;
 	yy++;
-	const float dx0 = *xx;
-	const float dy0 = *yy;
+	const float dx0 = (float)*xx;
+	const float dy0 = (float)*yy;
 	gdImageLine(currentImage_,ix0,iy0,dx0,dy0,currentLineStyleIndex_);
 }
 
@@ -528,33 +515,63 @@ MAGICS_NO_EXPORT void GDDriver::renderPolyline2(const int n, float* x, float* y)
   determined by what is described in the current LineStyle.
 
   \sa setLineParameters()
-  \param n number of points
+  \param nn number of points
   \param x array of x values
   \param y array of y values
 */
-MAGICS_NO_EXPORT void GDDriver::renderSimplePolygon(const int nn, float* x, float* y) const
+MAGICS_NO_EXPORT void GDDriver::renderSimplePolygon(const int nn, MFloat* x, MFloat* y) const
 {
-	int i;
 	int n = nn;
 	if ( (x[n-1] == x[0]) && (y[n-1] == y[0]) )  n--;
 
-	float *xx = x;
-	float *yy = y;
+	MFloat *xx = x;
+	MFloat *yy = y;
 
 	gdPoint *xyP = new gdPoint[n];
 	gdPoint *xy = xyP;
 
-	for( i = 0; i<n;xy++,i++)
+	for(int i = 0; i<n;xy++,i++)
 	{
-		xy->x = static_cast<int>(projectX(*(xx++)));
-		xy->y = static_cast<int>(projectY(*(yy++)));
+		xy->x = projectX(*(xx++));
+		xy->y = projectY(*(yy++));
 	}
 
 	currentLineStyle_ = currentLineStyleIndex_;
 	gdImageFilledPolygon(currentImage_,xyP,n,currentLineStyleIndex_);
-	delete [] xyP;	
+	delete [] xyP;
 }
 
+
+/*!
+  \brief renders a filled polygon
+
+  This method renders a filled polygon. The style is
+  determined by what is described in the current LineStyle.
+
+  \sa setLineParameters()
+  \param line polyline to be filled
+*/
+MAGICS_NO_EXPORT void GDDriver::renderSimplePolygon(const Polyline& line) const
+{
+	const unsigned int n = line.size();
+	if(n<3 || (currentColour_==Colour("none")) ) return;
+	line.getShading()->draw(*this);
+	setNewColour(line.getFillColour());
+
+	gdPoint *xy = new gdPoint[n];
+	for(unsigned int l = 0; l<n; l++)
+	{
+		const PaperPoint& pp = line.get(l);
+		xy[l].x = projectX(pp.x());
+		xy[l].y = projectY(pp.y());
+	}
+	currentLineStyle_ = currentLineStyleIndex_;
+	gdImageFilledPolygon(currentImage_,xy,n,currentLineStyleIndex_);
+	delete [] xy;
+}
+
+
+
 /*!
   \brief renders text strings
 
@@ -572,6 +589,7 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 {
 #ifdef MAGICS_TTF
 	if(text.empty()) return;
+	const string MAGPLUS_HOME=getEnvVariable("MAGPLUS_HOME");
 
 	const vector<NiceText>& niceT = text.getNiceText();
 	if(niceT.empty() && (text.empty()) ) return;
@@ -589,15 +607,15 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 	const enum Justification horizontal = text.getJustification();
 	const enum VerticalAlign vertical   = text.getVerticalAlign();
 
-	float HA = 0.; // MLEFT default
+	MFloat HA = 0.1; // MLEFT default
 	if (horizontal==MCENTRE)     HA = -0.5;
 	else if (horizontal==MRIGHT) HA = -1.;
 
-	float VA = 0.;
-	if (vertical==MBASE) VA = - PGNDESENT;
-	else if (vertical==MTOP)    VA = 1 - PNGASCENT - PGNDESENT;
-	else if (vertical==MHALF)   VA = 0.5 - PGNDESENT;
-	else if (vertical==MBOTTOM) VA = 0.;
+	MFloat VA = 0.;
+	if (vertical==MBASE) VA = + PGNDESENT;
+	else if (vertical==MTOP)    VA = 1. + PNGASCENT + PGNDESENT;
+	else if (vertical==MHALF)   VA = 0.5 + PGNDESENT;
+	else if (vertical==MBOTTOM) VA = -0.1;
 
 	string textString;
 
@@ -609,19 +627,21 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 	for(;niceText<niceTextEnd;niceText++)
 	{
 		const MagFont magfont	= (*niceText).font();
-		const string style	= (magfont.style()=="") ? "normal": magfont.style();
-//		const bool underlined	= (magfont.style()=="underline") ? true : false; 
-		const string font	= magfont.name()+"_"+style;
-		const string lowFont	= lowerCase(font);
-		fontMapIter iter	= FontMap_.find(lowFont);
+		const std::set<string>& styles = magfont.styles();
+		string style = "";
+		if(styles.find("bold") != styles.end()) style = "bold";
+		if(styles.find("italic") != styles.end()) style += "italic";
+		if(style == "") style = "normal";
+		const string lowFont = lowerCase(magfont.name()+"_"+style);
+		fontMapIter iter = FontMap_.find(lowFont);
 
-		string ttf = getEnvVariable("MAGPLUS_HOME") + "/" + MAGICS_TTF_PATH;
+		string ttf = MAGPLUS_HOME + "/" + MAGICS_TTF_PATH;
 		if(iter!=FontMap_.end())
 			ttf += iter->second.ttf_filename;
 		else
 		{
 			ttf += FontMap_["sansserif_normal"].ttf_filename; // if not found get default
-			Log::warning() << "GDDriver: Font "<< font << " is not registered! Default font is used."<< endl;
+			MagLog::warning() << "GDDriver: Font "<< lowFont << " is not registered! Default font is used."<< endl;
 		}
 
 		// test if font files exists
@@ -629,10 +649,10 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 		if(!fontfile)
 		{
 			const string new_font = lowerCase( "sansserif_"+style);
-			Log::warning() << "GDDriver (pretty): TTF font file "<< ttf <<" could not be found! Font "<< new_font<< " is used instead!"<< endl;
+			MagLog::warning() << "GDDriver (pretty): TTF font file "<< ttf <<" could not be found! Font "<< new_font<< " is used instead!"<< endl;
 			fontMapIter iter = FontMap_.find(new_font);
 
-			ttf = getEnvVariable("MAGPLUS_HOME") + "/" + MAGICS_TTF_PATH;
+			ttf = MAGPLUS_HOME + "/" + MAGICS_TTF_PATH;
 			if(iter!=FontMap_.end())
 				ttf += iter->second.ttf_filename;
 		}
@@ -642,7 +662,7 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 
 		int bbx[8];
 		char *err = gdImageStringFT(NULL,&bbx[0],0,(char *)ttf.c_str(),fontSize,0.,10,10,(char*)(*niceText).text().c_str());
-		if(err) Log::error() << "GDDriver::renderText() -> Could not determine text bounding box for >"<<(*niceText).text()<<"< !\n";
+		if(err) MagLog::error() << "GDDriver::renderText() -> Could not determine text bounding box for >"<<(*niceText).text()<<"< !\n";
 
 		text_length += (bbx[2] - bbx[0]);
 	 }
@@ -659,20 +679,22 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 //		const int length =textString.length()+1;
 
 		const MagFont magfont = (*niceText).font();
-		const string style	= (magfont.style()=="") ? "normal": magfont.style();
-//		const bool underlined = (magfont.style()=="underline") ? true : false; 
-		const string font	= magfont.name()+"_"+style;
-		const string lowFont	= lowerCase(font);
-		fontMapIter iter	= FontMap_.find(lowFont);
+		const std::set<string>& styles = magfont.styles();
+		string style = "";
+		if(styles.find("bold") != styles.end()) style = "bold";
+		if(styles.find("italic") != styles.end()) style += "italic";
+		if(style == "") style = "normal";
+		const string lowFont = lowerCase(magfont.name()+"_"+style);
+		fontMapIter iter = FontMap_.find(lowFont);
 		const float fontSize = scaleFactor_ * convertCM(magfont.size());
 
-		string ttf = getEnvVariable("MAGPLUS_HOME") + "/" + MAGICS_TTF_PATH;
+		string ttf = MAGPLUS_HOME + "/" + MAGICS_TTF_PATH;
 		if(iter!=FontMap_.end())
 			ttf += iter->second.ttf_filename;
 		else
 		{
 			ttf += FontMap_["sansserif_normal"].ttf_filename; // if not found get default
-			Log::warning() << "GDDriver: Font "<< font << " is not registered! Default font is used."<< endl;
+			MagLog::warning() << "GDDriver: Font "<< lowFont << " is not registered! Default font is used."<< endl;
 		}
 	
 		// test if font files exists
@@ -682,10 +704,10 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 		  if(!fontfile)
 		  {
 			const string new_font = lowerCase( "sansserif_"+style);
-			Log::warning() << "GDDriver (pretty): TTF font file "<< ttf <<" could not be found! Font "<< new_font<< " is used instead!"<< endl;
+			MagLog::warning() << "GDDriver (pretty): TTF font file "<< ttf <<" could not be found! Font "<< new_font<< " is used instead!"<< endl;
 			fontMapIter iter = FontMap_.find(new_font);
 
-			ttf = getEnvVariable("MAGPLUS_HOME") + "/" + MAGICS_TTF_PATH;
+			ttf = MAGPLUS_HOME + "/" + MAGICS_TTF_PATH;
 			if(iter!=FontMap_.end())
 				ttf += iter->second.ttf_filename;
 		  }
@@ -702,7 +724,7 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 	
 		if (err)
 		{
-			Log::error() << "GDDriver: gdImageStringFT (prettyText) -> "<< err <<" in "<<ttf<< endl;
+			MagLog::error() << "GDDriver: gdImageStringFT (prettyText) -> "<< err <<" in "<<ttf<< endl;
 			return;
 		}
 		else
@@ -714,7 +736,7 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 
 		const float E1x = 0;
 		const float E1y = H*VA;
-		const float E1xcsE1yss =  E1x*cs + E1y*ss + offset; 
+		const float E1xcsE1yss =  E1x*cs + E1y*ss + offset;
 		const float E1xssE1ycs = -E1x*ss + E1y*cs;
 		offset += W;
 
@@ -740,12 +762,12 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 				points[3].x = bbx[6];  points[3].y = bbx[7];
 				gdImageFilledPolygon(currentImage_, points, 4, white);
 			}
-			else Log::error() << "GDDriver: gdImageStringFT -> "<< err <<" -> NO blanking!!!"<< endl;
+			else MagLog::error() << "GDDriver: gdImageStringFT -> "<< err <<" -> NO blanking!!!"<< endl;
 		}
 		err = gdImageStringFT(currentImage_,&bbx[0],currentColourIndex_,(char *)ttf.c_str(),fontSize,LastAngle_,x0,y0,(char *)(*niceText).text().c_str());
 		if (err)
 		{
-			Log::error() << "GDDriver: gdImageStringFT -> "<< err << endl;
+			MagLog::error() << "GDDriver: gdImageStringFT -> "<< err << endl;
 			gdImageString(currentImage_,gdFontGetLarge(),x0,y0,(unsigned char*)(*niceText).text().c_str(),currentColourIndex_);
 		}
 		delete [] err;
@@ -753,7 +775,7 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
 	}// endfor all text segments
 	offsetX_ = offsetsX_.top();offsetsX_.pop();
 #else
-	Log::warning() << "GDDriver: No TTF fonts support in Magics++!" << endl;
+	MagLog::warning() << "GDDriver: No TTF fonts support in Magics++!" << endl;
 	return;
 #endif
 }
@@ -772,7 +794,7 @@ MAGICS_NO_EXPORT void GDDriver::renderText(const Text& text) const
   \param r Radius of circle
   \param s Style which determines how the circle is shaded
 */
-MAGICS_NO_EXPORT void GDDriver::circle(const float x, const float y, const float r, const int s) const
+MAGICS_NO_EXPORT void GDDriver::circle(const MFloat x, const MFloat y, const MFloat r, const int s) const
 {
 	const int rx = static_cast<int>(r*2.);
 	const int ry = static_cast<int>(r*2.);
@@ -808,10 +830,10 @@ MAGICS_NO_EXPORT void GDDriver::circle(const float x, const float y, const float
   \param w width of pixmap
   \param h height of pixmap
   \param pixmap contents
-  \param landscape says if contents is landscape
+  \param alpha says if array is transparent
 
 */
-MAGICS_NO_EXPORT bool GDDriver::renderPixmap(float x0,float y0,float x1,float y1,
+MAGICS_NO_EXPORT bool GDDriver::renderPixmap(MFloat x0,MFloat y0,MFloat x1,MFloat y1,
                                             int w,int h,unsigned char* pixmap,int, bool alpha) const
 {
 	unsigned char *p = pixmap;	
@@ -908,7 +930,7 @@ MAGICS_NO_EXPORT bool GDDriver::renderCellArray(const Image& image) const
 */
 MAGICS_NO_EXPORT void GDDriver::debugOutput(const string &s) const
 {
-	Log::debug() << s << endl;
+	MagLog::debug() << s << endl;
 }
 
 /*!
@@ -922,10 +944,10 @@ void GDDriver::print(ostream& out)  const
 
 //! Method to plot logo
 /*!
- Logo needs special treatment - much better quality when imported as GIF!
+ MagLogo needs special treatment - much better quality when imported as GIF!
  \sa convertToPixmap1()
 */
-MAGICS_NO_EXPORT void GDDriver::renderLogo(float x, float y) const
+MAGICS_NO_EXPORT void GDDriver::renderMagLogo(MFloat x, MFloat y) const
 {
 	GraphicsFormat format = PNG;
 	const string logofile = getEnvVariable("MAGPLUS_HOME") + MAGPLUS_PATH_TO_SHARE_ + "ecmwf_logo.png";
@@ -934,14 +956,14 @@ MAGICS_NO_EXPORT void GDDriver::renderLogo(float x, float y) const
 
 //! Method to plot symbols
 /*!
- Needs special treatment of Logo. Much better quality when imported as GIF!
+ Needs special treatment of MagLogo. Much better quality when imported as GIF!
 */
 MAGICS_NO_EXPORT void GDDriver::renderSymbols(const Symbol& symbol) const
 {
 	debugOutput("Start GDDriver Symbols");
 	
 	if(symbol.getSymbol()=="logo_ecmwf")
-		renderLogo(projectX(symbol[0].x()),projectY(symbol[0].y()));
+		renderMagLogo(projectX(symbol[0].x()),projectY(symbol[0].y()));
 	else 
 		BaseDriver::renderSymbols(symbol);
 }
