@@ -20,7 +20,7 @@
 
 /*! \file KMLDriver.cc
     \brief Implementation of KMLDriver.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Thu Oct 18 18:41:52 2007
 
@@ -32,6 +32,9 @@
 #include <Image.h>
 #include <Symbol.h>
 #include <Layer.h>
+#include <Arrow.h>
+#include <Flag.h>
+
 
 //! For generating KMZ files
 extern "C"{
@@ -39,6 +42,9 @@ extern "C"{
 #include "minizip/zip.h"
 #define MAXFILENAME 256
 #define WRITEBUFFERSIZE 16384
+#include <cstdio>    // BUFSIZ
+#include <fcntl.h>   // open
+#include <unistd.h>
 }
 
 
@@ -48,18 +54,46 @@ extern "C"{
 
 #ifdef MAGICS_CAIRO
 #include <cairo.h>
+#include <CairoDriver.h>
 #endif
 
+// #include <boost/filesystem.hpp>
+
 using namespace magics;
 
+bool copy_file(string from, string to)
+{
+    char buf[BUFSIZ];
+    size_t size;
+
+    int source = open(from.c_str(), O_RDONLY, 0);
+    int dest   = open(  to.c_str(), O_WRONLY | O_CREAT, 0644);
+
+    if(source >= 0 && dest >= 0)
+    {
+      while ((size = read(source, buf, BUFSIZ)) > 0)
+        write(dest, buf, size);
+      close(source);
+      close(dest);
+    }
+    else
+    {
+      MagLog::error() << "KMZ copy file> file "<< from << " could NOT be copied to "<< to << endl;
+      close(source);
+      close(dest);
+      return false;
+    }
+
+    return true;
+}
+
 /*!
   \brief Constructor
 */
 KMLDriver::KMLDriver() : currentTimeBegin_(""),currentTimeEnd_(""),kml_placemark_(false),
                          polyline_begin_(true),polygon_begin_(true),MultiGeometrySet_(false),layer_(false),
-			 render_(false),ecmwf_logo_(false)
+			 render_(true),ecmwf_logo_(false)
 {
-	kmz_ = getKmz();
 }
 
 /*!
@@ -74,35 +108,33 @@ KMLDriver::~KMLDriver()
 */
 void KMLDriver::open()
 {
+	kmz_ = kmz_;
 	currentPage_ = 0;
 
-	if(fileName_=="")
-	{
-		fileName_ = getFileName("kml");
-	}
+	fileName_ = getFileName("kml");
 	if(kmz_) fileName_ = "doc.kml";
 
 	pFile_.open(fileName_.c_str());
 	if(!pFile_)
 	{
-		Log::fatal() << "KMLDriver::open() --> Cannot open KML output file: " << fileName_ << "!\n";
-		exit(1);
+		MagLog::fatal() << "KMLDriver::open() --> Cannot open KML output file: " << fileName_ << "!\n";
+		terminate();
 	}
 	pFile_	<< "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n"
 		<< "<kml xmlns=\"http://www.opengis.net/kml/2.2\" \n"
 		<< "     xmlns:atom=\"http://www.w3.org/2005/Atom\">\n"
-		<< "<Document>\n"<< " <name>"<<getTitle()<<"</name>\n"
+		<< "<Document>\n"<< " <name>"<<title_<<"</name>\n"
 		<< " <open>1</open>\n";
-	if(!getAuthor().empty()) pFile_ << " <atom:author><atom:name>"<<getAuthor()<<"</atom:name></atom:author>\n";
-	if(!getLink().empty())   pFile_ << " <atom:link href=\""<<getLink()<<"\" />\n";
+	if(!author_.empty()) pFile_ << " <atom:author><atom:name>"<<author_<<"</atom:name></atom:author>\n";
+	if(!link_.empty())   pFile_ << " <atom:link href=\""<<link_<<"\" />\n";
 	pFile_	<< " <description>\n"
-		<< " <![CDATA["<<getDescription()<<"]]>\n"
+		<< " <![CDATA["<<description_<<"]]>\n"
 		<< " </description>\n"
 		<< " <LookAt>\n"
-		<< "\t<longitude>"<<getLongitude()<<"</longitude>\n"
-		<< "\t<latitude>"<<getLatitude()<<"</latitude>\n"
-		<< "\t<range>"<<getRange()*1000.<<"</range>\n"
-		<< "\t<tilt>"<<getTilt()<<"</tilt>\n"
+		<< "\t<longitude>"<<longitude_<<"</longitude>\n"
+		<< "\t<latitude>"<<latitude_<<"</latitude>\n"
+		<< "\t<range>"<<range_*1000.<<"</range>\n"
+		<< "\t<tilt>"<<tilt_<<"</tilt>\n"
 		<< "\t<heading>0</heading>\n"    // always 0 = north!
 		<< "\t<altitudeMode>absolute</altitudeMode>\n"    // possibly "relativeToGround"
 		<< " </LookAt>\n"
@@ -123,26 +155,26 @@ void KMLDriver::open()
 */
 void KMLDriver::close()
 {
-//	endPage();
+    if (kml_placemark_) closePlacemark();
 
-	if (kml_placemark_) closePlacemark();
+    // Wind icon file
+    const string iconfile="magics_kml_icons.png";
+    const string path=getEnvVariable("MAGPLUS_HOME") + MAGPLUS_PATH_TO_SHARE_ + iconfile;
+//    boost::filesystem::path from(path);
+//    boost::filesystem::path to(iconfile);
+//    boost::filesystem::copy_file(from, to);
+    bool is_copied = copy_file(path,iconfile);
+    if(is_copied) kml_output_resource_list_.push_back(iconfile);
 
 	if(ecmwf_logo_)
 	{
 	   const string logofilename = "kml_logo.png";
 	   const string logofile = getEnvVariable("MAGPLUS_HOME") + MAGPLUS_PATH_TO_SHARE_ + logofilename;
-	   const string cmd = "cp " + logofile + " kml_logo.png";
-	   if(getDebug()) Log::dev()<< "KMLDriver.close() > Execute command: "<< cmd<< endl;
-	   int status = system(cmd.c_str());
-	   if(status)
-	   {
-		Log::warning() << "KMLDriver.close() > Failed to copy logo file - Logo will be missing." << endl;
-	   }
-	   else
-	   {
-		kml_output_resource_list_.push_back(logofilename);
+           is_copied = copy_file(logofile,logofilename);
+           if(is_copied) kml_output_resource_list_.push_back(logofilename);
+	   
 		pFile_	<< "<ScreenOverlay id=\"logo\">\n"
-			<< "<name>Logo</name>\n"
+			<< "<name>MagLogo</name>\n"
 			<< "<Icon>\n"
 			<< " <href>"<<logofilename<<"</href>\n"
 			<< "</Icon>\n"
@@ -150,7 +182,6 @@ void KMLDriver::close()
 			<< "<screenXY x=\"0\" y=\"0\" xunits=\"fraction\" yunits=\"fraction\"/>\n"
 			<< "<size x=\"-1\" y=\"0.1\" xunits=\"fraction\" yunits=\"fraction\"/>\n"
 			<< "</ScreenOverlay>\n";
-	   }
 	}
 	pFile_ << "</Document>\n</kml>\n";
 	pFile_.close();
@@ -167,7 +198,6 @@ void KMLDriver::close()
            if (zf == 0)
            {
                printf("ERROR opening zip file %s\n",fileName_.c_str());
-               err= ZIP_ERRNO;
            }
            else
 	   {
@@ -175,14 +205,14 @@ void KMLDriver::close()
 		void* buf = (void*)malloc(size_buf);
 		if (buf==0)
 		{
-			Log::error() <<"Error allocating memory for KMZ generation!"<<endl;
+			MagLog::error() <<"Error allocating memory for KMZ generation!"<<endl;
 			return;
 		}
 		stringarray::iterator it    = kml_output_resource_list_.begin();
 		stringarray::iterator itend = kml_output_resource_list_.end();
 		for(; it != itend; it++)
 		{
-			if(getDebug()) Log::dev()<< "KMLDriver.close() > Start adding file " <<  *it << " to KMZ file.\n";
+			if(debug_) MagLog::dev()<< "KMLDriver.close() > Start adding file " <<  *it << " to KMZ file.\n";
 			FILE * fin;
 			int size_read;
 
@@ -191,13 +221,13 @@ void KMLDriver::close()
 			err = zipOpenNewFileInZip(zf,filename, 0, 0, 0, 0, 0, 0, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
 
 			if(err != ZIP_OK)
-				Log::error() << "Could NOT open ... KMZ file!" << endl;
+				MagLog::error() << "Could NOT open KMZ file "<< filename << endl;
 			else
 			{
 			  fin = fopen(filename,"rb");
 			  if(fin==0)
 			  {
-			     Log::fatal() << "Open file "<<filename<<" to be added to KMZ FAILED!"<< endl;
+			     MagLog::fatal() << "Open file "<<filename<<" to be added to KMZ FAILED!"<< endl;
 			     return;
 			  }
 			  else
@@ -208,7 +238,7 @@ void KMLDriver::close()
 				if (size_read < size_buf)
 				  if (feof(fin)==0)
 				  {
-					Log::error() << "Could NOT add "<<(*it) << endl;
+					MagLog::error() << "Could NOT add "<<(*it) << endl;
 					err = ZIP_ERRNO;
 				  }
 
@@ -217,7 +247,7 @@ void KMLDriver::close()
 				  err = zipWriteInFileInZip(zf,buf,size_read);
 				  if (err<0)
 				  {
-					Log::error() << "Could NOT write KMZ file "<< fileName_<< endl;
+					MagLog::error() << "Could NOT write KMZ file "<< fileName_<< endl;
 				  }
 				}
 			     } while ((err==ZIP_OK) && (size_read>0));
@@ -226,22 +256,17 @@ void KMLDriver::close()
 				fclose(fin);
 			}
 
-			if (err<0)
-				err=ZIP_ERRNO;
-			else
-			{
-				err = zipCloseFileInZip(zf);
-				if (err!=ZIP_OK)
-					printf("error in closing xxxx in the zipfile\n");
-			}
-//				delete [] filename;
+			err = zipCloseFileInZip(zf);
+			if (err!=ZIP_OK)
+				MagLog::error() << "Could NOT close KMZ file "<< fileName_<< endl;
+//			delete [] filename;
 		}
 		free(buf);
 
 		err = zipClose(zf,0);
 		if (err != ZIP_OK)
-			printf("error in closing %s\n",fileName_.c_str());
-		else if (!getDebug())
+			MagLog::error() << "Could NOT close KMZ file "<< fileName_<< endl;
+		else if (!debug_)
 		{
 			stringarray::iterator it = kml_output_resource_list_.begin();
 			stringarray::iterator itend = kml_output_resource_list_.end();
@@ -308,7 +333,43 @@ MAGICS_NO_EXPORT void KMLDriver::endPage() const
 */
 MAGICS_NO_EXPORT void KMLDriver::project(const magics::Layout& layout) const
 {
-Log::dev() << " KMLDriver::project("<<layout.id()<<")"<< endl;
+//MagLog::dev() << " KMLDriver::project("<<layout.id()<<")"<< endl;
+}
+
+MAGICS_NO_EXPORT void KMLDriver::redisplay(const magics::LegendLayout& layout) const
+{
+#ifdef MAGICS_CAIRO
+    //redisplay((const Layout&) legend);
+    const string filename = "legend.png";
+    cairo_surface_t* surface_ = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, layout.width(), layout.height());
+    cairo_t* cr_ = cairo_create(surface_);
+//    if(magCompare(transparent_,"off"))
+    {
+       cairo_set_source_rgb (cr_, 1.0, 1.0, 1.0); /* white */
+    }
+//    else
+//    {
+//       cairo_set_source_rgba (cr_, 1.0, 1.0, 1.0, 0.0); /* transparent */
+//    }
+    cairo_paint (cr_);
+    
+//    CairoDriver cD;
+//    cD.redisplay((const Layout&) layout);
+
+    cairo_surface_write_to_png(surface_, filename.c_str());
+    kml_output_resource_list_.push_back(filename);
+    pFile_  << "<ScreenOverlay id=\"legend\">\n"
+            << "<name>Legend</name>\n"
+            << "<Icon>\n"
+            << " <href>"<<filename<<"</href>\n"
+            << "</Icon>\n"
+            << "<overlayXY x=\"0\" y=\"0\" xunits=\"fraction\" yunits=\"fraction\"/>\n"
+            << "<screenXY x=\"0\" y=\"0\" xunits=\"fraction\" yunits=\"fraction\"/>\n"
+            << "<size x=\"-1\" y=\"0.1\" xunits=\"fraction\" yunits=\"fraction\"/>\n"
+            << "</ScreenOverlay>\n";
+#else
+    MagLog::warning() << " KMLDriver> legend could NOT be generated - Cairo support needs to be enabled!"<< endl;
+#endif
 }
 
 /*!
@@ -319,16 +380,37 @@ Log::dev() << " KMLDriver::project("<<layout.id()<<")"<< endl;
 
   \sa Layer
 */
-MAGICS_NO_EXPORT void KMLDriver::redisplay(const Layer& layer) const
+MAGICS_NO_EXPORT void KMLDriver::redisplay(const StaticLayer& layer) const
 {
-  if(!magCompare("coastlines",layer.name()) || getCoastlines() )
+//cout <<"KML staticlayer > "<<layer.name() <<" - "<<layer.kmlTimeBegin()<<endl;
+  if(!magCompare("coastlines",layer.name()) ||coastlines_ )
   {
 //	if(!magCompare(currentLayer_,layer.name()))
 //	{
-		currentLayer_     = layer.name();
+		currentLayer_     = (layer.name().empty()) ? "StaticLayer" : layer.name();
 		currentTimeBegin_ = layer.kmlTimeBegin();
 		currentTimeEnd_   = layer.kmlTimeEnd();
-cout << "layer "<<layer.name()<<" "<<layer.kmlTimeBegin()<< endl;		
+//cout << "layer "<<layer.name()<<" "<<layer.kmlTimeBegin()<< endl;
+		newLayer();
+		layer.visit(*this);
+		closeLayer();
+//	}
+//	else
+//		layer.visit(*this);
+  }
+}
+
+MAGICS_NO_EXPORT void KMLDriver::redisplay(const StepLayer& layer) const
+{
+//  cout <<"KML StepLayer> "<<layer.name() <<" - "<<layer.kmlTimeBegin()<<endl;
+  if(!magCompare("coastlines",layer.name()) || coastlines_ )
+  {
+//	if(!magCompare(currentLayer_,layer.name()))
+//	{
+		currentLayer_     = (layer.name().empty()) ? "Step" : layer.name();
+		currentTimeBegin_ = layer.kmlTimeBegin();
+		currentTimeEnd_   = layer.kmlTimeEnd();
+//cout << "layer "<<layer.name()<<" "<<layer.kmlTimeBegin()<< endl;
 		newLayer();
 		layer.visit(*this);
 		closeLayer();
@@ -338,12 +420,23 @@ cout << "layer "<<layer.name()<<" "<<layer.kmlTimeBegin()<< endl;
   }
 }
 
+/*!
+   \brief gets ignored in KML
+   
+   \sa BaseDriver::redisplay(const NoDataLayer&)
+ */
+MAGICS_NO_EXPORT void KMLDriver::redisplay(const NoDataLayer& layer) const
+{
+//   cout << "KML::NoDataLayer> "<<layer.name()<< endl;
+}
+
 /*!
   \brief open new layer
 
 */
 MAGICS_NO_EXPORT void KMLDriver::newLayer() const
 {
+//cout << "KML::newLayer()> "<<currentLayer_<< endl;
 	if (kml_placemark_) closePlacemark();
 
 	debugOutput("Start Layer - "+currentLayer_);
@@ -366,6 +459,7 @@ MAGICS_NO_EXPORT void KMLDriver::newLayer() const
 */
 MAGICS_NO_EXPORT void KMLDriver::closeLayer() const
 {
+//  cout << "KML::closeLayer()> "<<currentLayer_<< endl;
 	if (kml_placemark_) closePlacemark();
 
 	layer_=false;
@@ -413,9 +507,9 @@ MAGICS_NO_EXPORT void KMLDriver::setNewColour(const Colour &colour) const
 
   \sa setLineParameters()
 */
-MAGICS_NO_EXPORT void KMLDriver::setNewLineWidth(const float width) const
+MAGICS_NO_EXPORT void KMLDriver::setNewLineWidth(const MFloat width) const
 {
-	currentLineWidth_ = (width<1) ? currentLineWidth_=1. : width;
+	currentLineWidth_ = (width<1) ? 1. : width;
 }
 
 /*!
@@ -430,18 +524,18 @@ MAGICS_NO_EXPORT void KMLDriver::setNewLineWidth(const float width) const
   param w width of the line
 
 */
-MAGICS_NO_EXPORT int KMLDriver::setLineParameters(const LineStyle , const float w) const
+MAGICS_NO_EXPORT int KMLDriver::setLineParameters(const LineStyle , const MFloat w) const
 {
 	setNewLineWidth(w);
 
-	Log::debug() << "KMLDriver::setLineParameters needs implementing." <<endl;
+//	MagLog::debug() << "KMLDriver::setLineParameters needs implementing." <<endl;
 	return 0;
 }
 
 /*!
   \brief renders polylines
 
-  This method renders a polyline given as two float arrays. The two
+  This method renders a polyline given as two MFloat arrays. The two
   arrays given as X and Y values have to be at least the length of
   <i>n</i>. All values beyond <i>n</i> will be ignored. The style is
   determined by what is described in the current LineStyle.
@@ -451,7 +545,7 @@ MAGICS_NO_EXPORT int KMLDriver::setLineParameters(const LineStyle , const float
   \param x array of x values
   \param y array of y values
 */
-MAGICS_NO_EXPORT void KMLDriver::renderPolyline(const int n, float *x, float *y) const
+MAGICS_NO_EXPORT void KMLDriver::renderPolyline(const int n, MFloat *x, MFloat *y) const
 {
   if(render_ && n > 1 )
   {
@@ -464,20 +558,20 @@ MAGICS_NO_EXPORT void KMLDriver::renderPolyline(const int n, float *x, float *y)
 			pFile_ << "<name>"<<currentLayer_<<"</name>\n";
 			if(!currentTimeBegin_.empty())
 			{
-			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
-			   {
-				pFile_	<< "<TimeStamp>\n"
-					<< " <when>"<<currentTimeBegin_<<"</when>\n"
- 					<< "</TimeStamp>\n"
-					<< "<styleUrl>#hiker-icon</styleUrl>\n";
-			   }
-			   else
+//			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
+//			   {
+//				pFile_	<< "<TimeStamp>\n"
+//					<< " <when>"<<currentTimeBegin_<<"</when>\n"
+// 					<< "</TimeStamp>\n"
+//					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+//			   }
+//			   else
 			   {
 				pFile_	<< "<TimeSpan>\n"
 					<< " <begin>"<<currentTimeBegin_<<"</begin>\n"
 					<< " <end>"<<currentTimeEnd_<<"</end>\n"
- 					<< "</TimeSpan>\n"
-					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+ 					<< "</TimeSpan>\n";
+					//<< "<styleUrl>#hiker-icon</styleUrl>\n";
 			   }
 			}
 			pFile_	<< "<description><![CDATA["<<currentLayer_<<"]]></description>\n";
@@ -489,9 +583,11 @@ MAGICS_NO_EXPORT void KMLDriver::renderPolyline(const int n, float *x, float *y)
 		const int r = (int)currentColour_.red()*255;
 		const int g = (int)currentColour_.green()*255;
 		const int b = (int)currentColour_.blue()*255;
-		const int a = (int)(getTransparency() * 2.55);
+		const int a = (int)(transparency_ * 2.55);
 
-  		pFile_	<< "<Style>\n<LineStyle>\n"
+		pFile_	<< "<!-- r:"<<r<<" g:"<<g<<" b:"<<b <<" -->"<< endl;
+
+		pFile_	<< "<Style>\n<LineStyle>\n"
 			<< "\t<color>" <<hex;
 			if(a>15)	pFile_ <<a;
 			else		pFile_ <<"0"<< a;
@@ -511,13 +607,14 @@ MAGICS_NO_EXPORT void KMLDriver::renderPolyline(const int n, float *x, float *y)
 
 	pFile_	<< "<LineString>\n"
 		<< " <extrude>0</extrude>\n"
-		<< " <altitudeMode>relativeToGround</altitudeMode>\n"
-		<< " <tessellate>1</tessellate>\n"
+	//	<< " <altitudeMode>relativeToGround</altitudeMode>\n"
+		<< " <altitudeMode>clampToGround</altitudeMode>\n"
+		<< " <tessellate>0</tessellate>\n"
 		<< " <coordinates>\n";
 
 	for(int is=0;is<n;is++)
 	{
-		pFile_ <<"\t"<< x[is]<<","<<y[is]<<","<<getHeight()*1000<<"\n";
+		pFile_ <<"\t"<< x[is]<<","<<y[is]<<","<<height_*1000<<"\n";
 	}
 	pFile_ << " </coordinates>\n</LineString>\n";
   }
@@ -535,9 +632,8 @@ MAGICS_NO_EXPORT void KMLDriver::renderPolyline(const int n, float *x, float *y)
   \param x array of x values
   \param y array of y values
 */
-MAGICS_NO_EXPORT void KMLDriver::renderPolyline2(const int , float* , float* ) const
+MAGICS_NO_EXPORT void KMLDriver::renderPolyline2(const int , MFloat* , MFloat* ) const
 {
-	Log::debug() << "KMLDriver::renderPolyline2 needs implementing." <<endl;
 }
 
 /*!
@@ -548,28 +644,54 @@ MAGICS_NO_EXPORT void KMLDriver::renderPolyline2(const int , float* , float* ) c
 
   \sa setLineParameters()
   \param n number of points
-  \param x array of x values
-  \param y array of y values
+  \param xx array of x values
+  \param yy array of y values
 */
-MAGICS_NO_EXPORT void KMLDriver::renderSimplePolygon(const int n, float* xx, float* yy) const
+MAGICS_NO_EXPORT void KMLDriver::renderSimplePolygon(const int n, MFloat* xx, MFloat* yy) const
 {
 	if (!render_) return;
 	if (kml_placemark_) closePlacemark();
-	pFile_	<< "<Placemark>\n";
-	if(layer_)
-	{
-		pFile_ << "<name>"<<currentLayer_<<"</name>\n"
-		       << "<description><![CDATA["<<currentLayer_<<"]]></description>\n";
-	}
-//	    << "<altitudeMode>clampToGround</altitudeMode>\n"
-	pFile_<< "<visibility>1</visibility>\n<open>0</open>\n";
-
-//	kml_placemark_=true;
 
 	const int r = (int)(currentColour_.red()*255.);
 	const int g = (int)(currentColour_.green()*255.);
 	const int b = (int)(currentColour_.blue()*255.);
-	const int a = (int)(getTransparency() * 2.55);
+	if (r*g*b < 0) return;
+//pFile_	<< "<!-- r:"<<r<<" g:"<<g<<" b:"<<b <<" -->"<< endl;
+	pFile_	<< "<Placemark>\n";
+//	if(layer_)
+//	{
+//		pFile_ << "<name>"<<currentLayer_<<"</name>\n"
+//		       << "<description><![CDATA["<<currentLayer_<<"]]></description>\n";
+//	}
+//	    << "<altitudeMode>clampToGround</altitudeMode>\n"
+	if(layer_)
+	{
+		pFile_ << "<name>"<<currentLayer_<<"</name>\n";
+		if(!currentTimeBegin_.empty())
+		{
+//			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
+//			   {
+//				pFile_	<< "<TimeStamp>\n"
+//					<< " <when>"<<currentTimeBegin_<<"</when>\n"
+// 					<< "</TimeStamp>\n"
+//					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+//			   }
+//			   else
+		   {
+			pFile_	<< "<TimeSpan>\n"
+				<< " <begin>"<<currentTimeBegin_<<"</begin>\n"
+				<< " <end>"<<currentTimeEnd_<<"</end>\n"
+				<< "</TimeSpan>\n";
+		   }
+		}
+		pFile_	<< "<description><![CDATA["<<currentLayer_<<"]]></description>\n";
+	}
+
+	pFile_<< "<visibility>1</visibility>\n<open>0</open>\n";
+
+//	kml_placemark_=true;
+
+	const int a = (int)(transparency_ * 2.55);
 
 	pFile_	<< "<Style>\n<PolyStyle>\n"
 		<< "\t<color>" <<hex;
@@ -584,7 +706,7 @@ MAGICS_NO_EXPORT void KMLDriver::renderSimplePolygon(const int n, float* xx, flo
 	pFile_	<< "</color>\n" << dec
 		<< "\t<fill>1</fill>\n</PolyStyle>\n"
 		<< "<LineStyle>\n"
-		<< "\t<width>"<<currentLineWidth_<<"</width>\n"
+		<< "\t<width>"<<2<<"</width>\n"
 		<< "\t<color>" <<hex;
 		if(a>15)	pFile_ <<a;
 		else		pFile_ <<"0"<< a;
@@ -605,14 +727,14 @@ MAGICS_NO_EXPORT void KMLDriver::renderSimplePolygon(const int n, float* xx, flo
 	       << " <extrude>1</extrude>\n"
 	//       << " <altitudeMode>relativeToGround</altitudeMode>\n"
 	       << " <altitudeMode>clampToGround</altitudeMode>\n"
-	       << " <tessellate>1</tessellate>\n"
+	       << " <tessellate>0</tessellate>\n"
 	       << "  <outerBoundaryIs>\n"
 	       << "   <LinearRing>\n"
 	       << "    <coordinates>\n";
 
 	for(int it=0;it<n;it++)
 	{
-		pFile_ <<"\t"<< xx[it] <<","<< yy[it] <<","<<getHeight()*1000<<"\n";
+		pFile_ <<"\t"<< xx[it] <<","<< yy[it] <<","<<height_*1000<<"\n";
 	}
 
 	pFile_ << "    </coordinates>\n"
@@ -623,13 +745,32 @@ MAGICS_NO_EXPORT void KMLDriver::renderSimplePolygon(const int n, float* xx, flo
 	pFile_ << "</MultiGeometry>\n</Placemark>";
 }
 
+void KMLDriver::renderSimplePolygon(const Polyline& line) const
+{
+	const unsigned int n = line.size();
+	if(n<3) return;
+	Colour tmpcol = currentColour_;
+	setNewColour(line.getFillColour());
+
+	MFloat *x = new MFloat[n];
+	MFloat *y = new MFloat[n];
+	for(unsigned int i=0;i<n;i++)
+	{
+		const PaperPoint& pp = line.get(i);
+		x[i] = pp.x();
+		y[i] = pp.y();
+	}
+	renderSimplePolygon(line.size(),x,y);
+	delete [] x;
+	delete [] y;
+	setNewColour(tmpcol);
+}
+
 /*!
   \brief renders text strings
 
   This method renders given text strings.
 
-  \note As of version 2.0 there are two forms of describing text in Text.
-
   \sa Text
   \param text object containing the strings and their description
 */
@@ -652,9 +793,8 @@ MAGICS_NO_EXPORT void KMLDriver::renderText(const Text& text) const
   \param r Radius of circle
   \param s Style which determines how the circle is shaded
 */
-MAGICS_NO_EXPORT void KMLDriver::circle(const float , const float , const float , const int ) const
+MAGICS_NO_EXPORT void KMLDriver::circle(const MFloat , const MFloat , const MFloat , const int ) const
 {
-	Log::debug() << "KMLDriver::circle needs implementing." <<endl;
 }
 
 /*!
@@ -673,17 +813,18 @@ MAGICS_NO_EXPORT void KMLDriver::circle(const float , const float , const float
   param pixmap contents
 
 */
-MAGICS_NO_EXPORT bool KMLDriver::renderPixmap(float x0,float y0,float x1,float y1,
+MAGICS_NO_EXPORT bool KMLDriver::renderPixmap(MFloat x0,MFloat y0,MFloat x1,MFloat y1,
                                             int w,int h,unsigned char* pixmap,int, bool) const
 {
+  debugOutput("Start renderPixmap");
   if(render_)
   {
 	if (kml_placemark_) closePlacemark();
 #ifndef MAGICS_CAIRO
 #ifndef MAGICS_RASTER
-	Log::warning() << "Image import is not implemented for the used driver!!!" << endl; return false;
+	MagLog::warning() << "Image import is not implemented for the used driver!!!" << endl; return false;
 #else
-Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo backend."<< endl;
+MagLog::dev()<< "KML driver Image import uses GD -> for better results use Cairo backend."<< endl;
 	stringstream out;
 	out << 15 * kml_output_resource_list_.size();
 	const string filename = "KML_overlay_"+out.str()+"png";
@@ -693,14 +834,14 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 	{
 		if(!currentTimeBegin_.empty())
 		{
-			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
-			   {
-				pFile_	<< "<TimeStamp>\n"
-					<< " <when>"<<currentTimeBegin_<<"</when>\n"
- 					<< "</TimeStamp>\n"
-					<< "<styleUrl>#hiker-icon</styleUrl>\n";
-			   }
-			   else
+//			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
+//			   {
+//				pFile_	<< "<TimeStamp>\n"
+//					<< " <when>"<<currentTimeBegin_<<"</when>\n"
+// 					<< "</TimeStamp>\n"
+//					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+//			   }
+//			   else
 			   {
 				pFile_	<< "<TimeSpan>\n"
 					<< " <begin>"<<currentTimeBegin_<<"</begin>\n"
@@ -712,7 +853,7 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 		pFile_	<< "<name><![CDATA[Layer: "<<currentLayer_<<"]]></name>\n";
 	}
 
-	const int a = (int)(getTransparency() * 2.55);
+	const int a = (int)(transparency_ * 2.55);
 	pFile_ << "<styleUrl>#hiker-icon</styleUrl>\n"
 		<< "<color>"<<hex;
 		if(a>15)	pFile_ <<a;
@@ -749,8 +890,11 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 			const int r = (int) *(p++);
 			const int g = (int) *(p++);
 			const int b = (int) *(p++);
-			const int col = gdImageColorResolveAlpha(im,r,g,b,0);
-			gdImageSetPixel(im, w, h, col);
+			if(r*g*b >= 0.)
+			{
+			  const int col = gdImageColorResolveAlpha(im,r,g,b,0);
+			  gdImageSetPixel(im, w, h, col);
+			}
 		}
 	}
 	gdImageDestroy(im);
@@ -774,14 +918,14 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 	{
 		if(!currentTimeBegin_.empty())
 		{
-			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
-			   {
-				pFile_	<< "<TimeStamp>\n"
-					<< " <when>"<<currentTimeBegin_<<"</when>\n"
- 					<< "</TimeStamp>\n"
-					<< "<styleUrl>#hiker-icon</styleUrl>\n";
-			   }
-			   else
+//			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
+//			   {
+//				pFile_	<< "<TimeStamp>\n"
+//					<< " <when>"<<currentTimeBegin_<<"</when>\n"
+// 					<< "</TimeStamp>\n"
+//					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+//			   }
+//			   else
 			   {
 				pFile_	<< "<TimeSpan>\n"
 					<< " <begin>"<<currentTimeBegin_<<"</begin>\n"
@@ -793,7 +937,7 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 		pFile_	<< "<name><![CDATA[Time: "<<currentLayer_<<"]]></name>\n";
 	}
 
-	const int a = (int)(getTransparency() * 2.55);
+	const int a = (int)(transparency_ * 2.55);
 	pFile_	<< "<styleUrl>#hiker-icon</styleUrl>\n"
 		<< "<color>"<<hex;
 		if(a>15)	pFile_ <<a;
@@ -831,26 +975,26 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 //	cairo_restore (cr_);
 
 	unsigned char *p = pixmap;
-	const float dx =  (x1 - x0)/w;
-	const float dy = -(y1 - y0)/h;   // Minus needed for Y axis correction
+	const MFloat dx =  (x1 - x0)/w;
+	const MFloat dy = -(y1 - y0)/h;   // Minus needed for Y axis correction
 
-	const float X0 = x0;
-	const float Y0 = y0;
+	const MFloat X0 = x0;
+	const MFloat Y0 = y0;
 
 	for(int i=h-1;i>=0;i--)
 	{
 		for(int j=0;j<w; x0+=dx,j++)
 		{
-			const float r = *(p++);
-			const float g = *(p++);
-			const float b = *(p++);
+			const MFloat r = *(p++);
+			const MFloat g = *(p++);
+			const MFloat b = *(p++);
 
-			if( (r*g*b) > 0)
+			if( (r*g*b) >= 0.)
 			{
 				cairo_set_source_rgba(cr_,r,g,b,0.5);
 
-				const float x0 = X0+(j*dx)+.5;
-				const float y0 = Y0+(i*dy)+.5;
+				const MFloat x0 = X0+(j*dx)+.5;
+				const MFloat y0 = Y0+(i*dy)+.5;
 				cairo_rectangle (cr_, x0,y0,dx,-dy);
 				cairo_stroke_preserve(cr_);
 				cairo_fill (cr_);
@@ -866,6 +1010,7 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 	pFile_	<< "</GroundOverlay>\n";
 #endif
    }
+	debugOutput("End renderPixmap");
 	return true;
 }
 
@@ -881,14 +1026,15 @@ Log::dev()<< "KML driver Image import uses GD -> for better results use Cairo ba
 */
 MAGICS_NO_EXPORT bool KMLDriver::renderCellArray(const Image& image) const
 {
+   debugOutput("Start renderCellArray");
    if(render_)
    {
 	if (kml_placemark_) closePlacemark();
 #ifndef MAGICS_CAIRO
 #ifndef MAGICS_RASTER
-	Log::warning() << "Image import is not implemented for the used driver!!!" << endl; return false;
+	MagLog::warning() << "Image import is not implemented for the used driver!!!" << endl; return false;
 #else
-Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< endl;
+MagLog::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< endl;
 	stringstream out;
 	out << currentPage_;
 	const string layer_name = (!currentLayer_.empty()) ? currentLayer_ : "default_page";
@@ -899,14 +1045,14 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 	{
 		if(!currentTimeBegin_.empty())
 		{
-			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
-			   {
-				pFile_	<< "<TimeStamp>\n"
-					<< " <when>"<<currentTimeBegin_<<"</when>\n"
- 					<< "</TimeStamp>\n"
-					<< "<styleUrl>#hiker-icon</styleUrl>\n";
-			   }
-			   else
+//			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
+//			   {
+//				pFile_	<< "<TimeStamp>\n"
+//					<< " <when>"<<currentTimeBegin_<<"</when>\n"
+// 					<< "</TimeStamp>\n"
+//					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+//			   }
+//			   else
 			   {
 				pFile_	<< "<TimeSpan>\n"
 					<< " <begin>"<<currentTimeBegin_<<"</begin>\n"
@@ -918,7 +1064,7 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 		pFile_	<< "<name><![CDATA[Layer: "<<currentLayer_<<"]]></name>\n";
 	}
 
-	const int a = (int)(getTransparency() * 2.55);
+	const int a = (int)(transparency_ * 2.55);
 	pFile_	<< "<color>"<<hex;
 		if(a>15)	pFile_ <<a;
 		else		pFile_ <<"0"<< a << dec;
@@ -959,8 +1105,11 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 			const int b = static_cast<int>(lt[c].blue()*255.);
 			const int a = static_cast<int>(lt[c].alpha()*127.);
 
-			const int col = gdImageColorResolveAlpha(im,r,g,b,a);
-			gdImageSetPixel(im, j, i, col);
+			if(r*g*b >= 0.)
+			{
+			  const int col = gdImageColorResolveAlpha(im,r,g,b,a);
+			  gdImageSetPixel(im, j, i, col);
+			}
 		  }// point has colour
 		}
 	}
@@ -994,14 +1143,14 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 	{
 		if(!currentTimeBegin_.empty())
 		{
-			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
-			   {
-				pFile_	<< "<TimeStamp>\n"
-					<< " <when>"<<currentTimeBegin_<<"</when>\n"
- 					<< "</TimeStamp>\n"
-					<< "<styleUrl>#hiker-icon</styleUrl>\n";
-			   }
-			   else
+//			   if(magCompare(currentTimeBegin_,currentTimeEnd_))
+//			   {
+//				pFile_	<< "<TimeStamp>\n"
+//					<< " <when>"<<currentTimeBegin_<<"</when>\n"
+// 					<< "</TimeStamp>\n"
+//					<< "<styleUrl>#hiker-icon</styleUrl>\n";
+//			   }
+//			   else
 			   {
 				pFile_	<< "<TimeSpan>\n"
 					<< " <begin>"<<currentTimeBegin_<<"</begin>\n"
@@ -1013,7 +1162,7 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 		pFile_	<< "<name><![CDATA[Layer: "<<currentLayer_<<"]]></name>\n";
 	}
 
-	const int a = (int)(getTransparency() * 2.55);
+	const int a = (int)(transparency_ * 2.55);
 	pFile_	<< "<color>"<<hex;
 		if(a>15)	pFile_ <<a;
 		else		pFile_ <<"0"<< a << dec;
@@ -1049,8 +1198,8 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 	cairo_paint (cr_);
 //	cairo_restore (cr_);
 
-	const float dx = 1.;
-	const float dy = 1.;
+	const MFloat dx = 1.;
+	const MFloat dy = 1.;
 
 	for (int i=h-1;i>=0;i--)
 	{
@@ -1061,10 +1210,10 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 
  		  if(!(lt[c]=="undefined"))
 		  {
-			float r = lt[c].red();
-			float g = lt[c].green();
-			float b = lt[c].blue();
-			float a = lt[c].alpha();
+			MFloat r = lt[c].red();
+			MFloat g = lt[c].green();
+			MFloat b = lt[c].blue();
+			MFloat a = lt[c].alpha();
 
 			if( (r*g*b>1.) || (r*g*b<0.) )
 			{
@@ -1072,14 +1221,14 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 				g = 1.;
 				b = 1.;
 				a = 0.;
-//				Log::info()<< "PostScriptDriver-> Cellshading colour not defined in table! Colour index: "<<c<<endl;
+//				MagLog::info()<< "PostScriptDriver-> Cellshading colour not defined in table! Colour index: "<<c<<endl;
 //    PostScript will always 'overpaint' anything below missing data!!!!
 //
 			}
 			cairo_set_source_rgba(cr_,r,g,b,a);
 
-			const float x0 = (j*dx);
-			const float y0 = (i*dy);
+			const MFloat x0 = (j*dx);
+			const MFloat y0 = (i*dy);
 			cairo_rectangle (cr_, x0,y0,dx,-dy);
 			cairo_stroke_preserve(cr_);
 			cairo_fill (cr_);
@@ -1097,13 +1246,14 @@ Log::dev()<< "KML driver uses GD -> for better results use Cairo backend."<< end
 	pFile_	<< "</GroundOverlay>\n";
 #endif
    }
-	return true;
+   debugOutput("End renderCellArray");
+   return true;
 }
 
 
 //! Method to plot symbols
 /*!
- Needs special treatment of Logo. The logo is added on close().
+ Needs special treatment of MagLogo. The logo is added on close().
 
  \sa close
 */
@@ -1115,6 +1265,159 @@ MAGICS_NO_EXPORT void KMLDriver::renderSymbols(const Symbol& symbol) const
 //		BaseDriver::renderSymbols(symbol);
 }
 
+
+MAGICS_NO_EXPORT void KMLDriver::renderWindArrow(const Arrow &arrow) const
+{
+	const unsigned int arrPoNo = arrow.size();
+	if(arrPoNo<1) return;
+
+//	const ArrowPosition pos = arrow.getArrowPosition();
+
+    setNewColour(arrow.getColour());
+    const int r = (int)(currentColour_.red()*255.);
+    const int g = (int)(currentColour_.green()*255.);
+    const int b = (int)(currentColour_.blue()*255.);
+    const int a = (int)(transparency_ * 2.55);
+
+    Arrow::const_iterator arr = arrow.begin();
+	for(unsigned int pts=0;pts<arrPoNo;pts++)
+	{
+        pFile_  << "<Placemark>\n"
+                << "<Style>\n"
+                << "<IconStyle>\n"
+                << "<color>" <<hex;
+        if(a>15)    pFile_ <<a;
+        else        pFile_ <<"0"<< a;
+        if(b>15)    pFile_ <<b;
+        else        pFile_ <<"0"<< b;
+        if(g>15)    pFile_ <<g;
+        else        pFile_ <<"0"<< g;
+        if(r>15)    pFile_ <<r;
+        else        pFile_ <<"0"<< r;
+        pFile_  << "</color>\n" << dec
+                << "<heading>"<<90.+(arr->angle()*-57.29578)<<"</heading>\n"
+                << "<scale>"<<arr->norm()*0.2<<"</scale>\n"
+                << "<Icon>\n"
+                << "<href>magics_kml_icons.png</href>\n"
+                << "<x>0</x>\n"
+                << "<y>576</y>\n"
+                << "<w>64</w>\n"
+                << "<h>64</h>\n"
+                << "</Icon>\n"
+                << "</IconStyle>\n"
+                << "</Style>\n"
+                << "<Point>\n"
+                << "<coordinates>"<<arr->point_.x()<<","<<arr->point_.y()<<",0</coordinates>\n"
+                << "</Point>\n"
+                << "</Placemark>" << endl;
+	  ++arr;
+	}
+}
+
+
+MAGICS_NO_EXPORT void KMLDriver::renderWindFlag(const Flag &flag) const
+{
+/*
+ *	const MFloat thickness = (flag.getThickness()==1) ? 1 : 2.*flag.getThickness();
+	const LineStyle style = flag.getStyle();
+	const MFloat length = convertCM(flag.getLength() / coordRatioX_ );
+	const string marker = flag.getOriginMarker();
+	const MFloat markerHeight = convertCM(flag.getOriginHeight() *.5 / coordRatioX_);
+*/
+	setNewColour(flag.getColour());
+//	const unsigned int flaPoNo = flag.size();
+//	Flag::const_iterator fla = flag.begin();
+
+/*	Symbol origin;
+	if(markerHeight>0.)
+	{
+		origin.setSymbol(marker);
+		origin.setHeight(flag.getOriginHeight());
+		origin.setColour(flag.getColour());
+	}
+*/
+/*	for(unsigned int pts=0;pts<flaPoNo;pts++)
+	{
+		vector<PaperPoint> line;
+		const MFloat angle = setAngleY(fla->angle());
+		line.push_back(PaperPoint(-markerHeight,0.) );
+		line.push_back(PaperPoint(-length,0.) );
+		const MFloat ratio = (coordRatioY_==0) ? 1. : coordRatioX_/coordRatioY_;
+		for_each(line.begin(),line.end(),rotate(angle,ratio) );
+		for_each(line.begin(),line.end(),translate(fla->point_) );
+		MFloat len = fla->norm();
+
+		MFloat lev1 = 1.25;  // 2.5
+		MFloat lev2 = 3.75;  // 5.
+		MFloat lev3 = 23.;  // 25.
+		MFloat slev1 = 2.5;
+		MFloat slev2 = 5.;
+		MFloat slev3 = 25.;
+		
+		if(flag.getConvention()==KNOTS)
+		{
+			len *= 1.94384466;
+			lev1 = 3.;
+			lev2 = 7.;
+			lev3 = 47.;
+			slev1 = 5.;
+			slev2 = 10.;
+			slev3 = 50.;
+		}
+
+		if(markerHeight>0.) origin.push_back(PaperPoint(fla->point_) );
+
+		const int old_currentColourIndex = currentLineStyle_;
+		currentLineStyle_ = setLineParameters(style,thickness);
+		renderPolyline2(line);
+		currentLineStyle_ = old_currentColourIndex;
+		
+		MFloat tmp = 0.;
+		int i = 0;
+		const MFloat lengthY = setY(length * ratio);
+		const MFloat multi =  setFlagY((flag.getHemisphere()==NORTH) ? (0.2*lengthY) : (-0.2*lengthY));
+		bool fl=false;
+
+		if(len<lev2) i++;
+		while(len>(lev1))
+		{
+			if(len > lev3)      {tmp=1.0; len-= slev3;fl=true;}
+			else if(len >= lev2) {tmp=1.0; len-= slev2;fl=false;}
+			else                 {tmp=0.5; len-= slev1;fl=false;}
+
+			const MFloat dx   = sin(RAD(30.)) * tmp * 0.2 * length; // angle of flags
+			const MFloat step = i * (0.1*length);
+
+			if(!fl)
+			{
+				line.clear();
+				line.push_back(PaperPoint( -(length-step),0.) );
+				line.push_back(PaperPoint( -(length-step+dx),multi*tmp) );
+
+				for_each(line.begin(),line.end(),rotate(angle,ratio) );
+				for_each(line.begin(),line.end(),translate(fla->point_) );
+				renderPolyline2(line);
+			}
+			else
+			{
+				line.clear();
+				line.push_back(PaperPoint( -(length-step)   ,0.) );
+				line.push_back(PaperPoint( -(length-step),multi*tmp) );
+				line.push_back(PaperPoint( -(length-step-(0.1*length)),0.) );
+				for_each(line.begin(),line.end(),rotate(angle,ratio) );
+				for_each(line.begin(),line.end(),translate(fla->point_) );
+				renderSimplePolygon(line);
+				fl=false;
+				i++;
+			}
+			i++;
+		}// end while
+		++fla;
+	}// end for
+	//if(markerHeight>0.) renderSymbols(origin);
+*/}
+
+
 /*!
   \brief prints debug output
 
@@ -1126,7 +1429,7 @@ MAGICS_NO_EXPORT void KMLDriver::renderSymbols(const Symbol& symbol) const
 */
 MAGICS_NO_EXPORT void KMLDriver::debugOutput(const string &s) const
 {
-	if(getDebug()) pFile_ << "<!-- "<<s<<" -->\n";
+	if(debug_) pFile_ << "<!-- "<<s<<" -->\n";
 }
 
 /*!
