@@ -19,7 +19,7 @@
  ******************************** LICENSE ********************************/
 
 /*!
-    \file MgQPathIem.h
+    \file MgQPathItem.h
     \brief Definition of MgQPathItem.
     \author Graphics Section, ECMWF
 
@@ -31,9 +31,13 @@
 
 #include <QGraphicsPathItem>
 
+#include "MgQ.h"
+
 class MgQPathItem : public QGraphicsPathItem
 {
 public:
+	enum {Type = MgQ::PathItemType}; 
+	
 	MgQPathItem(const QPainterPath& ,QGraphicsItem* parent = 0);
 	~MgQPathItem();
 	
@@ -41,11 +45,10 @@ public:
 	QRectF boundingRect() const;
 	void paint(QPainter *, const QStyleOptionGraphicsItem *,
                 QWidget *widget=0);
+	int type() const {return Type;}
 
 protected:
 	float boundingRectSize_;
-
 };	
 
-
-#endif 
+#endif
