@@ -21,7 +21,7 @@
 /*!
     \file SVGDriver.h
     \brief Definition of SVGDriver.
-    \author Graphics Section, ECMWF
+    \author Meteorological Visualisation Section, ECMWF
 
     Started: Fri Oct 26 20:58:21 2007
 */
@@ -77,37 +77,41 @@ public:
 private:
 	MAGICS_NO_EXPORT void startPage() const;
 	MAGICS_NO_EXPORT void endPage() const;
-	MAGICS_NO_EXPORT void startAdditionalPage(const StandaloneLayout& lay) const;
-	MAGICS_NO_EXPORT void endAdditionalPage() const;
 	MAGICS_NO_EXPORT void project(const Layout& lay) const;
 	MAGICS_NO_EXPORT void unproject() const;
-	MAGICS_NO_EXPORT void newLayer(const Layer&) const;
-	MAGICS_NO_EXPORT void closeLayer(const Layer&) const;
+	MAGICS_NO_EXPORT void redisplay(const StaticLayer& layer) const;
+	MAGICS_NO_EXPORT void redisplay(const StepLayer& layer) const;
+	MAGICS_NO_EXPORT void redisplay(const NoDataLayer& layer) const;
+	MAGICS_NO_EXPORT void newLayer() const;
+	MAGICS_NO_EXPORT void closeLayer() const;
 //	MAGICS_NO_EXPORT void renderInteractiveBegin(const InteractiveBegin&) const;
 //	MAGICS_NO_EXPORT void renderInteractiveEnd(const InteractiveEnd&) const;
 
-	MAGICS_NO_EXPORT void setNewLineWidth(const float) const;
+	MAGICS_NO_EXPORT void setNewLineWidth(const MFloat) const;
 	MAGICS_NO_EXPORT void setNewColour(const Colour &col) const;
-	MAGICS_NO_EXPORT int setLineParameters(const LineStyle style, const float w) const;
+	MAGICS_NO_EXPORT int setLineParameters(const LineStyle style, const MFloat w) const;
+//	MAGICS_NO_EXPORT void printLine(const Polyline &line) const;
 
-	MAGICS_NO_EXPORT void renderPolyline(const int, float *, float *) const;
-	MAGICS_NO_EXPORT void renderPolyline2(const int n, float *x, float *y) const;
-	MAGICS_NO_EXPORT void renderSimplePolygon(const int, float *, float *) const;
+//	MAGICS_NO_EXPORT void redisplay(const Polyline& line) const;
+	MAGICS_NO_EXPORT void renderPolyline(const int, MFloat *, MFloat *) const;
+	MAGICS_NO_EXPORT void renderPolyline2(const int n, MFloat *x, MFloat *y) const;
+	MAGICS_NO_EXPORT void renderSimplePolygon(const int, MFloat *, MFloat *) const;
+	MAGICS_NO_EXPORT void renderSimplePolygon(const Polyline& lines) const;
 	MAGICS_NO_EXPORT void renderText(const Text& text) const;
-	MAGICS_NO_EXPORT bool renderPixmap(float,float,float,float,int,int,unsigned char*,int,bool) const;
+	MAGICS_NO_EXPORT bool renderPixmap(MFloat,MFloat,MFloat,MFloat,int,int,unsigned char*,int,bool) const;
 	MAGICS_NO_EXPORT void renderSymbols(const Symbol& symbol) const;
 	MAGICS_NO_EXPORT bool renderCellArray(const Image& obj) const;
 	MAGICS_NO_EXPORT void renderImage(const ImportObject& obj) const;
-	MAGICS_NO_EXPORT void circle(const float x, const float y, const float r, const int) const;
+	MAGICS_NO_EXPORT void circle(const MFloat x, const MFloat y, const MFloat r, const int) const;
 
 	//! Method to print string about this class on to a stream of type ostream (virtual).
 	MAGICS_NO_EXPORT void print(ostream&) const;
 	MAGICS_NO_EXPORT void debugOutput(const string &s) const;
 
-	MAGICS_NO_EXPORT float setY(const float y) const {return -y;}
-	MAGICS_NO_EXPORT float setSymbolY(const float y) const {return -y;}
-	MAGICS_NO_EXPORT float setAngleY(const float y) const {return y;}
-	MAGICS_NO_EXPORT float setFlagY(const float y) const {return -y;}
+	MAGICS_NO_EXPORT MFloat setY(const MFloat y) const {return -y;}
+	MAGICS_NO_EXPORT MFloat setSymbolY(const MFloat y) const {return -y;}
+	MAGICS_NO_EXPORT MFloat setAngleY(const MFloat y) const {return y;}
+	MAGICS_NO_EXPORT MFloat setFlagY(const MFloat y) const {return -y;}
 	MAGICS_NO_EXPORT double getRatio() const {return getYDeviceLength() / getXDeviceLength();}
 
 	MAGICS_NO_EXPORT void openGroup(string g) const;
@@ -118,11 +122,10 @@ private:
 	mutable string		tmp_pFile_;    //!< Tmp to hold output stream to file.
 	mutable string		groupString_;
 	mutable int		group_counter_;
-	mutable bool		showCoordinates_;
+//	mutable bool		showCoordinates_;
 	mutable bool		inkscape_;
-	mutable bool		interactive_;
-	mutable bool		line_visible_;
-	mutable unsigned short	interactiveCounter_;
+//	mutable bool		interactive_;
+//	mutable unsigned short	interactiveCounter_;
 
 	mutable std::vector<string> layers_;
 	mutable stringarray svg_output_resource_list_;
