@@ -16,89 +16,11 @@
 
  ******************************** LICENSE ********************************/
 
-/*! \mainpage Magics++
-
- \section intro What is Magics++?
-
- Magics++ is the second generation of a meteorological graphics 
- library developed at the <a href="http://www.ecmwf.int"><i>European 
- Centre for Medium Range Forecasts (ECMWF)</i></a>. This library is 
- developed in C++ and offers output in various formats such as PostScript, 
- PDF, PNG, SVG, KML and OpenGL. 
-
- \section install How-to install
-
- Before installation you have to compile Magics++. To do so, simply 
- unpack the tarball in an appropiate directory and run <i>./configure</i> 
- followed by <i>make</i>. You can type <i>./configure --help</i> to get all
- options of configuring Magics++. 
- 
- To install type <i>make install</i>. Depending on the 
- choosen installation directory you need root permission.
-
- For more information you should read the <a href="Magics++InstallationGuide.pdf">installation guide</a>.
-
- \section interfaces Magics++ interfaces
-
- Magics++ offers different interfaces to access its graphics 
- functionality: C++ (for higher-level applicatiosn such as Metview), C, Fortran 77
- Python and MagML. The Fortran interface is intended to be backwards compatible with older 
- versions (mainly the the 6.x series) of Magics.
-
- \section modules More information
-
- - \ref interpolation "Interpolation and contouring"
- - \ref hilo "Computations of High &amp; Lows"
- - \ref titles "Automatic titles"
- - \ref projections "Geographical projections"
- - \ref drivers "Output drivers"
- - \ref coastlines "Coastlines"
- - \ref colours "Colours"
- - \ref obs "Plotting of observations"
- 
- 
- \section links Links
- 
- - <a href="http://www.ecmwf.int/publications/manuals/magics/magplus/index.html">Magics++ homepage</a>
- - <a href="http://www.ecmwf.int/services/computing/training/material/com_mag.html">Training course</a>
- - <a href="file:///scratch/graphics/cgi/mpptestsuite/html/test/fortran/timings.html">Magics++ test suite [ECMWF internal]</a>
- 
- \section copyright License
- 
-  Copyright 2007 European Centre for Medium-Range Weather Forecasts (ECMWF)
-
-  Licensed under the Apache License, Version 2.0 (the "License"); 
-  you may not use this file except in compliance with the License. 
-  You may obtain a copy of the License at 
-
-	http://www.apache.org/licenses/LICENSE-2.0
-
-  Unless required by applicable law or agreed to in writing, software 
-  distributed under the License is distributed on an "AS IS" BASIS, 
-  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
-  See the License for the specific language governing permissions and 
-  limitations under the License.
-
-*/
-
-/*!
-  \defgroup coastlines Coastlines
-  
-  \section overviewCoast Overview
-  
-  Coastlines are stored in NetCDF files and are stored in the <i>share/</i> directory.
-  They are generated from GHSSH data files stored in the <i>gshhs/</i> directory.
-  
-  \section makeCoast Regeneration of the Coastlines
-  
-  Go into the main Magics++ directory and run <i>make clean-coast</i> followed 
-  by <i>make coast</i>.
-*/
 
 /*! \file magics.h
 
   \brief This file contains all global definition for the MagPlus library.
-  \author Graphics Section, ECMWF
+  \author Meteorological Visualisation Section, ECMWF
   \license Apache License 2.0
 
   Started: January 2004
@@ -109,8 +31,12 @@
 #ifndef magicsplusplus_H
 #define magicsplusplus_H
 
+//#define BOOST_VERSION 104700
+
 #include <magics_config.h>
-#include <unistd.h>           // for AIX
+#if defined(MAGICS_AIX_XLC)
+ #include <unistd.h>           // for AIX
+#endif
 #include <climits>
 
 // do the export restriction only if NOT for Metview
@@ -148,11 +74,11 @@
   #define MAGICS_DLLPUBLIC
 #endif
 
-//#if __GNUC__ - 0 > 3 || (__GNUC__ - 0 == 3 && __GNUC_MINOR__ - 0 >= 2)
-//#define MAGICS_DEPRECATED __attribute__ ((deprecated))
-//#else
-//#define MAGICS_DEPRECATED
-//#endif
+#ifdef __GNUC__
+#define MAGICS_DEPRECATED __attribute__((deprecated))
+#else
+#define MAGICS_DEPRECATED
+#endif
 
 #include <iostream>
 #include <cstring>
@@ -178,7 +104,7 @@ using std::ifstream;
 #include <set>
 #include <iterator>
 #include <numeric>
-#include <cfloat>
+//#include <cfloat>
 #include <algorithm>
 
 using std::allocator;
@@ -195,13 +121,13 @@ using std::exception;
 #include <cassert>
 #include <cmath>
 
-#if defined(MAGICS_AIX_XLC) || defined(MAGICS_WINDOWS_CYGWIN)
-inline MAGICS_NO_EXPORT double exp10(const double r) { return exp(r*2.302585093);} // exp(r*ln(10))
+#if defined(MAGICS_AIX_XLC) || defined(MAGICS_WINDOWS_CYGWIN) || defined(MAGICS_MAC_GCC)
+inline double exp10(const double r) { return exp(r*2.302585093);} // exp(r*ln(10))
 #endif
 
 /*! \namespace magics
 
- The <I>magics</I> namespace ancapsulate all elements of Magics++.
+ The <I>magics</I> namespace encapsulates all elements of Magics++.
  The namespace prevents confusions with other libraries or 
  older versions of Magics.
  
@@ -213,10 +139,7 @@ inline MAGICS_NO_EXPORT double exp10(const double r) { return exp(r*2.302585093)
 namespace magics {
 
 
-const int int_MAX = INT_MAX;
-const int int_MIN = INT_MIN;
-
-const double double_MIN = DBL_MIN;
+//const double double_MIN = DBL_MIN;
 
 
 template <class P>
@@ -265,7 +188,8 @@ enum Shading { M_SH_NONE, M_SH_SOLID, M_SH_HATCH, M_SH_DOT };
 enum ArrowPosition { M_TAIL, M_CENTRE };
 enum DisplayType { ABSOLUTE, INLINE, BLOCK, NONE, HIDDEN };
 enum ListPolicy { M_LASTONE, M_CYCLE };
-enum GraphicsFormat {PS, EPS, PDF, SVG, KML, PNG, X, CPS, CSVG, GIF, AGIF, JPG, OPENGL};
+enum GraphicsFormat {PS, EPS, PDF, SVG, KML, PNG, X, CPS, CSVG, GIF, AGIF, JPG, QT};
+enum AxisAutomaticSetting {m_off, m_both, m_min_only, m_max_only};
 
 /*
 inline MAGICS_NO_EXPORT string getOutputString()
@@ -274,7 +198,7 @@ inline MAGICS_NO_EXPORT string getOutputString()
 	return outputs;
 }
 */
-const double epsilon = 1.25e-7;
+const double epsilon = 1.25e-10;
 
 template<class T>
 inline MAGICS_NO_EXPORT T abs(const T a) { return (a < 0) ? -a : a; }
@@ -363,6 +287,7 @@ inline string tostring(const T& in)
 /*!
   \brief checks if file readable or not
 */
+/*
 inline MAGICS_NO_EXPORT bool fileReadable(const string &strFile)
 {
     FILE *fp = fopen (strFile.c_str(), "r");
@@ -376,7 +301,7 @@ inline MAGICS_NO_EXPORT bool fileReadable(const string &strFile)
         return false;
     }
 }
-
+*/
 
 #define MAGPLUS_PATH_TO_SHARE_ "/share/magics/"
 #define MAGPLUS_LINK_  "http://www.ecmwf.int/publications/manuals/magics"
