@@ -17,7 +17,7 @@
  ******************************** LICENSE ********************************/
 
 /*! \file Akima760.cc
-    \Implemtation of Akima760 class.
+    Implementation of Akima760 class.
     
     Magics Team - ECMWF 2004
    
@@ -26,28 +26,28 @@
 */
 
 #include "Akima760Method.h"
-#include "Log.h"
+#include "MagLog.h"
 #include "Timer.h"
 
 using namespace magics;
 
-template <class P>	
-Akima760<P>::Akima760(const  AbstractMatrix& matrix, const Akima760MethodAttributes& attr) : 
-    MatrixHandler<P>(matrix), attr_(attr)
+
+Akima760::Akima760(const  AbstractMatrix& matrix, const Akima760MethodAttributes& attr) :
+    MatrixHandler(matrix), attr_(attr)
 {
-    	Log::debug() << "Akima760 Constructor" << "\n";
+	MagLog::debug() << "Akima760 Constructor" << "\n";
 
        int j;
-//     monoColumns_ = MatrixHandler<P>::columns(); //syntax also valid
+//     monoColumns_ = MatrixHandler::columns(); //syntax also valid
        monoColumns_ = this->matrix_.columns();
        monoRows_ = this->matrix_.rows();
 
       // Compute matrix output sizes
-       double aux = (this->matrix_.regular_column( monoColumns_-1) - this->matrix_.regular_column(0)) / attr_.getResolutionX();
+       double aux = (this->matrix_.regular_column( monoColumns_-1) - this->matrix_.regular_column(0)) / attr_.resolutionX_;
        if ( (double)(int(aux)) != aux ) aux += 1.; //next integer number
        ncols_ = int(aux + 1.);                  //must include the first and the last input coordinates
 
-       aux = (this->matrix_.regular_row( monoRows_-1) - this->matrix_.regular_row(0)) / attr_.getResolutionY();
+       aux = (this->matrix_.regular_row( monoRows_-1) - this->matrix_.regular_row(0)) / attr_.resolutionY_;
        if ( (double)(int(aux)) != aux ) aux += 1.; //next integer number
        nrows_ = int(aux + 1.);                  //must include the first and the last input coordinates
 
@@ -72,38 +72,49 @@ Akima760<P>::Akima760(const  AbstractMatrix& matrix, const Akima760MethodAttribu
       rgpd3p();
       //}
       
-     
+      double from = this->matrix_.regular_row(0);
+      for ( int i = 0; i < nrows_; i++) {
+    	  double pos = from + i *attr_.resolutionY_;
+    	  rowsMap_.insert(make_pair(pos, i));
+    	  rows_.push_back(pos);
+      }
+      from = this->matrix_.regular_column(0);
+      for ( int i = 0; i < ncols_; i++) {
+    	  double pos = from + i *attr_.resolutionX_;
+          	 columnsMap_.insert(make_pair(pos, i));
+          	columns_.push_back(pos);
+      }
          
           
 #if 0
       // Print partial derivatives
       int i;
-      Log::debug() << "WKZX" << "\n";;
+      MagLog::debug() << "WKZX" << "\n";;
       for (i = 0; i < this->matrix_.rows(); i++)
       {
-	      Log::debug() << "\n";;
+	      MagLog::debug() << "\n";;
 	      for (j = 0; j < this->matrix_.columns(); j++)
-		      Log::debug() << WKZX_[i][j] << " ";
+		      MagLog::debug() << WKZX_[i][j] << " ";
        }
-      Log::debug() << "\n" << "WKZY" << "\n";;
+      MagLog::debug() << "\n" << "WKZY" << "\n";;
       for (i = 0; i < this->matrix_.rows(); i++)
       {
-	      Log::debug() << "\n";;
+	      MagLog::debug() << "\n";;
 	      for (j = 0; j < this->matrix_.columns(); j++)
-	      Log::debug() << WKZY_[i][j] << " ";
+	      MagLog::debug() << WKZY_[i][j] << " ";
        }
-      Log::debug() << "\n" << "WKZXY" << "\n";;
+      MagLog::debug() << "\n" << "WKZXY" << "\n";;
       for (i = 0; i < this->matrix_.rows(); i++)
       {
-	      Log::debug() << "\n";;
+	      MagLog::debug() << "\n";;
 	      for (j = 0; j < this->matrix_.columns(); j++)
-	      Log::debug() << WKZXY_[i][j] << " ";
+	      MagLog::debug() << WKZXY_[i][j] << " ";
        }
 #endif
 }
 
-template <class P>
-Akima760<P>::~Akima760()
+
+Akima760::~Akima760()
 {
       int j;
 
@@ -126,20 +137,20 @@ Akima760<P>::~Akima760()
       WKZXY_ = NULL; 
 }
 
-template <class P>
-double Akima760<P>::regular_row(int i) const
+
+double Akima760::regular_row(int i) const
 {
 //Remove later. Why this function is called so many times ???
 //static long itest=0;
-//Log::debug() << "Akima760 row=" << itest++ << "\n";
+//MagLog::debug() << "Akima760 row=" << itest++ << "\n";
 
 	// Check if the coordinate does not go beyond the upper limit
-	double aux = i*attr_.getResolutionY() + this->matrix_.regular_row(0);
+	double aux = i*attr_.resolutionY_ + this->matrix_.regular_row(0);
         return (aux > this->matrix_.regular_row(monoRows_-1) ? this->matrix_.regular_row(monoRows_-1) : aux);
 }
 
-template <class P>
-double Akima760<P>::row(int i, int j) const {
+
+double Akima760::row(int i, int j) const {
 	return this->real_row(regular_row(i), regular_column(j));
 	/*
 	double r = regular_row(i);
@@ -180,19 +191,19 @@ double Akima760<P>::row(int i, int j) const {
     
 }
 
-template <class P>
-double Akima760<P>::regular_column(int j) const
+
+double Akima760::regular_column(int j) const
 { 
 //Remove later. Why this function is called so many times ???
 //static long jtest=0;
-//Log::debug() << "Akima760 column=" << jtest++ << "\n";
+//MagLog::debug() << "Akima760 column=" << jtest++ << "\n";
 
 	// Check if the coordinate does not go beyond the upper limit
-	double aux = j*attr_.getResolutionX() + this->matrix_.regular_column(0);
+	double aux = j*attr_.resolutionX_ + this->matrix_.regular_column(0);
         return (aux > this->matrix_.regular_column(monoColumns_-1) ? this->matrix_.regular_column(monoColumns_-1) : aux);
 }
-template <class P>
-double Akima760<P>::column(int i, int j) const {
+
+double Akima760::column(int i, int j) const {
 	
 	return this->real_column(regular_row(i), regular_column(j));
 	/*
@@ -231,16 +242,16 @@ double Akima760<P>::column(int i, int j) const {
 	return row;
     */	
 }
-template <class P>	
-double Akima760<P>::operator()(int  i, int  j) const
+
+double Akima760::operator()(int  i, int  j) const
 {
 	double value =  rgbi3p(regular_column(j), regular_row(i));
 	
 	return value;
 }
 
-template <class P>
-void Akima760<P>::rgpd3p()
+
+void Akima760::rgpd3p()
 {
 //*
 //* Partial derivatives of a bivariate function on a rectangular grid
@@ -791,8 +802,8 @@ void Akima760<P>::rgpd3p()
 }
 
 // A 7x7 area surround the point is searched
-template <class P>
-int Akima760<P>::CheckMissingValues (int col, int lin) const
+
+int Akima760::CheckMissingValues (int col, int lin) const
 {
       int i,j,NXDF,NYDF;
 
@@ -819,8 +830,8 @@ int Akima760<P>::CheckMissingValues (int col, int lin) const
 
 #if 0
 // This subroutine is obsolet, no longer needed
-template <class P>
-void Akima760<P>::rglctn(double XII, double YII, int& INXI, int& INYI) const
+
+void Akima760::rglctn(double XII, double YII, int& INXI, int& INYI) const
 {
 //*
 //* Location of the desired points in a rectangular grid
@@ -919,11 +930,11 @@ ll:
 
 #if 0  //REMOVE LATER
      if (JX1 != IX){
-       Log::dev()<< "SERIOUS ERROR FINDING INDEXES" << endl;
+       MagLog::dev()<< "SERIOUS ERROR FINDING INDEXES" << endl;
        return -9999999.;
      }
 #endif
-//     Log::dev()<< XII << " " << INXI << " " << ji << endl;
+//     MagLog::dev()<< XII << " " << INXI << " " << ji << endl;
        }
        else
               INXI = NXDF-1;
@@ -951,11 +962,11 @@ ll1:
 	      ji = this->matrix_.lowerRow(YII);
 #if 1  //REMOVE LATER
      if (ji != INYI){
-       Log::dev()<< "SERIOUS ERROR FINDING INDEXES" << ji << " " <<  INYI << endl;
+       MagLog::dev()<< "SERIOUS ERROR FINDING INDEXES" << ji << " " <<  INYI << endl;
        return -9999999.;
      }
 #endif
-//      Log::dev()<< YII << " " << INYI << " " << ji << endl;
+//      MagLog::dev()<< YII << " " << INYI << " " << ji << endl;
       	}
         else
                INYI = NYDF-1;
@@ -964,8 +975,8 @@ ll1:
 }
 #endif
 
-template <class P>
-void Akima760<P>::rgplnl(double XII, double YII, int IXDI, int IYDI, double& ZII) const
+
+void Akima760::rgplnl(double XII, double YII, int IXDI, int IYDI, double& ZII) const
 {
 
 ////*
@@ -1237,8 +1248,9 @@ void Akima760<P>::rgplnl(double XII, double YII, int IXDI, int IYDI, double& ZII
       return;
 }
 
-template <class P>
-double Akima760<P>::rgbi3p(double XI, double YI) const
+
+
+double Akima760::rgbi3p(double XI, double YI) const
 {
 //*
 //* Rectangular-grid bivariate interpolation
@@ -1347,7 +1359,7 @@ double Akima760<P>::rgbi3p(double XI, double YI) const
       else if (YI < this->matrix_.regular_row(this->matrix_.rows()-1)) INYI = this->matrix_.lowerRow(YI);
       else INYI = this->matrix_.rows()-1;
 
-//Log::dev()<< XI << " " << YI << "   " << INXI << " " << INYI << "   " << col << " " << row << endl;
+//MagLog::dev()<< XI << " " << YI << "   " << INXI << " " << INYI << "   " << col << " " << row << endl;
 
       // Calculates the z values at the output point
       rgplnl(XI,YI,INXI,INYI,ZI);
@@ -1355,219 +1367,77 @@ double Akima760<P>::rgbi3p(double XI, double YI) const
       return ZI;
 }
 
-//TEST, REMOVE LATER
-#if 0
-#define   NXI 19
-#define   NYI 23
-template <class P>
-void Akima760<P>::test_build_data()
+
+
+int Akima760::rowIndex(double row) const
 {
-float     XIMN=-0.5;
-float	  XIMX= 8.5;
-float     YIMN=-0.5;
-float	  YIMX=10.5;
-
-double    XI[NXI],YI[NYI];
-double    ZI[NYI][NXI];      
-
-      float     ANXIM1,ANYIM1,DXI,DYI;
-      int       IER,ISEC,IXD,IXI,IXIMN,IXIMX,IYD,IYI,IYIR,MD,NXDF,NYDF;
-
-      //     ..Local Arrays ..
-      double     DZI[NYI][NXI];
-
-      // Initializations
-
-     double ZIE[NYI][NXI] = { -.847,-.533,
-                      -.274,-.117,-.031,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .401,.250,.119,.043,.011,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,-.665,-.376,-.143,-.033,-.007,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-		      .000,.000,.000,.000,
-		      .000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,2.449,
-                      1.368,.537,.149,.025,.000,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      5.083,3.200,1.642,.700,.187,.000,.000,.000,.000,
-                      .000,.000,.000,.000,.000,.000,.000,.000,.000,
-                      .000,6.588,5.234,3.878,2.542,1.188,.253,.026,
-                      .026,.007,.000,.000,.000,.000,.000,.000,.000,
-                      .000,.000,.000,8.017,7.400,6.400,4.800,2.963,
-                      1.400,.457,.100,.027,.000,.000,.000,.000,.000,
-		      .000,.000,.000,.000,.000,
-		      11.055,
-                      9.670,8.083,6.305,4.786,3.421,2.043,1.112,.565,
-                      .131,-.019,.000,.000,.000,.000,.000,.000,.000,
-                      .000,14.492,12.000,9.746,8.000,6.594,5.300,4.081,
-                      2.900,1.697,.600,.059,.000,.000,.000,.000,.000,
-                      .000,.000,.000,15.999,14.376,12.657,10.774,8.620,
-                      6.659,5.291,4.392,3.926,3.005,1.223,.139,.051,
-                      .025,.009,.000,.000,.000,-.005,15.525,16.800,
-                      16.749,14.400,10.956,8.100,6.735,6.900,7.298,
-                      6.200,3.010,.600,.248,.100,.024,.000,.006,.000,
-                      -.025,15.876,19.280,20.563,17.856,13.242,10.219,
-                      10.577,11.999,10.170,7.053,5.198,3.543,1.831,
-		      .350,-.130,.168,.408,.168,-.224,
-		      17.700,
-                      21.800,23.531,20.500,15.087,12.800,15.817,17.600,
-                      11.477,5.800,6.988,7.600,4.410,.800,-.392,.600,
-                      1.261,.600,-.417,17.913,22.788,24.944,21.881,
-                      16.302,14.382,18.557,20.807,11.916,4.561,7.327,
-                      8.518,5.133,1.284,-.013,1.201,1.998,1.200,-.065,
-                      16.383,22.400,25.330,22.500,16.796,14.600,19.172,
-                      22.500,13.159,4.700,6.689,7.200,4.392,1.800,
-                      1.150,2.100,2.734,2.100,1.025,18.109,26.756,
-                      31.311,28.143,21.004,18.237,24.236,28.979,17.970,
-                      7.469,10.467,11.985,9.022,6.833,6.901,8.292,
-                      9.186,8.524,7.101,24.667,37.200,44.007,40.000,
-                      30.508,27.000,34.974,41.300,27.136,14.100,20.473,
-                      24.500,20.557,17.300,17.639,20.200,21.826,20.800,
-		      18.458,
-		      33.414,
-                      48.009,56.017,51.561,40.817,36.922,45.856,52.860,
-                      37.376,23.200,30.839,36.192,31.969,28.037,28.437,
-                      31.604,33.579,32.332,29.561,44.842,58.200,65.537,
-                      61.500,51.657,47.900,55.899,62.300,47.891,34.600,
-                      41.239,45.500,41.479,38.200,38.591,41.200,42.823,
-                      41.700,39.192,58.284,68.917,74.644,71.333,63.413,
-                      60.125,66.293,71.400,59.129,47.725,52.451,54.592,
-		      50.842,48.483,48.639,50.142,51.089,50.200,48.268 };
-
-	int i,j;
-
-//*     ..
-//* Calculation
-//* Opens the output file and writes the input data.
-//
-      NXDF = this->matrix_.columns();
-      NYDF = this->matrix_.rows();
-      printf("\nPRG3P  Original Data  ZD(YD,XD)\n\n       ");
-      for (IXD = 0; IXD < NXDF; IXD++) printf("%7.1f",this->matrix_.regular_column(IXD));
-      for (IYD = NYDF-1; IYD >= 0; IYD--)
-      {
-	      printf("\n%7.1f",this->matrix_.regular_row(IYD));
-	      for (IXD = 0; IXD < NXDF; IXD++) printf("%7.1f",this->matrix_(IYD,IXD));
-      }
-
-//* Program check for the RGBI3P subroutine
-//* - Performs interpolation and calculates the differences.
-      DXI = XIMX - XIMN;
-      ANXIM1 = NXI - 1;
-      printf("\n\nXI %d\n",NXI);
-      for (IXI = 0; IXI < NXI; IXI++)
-      {
-	      XI[IXI] = XIMN + DXI*float(IXI)/ANXIM1;
-	      printf("%5.1f",XI[IXI]);
-      }
-
-      DYI = YIMX - YIMN;
-      ANYIM1 = NYI - 1;
-      printf("\n\nYI %d\n",NYI);
-      for (IYI = 0; IYI < NYI; IYI++)
-      {
-	      YI[IYI] = YIMN + DYI*float(IYI)/ANYIM1;
-	      printf("%5.1f",YI[IYI]);
-      }
-
-      printf("\n\n PROCESSING\n");
-
-      // Estimates partial derivatives at all input-grid data points
-      Log::debug() << "Starting RGPD3P" << "\n";;
-      rgpd3p();
-
-      Log::debug() << "WKZX" << "\n";;
-      for (i = 0; i < this->matrix_.rows(); i++)
-      {
-	      Log::debug() << "\n";;
-	      for (j = 0; j < this->matrix_.columns(); j++)
-		      Log::debug() << WKZX_[i][j] << " ";
-       }
-      Log::debug() << "\n" << "WKZY" << "\n";;
-      for (i = 0; i < this->matrix_.rows(); i++)
-      {
-	      Log::debug() << "\n";;
-	      for (j = 0; j < this->matrix_.columns(); j++)
-	      Log::debug() << WKZY_[i][j] << " ";
-       }
-      Log::debug() << "WKZXY" << "\n";;
-      for (i = 0; i < this->matrix_.rows(); i++)
-      {
-	      Log::debug() << "\n";;
-	      for (j = 0; j < this->matrix_.columns(); j++)
-	      Log::debug() << WKZXY_[i][j] << " ";
-       }
-
-      for (IYI = 0; IYI < NYI; IYI++)
-      {
-	      for (IXI = 0; IXI < NXI; IXI++)
-	      {
-		      rgbi3p(XI[IXI],YI[IYI],ZI[IYI][IXI]);
-		      DZI[IYI][IXI] = ZI[IYI][IXI] - ZIE[IYI][IXI];
-	      }
-      }
-
-//* - Writes the calculated results.
-      printf("\n\n\nTPRG3P  Program Check for RGBI3P   Calculated ZI Values\n");
-      for (ISEC = 1; ISEC < 3; ISEC++)
-      {
-	      if (ISEC == 1)
-	      {
-		  IXIMN = 0;
-		  IXIMX = 11;
-	      }
-	      else
-	      {
-		  IXIMN = 8;
-		  IXIMX = NXI;
-	      }
-
-	      printf("\n\n                ZI(XI,YI)\n\n     ");
-	      for (IXI = IXIMN; IXI < IXIMX; IXI++)
-		  printf("%7.1f",XI[IXI]);
-
-	      for (IYI = NYI-1; IYI >= 0; IYI--)
-	      {
-		      printf("\n%7.1f",YI[IYI]);
-		      for (IXI = IXIMN; IXI < IXIMX; IXI++)
-			      printf("%7.2f",ZI[IYI][IXI]);
-	      }
-      }
-
-//* - Writes the differences.
-      printf("\n\n\nTPRG3P  Program Check for RGBI3P   Differences\n");
-      for (ISEC = 1; ISEC < 3; ISEC++)
-      {
-	      if (ISEC == 1)
-	      {
-		  IXIMN = 0;
-		  IXIMX = 11;
-	      }
-	      else
-	      {
-		  IXIMN = 8;
-		  IXIMX = NXI;
-	      }
-
-	      printf("\n\n           DZI(YI,XI)\n\n       ");
-	      for (IXI = IXIMN; IXI < IXIMX; IXI++)
-		  printf("%7.1f",XI[IXI]);
-
-	      for (IYI = NYI-1; IYI >= 0; IYI--)
-	      {
-		      printf("\n%7.1f",YI[IYI]);
-		      for (IXI = IXIMN; IXI < IXIMX; IXI++)
-			      printf("%7.2f",DZI[IYI][IXI]);
-	      }
-      }
-
-      printf("\n\nEND TEST\n");
-
+     map<double, int>::const_iterator i = rowsMap_.find(row);
+     if ( i != rowsMap_.end() ) return (*i).second;
+     return -1;
+}
+
+
+
+int Akima760::columnIndex(double row) const
+{
+      map<double, int>::const_iterator i = columnsMap_.find(row);
+      if ( i != columnsMap_.end() ) return (*i).second;
+      return -1;
+}
+
+
+
+void Akima760::boundRow(double r, double& row1, int& index1, double& row2, int& index2) const
+{
+//	unsigned int  nb =  rows_.size();
+
+	// first test if increasing!
+	if ( rows_.back() - rows_.front() > 0 ) {
+		index2 = 0;
+		while ( index2 < rows_.size() && rows_[index2] < r ) {
+			index2++;
+		}
+		index1 = (index2) ? index2-1 : 0;
+		row1 = rows_[index1];
+		row2 = rows_[index2];
+	}
+	else {
+		index1 = 0;
+		while ( index1 < rows_.size() && rows_[index1] >  r ) {
+			index1++;
+		}
+		index2 = (index1 == rows_.size() -1 ) ? index1 :  index1+1;
+		row1 = rows_[index1];
+		row2 = rows_[index2];
+	}
+}
+
+
+
+void Akima760::boundColumn(double r, double& column1, int& index1, double& column2, int& index2) const
+{
+//	unsigned int  nb =  rows_.size();
+
+	// first test if increasing!
+	if ( columns_.back() - columns_.front() > 0 )
+	{
+		index2 = 0;
+		while ( index2 < columns_.size() && columns_[index2] < r )
+		{
+				index2++;
+		}
+		index1 = (index2) ? index2-1 : 0;
+		column1 = columns_[index1];
+		column2 = columns_[index2];
+	}
+	else {
+		index1 = 0;
+		while (  index1 < columns_.size() && columns_[index1] >  r )
+		{
+			index1++;
+		}
+		index2 = (index1 == columns_.size() -1 ) ? index1 : index1+1;
+		column1 = columns_[index1];
+		column2 = columns_[index2];
+	}
 }
-#endif
