@@ -67,13 +67,16 @@ public:
 	
 	
 
-	virtual void operator()(const PaperPoint& point, double x, double y, double val);
+	virtual void operator()(const UserPoint&, const PaperPoint& point, double x, double y, double val);
+
 	virtual void prepare(BasicGraphicsObjectContainer& out, double res);	
+	virtual void prepare(BasicGraphicsObjectContainer&);	
 	virtual void finish(BasicGraphicsObjectContainer&);
 	void visit(LegendVisitor& legend);
+	void visit(Data&, PointsHandler&, HistoVisitor&);
 	
 protected:
-	//! Method to print string about this class on to a stream of type ostream (virtual).
+        //! Method to print string about this class on to a stream of type ostream (virtual).
 	virtual void print(ostream& out) const
 	 	{ out << "ArrowPlotting<P>"; } 
 	map<Colour, Arrow*> southArrows_;
