@@ -35,32 +35,76 @@
 #include "BasicSceneObject.h"
 #include "AxisAttributes.h"
 #include "XmlNode.h"
+#include "MagicsEvent.h"
 namespace magics {
 	
 class XmlNode;
 
+class SceneLayer;
+class LayoutVisitor;
+class AxisItem;
 
-
-class Axis: public BasicSceneObject, public AxisAttributes {
+class Axis: public BasicSceneObject, public AxisAttributes  {
 
 public:
 	Axis();
 	virtual ~Axis();
-    
  
-    
     void set(const XmlNode& node ) { AxisAttributes::set(node); }
     void set(const map<string, string>& map ) { AxisAttributes::set(map); }
-  
-   
+    
+    void visit(SceneLayer& layer, vector<LayoutVisitor*>& visitors);
+    void visit(TextVisitor& );
+
+    virtual void tick(VerticalAxisVisitor&) {}
+    virtual void tick(HorizontalAxisVisitor&) {}
+
+    virtual void minortick(VerticalAxisVisitor&) {}
+    virtual void minortick(HorizontalAxisVisitor&) {}
+
+    virtual void label(VerticalAxisVisitor&) {}
+    virtual void label(HorizontalAxisVisitor&) {}
+
+    virtual void title(VerticalAxisVisitor&) {}
+    virtual void title(HorizontalAxisVisitor&) {}
+
+    virtual void line(TopAxisVisitor& out) const 		{}
+    virtual void line(BottomAxisVisitor& out) const 	{}
+    virtual void line(LeftAxisVisitor& out) const 		{}
+    virtual void line(RightAxisVisitor& out) const 	{}
+
+    virtual void tip(TopAxisVisitor& out) const 		{}
+       virtual void tip(BottomAxisVisitor& out) const 	{}
+       virtual void tip(LeftAxisVisitor& out) const 		{}
+       virtual void tip(RightAxisVisitor& out) const 	{}
+
+    virtual void grid(DrawingVisitor&) const {}
+
+    string createLabel(const AxisItem&);
+
+    double min() const { return method_->min(); }
+    double max() const { return method_->max(); }
+
+
 
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream&) const; 
      void ticks(double, double, vector<double>&);
      AxisItems items_;
-   
-     
+
+
+     typedef string (Axis::*LabelHelper)(const AxisItem&);
+     map<string,  LabelHelper> labelHelpers_;
+
+     string number(const AxisItem&);
+     string labellist(const AxisItem&);
+     string latitude(const AxisItem&);
+     string longitude(const AxisItem&);
+     string basic(const AxisItem&);
+     double title_position_;
+
+     int currentLabel_;
 private:
     //! Copy constructor - No copy allowed
 	Axis(const Axis&);
@@ -85,22 +129,25 @@ public :
     void build(V& visitor) {
     	if ( items_ .empty() )  {   		
     		method_->updateX(visitor.transformation());
-    	    method_->prepare( visitor.transformation().getMinX(),  visitor.transformation().getMaxX(), items_);
+    	    method_->prepare(*this, items_);
     	}
-    	(*tick_).horizontal(items_, (*line_).colour(), visitor);
-    	(*minor_tick_).horizontal(items_, (*line_).colour(), visitor);
-    	(*label_).horizontal(items_, (*line_).colour(), visitor);
+    	line(visitor);
+    	tick(visitor);
+    	label(visitor);
+    	title(visitor);
+    	minortick(visitor);
+    	tip(visitor);
+
+
     	
-    	(*line_).horizontal(visitor);
-    	(*title_).horizontal((*line_).colour(), visitor);
     }
     
     void visit(DrawingVisitor& visitor) {
     	if ( items_ .empty() ) {
     		method_->updateX(visitor.transformation());
-    		method_->prepare( visitor.transformation().getMinX(),  visitor.transformation().getMaxX(), items_);
+    		method_->prepare(*this, items_);
     	}
-    	(*grid_).vertical(items_, visitor);
+    	grid(visitor);
     }
     
     void visit(TopAxisVisitor& visitor) {
@@ -120,7 +167,19 @@ public :
     	} 
     }
 
-  
+    void minortick(HorizontalAxisVisitor&);
+    void tick(HorizontalAxisVisitor&);
+    void label(HorizontalAxisVisitor&);
+    void title(HorizontalAxisVisitor&);
+
+    void line(TopAxisVisitor& out) const;
+    void line(BottomAxisVisitor& out) const;
+
+    void tip(TopAxisVisitor& out) const;
+    void tip(BottomAxisVisitor& out) const;
+
+    void grid(DrawingVisitor&) const;
+
 };
 
 class VerticalAxis : public Axis 
@@ -131,23 +190,24 @@ public :
     template <class V>
     void build(V& visitor) {
     	if ( items_ .empty() ) {
-          		method_->updateX(visitor.transformation());
-          		method_->prepare( visitor.transformation().getMinY(),  visitor.transformation().getMaxY(), items_);
-       }
-       (*tick_).vertical(items_, (*line_).getColour(), visitor);
-       (*minor_tick_).vertical(items_, (*line_).getColour(), visitor);
-       (*label_).vertical(items_,(*line_).getColour(), visitor);
-       (*grid_).horizontal(items_, visitor);
-       (*line_).vertical(visitor);
-       (*title_).vertical((*line_).getColour(), visitor);
+    		method_->updateY(visitor.transformation());
+    		method_->prepare(*this, items_);
+    	}
+       line(visitor);
+       tick(visitor);
+       label(visitor);
+       title(visitor);
+       minortick(visitor);
+       tip(visitor);
+
         
     }
     void visit(DrawingVisitor& visitor) {
       	if ( items_ .empty() ) {
       		method_->updateY(visitor.transformation());
-      		method_->prepare( visitor.transformation().getMinY(),  visitor.transformation().getMaxY(), items_);
+      		method_->prepare(*this, items_);
       	}
-      	(*grid_).horizontal(items_, visitor);
+      	grid(visitor);
       }
     void visit(LeftAxisVisitor& visitor) { 
     	if ( magCompare(position_, "left") )
@@ -164,7 +224,18 @@ public :
     		AxisAttributes::set(axis);
     	} 
     }
+    void tick(VerticalAxisVisitor&);
+    void minortick(VerticalAxisVisitor&);
+    void label(VerticalAxisVisitor&);
+    void title(VerticalAxisVisitor&);
 
+    void line(LeftAxisVisitor& out) const;
+    void line(RightAxisVisitor& out) const;
+
+    void tip(LeftAxisVisitor& out) const;
+    void tip(RightAxisVisitor& out) const;
+
+    void grid(DrawingVisitor&) const;
 };
 
 } // namespace magics
