@@ -16,7 +16,7 @@
 
  ******************************** LICENSE ********************************/
 
-/*! \file Coastlines.cc
+/*! \file Coastlines.cc 
     \brief Implementation of the Template class Coastlines.
     
     Magics Team - ECMWF 2004
@@ -31,13 +31,13 @@
 #include "Layer.h"
 
 using namespace magics;
-
+ 
 Coastlines::Coastlines() 
 {
-  
+	setInfo("_datatype","Coastlines");
 }
 
-Coastlines::~Coastlines() {}
+Coastlines::~Coastlines() { if ( layer_ ) delete layer_; }
 
 /*!
  Class information are given to the output-stream.
@@ -54,23 +54,17 @@ void Coastlines::print(ostream& out)  const
 
 void Coastlines::visit(DrawingVisitor& parent)
 {
-	 Layer* layer =  LayerManager::layer("coastlines");
-	 
-	
-	 parent.push_back(layer);
-	 
-	 coastlines_->layer(layer);
+
 	(*coastlines_)(parent);
-	coastlines_->layer(0);
-	 
-	grid_->layer(layer);
+	
+	
 	(*grid_)(parent);
-	 grid_->layer(0);
+	 
 
 	(*label_).prepare(*grid_);
-	label_->layer(layer);
+	
 	(*label_)(parent);
-	label_->layer(0);
+
 }
 
 void Coastlines::visit(LeftAxisVisitor& parent)
@@ -78,7 +72,14 @@ void Coastlines::visit(LeftAxisVisitor& parent)
 	(*label_).prepare(*grid_); 
 	(*label_)(parent);
 }
-
+void Coastlines::visit(TextVisitor& )
+{
+	// Nothing to put in the title
+}
+void Coastlines::visit(LegendVisitor& legend)
+{
+	coastlines_->visit(legend);
+}
 void Coastlines::visit(RightAxisVisitor& parent)
 {
 	(*label_).prepare(*grid_); 
@@ -102,3 +103,40 @@ void Coastlines::visit(PreviewVisitor& preview)
 {	
 	(*coastlines_)(preview);
 }
+
+void Coastlines::visit(SceneLayer& layer, vector<LayoutVisitor*>& visitors)
+{
+#ifdef MAG_NEXT
+	// First we create the layer!
+	// and push It to the parent layer! 
+	if (layer.state() == geometry_changed ) {
+		assert (layer_);
+		layer_->clean();
+	}
+	else 
+#endif
+    {
+    // First we create the layer!
+	// and push It to the parent layer! 
+		layer_ = new NoDataLayer(this);
+		layer_->id(iconName_);
+		layer_->name(iconName_);
+		layer.add(layer_);
+	}
+	for  (vector<LayoutVisitor*>::iterator visitor = visitors.begin(); visitor != visitors.end(); ++visitor) {
+		layer_->set(*visitor);
+		(*visitor)->visit(*this);
+	}
+	
+}
+void Coastlines::visit(MetaDataCollector& meta)
+{
+
+	MetviewIcon::visit(meta);
+	coastlines_->visit(meta);
+}
+
+void Coastlines::visit(Transformation& transformation)
+{
+	label_->label(transformation);
+}
