@@ -42,38 +42,38 @@
 
 using namespace magics;
 
-template <class P>
-ImagePlotting<P>::ImagePlotting()
+
+ImagePlotting::ImagePlotting()
 {
 }
 
-template <class P>
-ImagePlotting<P>::~ImagePlotting() 
+
+ImagePlotting::~ImagePlotting()
 {
 }
 
 /*!
  Class information are given to the output-stream.
 */	
-template <class P>	
-void ImagePlotting<P>::print(ostream& out)  const
+
+void ImagePlotting::print(ostream& out)  const
 {
-	out << "ImagePlotting<P>[";
+	out << "ImagePlotting[";
 	ImagePlottingAttributes::print(out);
 	out << "]";
 }
 
-template <class P>	
-void ImagePlotting<P>::operator()(Data<P>& data, BasicGraphicsObjectContainer& out)
+
+void ImagePlotting::operator()(Data& data, BasicGraphicsObjectContainer& out)
 {
 	// Reproject input data using Terralib routines
-	RasterData<P>& rdin = data.raster();  //input data
-	RasterData<P>  rdout;                 //output data
-	Log::dev() << "ImagePlotting<P>::operator()---> NEEDS TO BE IMPLENTED" << endl;
+	RasterData& rdin = data.raster(out.transformation());  //input data
+	RasterData  rdout;                 //output data
+	MagLog::dev() << "ImagePlotting::operator()---> NEEDS TO BE IMPLENTED" << endl;
 	
 	if ( !createOutputRaster(out, rdin,rdout) )
 	{
-	     Log::dev() << "ERROR: ImagePlotting<P>::createOutputRaster:" << *this << "\n";
+	     MagLog::dev() << "ERROR: ImagePlotting::createOutputRaster:" << *this << "\n";
 	     return;
 	}
 
@@ -86,10 +86,12 @@ void ImagePlotting<P>::operator()(Data<P>& data, BasicGraphicsObjectContainer& o
 	object->set(rdout.getRows(),rdout.getColumns());
 //	    object->setFormat(format_);
 
-	int count = getCount();
-        ColourTable& table = object->getColourTable();
-        (*colourMethod_).set(table, count);
-//        Log::dev()<< table << endl;
+
+    ColourTable& table = object->getColourTable();
+	table.push_back(Colour("none"));
+	//count--;
+        (*colourMethod_).set(table, count_-1);
+//
 
 	(*mode_)(*object, rdout);
 	//table.visit(legend_);
@@ -106,9 +108,10 @@ void ImagePlotting<P>::operator()(Data<P>& data, BasicGraphicsObjectContainer& o
 //    structure) is copied to the Terralib environment and, after the 
 //    processing, it is copied back to the Magics++ structure.
 // 2. Pixel size: currently, this value is hardcoded to TeUNSIGNEDSHORT
-//
-template <class P>
-bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& owner, RasterData<P>& rdin, RasterData<P>& rdout)
+//     *IF THIS IS CHANGED, THEN metview/src/Reprojection/ReprojectService.cc
+//     WILL ALSO NEED TO BE UPDATED*
+
+bool ImagePlotting::createOutputRaster(const BasicGraphicsObjectContainer& owner, RasterData& rdin, RasterData& rdout)
 {
 	int i,j;
 	static TeDecoderMemoryFactory theDecoderMemoryFactory("MEM");
@@ -117,7 +120,6 @@ bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& ow
 	// Initialise parameters
 	
 	TeRasterParams parin;
-	
 	parin.ncols_  = rdin.getColumns();
 	parin.nlines_ = rdin.getRows();
 	parin.resx_   = abs(rdin.getXResolution());
@@ -125,26 +127,20 @@ bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& ow
 	parin.decName("MEM");
 	
 	parin.projection(&rdin.getProjection());
-        P ll = rdin.getLowerLeftCorner();
-	P ur = rdin.getUpperRightCorner();
-        parin.boundingBoxLinesColumns(ll.x(),ll.y(),ur.x(),ur.y(),rdin.getRows(),  rdin.getColumns(), TeBox::TeLOWERLEFT);
+	UserPoint ll = rdin.getLowerLeftCorner();
+	UserPoint ur = rdin.getUpperRightCorner();
+	parin.boundingBoxLinesColumns(ll.x(),ll.y(),ur.x(),ur.y(),rdin.getRows(),  rdin.getColumns(), TeBox::TeLOWERLEFT);
 	parin.nBands(1);
 	parin.setDataType(TeUNSIGNEDSHORT);
 	parin.setCompressionMode(TeRasterParams::TeNoCompression);
-//Log::dev()<< "min/max = " << rdin.min() << " " << rdin.max() << endl;
-
-        // Set offset value. If the min image value is not zero then
-        // it means that an offset was applied to the original image 
-        // (e.g. added 145 if value is temperature in degrees K)
-        int offset = (int)rdin.min();
 
 	// Copy input image to the raster structure
-	TeRaster rastin;
+	TeRaster rastin(parin);
 	rastin.params().decName("MEM");
 	bool ok = rastin.init(parin);
 	for (i = 0; i < parin.nlines_; i++)
 		for (j = 0; j < parin.ncols_; j++)
-			rastin.setElement(j,i,rdin(i,j),0); //too slow???
+			rastin.setElement(j,i,rdin(i,j),0);
 
 	// Initialize Terralib raster structure for output data
 	// Initialise parameters
@@ -163,7 +159,7 @@ bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& ow
 	// Set output resolution and compute image size
 	// Set the same x/y resolution
 	double width  = owner.absoluteWidth();
-	int ifreq = max(int(parin.ncols_/(getPixelFrequency()*width)),1);
+	int ifreq = max(int(parin.ncols_/(pixelFrequency_*width)),1);
 	int nnx   = (parin.ncols_ - 1) / ifreq + 1;
 	double res = (maxX-minX)/double(nnx-1);
 
@@ -171,11 +167,11 @@ bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& ow
 	parout.decName("MEM");
 	parout.nBands(1);
 	parout.setDummy(0);
-	parout.setDataType(TeUNSIGNEDSHORT);
+	parout.setDataType(TeUNSIGNEDSHORT); // see note 2 above
 	parout.setCompressionMode(TeRasterParams::TeNoCompression);
 
 	// Initialise raster structure
-	TeRaster rastout;
+	TeRaster rastout(parout);
 	rastout.params().decName("MEM");
 	ok = rastout.init(parout);
 
@@ -184,9 +180,9 @@ bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& ow
 	ok = reproj.apply();
 
 	// Copy output raster structure to output image
-        rdout.setColumns(parout.ncols_);
+    rdout.setColumns(parout.ncols_);
 	rdout.setRows(parout.nlines_);
-// no need	rdout.setProjection(&projout);
+
 	rdout.setXResolution(parout.resx_);
 	rdout.setYResolution(parout.resy_);
 	// boundingBox or box function ???
@@ -198,30 +194,26 @@ bool ImagePlotting<P>::createOutputRaster(const BasicGraphicsObjectContainer& ow
 	{
 		for (j = 0; j < parout.ncols_; j++)
 		{
-			rastout.getElement(j,i,val,0); //too slow???
-
-                        //points outside the rectangle image area is set 
-                        //to 0 by the Terralib interpolation procedure 
-                        val = val ? val : offset;
-			rdout.push_back(val);
+			if (!rastout.getElement(j,i,val,0) )
+				val = 0;
+           //points outside the rectangle image area is set
+           //to 0 by the Terralib interpolation procedure
+  		   rdout.push_back(val);
 		}
 	}
 
 	return ok;
 }
 
-template <class P>
-void ImagePlotting<P>::visit(LegendVisitor& legend)
+
+void ImagePlotting::visit(LegendVisitor& legend)
 {
-	for (vector<LegendEntry* > ::iterator entry = legend_.begin(); entry != legend_.end(); ++entry) {
-		legend.add(*entry);
-	}
+	//(*mode_)(legend);
 }
-template <class P>	
-bool ImagePlotting<P>::reproject(RasterData<P>& rdin, RasterData<P>& rdout, 
+
+bool ImagePlotting::reproject(RasterData& rdin, RasterData& rdout,
 	Transformation& transformation, double resx, double resy)
 {
-#ifdef later
 	int i,j;
 	
 	static TeDecoderMemoryFactory theDecoderMemoryFactory("MEM");
@@ -233,16 +225,18 @@ bool ImagePlotting<P>::reproject(RasterData<P>& rdin, RasterData<P>& rdout,
 	parin.nlines_ = rdin.getRows();
 	parin.resx_   = abs(rdin.getXResolution());
 	parin.resy_   = abs(rdin.getYResolution());
-	parin.projection_ = &rdin.getProjection();
-        P ll = rdin.getLowerLeftCorner();
-	P ur = rdin.getUpperRightCorner();
-        parin.boundingBox(ll.x(),ll.y(),ur.x(),ur.y());
+	parin.projection (&rdin.getProjection());
+	UserPoint ll = rdin.getLowerLeftCorner();
+	UserPoint ur = rdin.getUpperRightCorner();
+	parin.boundingBoxResolution(ll.x(),ll.y(),ur.x(),ur.y(), parin.resx_, parin.resy_);
 	parin.nBands(1);
-	parin.dataType_ = TeUNSIGNEDSHORT;
-	parin.compressionType_ = TeNoCompression;
+	parin.setDataType(TeUNSIGNEDSHORT);
+	//parin.compressionType_ = TeNoCompression;
+    parin.setCompressionMode(TeRasterParams::TeNoCompression);
+	parin.decName("MEM");
 
 	// Copy input image to the raster structure
-	TeRaster rastin;
+	TeRaster rastin(parin);
 	bool ok = rastin.init(parin);
 	for (i = 0; i < parin.nlines_; i++)
 		for (j = 0; j < parin.ncols_; j++)
@@ -252,8 +246,9 @@ bool ImagePlotting<P>::reproject(RasterData<P>& rdin, RasterData<P>& rdout,
 	// Initialise parameters
 	TeRasterParams parout;
 	TeProjection& projout = const_cast<Transformation * >(&transformation)->getProjection();
-	parout.projection_ = &projout;
-	
+	parout.projection(&projout);
+	parout.decName("MEM");
+
 	double minX = transformation.getMinPCX();  
 	double maxX = transformation.getMaxPCX();  
 	double minY = transformation.getMinPCY();  
@@ -263,11 +258,12 @@ bool ImagePlotting<P>::reproject(RasterData<P>& rdin, RasterData<P>& rdout,
 	parout.boundingBoxResolution(minX,minY,maxX,maxY,resx,resx);
 
 	parout.nBands(1);
-	parout.dataType_ = TeUNSIGNEDSHORT;
-	parout.compressionType_ = TeNoCompression;
+	parout.setDataType(TeUNSIGNEDSHORT);   // see note 2 above
+	//parout.compressionType_ = TeNoCompression;
+    parout.setCompressionMode(TeRasterParams::TeNoCompression);
 
 	// Initialise raster structure
-	TeRaster rastout;
+	TeRaster rastout(parout);
 	ok = rastout.init(parout);
 
 	// Reproject input data
@@ -293,6 +289,4 @@ bool ImagePlotting<P>::reproject(RasterData<P>& rdin, RasterData<P>& rdout,
 		}
 
 	return ok;
-#endif	
-	return true;
 }
