@@ -35,19 +35,19 @@
 #include "ImportPlotAttributes.h"
 #include "Visdef.h"
 #include "ImportObject.h"
-
+#include "LegendVisitor.h"
 
 namespace magics {
 
-template <class P>
-class OverlayPlot: public ImportPlotAttributes, public Visdef<P> {
+
+class OverlayPlot: public ImportPlotAttributes, public Visdef {
 
 public:
 	OverlayPlot() {}
 	virtual ~OverlayPlot() {}
 	void set(const map<string, string>& map) { ImportPlotAttributes::set(map); }
 
-	virtual void operator()(Data<P>& data, BasicGraphicsObjectContainer& visitor)
+	virtual void operator()(Data& data, BasicGraphicsObjectContainer& visitor)
 	{
 		// Here we work on projection coordinates!
 		ImportObject* object = new ImportObject();
@@ -56,12 +56,26 @@ public:
 	    object->setWidth(width_);
 	    object->setHeight(height_);
 	    object->setFormat(format_);
-	    visitor.push_back(object);
+	    object->setOriginReference(ImportObject::bottom_left);
+	   
+	    if ( crs_.empty() ) {
+	    	    	visitor.push_back(object);
+	    	    	return;
+	    	    }
+	    	    // here we are doing a bit of cheking for metview 
+	    	    if ( crs_ != "EPSG:4326" ) {
+	    	    	MagLog::warning() << " Magics canot handle crs:" << crs_ << endl;
+	    	    	return;
+	    	    }
+	    	    
+	    
+	    
+	    
 	}
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream& out) const 
-	 	{ out << "OverlayPlot<P>["; ImportPlotAttributes::print(out); out << "]";  }  
+	 	{ out << "OverlayPlot["; ImportPlotAttributes::print(out); out << "]";  }
  
 private:
     //! Copy constructor - No copy allowed
@@ -71,35 +85,52 @@ private:
     
 // -- Friends
     //! Overloaded << operator to call print().
-	friend ostream& operator<<(ostream& s,const OverlayPlot<P>& p)
+	friend ostream& operator<<(ostream& s,const OverlayPlot& p)
 		{ p.print(s); return s; }
 
 };
 
-template <class P>
-class ImportPlot: public ImportPlotAttributes, public Visdef<P> {
+
+class ImportPlot: public ImportPlotAttributes, public Visdef {
 
 public:
 	ImportPlot() {}
 	virtual ~ImportPlot() {}
 	void set(const map<string, string>& map) { ImportPlotAttributes::set(map); }
 
-	virtual void operator()(Data<P>& data, BasicGraphicsObjectContainer& visitor)
+	virtual void operator()(Data& data, BasicGraphicsObjectContainer& visitor)
 	{				
-		
-		
 		ImportObject* object = new ImportObject();
 	    object->setPath(data.path());
 	    object->setOrigin(PaperPoint(x_, y_));
 	    object->setWidth(width_);
 	    object->setHeight(height_);
 	    object->setFormat(format_);
+	    object->setOriginReference(ImportObject::bottom_left);
+
+	    if ( crs_.empty() ) {
+	    	visitor.push_back(object);
+	    	return;
+	    } 
+	    // here we are doing a bit of cheking for metview 
+	    if ( !visitor.transformation().verifyDef(crs_) ) {
+	    	MagLog::warning() << " incompatible projections ( in " << crs_  << " and out) " << crs_ << endl;
+	    	return;
+	    }
+	    
+	   
+	    object->setOrigin(PaperPoint(crs_minx_, crs_miny_));
+	    object->setWidth(crs_maxx_ - crs_minx_);
+	    object->setHeight(crs_maxy_ - crs_miny_);
 	    visitor.push_back(object);
 	}
+	void visit(LegendVisitor& legend) {
+		legend.add(new EmptyEntry());
+	}
 protected:
      //! Method to print string about this class on to a stream of type ostream (virtual).
 	 virtual void print(ostream& out) const 
-	 	{ out << "ImportPlot<P>["; ImportPlotAttributes::print(out); out << "]";  }  
+	 	{ out << "ImportPlot["; ImportPlotAttributes::print(out); out << "]";  }
  
 private:
     //! Copy constructor - No copy allowed
@@ -109,7 +140,7 @@ private:
     
 // -- Friends
     //! Overloaded << operator to call print().
-	friend ostream& operator<<(ostream& s,const ImportPlot<P>& p)
+	friend ostream& operator<<(ostream& s,const ImportPlot& p)
 		{ p.print(s); return s; }
 
 };
