@@ -75,8 +75,8 @@ void LinearTableMode::operator()(Image& im, Raster& rd)
     double level = imin;
     for( i = 0; i < nlevels; i++)
     {
-         map[ Interval(level, level+rint) ] = i;
-//Log::dev()<< i << " " << level << " " << level+rint << endl;
+         map[  Interval(level, level+rint) ] = i;
+//MagLog::dev()<< i << " " << level << " " << level+rint << endl;
 	 level += rint;
     }
 
@@ -93,8 +93,8 @@ void LinearTableMode::operator()(Image& im, Raster& rd)
      }
 
 //for( i = 0; i < nlevels; i++)
-//       Log::dev()<< i << " " << hist1[i] << endl;
-//Log::dev()<< "total=" << iii<< endl;
+//       MagLog::dev()<< i << " " << hist1[i] << endl;
+//MagLog::dev()<< "total=" << iii<< endl;
 
      return;
 }
@@ -111,8 +111,8 @@ void LinearTableMode::operator()(Image& im, Raster& rd)
 #if 0 //remove later
      if (imax > ETM_MLEN || imin < 0 )
      {
-	  Log::dev()<< "LinearTableMode: Invalid range of image values (" << imin << "," << imax << ")" << endl;
-	  Log::dev()<< "Current implementation accepts values between 0 to " << ETM_MLEN << endl;
+	  MagLog::dev()<< "LinearTableMode: Invalid range of image values (" << imin << "," << imax << ")" << endl;
+	  MagLog::dev()<< "Current implementation accepts values between 0 to " << ETM_MLEN << endl;
 	  return;
      }
 #endif
@@ -147,9 +147,9 @@ vector<long> hist1(nlevels+1,0L); //test
 hist1[ii]++;
      }
 
-     Log::dev()<< "LinearTableMode::operator()" << endl;
+     MagLog::dev()<< "LinearTableMode::operator()" << endl;
      for( i = 0; i < nlevels+1; i++)
-	     Log::dev()<< i << " " << hist1[i] << endl;
+	     MagLog::dev()<< i << " " << hist1[i] << endl;
 
      return;
 }
