@@ -36,7 +36,7 @@
 
 #include "LegendVisitor.h"
 #include "Transformation.h"
-
+#include <cfloat>
 
 using namespace magics;
 
@@ -75,10 +75,10 @@ void MetgramFlags::print(ostream& out)  const
 
 DateTime readDate(CustomisedPoint& point)
 {
-	Date date1((long)point["year"], (long)point["month"], (long)point["day"]);
-	magics::Time time((long)point["hours"], (long)point["minutes"], (long)point["seconds"]);		
+	MagDate date1((long)point["year"], (long)point["month"], (long)point["day"]);
+	magics::MagTime time((long)point["hours"], (long)point["minutes"], (long)point["seconds"]);
 
-	time -= Second(point["shift"] * 3600);
+	//time -= Second(point["shift"] * 3600);
 	return DateTime(date1, time); 
 }
 
@@ -99,10 +99,9 @@ void MetgramBar::operator()(CustomisedPointsList& points, BasicGraphicsObjectCon
 
 	visitor.push_back(last);
 
-	Date date((long)(*points.front())["year"], (long)(*points.front())["month"], (long)(*points.front())["day"]);
-	Time time((long)(*points.front())["hours"], (long)(*points.front())["minutes"], (long)(*points.front())["seconds"]);
-	DateTime  base = DateTime(date, time);	
-	
+	MagDate date((long)(*points.front())["year"], (long)(*points.front())["month"], (long)(*points.front())["day"]);
+	MagTime time((long)(*points.front())["hours"], (long)(*points.front())["minutes"], (long)(*points.front())["seconds"]);
+	DateTime  base = readDate(*points.front());
 	double left, right = 0.;
 
 	const Transformation& transformation = visitor.transformation();
@@ -122,7 +121,7 @@ void MetgramBar::operator()(CustomisedPointsList& points, BasicGraphicsObjectCon
 			next = readDate(**point);			
 		else next = step;
 		point--;
-		
+		  x -= (**point)["shift"];
 		xpos.push_back(x);
 	
 		Polyline* box  = new Polyline();	
@@ -175,37 +174,36 @@ void MetgramBar::operator()(CustomisedPointsList& points, BasicGraphicsObjectCon
 void MetgramCurve::operator()(CustomisedPointsList& points, BasicGraphicsObjectContainer& visitor) 
 {	
 	Polyline* curve1  = new Polyline();
-	curve1->setColour(Colour("red"));
-	curve1->setThickness(2);
+	curve1->setColour(*colour_);
+	curve1->setThickness(thickness_);
+	//curve1->setLineStyle(line_style_);
 	
 	Polyline* curve2  = new Polyline();
 	curve2->setColour(Colour("blue"));
-	curve2->setThickness(2);
+	curve2->setThickness(2); 
 	
 	vector<double> xpos;
 	vector<double> ypos;
 
-	Date date((long)(*points.front())["year"], (long)(*points.front())["month"], (long)(*points.front())["day"]);
-	Time time((long)(*points.front())["hours"], (long)(*points.front())["minutes"], (long)(*points.front())["seconds"]);
+	MagDate date((long)(*points.front())["year"], (long)(*points.front())["month"], (long)(*points.front())["day"]);
+	MagTime time((long)(*points.front())["hours"], (long)(*points.front())["minutes"], (long)(*points.front())["seconds"]);
 	DateTime  base = DateTime(date, time);	
-	
+
 	double x1 = 0.;
 	double x2 = 0.;
-	double y1 = int_MAX;
-	double y2 = int_MIN;
+	double y1 = DBL_MAX;
+	double y2 = DBL_MIN;
 
 	for (CustomisedPointsList::const_iterator point = points.begin(); point != points.end(); ++point) {
 		
-		Date date((long)(**point)["year"], (long)(**point)["month"], (long)(**point)["day"]);
-		Time time((long)(**point)["hours"], (long)(**point)["minutes"], (long)(**point)["seconds"]);
+		MagDate date((long)(**point)["year"], (long)(**point)["month"], (long)(**point)["day"]);
+		MagTime time((long)(**point)["hours"], (long)(**point)["minutes"], (long)(**point)["seconds"]);
 
 		DateTime orig = DateTime(date, time);		
 		
 		double x = orig - base;		
 		xpos.push_back(x);		
-				
-		DateTime step =  DateTime(date, time);		
-		x = step - base;
+	    x -= (**point)["shift"];
 		
 		if ( (*point)->find("curve1") != (*point)->end() )   {
 			curve1->push_back(PaperPoint(x, (**point)["curve1"]));
@@ -273,14 +271,14 @@ void MetgramCurve::operator()(CustomisedPointsList& points, BasicGraphicsObjectC
 
 void MetgramFlags::operator()(CustomisedPointsList& points, BasicGraphicsObjectContainer& visitor)
 {
-	Date date((long)(*points.front())["year"], (long)(*points.front())["month"], (long)(*points.front())["day"]);
-	magics::Time time((long)(*points.front())["hours"], (long)(*points.front())["minutes"], (long)(*points.front())["seconds"]);
+	MagDate date((long)(*points.front())["year"], (long)(*points.front())["month"], (long)(*points.front())["day"]);
+	MagTime time((long)(*points.front())["hours"], (long)(*points.front())["minutes"], (long)(*points.front())["seconds"]);
 	DateTime  base = DateTime(date, time);	
 	
 	Flag* flags = new Flag();
 	flags->setColour(*colour_); 
 	flags->setLength(length_);
-    cout << "LENGTH--> " << length_ << endl;
+   
 	flags->setOriginMarker("magics_15");
 	flags->setOriginHeight(0.05);
 	flags->setConvention(KNOTS);
@@ -289,17 +287,15 @@ void MetgramFlags::operator()(CustomisedPointsList& points, BasicGraphicsObjectC
 		i++;
         if ( i % frequency_ ) 
             continue;
-		Date date((long)(**point)["year"], (long)(**point)["month"], (long)(**point)["day"]);
-		Time time((long)(**point)["hours"], (long)(**point)["minutes"], (long)(**point)["seconds"]);
+		MagDate date((long)(**point)["year"], (long)(**point)["month"], (long)(**point)["day"]);
+		MagTime time((long)(**point)["hours"], (long)(**point)["minutes"], (long)(**point)["seconds"]);
 		
 		
 		DateTime orig = DateTime(date, time);				
 		double x = orig - base;		
-	
-		time -= Second((**point)["shift"] * 3600);	
-		DateTime step =  DateTime(date, time);		
-		x = step - base;
-
+		  x -= (**point)["shift"];
+		
+        
 		if ( (*point)->find("curve1") != (*point)->end() && (*point)->find("curve2") != (*point)->end() )   {
 			PaperPoint pos(x, 0);
 			flags->push_back(ArrowPoint((**point)["curve1"], (**point)["curve2"], pos));
@@ -310,7 +306,7 @@ void MetgramFlags::operator()(CustomisedPointsList& points, BasicGraphicsObjectC
 
 void MetgramCurve::visit(LegendVisitor& legend) 
 {
-	Log::dev() << "MetgramGraph::visit(LegendBase&) " << endl;
+	MagLog::dev() << "MetgramGraph::visit(LegendBase&) " << endl;
 	Polyline* curve1  = new Polyline();
 	curve1->setColour(Colour("red"));
 	curve1->setThickness(2);
@@ -321,3 +317,4 @@ void MetgramCurve::visit(LegendVisitor& legend)
 	legend.add(new LineEntry("curve1", curve1));
 	legend.add(new LineEntry("curve2", curve2));
 }
+
