@@ -32,7 +32,7 @@
 
 #include "magics.h"
 #include "Factory.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 #include "CustomisedPoint.h"
 #include "ObsItemAttributes.h"
 #include "Symbol.h"
@@ -53,10 +53,10 @@ public:
 	virtual void operator()(CustomisedPoint&, ComplexSymbol&) const {} 
 	virtual void visit(std::set<string>&) {}
 	
-	string find(const map<string, string>& def, string keyword)
+	string find(const map<string, string>& def, const string& keyword, const string& defaut = "")
 	{
 		map<string, string>::const_iterator val = def.find(keyword);
-		return ( val != def.end() ) ?  val->second : "-1";
+		return ( val != def.end() ) ?  val->second : defaut;
 	}
  
 protected:
@@ -77,7 +77,7 @@ private:
 };
 
 template<>
-class Translator<string, ObsItem> { 
+class MagTranslator<string, ObsItem> { 
 public:
 	ObsItem* operator()(const string& val )
 	{
