@@ -37,9 +37,68 @@
 using namespace magics;
 
 
-map<int, string> ObsWind::origins_;
+map<int, string> ObsCloudAndWind::origins_;
 
 void  ObsWind::setOrigins() 
+{
+	
+}
+void ObsWind::visit(std::set<string>& tokens)
+{
+	if (!attributes_ ) attributes_ = new ObsWindAttributes();
+	if ( !colour_.empty() )  attributes_->colour_ = auto_ptr<Colour>(new Colour(colour_));
+	if (!attributes_->visible_) return;
+	tokens.insert(speed_);
+	tokens.insert(direction_);
+
+}
+void ObsWind::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
+{
+	if (!attributes_->visible_) return;
+
+	map<string, double>::iterator it = point.begin();
+	map<string, double>::iterator en = point.end();
+	for(;it != en; ++it)
+	{
+	 MagLog::debug() << " >>> "<<it->first<<" -> " << it->second << endl;
+	}
+
+	CustomisedPoint::const_iterator speed = point.find(speed_);
+	if ( speed == point.end() ) return; 
+	CustomisedPoint::const_iterator direction = point.find(direction_);
+	if ( direction == point.end() ) return; 
+
+	FlagItem* flag = new FlagItem();
+	flag->setColour(*attributes_->colour_);
+	flag->length(size_ * 1.5); // Size to be adjusted later!
+
+	const string origin = "circle";
+
+
+MagLog::debug() << "OBS ITEM - ObsWind - Lon/Lat: "<<point.longitude()<<" / "<<point.latitude()
+     << "\n\twind_speed:     " << point[speed_]
+     << "\n\twind_direction: " << point[direction_]
+     << "\n\tcloud amount:   " << point["cloud_amount"]<<" -> "<<origin <<endl;
+
+
+	flag->setOriginHeight(0.01);
+	flag->setOriginMarker(origin);
+	flag->x(0);
+	flag->y(0);
+	
+	const Transformation& transformation = symbol.parent().transformation();
+	PaperPoint pp(point.longitude(), point.latitude());
+	pair<double, double> wind = make_pair(speed->second, direction->second);
+	transformation.reprojectSpeedDirection(pp, wind);
+	
+	flag->speed(speed->second);
+	flag->direction(direction->second);
+
+	if (point.latitude() <0 ) 
+		flag->setHemisphere(SOUTH);
+	symbol.add(flag);
+}
+void  ObsCloudAndWind::setOrigins() 
 {
 	if ( !origins_.empty() ) return;
 	origins_[0] = "N_0";
@@ -53,36 +112,36 @@ void  ObsWind::setOrigins()
 	origins_[8] = "N_8";
 }
 
-void ObsWind::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
+void ObsCloudAndWind::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
-#ifdef OBS_DEBUG_
+	if (!attributes_->visible_) return;
+
 	map<string, double>::iterator it = point.begin();
 	map<string, double>::iterator en = point.end();
 	for(;it != en; ++it)
 	{
-	 cout << " >>> "<<it->first<<" -> " << it->second << endl;
+	 MagLog::debug() << " >>> "<<it->first<<" -> " << it->second << endl;
 	}
-#endif
+
 	CustomisedPoint::const_iterator speed = point.find("wind_speed");
 	if ( speed == point.end() ) return; 
 	CustomisedPoint::const_iterator direction = point.find("wind_direction");
 	if ( direction == point.end() ) return; 
 
 	FlagItem* flag = new FlagItem();
-	flag->setColour(attributes_->getColour());
-	flag->length(size_ * 4); // Size to be adjusted later!
+	flag->setColour(*attributes_->colour_);
+	flag->length(size_ * 1.5); // Size to be adjusted later!
 
 	map<int, string>::const_iterator marker = origins_.find(int(point["cloud_amount"]));
 	const string origin = ( marker != origins_.end()) ? marker->second : "magics_13";
 
-#ifdef OBS_DEBUG_
-cout << "OBS ITEM - ObsWind - Lon/Lat: "<<point.longitude()<<" / "<<point.latitude()
+
+MagLog::debug() << "OBS ITEM - ObsWind - Lon/Lat: "<<point.longitude()<<" / "<<point.latitude()
      << "\n\twind_speed:     " << point["wind_speed"] 
      << "\n\twind_direction: " << point["wind_direction"]
      << "\n\tcloud amount:   " << point["cloud_amount"]<<" -> "<<origin <<endl;
-#endif
 
+	flag->setOriginHeight(size_);
 	flag->setOriginMarker(origin);
 	flag->x(0);
 	flag->y(0);
@@ -103,13 +162,14 @@ cout << "OBS ITEM - ObsWind - Lon/Lat: "<<point.longitude()<<" / "<<point.latitu
 void ObsPressure::visit(std::set<string>& tokens)
 {
 	if ( !attributes_ ) attributes_ = new ObsPressureAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
+	if ( !colour_.empty() )  attributes_->colour_ = auto_ptr<Colour>(new Colour(colour_));
 	tokens.insert("msl_pressure");
 }
 
 void ObsPressure::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("msl_pressure");
 	if ( value == point.end() ) return;
 	
@@ -118,7 +178,7 @@ void ObsPressure::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) con
 	object->x(column_);
 	object->y(row_);
 	MagFont font;
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.name("sansserif");
 	font.size(size_);
 	
@@ -127,9 +187,9 @@ void ObsPressure::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) con
 	os <<  setw(3) << setfill('0') << pressure;
 	object->text(os.str());
 
-#ifdef OBS_DEBUG_
-	cout << "\tPressure:  " << value->second << " = " << pressure << " -> "<<os.str() << "\n";
-#endif
+
+	MagLog::debug() << "\tPressure:  " << value->second << " = " << pressure << " -> "<<os.str() << "\n";
+
 	object->font(font);
 	symbol.add(object);
 }
@@ -137,13 +197,13 @@ void ObsPressure::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) con
 void ObsPressureLevel::visit(std::set<string>& tokens)
 {
 	if ( !attributes_ ) attributes_ = new ObsPressureLevelAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("pressure");
 }
 
 void ObsPressureLevel::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("pressure");
 	if ( value == point.end() ) return;
 	TextItem*  object = new TextItem();
@@ -151,14 +211,13 @@ void ObsPressureLevel::operator()(CustomisedPoint& point,  ComplexSymbol& symbol
 	object->y(row_);
 	
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	
 	const double pressure = value->second/100.;
 
 	object->text(tostring(pressure));
-#ifdef OBS_DEBUG_
-	cout << "\tPressureLevel: " << value->second << " = " << pressure << "\n";
-#endif
+
+	MagLog::debug() << "\tPressureLevel: " << value->second << " = " << pressure << "\n";
 	font.size(size_);
 	object->font(font);
 	symbol.add(object);
@@ -167,14 +226,14 @@ void ObsPressureLevel::operator()(CustomisedPoint& point,  ComplexSymbol& symbol
 void ObsPressureTendency::visit(std::set<string>& tokens)
 {
 	if ( !attributes_ ) attributes_ = new ObsPressureTendencyAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("pressure_tendency_amount");
 	tokens.insert("pressure_tendency_characteristic");
 }
 	
 void ObsPressureTendency::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("pressure_tendency_amount");
 	if ( value == point.end() ) return; 
 	TextItem*  object = new TextItem();
@@ -187,7 +246,7 @@ void ObsPressureTendency::operator()(CustomisedPoint& point, ComplexSymbol& symb
 	
 	// The Pressure tendancy is red if negative!
 	MagFont font("sansserif");
-	font.colour(( value->second < 0 ) ? Colour("red") : attributes_->getColour());
+	font.colour(( value->second < 0 ) ? Colour("red") : *attributes_->colour_);
 
 	object->text(os.str());
 	font.size(size_);
@@ -199,12 +258,12 @@ void ObsPressureTendency::operator()(CustomisedPoint& point, ComplexSymbol& symb
 	tendancy->x(column_+1);
 	tendancy->y(row_);	
 	
-	tendancy->colour(attributes_->getColour());
+	tendancy->colour(*attributes_->colour_);
 	ostringstream oss;	
 	oss << "a_"  << value->second;
 	tendancy->symbol(oss.str());
 #ifdef OBS_DEBUG_
-	cout << "\tPressure tendency--->" << oss.str() << "\n";
+	MagLog::debug() << "\tPressure tendency--->" << oss.str() << "\n";
 #endif
 	tendancy->height(size_);
 	symbol.add(tendancy);
@@ -214,19 +273,20 @@ void ObsPressureTendency::operator()(CustomisedPoint& point, ComplexSymbol& symb
 void ObsDewPoint::visit(std::set<string>& tokens)
 {
 	if ( !attributes_ ) attributes_ = new ObsDewPointAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
+	if ( !colour_.empty() )  attributes_->colour_ = auto_ptr<Colour>(new Colour(colour_));
 	tokens.insert("dewpoint");
 }
 
 void ObsDewPoint::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("dewpoint");
 	if ( value == point.end() ) return; 
 	TextItem*  object = new TextItem();
 	MagFont font("sansserif");
 	
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.size(size_);
 	object->x(column_);
 	object->y(row_);
@@ -234,7 +294,7 @@ void ObsDewPoint::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) con
 	// The temperature is displayed in Celsius.
 	const double tempe = maground(value->second-273.25);
 #ifdef OBS_DEBUG_
-	cout << "\tDewPoint--->" << point["dewpoint_2meters"] << " = " << tempe << "\n";
+	MagLog::debug() << "\tDewPoint--->" << point["dewpoint_2meters"] << " = " << tempe << "\n";
 #endif
 	object->text(tostring(tempe));
 	object->font(font);
@@ -247,7 +307,7 @@ void ObsDewPoint::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) con
 void ObsVisibility::visit(std::set<string>& tokens)
 {
 	if ( !attributes_ ) attributes_ = new ObsVisibilityAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("horizontal_visibility");
 }
 /*! 
@@ -256,7 +316,7 @@ void ObsVisibility::visit(std::set<string>& tokens)
 */
 void ObsVisibility::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("horizontal_visibility");
 	if ( value == point.end() ) return;
 
@@ -265,7 +325,7 @@ void ObsVisibility::operator()(CustomisedPoint& point, ComplexSymbol& symbol) co
 	object->y(row_);
 
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	
 	const double vv = value->second;
 	string val;
@@ -279,9 +339,9 @@ void ObsVisibility::operator()(CustomisedPoint& point, ComplexSymbol& symbol) co
 		val = "99";
 
 	object->text(val);
-#ifdef OBS_DEBUG_
-	cout << "\tVisibility: " << vv << " = " << val << "\n";
-#endif
+
+	MagLog::debug() << "\tVisibility: " << vv << " = " << val << "\n";
+
 	font.size(size_);
 	object->font(font);
 	//object->setJustification(MRIGHT);
@@ -292,20 +352,21 @@ void ObsVisibility::operator()(CustomisedPoint& point, ComplexSymbol& symbol) co
 void ObsPresentWeather::visit(std::set<string>& tokens)
 {
 	if ( !attributes_ ) attributes_ = new ObsPresentWeatherAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("present_weather");
+	if ( !colour_.empty() )  attributes_->colour_ = auto_ptr<Colour>(new Colour(colour_));
 }
 
 void ObsPresentWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("present_weather");
 	if ( value == point.end() ) return; 
 	SymbolItem*  object = new SymbolItem();
 	object->x(column_);
 	object->y(row_);
 	
-	object->colour(attributes_->getColour());
+	object->colour(*attributes_->colour_);
 	//object->setFont("sansserif");
 	ostringstream os;
 	
@@ -313,7 +374,7 @@ void ObsPresentWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbo
 
 	object->symbol(os.str());
 #ifdef OBS_DEBUG_
-	cout << "\tPresent Weather--->" << os.str() << "\n";
+	MagLog::debug() << "\tPresent Weather--->" << os.str() << "\n";
 #endif
 	//time->setJustification(MRIGHT);
 	object->height(size_);
@@ -323,13 +384,13 @@ void ObsPresentWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbo
 
 void ObsTemperature::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
 {
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("temperature");
 	if ( value == point.end() )  return;
 
 	TextItem*  object = new TextItem();
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.size(size_);
 	object->font(font);
 	object->x(column_);
@@ -338,7 +399,7 @@ void ObsTemperature::operator()(CustomisedPoint& point,  ComplexSymbol& symbol)
 	// The temperature is displayed in Celsius.		
 	double tempe = maground(value->second-273.25);
 #ifdef OBS_DEBUG_
-	cout << "\tTemperature: " << tempe << " from "<<value->second<<"\n";
+	MagLog::debug() << "\tTemperature: " << tempe << " from "<<value->second<<"\n";
 #endif
 	object->text(tostring(tempe));
 	//object->setJustification(MCENTRE);
@@ -351,20 +412,20 @@ void ObsTemperature::operator()(CustomisedPoint& point,  ComplexSymbol& symbol)
 void ObsTimePlot::visit(std::set<string>&) 
 {
 	if ( !attributes_ ) attributes_ = new ObsTimePlotAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 }
 	
 void ObsTimePlot::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const 
 { 
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("time");
 	if ( value == point.end() ) return;
 #ifdef OBS_DEBUG_
-	cout << "\tTimePlot: " << value->second << "at[" << column_ << ", " << row_ << "]" << endl;
+	MagLog::debug() << "\tTimePlot: " << value->second << "at[" << column_ << ", " << row_ << "]" << endl;
 #endif
 	TextItem*  time = new TextItem();
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.size(size_);
 	time->x(column_);
 	time->y(row_);
@@ -380,22 +441,22 @@ void ObsTimePlot::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) con
 void ObsHeight::visit(std::set<string>& tokens ) 
 {
 	if ( !attributes_ ) attributes_ = new ObsHeightAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("geopotential");
 }
 	
 void ObsHeight::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const 
 { 
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("geopotential");
 	if ( value == point.end() ) return;
 	double geop = maground(value->second/98.1);
 #ifdef OBS_DEBUG_
-	cout << "\tGeopotential: " << geop << "at[" << column_ << ", " << row_ << "]" << endl;
+	MagLog::debug() << "\tGeopotential: " << geop << "at[" << column_ << ", " << row_ << "]" << endl;
 #endif
 	TextItem*  height = new TextItem();
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.size(size_);
 	height->x(column_);
 	height->y(row_);
@@ -411,23 +472,23 @@ void ObsHeight::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
 void ObsThickness::visit(std::set<string>& tokens ) 
 {
 	if ( !attributes_ ) attributes_ = new ObsThicknessAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("thickness");
 }
 
 
 void ObsThickness::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const 
 { 
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("thickness");
 	if ( value == point.end() ) return;
 	const double thickness = maground(value->second/98.1);
 #ifdef OBS_DEBUG_
-	cout << "\tThickness: " << thickness << "at[" << column_ << ", " << row_ << "]" << endl;
+	MagLog::debug() << "\tThickness: " << thickness << "at[" << column_ << ", " << row_ << "]" << endl;
 #endif
 	TextItem*  object = new TextItem();
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.size(size_);
 	object->x(column_);
 	object->y(row_);
@@ -444,18 +505,19 @@ void ObsThickness::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) co
 void ObsIdentifier::visit(std::set<string>&) 
 {
 	if ( !attributes_ ) attributes_ = new ObsIdentifierAttributes();
-	if (!attributes_->getVisible()) return;
+	MagLog::debug() << " ObsIdentifier" << *attributes_ << endl;
+	if (!attributes_->visible_) return;
 }
 	
 void ObsIdentifier::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const 
 { 
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	TextItem*  time = new TextItem();
 #ifdef OBS_DEBUG_
-	cout << "Identification for " << point.identifier() << "at[" << column_ << ", " << row_ << "]" << endl;
+	MagLog::debug() << "Identification for " << point.identifier() << "at[" << column_ << ", " << row_ << "]" << endl;
 #endif
 	MagFont font("sansserif");
-	font.colour(attributes_->getColour());
+	font.colour(*attributes_->colour_);
 	font.size(size_);
 	time->x(column_);
 	time->y(row_);
@@ -471,14 +533,14 @@ void ObsIdentifier::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) c
 void ObsPastWeather::visit(std::set<string>& tokens) 
 {
 	if ( !attributes_ ) attributes_ = new ObsPastWeatherAttributes();
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	tokens.insert("past_weather_1");
 	tokens.insert("past_weather_2");
 }
 
 void ObsPastWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const 
 { 
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	CustomisedPoint::const_iterator value = point.find("past_weather_1");
 	if ( value == point.end() ) return;
 	if(value->second > 2)
@@ -486,12 +548,12 @@ void ObsPastWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbol)
 		SymbolItem*  object = new SymbolItem();
 		object->x(column_);
 		object->y(row_);
-		object->colour(attributes_->getColour());
+		object->colour(*attributes_->colour_);
 		ostringstream os;	
 		os << "W_"  << value->second;	
 		object->symbol(os.str());
 #ifdef OBS_DEBUG_
-		cout << "\tPast Weather 1-> " << os.str() << "\n";
+		MagLog::debug() << "\tPast Weather 1-> " << os.str() << "\n";
 #endif
 		object->height(size_);
 		symbol.add(object);
@@ -504,12 +566,12 @@ void ObsPastWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbol)
 			SymbolItem* object2 = new SymbolItem();
 			object2->x(column_*2);
 			object2->y(row_);
-			object2->colour(attributes_->getColour());
+			object2->colour(*attributes_->colour_);
 			ostringstream os2;	
 			os2 << "W_"  << value->second;	
 			object2->symbol(os2.str());
 #ifdef OBS_DEBUG_
-			cout << "\tPast Weather 2-> " << os2.str() << "\n";
+			MagLog::debug() << "\tPast Weather 2-> " << os2.str() << "\n";
 #endif
 			object2->height(size_);
 			symbol.add(object2);
@@ -521,21 +583,28 @@ void ObsPastWeather::operator()(CustomisedPoint& point,  ComplexSymbol& symbol)
 
 void ObsCloud::visit(std::set<string>& tokens) 
 {
+
 	if (!attributes_ ) attributes_ = new ObsCloudAttributes();
-	if (!attributes_->getVisible()) return;
-	if ( attributes_->getLow() )
+	if (!attributes_->visible_) return;
+	if ( attributes_->low_ )
 	{
 		tokens.insert("low_cloud");
 		tokens.insert("low_cloud_nebulosity");
 		tokens.insert("low_cloud_height");
 	}
-	if ( attributes_->getMedium() ) tokens.insert("medium_cloud");
-	if ( attributes_->getHigh() ) tokens.insert("high_cloud");
+	if ( attributes_->medium_ ) tokens.insert("medium_cloud");
+	if ( attributes_->high_ ) tokens.insert("high_cloud");
+	if ( !colour_.empty() )  {
+
+		attributes_->medium_colour_ = auto_ptr<Colour>(new Colour(colour_));
+		attributes_->low_colour_ = auto_ptr<Colour>(new Colour(colour_));
+		attributes_->high_colour_ = auto_ptr<Colour>(new Colour(colour_));
+	}
 }
 	
 void ObsCloud::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const 
 { 
-	if (!attributes_->getVisible()) return;
+	if (!attributes_->visible_) return;
 	
 	CustomisedPoint::const_iterator value = point.find("low_cloud");
 	if ( value != point.end() )
@@ -543,13 +612,13 @@ void ObsCloud::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 		SymbolItem*  cloud = new SymbolItem();
 		cloud->x(lowColumn_);
 		cloud->y(lowRow_);
-		cloud->colour(attributes_->getLow_colour());
+		cloud->colour(*attributes_->low_colour_);
 		ostringstream oss;	
 		int type = int(value->second - (int (value->second/10) *10));
 		oss << "CL_"  << type;
 		cloud->symbol(oss.str());
 #ifdef OBS_DEBUG_
-		cout << "\tLow Cloud--->" << oss.str() << "\n";
+		MagLog::debug() << "\tLow Cloud--->" << oss.str() << "\n";
 #endif
 		cloud->height(size_);
 		symbol.add(cloud);
@@ -588,10 +657,10 @@ void ObsCloud::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 		object->y(lowRow_-1);
 		MagFont font;
 		font.name("sansserif");
-		font.colour(attributes_->getLow_colour());
+		font.colour(*attributes_->low_colour_);
 		font.size(size_*.6);
 #ifdef OBS_DEBUG_
-		cout << "\tLow Cloud Nebulosity--->" << value->second << "\n";
+		MagLog::debug() << "\tLow Cloud Nebulosity--->" << value->second << "\n";
 #endif
 		object->text(nh.str());
 		object->font(font);
@@ -608,14 +677,14 @@ void ObsCloud::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 		cloud->x(mediumColumn_);
 		cloud->y(mediumRow_);
 		
-		cloud->colour(attributes_->getMedium_colour());
+		cloud->colour(*attributes_->medium_colour_);
 		ostringstream oss;
 		int type = int(value->second - (int (value->second/10.) *10));
 		oss << "CM_"  << type;
 		cloud->symbol(oss.str());
-#ifdef OBS_DEBUG_
-		cout << "\tMedium Cloud--->" <<value->second<<" > "<< oss.str() << "\n";
-#endif
+
+		MagLog::debug() << "\tMedium Cloud--->" <<value->second<<" > "<< oss.str() << "\n";
+
 		cloud->height(size_);
 		symbol.add(cloud);
 	}
@@ -626,13 +695,13 @@ void ObsCloud::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
 		SymbolItem*  cloud = new SymbolItem();
 		cloud->x(highColumn_);
 		cloud->y(highRow_);
-		cloud->colour(attributes_->getHigh_colour());
+		cloud->colour(*attributes_->high_colour_);
 		ostringstream oss;	
 		int type = int(value->second - (int (value->second/10) *10));
 		oss << "CH_"  << type;
 		cloud->symbol(oss.str());
 #ifdef OBS_DEBUG_
-		cout << "\tHigh Cloud--->" << oss.str() << "\n";
+		MagLog::debug() << "\tHigh Cloud--->" << oss.str() << "\n";
 #endif
 		cloud->height(size_);
 		symbol.add(cloud);
@@ -649,4 +718,86 @@ void ObsCloud::set(const map<string, string>& def)
 
 	highRow_ = atoi(find(def, "high_row").c_str());
 	highColumn_ = atoi(find(def, "high_column").c_str());
+
+	colour_ = find(def, "colour");
+}
+
+void ObsDemoItem1::visit(std::set<string>& tokens)
+{
+	tokens.insert("temperature");
+}
+
+void ObsDemoItem1::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
+{
+	if ( point.find("temperature") == point.end() ) {
+			MagLog::debug() << "No info for temperature given!" << endl;
+			return;
+		}
+
+		TextItem*  object = new TextItem();
+		object->x(column_);
+		object->y(row_);
+
+		object->text(tostring(point["temperature"]/3));
+		symbol.add(object);
+}
+
+void ObsDemoItem2::visit(std::set<string>& tokens)
+{
+	tokens.insert("info_for_demo2");
+}
+
+void ObsDemoItem2::operator()(CustomisedPoint& point, ComplexSymbol& symbol) const
+{
+	string text;
+	if ( point.find("info_for_demo2") == point.end() )
+			text = attributes_->text_;
+
+	else
+		text=tostring(point["info_for_demo2"]);
+
+	TextItem*  object = new TextItem();
+	object->x(column_);
+	object->y(row_);
+	MagFont font;
+	font.name("sansserif");
+	font.colour(attributes_->text_colour_);
+	font.size(size_);
+	//object->colour(attributes_text_colour_);
+	object->text(text);
+	object->font(font);
+	symbol.add(object);
+}
+
+void ObsEra::set(const map<string, string>& def)
+{
+	key_ = find(def, "key");
+	colour_ = find(def, "colour");
+	row_ = atoi(find(def, "row").c_str());
+	column_ = atoi(find(def, "column").c_str());
+}
+
+void  ObsEra::visit(std::set<string>& tokens)
+{
+	tokens.insert(key_);
+}
+
+void  ObsEra::operator()(CustomisedPoint& point,  ComplexSymbol& symbol) const
+{
+	CustomisedPoint::iterator pt = point.find(key_);
+	if (pt == point.end())
+		return;
+	string text = tostring(point[key_]);
+	TextItem*  object = new TextItem();
+	object->x(column_);
+	object->y(row_);
+	MagFont font;
+	font.name("sansserif");
+	font.colour(colour_);
+	font.size(size_);
+
+	object->text(text);
+	object->font(font);
+	symbol.add(object);
+
 }
