@@ -23,7 +23,7 @@
 
 
 #include "ObsItem.h"
-#include "GeoPoint.h"
+#include "UserPoint.h"
 #include "ObsWindAttributes.h"
 #include "ObsStationRingAttributes.h"
 #include "ObsIdentifierAttributes.h"
@@ -39,7 +39,7 @@
 #include "ObsCloudAttributes.h"
 #include "ObsHeightAttributes.h"
 #include "ObsThicknessAttributes.h"
-
+#include "ObsDemoItem2Attributes.h"
 #include "Symbol.h"
 
 class ObsItemBox : public ObsItem
@@ -48,14 +48,16 @@ public:
 	ObsItemBox() {}
 	~ObsItemBox() {}
 	
-	void set(const map<string, string>& def) 
+	virtual void set(const map<string, string>& def)
 	{   
 		row_ = atoi(find(def, "row").c_str());
 		column_ = atoi(find(def, "column").c_str());
+		colour_ = find(def, "colour");
 	}
 protected:		
 	int row_;
 	int column_;
+	string colour_;
 };
 
 class ObsStationRing : public ObsItemBox
@@ -66,7 +68,7 @@ public:
 	void visit(std::set<string>& tokens)
 	{
 		if ( !attributes_ ) attributes_ = new ObsStationRingAttributes();
-		if (!attributes_->getVisible()) return;
+		if (!attributes_->visible_) return;
 		tokens.insert("latitude");
 		tokens.insert("longitude");
 		tokens.insert("cloud_amount");
@@ -74,13 +76,13 @@ public:
 
 	void operator()(CustomisedPoint&,  ComplexSymbol& symbol) const
 	{ 
-		if (!attributes_->getVisible()) return;
+		if (!attributes_->visible_) return;
 		SymbolItem*  station = new SymbolItem();
 		station->x(column_);
 		station->y(row_);		
-		station->colour(attributes_->getColour());		
+		station->colour(*attributes_->colour_);
 		station->symbol("circle");
-		station->height(getSize());
+		station->height(size_);
 		symbol.add(station);
 	} 
 
@@ -104,15 +106,41 @@ protected:
 };
 
 
-class ObsWind : public ObsItem
+class ObsWind : public ObsItemBox
 {
 public:
 	ObsWind() : attributes_(0) { setOrigins(); }
 	~ObsWind()  {}
+	void visit(std::set<string>& tokens);
+
+	virtual void operator()(CustomisedPoint&,  ComplexSymbol&) const;
+	virtual void set(const map<string, string>& def)
+	{
+		row_ = atoi(find(def, "row").c_str());
+		column_ = atoi(find(def, "column").c_str());
+		colour_ = find(def, "colour");
+		speed_ = find(def, "wind_speed", "wind_speed");
+		direction_ = find(def, "wind_direction",  "wind_direction");
+	}
+protected:
+	string speed_;
+	string direction_;
+	void setOrigins();
+	void print(ostream& out) const { out << "ObsWind";  }
+	ObsWindAttributes* attributes_; // Here we are delegating instead of inheriting ...
+	
+};
+
+class ObsCloudAndWind : public ObsItemBox
+{
+public:
+	ObsCloudAndWind() : attributes_(0) { setOrigins(); }
+	~ObsCloudAndWind()  {}
 	void visit(std::set<string>& tokens)
 	{
 		if (!attributes_ ) attributes_ = new ObsWindAttributes();
-		if (!attributes_->getVisible()) return;
+		if ( !colour_.empty() )  attributes_->colour_ = auto_ptr<Colour>(new Colour(colour_));
+		if (!attributes_->visible_) return;
 		tokens.insert("wind_speed");
 		tokens.insert("wind_direction");
 		tokens.insert("cloud_amount");
@@ -128,6 +156,7 @@ protected:
 };
 
 
+
 class ObsTemperature : public ObsItemBox
 {
 public:
@@ -136,7 +165,8 @@ public:
 	void visit(std::set<string>& tokens)
 	{
 		if ( !attributes_ ) attributes_ = new ObsTemperatureAttributes();
-		if (!attributes_->getVisible()) return;
+		if ( !colour_.empty() )  attributes_->colour_ = auto_ptr<Colour>(new Colour(colour_));
+		if (!attributes_->visible_) return;
 		tokens.insert("temperature");
 	}
 	void operator()(CustomisedPoint&,  ComplexSymbol& symbol) const;
@@ -307,4 +337,48 @@ protected:
 	int highColumn_;
 };
 
+class ObsDemoItem1 : public ObsItemBox
+{
+public:
+	ObsDemoItem1()  {}
+	~ObsDemoItem1()  {}
+
+	void visit(std::set<string>& tokens);
+	void operator()(CustomisedPoint&,  ComplexSymbol&) const;
+
+protected:
+	void print(ostream& out) const { out << "ObsDemoItem1";  }
+
+
+};
+class ObsDemoItem2 : public ObsItemBox
+{
+public:
+	ObsDemoItem2()   { attributes_ = new ObsDemoItem2Attributes(); }
+	~ObsDemoItem2()  {}
+
+	void visit(std::set<string>& tokens);
+	void operator()(CustomisedPoint&,  ComplexSymbol&) const;
+
+protected:
+	void print(ostream& out) const { out << "ObsDemoItem2";  }
+	ObsDemoItem2Attributes* attributes_; // Here we are delegating instead of inheriting ...
+
+};
+
+class ObsEra : public ObsItemBox
+{
+public:
+	ObsEra()   { }
+	~ObsEra()  {}
+	void set(const map<string, string>& def);
+	void visit(std::set<string>& tokens);
+	void operator()(CustomisedPoint&,  ComplexSymbol&) const;
+
+protected:
+	void print(ostream& out) const { out << "ObsEra";  }
+	string key_;
+	string colour_;
+
+};
 #endif
