@@ -23,6 +23,7 @@
 #include "ObsItem.h"
 #include "PaperPoint.h"
 #include "BasicSceneObject.h"
+#include "ObsTableAttributes.h"
 
 namespace magics {
 
@@ -65,7 +66,7 @@ protected:
 		{ p.print(s); return s; }
 };
 
-class ObsTable : public map<string, ObsTemplate*>
+class ObsTable : public map<string, ObsTemplate*>, public ObsTableAttributes
 {
 public:
 	ObsTable();
