@@ -3,15 +3,16 @@
 use XML::Parser;
 use Time::localtime;
 
-my $file = shift;
-my $dir = shift;
+my $file  = shift;
+my $dir   = shift;
+my $nosubdir = shift;
+
 my $Author="MagicsTeam";
 my $ecmwf="http://wms.ecmwf.int";
 my $info = {};
 my $element = {};
 
 
-
 my %basetype = (
     "int" => 1,
     "float" => 1,
@@ -24,11 +25,14 @@ my %basetype = (
     "ListPolicy" =>1,
     "Hemisphere" =>1,
     "ArrowPosition" => 1,
+    "AxisAutomaticSetting" => 1,
     "Justification" => 1,
     "DisplayType" => 1,
     "OpenGLDriverObserverPtr" => 1,
     "Widget" =>1, 
     "cairo_t*" =>1,
+    "QWidget*" =>1,
+    "QGraphicsScene*" => 1,
     "Matrix" => 1,
     "GribHandlePtr" =>1,  
 );
@@ -36,12 +40,20 @@ my %basetype = (
 my %translator = (
     "int" => "ParameterManager::getInt",
     "float" => "ParameterManager::getDouble",
-    "string" => "ParameterManager::getString"
+    "string" => "ParameterManager::getString",
+    "stringarray" => "ParameterManager::getStringArray",
+    "bool" => "ParameterManager::getBool",
+    "doublearray" => "ParameterManager::getDoubleArray",
+    "floatarray" => "ParameterManager::getDoubleArray",
+    "intarray" => "ParameterManager::getIntArray"
     
 );
 my %magtype = (
 	"float" =>'double',
 );
+my %consttype = (
+	"string" =>'const string&',
+);
 
 my %classtype = (
 	"Colour" =>1,
@@ -123,10 +135,18 @@ foreach my $object (keys %{$info->{magics}->{class}})
     $directory = $info->{magics}->{class}->{$object}->{attributes}->{directory};
     $tag = $info->{magics}->{class}->{$object}->{attributes}->{xmltag};
 
+    my $location;
+    if( defined $nosubdir and $nosubdir eq "nosubdir" ) { 
+        $location  = $dir; 
+    }
+    else { 
+        $location  = "$dir/$directory"; 
+    }
+        
 ######################################################
 #####                Include file
 
-    open STDOUT, ">$dir/$directory/$object\Attributes.h";
+    open STDOUT, ">$location/$object\Attributes.h";
 
     my $string =  ctime();
     print  <<EOF;
@@ -138,7 +158,7 @@ foreach my $object (keys %{$info->{magics}->{class}})
 
 /*! \\file $object\Attributes.h
     \\brief Definition of $object Attributes class.
-    \\author Graphics Section, ECMWF
+    \\author Meteorological Visualisation Section, ECMWF
 
     This file is automatically generated.
     Do Not Edit!
@@ -238,84 +258,17 @@ public:
     void setTag(const string& tag) { tag_ = tag; }
 
 EOF
-    my $xml_data = $current->{attributes}->{xml_data};
-    if ($xml_data eq '' ) {
-        print <<EOF;
-    void setXmlData(const string&)  {}
-EOF
-    }
-    else {
-    	my $to = $current->{parameter}->{$xml_data}->{attributes}->{to};
-    	my $member = $current->{parameter}->{$xml_data}->{attributes}->{member};
-    	if ( $arraytype{$to} ) {    		
-    		print("\tvoid setXmlData(const string& data)  { $member\_.push_back($arraytype{$to}); } \n");
-    	}
-    	else {
-    		print("\tvoid setXmlData(const string& data)  { $member\_ = data; }\n");
-    	}
-    }
-
-    foreach  my $param (@{$current->{parameter_list}}) 
-    {      
-        my $todo = $current->{parameter}->{$param}->{attributes}->{implemented};
-        next if $todo eq 'no';
-        my $name = $current->{parameter}->{$param}->{attributes}->{name};
-        my $default = $current->{parameter}->{$param}->{attributes}->{default};
-        my $member = $current->{parameter}->{$param}->{attributes}->{member};
-        my $from = $current->{parameter}->{$param}->{attributes}->{from};
-        my $to = $current->{parameter}->{$param}->{attributes}->{to};
-        my $tpl = $current->{parameter}->{$param}->{attributes}->{template};
-        $to = "$to<$tpl> " unless $tpl eq "";
-       
-        my $value = lc $member;
-        my $method = ucfirst $member;
-        
-          my $dox = "\\brief $current->{parameter}->{$param}->{documentation}->{data}";
-        
-        foreach my $doc (@{$current->{parameter}->{$param}->{documentation_list}}) 
-        {
-         	 $dox = "$dox  $doc xxx $current->{parameter}->{$param}->{documentation}->{$doc}->{data}";
-        }
-        
-        print <<EOF;
-/*!
-    Magics Information:
-    $dox
-    \\param $value Setting $name - default=$default
-*/
-EOF
-    my $magto = $to;
-    $magto = $magtype{$magto} if $magtype{$magto};
     
 
-    if ( $basetype{$to} ) 
-    { 
-        print <<EOF;   
-	void set$method($magto $value) 
-		{ $member\_ =  $value; } 
-	$magto get$method() const 
-		{ return $member\_; } 
-        
-EOF
-    }
-    else
-    {
-        print <<EOF;   
-	void set$method($to* $value) 
-		{ auto_ptr<$to> tmp($value); $member\_ = tmp; } 
-	const $to&  get$method() const 
-		{ return *$member\_; }
-EOF
-    }
-    }
-print <<EOF;
-	virtual void toxml(ostream& out, int tabs = 0) const { toxml(out, tag_, tabs); }
 
-protected:
+    
+print <<EOF;
+	
+
+public:
 	//  --  method
 	virtual void print(ostream&) const;
-	virtual void toxml(ostream&, const string&, int tabs) const;
-
+	virtual void toxml(ostream& out) const;
 	//  --  members:
 	string tag_;
 EOF
@@ -369,7 +322,7 @@ EOF
 ######################################################
 #####                Source file
 
-    open STDOUT, ">$dir/$directory/$object\Attributes.cc";
+    open STDOUT, ">$location/$object\Attributes.cc";
 
     print  <<EOF;
 /*******************************  LICENSE  *******************************
@@ -379,8 +332,8 @@ EOF
  *******************************  LICENSE  *******************************/
 
 /*! \\file $object\Attributes.cc
-    \\brief Implemtation of $object Attributes class.
-    \\author Graphics Section, ECMWF
+    \\brief Implementation of $object Attributes class.
+    \\author Meteorological Visualisation Section, ECMWF
 
     This file is automatically generated.
     Do Not Edit!
@@ -391,7 +344,7 @@ EOF
 #include "$object\Attributes.h"
 #include "MagicsParameter.h"
 #include "Factory.h"
-#include "Translator.h"
+#include "MagTranslator.h"
 #include "XmlNode.h"
 #include "ParameterSettings.h"
 
@@ -450,7 +403,7 @@ EOF
                 print "$sep\n\t$member\_($translator{$to}(\"$default\"))";
             }
             else {
-        	    print "$sep\n\t$member\_(Translator<$from, $magto>()(\"$default\"))"; 
+        	    print "$sep\n\t$member\_(MagTranslator<$from, $magto>().magics(\"$default\"))"; 
             }
         } 
         else {
@@ -459,7 +412,7 @@ EOF
                 print "$sep\n\t$member\_($translator{$to}(\"$name\"))";
             }
             else {
-        	    print "$sep\n\t$member\_(Translator<$from, $magto>().magics(\"$name\"))"; 
+        	    print "$sep\n\t$member\_(MagTranslator<$from, $magto>().magics(\"$name\"))"; 
             }
         }
 		$sep = ",";
@@ -485,8 +438,8 @@ EOF
         my $magto = $to;
         $magto = $magtype{$to} if $magtype{$to};
         $from = $magtype{$from} if $magtype{$from};
-        print "\n\tTranslator<string, $magto> $member;\n";
-        print "\tset$method($member.magics(\"$name\"));\n";
+	
+        print "\tset$method(MagTranslator<string, $magto>()(\"$name\"));\n";
    }
    print <<EOF;
 } 
@@ -586,14 +539,14 @@ EOF
         }
         else 
         {
-           print "\tset$method(other.$member\_->clone());\n";          
+           print "\t$member\_ = auto_ptr<$to>(other.$member\_->clone());\n";          
         }
        
         print "";
        
         
     }
-     print  <<EOF;
+print  <<EOF;
 } 
 
 $header
@@ -711,64 +664,47 @@ EOF
         
     print "\tout << \"]\" << \"\\n\";\n";
     print "}\n\n";
-    print <<EOF; 
+
+     print <<EOF; 
 $header
-void $object\Attributes$p\::toxml(ostream& out, const string& tag, int tabs)  const
+void $object\Attributes$p\::toxml(ostream& out)  const
 {
-	string tab;
-	for ( int t = 0; t < tabs; t++) tab = tab + "\t";
-	
-	out << tab << "<" << tag << "\\n";
+     out <<  \"\\\"$tag\\\"\";
 EOF
-    $sep = " ";
-     my $calls = "";
     
     foreach  my $param (@{$current->{parameter_list}}) 
-    {
-        my $todo = $current->{parameter}->{$param}->{attributes}->{implemented};
-        next if $todo eq 'no';
+    {      
+         my $todo = $current->{parameter}->{$param}->{attributes}->{implemented};
+       next if $todo eq 'no';
         my $name = $current->{parameter}->{$param}->{attributes}->{name};
-        my $default = $current->{parameter}->{$param}->{attributes}->{default};
         my $member = $current->{parameter}->{$param}->{attributes}->{member};
-        my $from = $current->{parameter}->{$param}->{attributes}->{from};
         my $to = $current->{parameter}->{$param}->{attributes}->{to};
         my $tpl = $current->{parameter}->{$param}->{attributes}->{template};
-        my $xml = $current->{parameter}->{$param}->{attributes}->{xml};
-         $xml = $name;
-         my $xml_data = $current->{attributes}->{xml_data};
+       
+       
         $to = "$to<$tpl> " unless $tpl eq "";
-        my $magto = $to;
-        my $method = ucfirst $member;
-        $magto = $magtype{$to} if $magtype{$to};
-        $from = $magtype{$from} if $magtype{$from};
-       
-        if ( $basetype{$to}  ) 
+        if ( $basetype{$to} ) 
         {         	
-             print "\tout << tab << \"\\t$xml = \\\'\" << $member\_ << \"\\\'\\n\";\n"; 
+             print "\t out << \", \\\"$name\\\":\";\n";
+             print "\t niceprint(out,$member\_);\n";
         }
-        elsif ( $classtype{$to} ) {
-        	print "\tout << tab << \"\\t$xml = \\\'\" << *$member\_ << \"\\\'\\n\";\n"; 
+        elsif ( $classtype{$to}  )
+		{
+             print "\t out << \", \\\"$name\\\":\";\n";
+             print "\t niceprint(out, *$member\_);\n";
+		}
+		else 
+        {
+            print "\t out << \", \\\"$name\\\":\";\n";
+            print "\t $member\_->toxml(out);\n";       
         }
-        else {
-        	 $calls = "$calls \t$member\_->toxml(out, tabs+1);\n";
-        	
-        }
-       
         
-      
-        print "";
-
+        
+        
     }
-    print "\tout << tab << \">\";\n\n";
-    print $calls;
-    
-    print "\n\tout << tab << \"</\" << tag << \">\\n\";\n";
-
-  
-        
-    
-    print "}\n\n";
+    print "}\n";       
     
+   
     foreach  my $param (@{$current->{parameter_list}}) 
     {     
          my $todo = $current->{parameter}->{$param}->{attributes}->{implemented};
@@ -821,17 +757,24 @@ EOF
         	my $xml = $current->{parameter}->{$param}->{option}->{$option}->{attributes}->{xml};
         	my $base = $current->{parameter}->{$param}->{attributes}->{to};  
         	my $templates = $current->{parameter}->{$param}->{option}->{$option}->{attributes}->{template};
+			my @fortrans = split("/", $fortran);
         	if ($template eq "") {
-        	print "static SimpleObjectMaker<$class, $base> $fortran\_$class(\"$fortran\");\n" if $fortran ne "";
-        	print "static SimpleObjectMaker<$class, $base> $xml\_$class(\"$xml\");\n" if $xml ne "" && $xml ne $fortran;
+        		for my $o (@fortrans) {
+        			print "static SimpleObjectMaker<$class, $base> $o\_$class(\"$o\");\n" if $o ne "";
+        			print "static SimpleObjectMaker<$class, $base> $xml\_$class(\"$xml\");\n" if $xml ne "" && $xml ne $fortran;
+				}
         	}
         	else {
         		my @templates = split("/", $template);	
         		for my $t (@templates) {
-        			print "static SimpleObjectMaker<$class<$t>, $base<$t> > $fortran\_$class\_$t(\"$fortran\");\n" if $fortran ne "";
-        	        print "static SimpleObjectMaker<$class<$t>, $base<$t> > $xml\_$class\_$t(\"$xml\");\n" if $xml ne "" && $xml ne $fortran;
+        			for my $o (@fortrans) {
+        				print "static SimpleObjectMaker<$class<$t>, $base<$t> > $o\_$class\_$t(\"$o\");\n" if $o ne "";
+        	        	print "static SimpleObjectMaker<$class<$t>, $base<$t> > $xml\_$class\_$t(\"$xml\");\n" if $xml ne "" && $xml ne $fortran;
+					}
         		}
         	}
         }
     }
 }
+
+
