@@ -1,12 +1,10 @@
 #
-# Package name and versioning information for Magics++
+# Package name and versioning information for Magics
 #
-#  DO NOT FORGET to change MagPlus.dox !!!!
-#
-MAGICS_PACKAGE_NAME='Magics++'
-MAGICS_PACKAGE_VERSION='2.10.0'
-MAGICS_PACKAGE_COPYRIGHT_YEAR='2010'
-MAGICS_PACKAGE_COPYRIGHT_PERIOD='2007-2010'
+MAGICS_PACKAGE_NAME='Magics'
+MAGICS_PACKAGE_VERSION='2.18.15'
+MAGICS_PACKAGE_COPYRIGHT_YEAR='2013'
+MAGICS_PACKAGE_COPYRIGHT_PERIOD='2007-2013'
 MAGICS_PACKAGE_RELEASE_DATE=`date +%d-%B-%Y`
 #
 #
